module(..., package.seeall)
--just helps
local helps = {
   _H = "webJobs",
   _H = "pleaseWait",
   _H = "runFirst",
   _H = "Welcome to NSG Linux",
}

local nsg = require'nsg'
local mod_scheme = require"scheme"
local deepCopy = nsg.deepcopy
local clonePath = mod_scheme.clonePath
local ser = nsg.serialize
local utf8 = require"utf8"
local locale = "en"
localeString =  mod_scheme.localeString
local safe
local ifThen
local fstr
local genDir
local genParam
local isExpanded
local expandedOrCollapsed
local normalOrDisabled
local genLiClass
local genParamName
local genParamValue
local genParamControls
local genDirControls
local genDirBlock
local genDirItem
local htmlConf
local mainPage
local htmlEncode
local escapeForScript
local tag
local replaceNil
local toTypeString
local findSchemeNode
local findSchemeKey
local hint
local btnSave
local btnEditSave
local btnValidate
local btnManual
local btnHTML
local btnEdit
local btnUndo
local btnRefresh
local btnApply
local btnClearDir
local btnDisable
local btnNewItem
local btnCopyItem
local genCursor
local btnDelete
local btnExecute
local genInputString
local genInputChoice
local genInputBoolean
local strongValue
local genInput
local splitString

splitString =
   function (s)
      s = s or ""
      local out = {}
      local i = 70
      local i2 = 0
      local len = utf8.len(s) or 0
      while i2<len do
         out[#out+1] = utf8.sub(s,i2 + 1, i2 + i)
         i2 = i2 + i
      end
      return out
   end

escapeForScript =
   function (s)
      s = splitString(s)
      for k,v in ipairs(s) do
         s[k] = '"'..((((((tostring(v or "" )):gsub("\\","\\\\")):gsub("\r","\\r")):gsub("\n","\\n")):gsub('"','\\"')):gsub("'","\\'") or "")..'"'
      end
      s[#s+1] = '""'
      return table.concat(s,"+")
   end

htmlEncode =
   function(s)
      if type(s) == 'boolean' then
         return tostring(s)
      end
      return ((((tostring(s) or ""):gsub( "&", "&amp;")):gsub("\"", "&quot;")):gsub("<", "&lt;")):gsub(">", "&gt;")
   end

tag =
   function ( shift, tag, param, inner, ret_inner)
      if(tag > "") then
         local white=""
         local end_white=""

         if(type(ret_inner) == 'nil') then
            ret_inner= "\n"
         else
            if(ret_inner ~= "" ) then
            end
         end

         while (shift > 0 ) do
            shift=shift - 1
            white = white.."  "
         end
         end_white=white

         if(ret_inner == "" ) then
            end_white=""
         end

         if(inner ~= "") then
            inner = fstr(ret_inner,inner)
         else
            inner = ""
         end

         return fstr(white, "<", tag," ", param,">",
                     inner, ret_inner, end_white, "</", tag, ">")
      else
         return inner
      end
   end

replaceNil =
   function (a, ...)
      if(#{...} == 0) then
         return a
      else
         return (a or ""), replaceNil(...)
      end
   end

fstr =
   function (...)
      return table.concat({replaceNil(...)})
   end


ifThen =
   function (test, ontrue, onfalse)
      if(test) then
         return ontrue
      else
         return onfalse
      end
   end

toTypeString =
   function (...)
      local out = {}
      for k,v in pairs({...}) do
         table.insert(out,
                      fstr(tostring(k),
                           ":",
                           type(v),
                           "(",
                           ifThen(type(v)=='table',ser(v or {}),tostring(v))
                           ,")"
                        )
                   )
      end
      return table.concat(out,", ")
   end

local debugBuf = ""
local lastRet = nil

xsafe =
   function (func, funcName, ...)
      return func(...)
   end

safe =
   function (func, funcName, ...)
      local res = { pcall(func, ...) }
      if res[1] then
         table.remove(res,1)
         lastRet = res
         return unpack(res)
      else
         return error(
            fstr(funcName,
                 "=",
                 type(func),
                 "(",
                 toTypeString(...),
                 ")-><br>lastRet=",
                 toTypeString(lastRet),
                 "<br>",
                 toTypeString(debugBuf),
                 "<br>",
                 tostring(res[2])
              )
         )
      end
   end

hint =
   function (header, body, delay, ext)
      out = {}
      out[#out+1] = " title=\""
      out[#out+1] = fstr("header=[", htmlEncode(tostring(header)), "] ")

      if( body ) then
         out[#out+1] = fstr("body=[", htmlEncode(tostring(body)), "] ")
      else
         out[#out+1] = "body=[]"
      end
      if( delay ) then
         out[#out+1] = fstr("delay=[", htmlEncode(tostring(delay)), "] ")
      end
      for k,v in pairs(ext or {}) do
         out[#out+1] = fstr(tostring(k), "=[", htmlEncode(tostring(v)), "] ")
      end
      out[#out+1] = "\" "
      return table.concat(out)
   end

btnSave =
   function (path)
      local _H = "webBtnSave"
      return fstr("<img onkeydown=\"return btnKeyDown(event,this);\" tabindex=0 src=\"/nil.png\" alt=\"Save\" class=\"bsave\" ",
                  "onclick=\"return saveCfg(this);\"",
                  hint(_H),
                  ">")
   end

btnEditSave =
   function (path)
      local _H = "webBtnEditSave"
      return fstr("<img onkeydown=\"return btnKeyDown(event,this);\" tabindex=0 src=\"/nil.png\" alt=\"Save\" class=\"beditsave\" ",
                  "onclick=\"return saveEditCfg(this);\"",
                  hint(_H),
                  ">")
   end

btnValidate =
   function (path)
      local _H = "webBtnValidate"
      return fstr("<img onkeydown=\"return btnKeyDown(event,this);\" tabindex=0 src=\"/nil.png\" alt=\"Validate\" class=\"bvalidate\" ",
                  "onclick=\"return validateCfg(this);\"",
                  hint(_H),
                  ">")
   end

btnManual =
   function (path)
      local _H = "webBtnManual"
      return fstr("<img onkeydown=\"return btnKeyDown(event,this);\" tabindex=0 src=\"/nil.png\" alt=\"Man\" class=\"bmanual\" ",
                  "onclick=\"return getManual(this);\"",
                  hint(_H),
                  ">")
   end

btnHTML =
   function (path, scheme)
      if scheme["_HTML"] then
         local _H = "webBtnHTML"
         return fstr("<img onkeydown=\"return btnKeyDown(event,this);\" tabindex=0 src=\"/nil.png\" alt=\"Html\" class=\"bhtml\" ",
                     "onclick=\"return getHTML(this,'"..htmlEncode(scheme["_HTML"]).."');\"",
                     hint(_H),
                     ">")
      end
      return ""
   end

btnEdit =
   function (scheme,path)
      local _H = "webBtnEdit"
      return fstr("<img onkeydown=\"return btnKeyDown(event,this);\" tabindex=0 src=\"/nil.png\" alt=\"Edit\"  class=\"bedit\" ",
                  "onclick=\"return Edit(this);\"",
                  hint(_H),
                  ">")
   end

btnUndo =
   function (scheme,path)
      local _H = "webBtnUndo"
      return fstr("<img onkeydown=\"return btnKeyDown(event,this);\" tabindex=0 src=\"/nil.png\" alt=\"Edit\" class=\"bedit\"",
                  "onclick=\"return Edit(this);\"",
                  hint(_H),
                  ">")
   end

btnRefresh =
   function (scheme, path)
      local _H = "webBtnRefresh"
      return not (scheme._T or scheme._X) and fstr("<img onkeydown=\"return btnKeyDown(event,this);\" tabindex=0 src=\"/nil.png\" alt=\"Refresh\"  class=\"brefresh\" ",
                                                   "onclick=\"return loadDir(this);\"",
                                                   hint(_H),
                                                   ">") or ""
   end

btnApply =
   function (scheme, path)
      local _H = "webBtnApply"
      return fstr("<img onkeydown=\"return btnKeyDown(event,this);\"  tabindex=0 src=\"/nil.png\" alt=\"Apply\" class=\"bapply\" ",
                  "onclick=\"applyCfg(this); return false;\"",
                  hint(_H),
                  ">")
   end

btnClearDir =
   function ()
      local _H = "webBtnClearDirConfirm"
      local _H = "webBtnClearDir"
      return fstr("<img onkeydown=\"return btnKeyDown(event,this);\"  tabindex=0 src=\"/nil.png\" alt=\"Clear\" class=\"bclear\" ",
                  hint(_H),
                  " onclick=\"return clearDir(this);\">"
               )
   end

btnDisable =
   function ()
      local _H = "webBtnDisable"
      return fstr("<img onkeydown=\"return btnKeyDown(event,this);\"  tabindex=0 src=\"/nil.png\" alt=\"Disable\" class=\"bdisable\" ",
                  hint(_H),
                  " onclick=\"return setNil(this);\">"
               )
   end


btnNewItem =
   function (scheme, conf, path)
      local _H = "webDlgAddName"
      local _H = "webDlgAddList"
      local _H = "webBtnAdd" 
      local hiddenItems = {}
      if (scheme._N )then
         hiddenItems = scheme._N(scheme, conf or {}, path or {}) or {}
      end

      if (mod_scheme.getSubScheme(mod_scheme.clonePath(path, nil, "_NAME"), scheme) and  hiddenItems[1] == hiddenItems._NAME) or
      (mod_scheme.getSubScheme(mod_scheme.clonePath(path, nil, "_LIST"), scheme)  and  hiddenItems[1] == hiddenItems._LIST)
   then
      if (scheme._NAME) then
         return fstr("<img onkeydown=\"return btnKeyDown(event,this);\"  tabindex=0 src=\"/nil.png\" alt=\"New\" class=\"bnew\" id=\"",
                     htmlEncode("root."..table.concat(path, ".")),
                     "\"",
                     hint(_H),
                     "onclick=\"return newItem(this,getHelp('",
                     (scheme._NAME_H or "webDlgAddName"),
                     "'),'newKey');\">")
      elseif (scheme._LIST) then
         return fstr("<img onkeydown=\"return btnKeyDown(event,this);\" tabindex=0 src=\"/nil.png\" alt=\"New\" class=\"bnew\" id=\"",
                     htmlEncode("root."..table.concat(path, ".")),
                     "\" ",
                     hint(_H),
                     " onclick=\"return newItem(this,getHelp('webDlgAddList'),'insertKey');\">")
      else
         return ""
      end
   else
      return ""
   end
end

btnCopyItem =
   function ()
      return  fstr("<img onkeydown=\"return btnKeyDown(event,this);\" tabindex=0 src=\"/nil.png\" alt=\"New\" class=\"bnew\"",
                   hint("webBtnCopy"),
                   " \"onclick=\"return CopyItem(this);\">");
   end

genCursor =
   function (scheme)
      local _H = "webBtnExpand"
      local _H = "webBtnCollapse"
      if  scheme and (((not scheme._T)) or  (scheme._X) or (scheme._T == 'set'))  then
         return fstr("<img onkeydown=\"return btnKeyDown(event,this);\" tabindex=0 class=\"bexpand\" alt=\"Expand\" src=\"/nil.png\"",
                     hint("webBtnExpand"),
                     "onclick=\"UnHide(this); return false;\">",
                     "<img onkeydown=\"return btnKeyDown(event,this);\" tabindex=0 class=\"bcollapse\" alt=\"Collapse\" src=\"/nil.png\"",
                     hint("webBtnCollapse"),
                     "onclick=\"UnHide(this); return false;\">")
      else
         return nil
      end
   end

btnDelete =
   function ()
      local _H = "webDlgDelete"
      return fstr("<img onkeydown=\"return btnKeyDown(event,this);\"  tabindex=0 src=\"/nil.png\" alt=\"Delete\" class=\"bdel\" ",
                  hint(_H),
                  "onclick=\"return deleteItem(this,getHelp('webDlgDelete'));\">")
   end

btnExecute =
   function (scheme)
      local _H = "webBtnRun"
      return fstr("<img  onkeydown=\"return btnKeyDown(event,this);\" tabindex=0 src=\"/nil.png\" alt=\"Execute\" class=\"bexec\" ",
                  hint(_H),
                  ifThen(scheme._XI,
                         "onclick=\"return executeTask(this,'interactive');\">",
                         "onclick=\"return executeTask(this,'');\">"),
                  ""
               )
   end

genInputString =
   function (value, stype, params, default, X, EX)
      local text = (((tostring(value or default):match("\\n")or tostring(value or default):match("\\r"))~= nil) and {}) or (EX and EX.TEXT)
      local big = (stype=="string" and #(tostring(value or default) or "") > 15) and "style=\"width:80ex;\"" or ""
      if text then
         return fstr("<textarea class=\"inline\" rows=\""..(text.ROWS or "10").."\" cols=\""..(text.COLS or "80").."\" wrap=\""..(text.WRAP or "off").."\" ",
                     ifThen(X, "onblur=\"inLineOnBlur(this, 'normal');\"",
                            "onblur=\"inLineOnBlur(this); setValue(this);\" "),
                     " onkeydown=\"inLineKeyDown(event, this);\" ",
                     ">",
                     htmlEncode(loadstring("return "..(value or default or '""'))()),
                     "</textarea>")
      else
         return fstr("<input type=text class=\"inline\" ",
                     big,
                     "  value=\""..htmlEncode(value or default or ""),
                     "\" ",
                     ifThen(X, "onblur=\"inLineOnBlur(this, 'normal');\"",
                            "onblur=\"inLineOnBlur(this); setValue(this);\" "),
                     " onkeydown=\"inLineKeyDown(event, this);\" ",
                     ">")
      end
   end

genInputChoice =
   function (value, stype, params, default, X)
      local choice = fstr("<select class=\"inline\" name=\"TODOgenInputChoice\" ",
                          ifThen(X, " onblur=\"inLineOnBlur(this, 'normal');\"", " onblur=\"inLineOnBlur(this);setValue(this);\""),
                          " onkeydown=\"inLineKeyDown(event, this);\" ",
                          ">");
      for k,v in pairs(params or {}) do
         local Help =(type(v) == "table" and hint(tostring(v[2]))) or ""
         v = tostring((type(v) == "table" and v[1]) or v)
         choice = fstr(choice, "<option value=\"", htmlEncode(v), "\" ",Help," ")
         if(v == tostring(value or default or "")) then
            choice = choice.."selected";
         end
         if(type(v) == 'string') then
            choice = fstr(choice, ">", v, "</option>")
         elseif (type(v) == 'table') then
            choice = fstr(choice, ">", v, "</option>")
         end
      end
      choice = choice.."</select>"
      return choice
   end

genInputBoolean =
   function (value, stype, params, default, X)
      return fstr("<select name=\"TODOgIB\" ",
                  " class=\"inline\" ",
                  ifThen(X, " onblur=\"inLineOnBlur(this, 'normal');\""," onblur=\"inLineOnBlur(this);setValue(this);\""),
                  " onkeydown=\"inLineKeyDown(event, this);\" ",
                  ">",
                  "<option value=\"true\"  ",
                  ifThen("true"  == tostring(value or default), "selected", "")," >true </option>",
                  "<option value=\"false\" ",
                  ifThen("false" == tostring(value or default or "false"), "selected", "")," >false</option>",
                  "</select>"
               )
   end

local specChar = {
   ["\a"] = "\\a",
   ["\b"] = "\\b",
   ["\t"] = "\\t",
   ["\r"] = "\\r",
   ["\n"] = "\\n",
   ["\v"] = "\\v",
   ["\f"] = "\\f"
}

strongValue =
   function (conf, scheme)
      local val = conf
      if conf == nil then
         if scheme._D == nil then
            if scheme._T == "boolean" then
               val =  "false"
            else
               val = ""
            end
         else
            if scheme._T == "boolean" then
               val =  tostring(scheme._D)
            else
               val = scheme._D
            end
         end
      end
      if type(conf or scheme._D) == "string" and (scheme._T ~= "choice") then
         val = ("%q"):format(val):gsub("\\\n","\\n"):gsub("[%c%z]", function (char) return specChar[char] or ("\\%.3d"):format((char:byte())) end)
      end
      return tostring(val)
   end


genInput =
   function (conf, scheme)
      local value=strongValue(conf, scheme)
      local stype=scheme._T
      local params=scheme._A
      local defvalue=strongValue(scheme._D, scheme)
      local X = (scheme and (scheme._X or scheme._XI)) ~= nil
      if (stype == 'choice') then
         return safe(genInputChoice, "genInputChoice", value, stype, params, defvalue, X)
      elseif (stype == 'boolean') then
         return genInputBoolean(value, stype, params, defvalue, X)
      else -- if(stype == 'string' or stype == 'integer'
         --or stype == 'number' or stype == 'ipaddr') then
         stype = 'string'
         return genInputString(value, stype, params, defvalue, X, scheme._EX)
      end
   end

genParam =
   function (scheme, conf, path, key, schemeKey, level)
      if(scheme and scheme[schemeKey]) then
         if(scheme[schemeKey]._T == "set") then
            html = safe(genParamSet,"genParamSet", scheme, conf, path, key, schemeKey)
         else
            html = safe(genParamSimple,"genParamSimple", scheme, conf, path, key, schemeKey)
         end
         return html
      else
         return "Error:scheme["..tostring(scheme).."], schemeKey["..tostring(schemeKey).."]."
      end
   end

isExpanded =
   function (htmlEnv, path)
      if htmlEnv and (htmlEnv.data[
                         fstr(
                            "LC",
                            path[-1],
                            ".",
                            ifThen((type(path[#path])== 'string'), ("%q"):format(path[#path]), tostring(path[#path]))
                         )
                      ])then
         return true
      else
         return false
      end
   end

expandedOrCollapsed =
   function (htmlEnv, path)
      return isExpanded(htmlEnv, path) and "\"ex\"" or "\"cl\""
   end

normalOrDisabled =
   function (conf, scheme)
      if (not scheme._T) or (conf ~= nil) then
         return "\"normal\""
      else
         return "\"disabled\""
      end
   end

genLiClass =
   function (scheme, conf,  path, htmlEnv, level)
      local name
      if type(path[#path])== 'string' then
         name = htmlEncode(("%q"):format(path[#path]))
      else
         name = tostring(path[#path])
      end
      return fstr("class=",
                  expandedOrCollapsed(htmlEnv, path, level),
                  " name =\"",
                  name,
                  "\"")
   end

genParamName =
   function (htmlEnv, scheme, schemeParent, conf, path, level)
      local _H = "webBtnKey"
      if (not scheme._T) then
         return fstr(ifThen(scheme['_DEP'] and scheme._X,
                            fstr("<img src=\"/nil.png\" alt=\"!\" class=\"bkey\"",
                                 hint(_H),
                                 ">"
                              ),
                            ""
                         ),
                     "<span class=\""..(scheme._X and "exec" or "dir").."\">",
                     "<a class=\"name\" href=\"javascript:void(0)\" onkeydown=\"paramKeyDown(event, this)\" onclick=\" UnHide(this); return false;\">",
                     htmlEncode(path[#path]),
                     "</a>",
                     "</span>",
                     "<input class=\"value\" type=hidden name=\"",
                     htmlEncode(path[#path]),
                     "\" value=\"",
                     htmlEncode(path[#path]),
                     "\">",
                     "<input class=\"stype\" type=hidden value=\"dir\">",
                     "<input class=\"conf\" type=hidden value=\"",
                     htmlEncode(ser(((conf ~= nil) and conf) or ((conf == nil) and"{}"))),
                     "\">",
                     btnManual(),
                     btnHTML(path, scheme))
      else
         return fstr(ifThen(scheme['_DEP'],
                            fstr("<img src=\"/nil.png\" alt=\"!\" class=\"bkey\" ",
                                 hint(_H),
                                 ">"
                              ),
                            ""
                         ),
                     "<span class=\""..(scheme._X and "exec" or "parm").."\">",
                     "<a class=\"name\" href=\"javascript:void(0)\" onkeydown=\"paramKeyDown(event, this)\" onclick=\"",ifThen(scheme._X,"UnHide(this);",""),"return false;\">",
                     htmlEncode(path[#path]),
                     "</a>",
                     "</span>",
                     (scheme._X and "<input class=\"loaded\" type=hidden value=\"no\">" or ""),
                     btnManual(),
                     btnHTML(path, scheme))
      end
   end

genParamValue =
   function (scheme, schemeParent,  conf, path, level)
      local _H = "webEditHelp"
      local _h
      if scheme._T == 'choice' and scheme._A then
         for k,v in ipairs(scheme._A) do
            if type(v)=='table' and v[1]== (conf or scheme._D) and v[2] then
               _h=v[2]
               _H,_h = _h, _H
               break;
            end
         end
      end

      if (not scheme._T) then
         --_LIST and _NAME have deps by there nature and _DEP  is equal 0 by def for subitems
         return ifThen(scheme['_DEP'] or scheme["_LIST"] or scheme["_NAME"], "<input class=\"dep\"  type=hidden value=\""..tostring(scheme['_DEP'] or 0 ).."\">", "")
      else
         local big = (scheme._T and scheme._T=="string" and #(tostring(conf or scheme._D) or "") > 15) and "style=\"width:80ex;\"" or ""
         return fstr("<span class="..normalOrDisabled(conf, scheme)..">",
                     safe(genInput, "genInput", conf, scheme),
                     "<input class=\"value\" ",big," type=text name=\"",
                     htmlEncode(path[#path]),
                     "\" value=\"",
                     htmlEncode(strongValue(conf,scheme)),
                     "\"  readonly ",
                     " ondblclick=\"startEdit(this); return false;\" ",
                     " onkeydown=\"valueKeyDown(event, this)\" ",
                     hint(_H, _h),
                     ">",
                     "<input class=\"lastvalue\" type=hidden value=\"",
                     htmlEncode(strongValue(conf,scheme)),
                     "\" ",
                     ifThen(conf ~= nil, "", "disabled"),">",
                     "<input class=\"defvalue\"  type=hidden value=\"",
                     htmlEncode(strongValue(scheme._D, scheme)),
                     "\" ",
                     ifThen(scheme._D, "", "disabled"),
                     ">",
                     "<input class=\"stype\"     type=hidden value=\"",
                     htmlEncode(scheme._T),
                     "\">",
                     ifThen(scheme['_DEP'], "<input class=\"dep\"  type=hidden value=\""..tostring(scheme['_DEP']).."\">", ""),
                     "</span>"
                  )
      end
   end

genParamControls =
   function (scheme, schemeParent, cfg, path)
      if (schemeParent and (schemeParent["_LIST"] or schemeParent["_NAME"]) and (not schemeParent[path[#path]])) then
         return btnDelete()
         --..btnCopyItem()
      else
         if scheme and scheme._X then
            return btnExecute(scheme)
         elseif scheme and scheme._T then
            return btnDisable()
         else
            return ""
         end
      end
   end

--+
genDirControls =
   function (scheme, conf, path, level, isTop)
      local _H = "webBtnKey"
      if (scheme and (not scheme._T)) then
         return
         fstr(
            ifThen(scheme['_DEP'] and not scheme._X,
                   fstr("<img src=\"/nil.png\" alt=\"!\" class=\"bkey\"",
                        hint(_H),
                        ">"
                     ),
                   ""
                ),
            (isTop and btnManual() or ""),
            (isTop and btnHTML(path, scheme).."&nbsp;&nbsp;&nbsp;" or ""),
            (not scheme._X and btnEdit(scheme, path)),
            (btnNewItem(scheme, conf, path) or ""),
            (isTop and btnRefresh(scheme, path) or ""),
            (scheme._Z and btnApply(path) or ""),
            (isTop and btnSave(scheme, path) or ""),
            (not scheme._X and btnClearDir() or "")
         )
      else
         return nil
      end
   end

genDirBlock =
   function (path, dir, dirControls)
      return ifThen(dir,
                    fstr(tag(0,
                             "div",
                             "class=\"dircontrol\"",
                             dirControls)
                         ,tag(0,
                              "ul",
                              "",
                              dir)),
                    "")
   end

genDirItem =
   function (scheme, schemeParent, conf, path, htmlEnv, level)
      if type(scheme) ~= 'table' then
         return ""
      end

      local liClass = safe(genLiClass, "genLiClass", scheme, conf, path, htmlEnv, level) or "class=\"some error\""
      local Help = scheme and scheme._H or "No help..."
      local cursor =  safe(genCursor, "genCursor", scheme) or ""
      local paramName = safe(genParamName,"genParamName", htmlEnv, scheme, schemeParent, conf, path, level) or "noname"
      local paramValue = safe(genParamValue,"genParamValue", scheme, schemeParent, conf, path, level) or ""
      local paramControls = safe(genParamControls, "genParamControls", scheme, schemeParent,conf, path) or ""
      local paramBlock = fstr(
         "<table class=\"node\">",
         "<tr valign=\"middle\">",
         "<td>",
         cursor,
         "</td><td class=\"param\"",
         ifThen(Help, hint(Help),""),
         ">",
         paramName,
         "</td><td class=\"value\">",
         paramValue,
         "</td><td class=\"itemcontrols\">",
         paramControls,
         "</td></tr></table>"
      )

      local dirControls = safe(genDirControls, "genDirControls", scheme, conf, path, level) or ""
      local dir = safe(genDir, "genDir", scheme, schemeParent, conf, path, htmlEnv, (isExpanded(htmlEnv, path) and (level+1) or level)) or ""
      return tag(0,
                 "li",
                 liClass,
                 paramBlock
                    ..ifThen(dir,
                             tag(0,
                                 "div",
                                 "class=\"dir\"",
                                 genDirBlock(path, dir or "", dirControls or "")
                              )
                          )
              )
   end

genDir =
   function (scheme, schemeParent, conf, path, htmlEnv, level)
      if type(scheme) ~= 'table' then
         return ""
      end
      if scheme._X then
         return "<input class=\"runfirst\" type=hidden>"
      end
      if level < 1 then
         return "<input class=\"pleasewait\" type=hidden>"
      end
      conf = conf or {}

      if path[-1]==nil and #path>1 then
         for i=1,#path -1 do
            path[-1] =  fstr((path[-1] or "") ,".",type(path[i])=="string" and  ("%q"):format(path[i]) or tostring(path[i]))
         end
      end

      local newPath = clonePath(path, nil, "")

      newPath[-1] = (#path>0) and  (fstr((path[-1] or "") ,".",type(path[#path])=="string" and  ("%q"):format(path[#path]) or tostring(path[#path]))) or ""

      local html = {}

      local confScheme = mod_scheme.confScheme(scheme or {}, conf or {}, path)
      -- Generate sub items of DIR
      local iter = confScheme._pairs or pairs
      for k,v in iter(confScheme or {}, path) do
         if not (tostring(k):find('^[_].*$')) then
            newPath[#newPath] = k
            local subsch = v(nil, newPath)
            html[#html + 1] = safe(genDirItem, "genDirItem", subsch, scheme, conf[k], newPath, htmlEnv, level-1)
         end
      end
      --         io.write("MEMB:"..collectgarbage('count').."\n")
      if collectgarbage('count')>4000 then
         collectgarbage('collect')
      end
      --         io.write("MEMA:"..collectgarbage('count').."\n")
      return table.concat(html)
end

local preloadLevels = 1

htmlConf =
   function (conf, confParent, scheme, schemeParent, path, htmlEnv, isTop)
      dirControls = safe(genDirControls, "genDirControls", scheme, conf, path, preloadLevels, isTop)
      dir = safe(genDir, "genDir", scheme, schemeParent, conf, path, htmlEnv, preloadLevels)
      return   genDirBlock(path, dir, dirControls, htmlEnv, level)
   end

mainPage =
   function (conf, confParent, scheme, schemeParent, path, htmlEnv)
      local _H = "webBtnLogout"
      local _H1 = _H
      local _H = "webBtnUsage"
      local _H2 = _H
      local release = C.getVersion()
--XHTML 1.0 Strict
--HTML 4.01 Transitional
      return fstr([[<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" >
                        <html>
                        <meta http-equiv="X-UA-Compatible" content="IE=7" />
                        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
                        <script src="/jquery.js?]]..release..[[" type="text/javascript" ></script>
                        <script src="/boxover.js?]]..release..[[" type="text/javascript" ></script>
                        <script src="/webtty.js?]]..release..[[" type="text/javascript" ></script>
                        <link rel="stylesheet" type="text/css" href="/style.css?]]..release..[[">
                        <link rel="icon" type="image/x-icon" href="/nsg.ico">
                        <link rel="shortcut icon" type="image/x-icon" href="/nsg.ico">
                        <head>
                        <title>NSG Web Configuration Manager</title>
                        </head>
                  ]],
               "<script type=\"text/javascript\">var NSGVERSION=\"",release:gsub("[^%w]*",""),"\";var activity=0;var SID=\"",
               htmlEnv.data.sesnum,
               "\";var webMode='html';var LOGIN=",
               escapeForScript(htmlEnv.data.login),
               ";var PASS=",
               escapeForScript(htmlEnv.data.pass),
               ";var AUTH=\"mode="..(htmlEnv.data.mode or "html").."&login=\"+encodeURIComponent(LOGIN)+ \"&pass=\" + encodeURIComponent(PASS) +\"&sesnum=\"+ SID +\"&\";var PAGENAME=\""..htmlEnv.data.pagename.."\";var LOCALE=\"",
               locale,"\";</script>\n",
               "<script src=\"/nsghelp.js?"..release.."\" type=\"text/javascript\" ></script>\n",
               "<script src=\"/script.js?"..release.."\" type=\"text/javascript\" ></script>\n",
               "<body>",
               "<div class=\"welcome\">",
               "<img title=\"header=[webJobs] body=[1]\" class=\"activityoff\" src=\"/loading.gif\" alt=\"1\">",
               mod_scheme.localeString("Welcome to NSG Linux",htmlEnv.data.locale)," ",
               "<span class=\"welcome\" onclick=\"showLeakChecker();\">",
               htmlEncode(release),
               "</span>",
               (BI and BI.mach and ', '..BI.mach),
               "</div>",
               htmlEnv.data.sessionMessage:match("[%a%d]+") and 
               fstr([[<ul>
                          <li class="cl">
                          <table class="node" ]], hint("webBtnExpand"),[[>
                          <tbody>
                          <tr valign="middle">
                          <td>
                          <img ]], hint("webBtnExpand"),[[class="bexpand" onclick="UnHide(this); return false;" title="" src="/nil.png" alt="Expand" tabindex="0" onkeydown="return btnKeyDown(event,this);"/>
                          <img ]], hint("webBtnCollapse"),[[ class="bcollapse" onclick="UnHide(this); return false;" title="" src="/nil.png" alt="Collapse" tabindex="0" onkeydown="return btnKeyDown(event,this);"/>
                       </td><td><a class="message" onclick="UnHide(this); return false;" onkeydown="return btnKeyDown(event,this);" >]],mod_scheme.localeString("Message",htmlEnv.data.locale) ,[[:<span class="messagehead">]],
                 htmlEncode(htmlEnv.data.sessionMessage:sub(1,40).."..."):match("^[\n\r]*(.-)[\n\r]*$"),
                 [[</span></td></tr></tbody></table><div class="dir"><pre ondblclick="UnHide(this);">]],
                 (htmlEncode(htmlEnv.data.sessionMessage):match("^[\n\r]*(.-)[\n\r]*$")),
                 "</pre></div></li></ul>") or "",
               "<ul><li class=\"root\" name=\"",
               htmlEncode(htmlEnv.data.login),
               "\">",
               "<table class=\"node\" width=\"100%\"><tbody><tr>",
               "<td class=\"param\">",
               htmlEncode(io.popen("hostname", "r"):read("*l")),
               ":",
               htmlEncode((htmlEnv.data.login).."(".. htmlEnv.data.sesnum..")"),
               "</td><td>",
               "<select class=\"locale\" onchange=\"setLocale(this);\"><option value=\"ru\" ",
               ((locale == "ru") and "selected" or ""),
               ">ru</option><option value=\"en\" ",
               (((locale == "en") or not locale) and "selected" or ""),
               ">en</option></select>",
               "</td><td>",
               "<img  onkeydown=\"return btnKeyDown(event,this);\" tabindex=0 src=\"/nil.png\" alt=\"Logout\" class=\"blogout\"",
               hint(_H1),
               "onclick=\"logOut(); return false;\">",
               "</td>",
               "<td width=\"90%\" class=\"usage\">",
               "<a tabindex=0  onkeydown=\"return btnKeyDown(event,this);\" onclick=\"getUsage(this);return false;\" ",hint(_H2)," >",
               htmlEncode(htmlEnv.usage),
               '<img alt="refresh" tabindex=0 class="brefresh" src="/nil.png">',
               "</a>",
               "</tr></tbody></table>",
               "<div class=\"dir\">",
               htmlConf(conf, confParent, scheme, schemeParent, {}, htmlEnv, true),
               "</div></li></ul>",
               "<div class=\"scriptexecutionarea\" id=\"scriptexecutionarea\"></div>",
               "</body></html>")
   end

----------------------------------------------------------------------
--  global methods

function getConf(args)
   locale = args.locale
   if(args.par[1] == false) then
      local editControls = btnEditSave(args.path)..btnValidate(args.path)..btnUndo(args.sch, args.path)
      local conf = args.getConfCommon(args) or ""
      local _, rows = (conf):gsub("([^\n]*[\n])","")

      local dir = "<textarea class=\"edit\" wrap=off cols=\"80\" rows=\""..(rows > 10 and (rows < 25 and rows or 25) or 10).."\">"..htmlEncode(conf).."</textarea>"
      return genDirBlock(args.path, dir, editControls)
   else
      if args.virtPath then
         collectgarbage("collect")
         return (htmlConf(args.conf, args.confParent, args.sch, args.schParent, args.path or args.virtPath, args.par[1].htmlEnv,#(args.virtPath or args.path)==0))
      else
         collectgarbage("collect")
         args.par[1].htmlEnv.usage = args.getUsage()
         return (mainPage(args.conf, args.confParent, args.sch, args.schParent, nil, args.par[1].htmlEnv))
      end
      --..ser(XXX)..ser( args.par[1].htmlEnv.data)
   end
end

function setValue(args)
   locale = args.locale
   local res, err, val
   if args.val == nil then
      res, err, val = true, "", nil 
   else
      res, err, val = args.setValueCommon(args)
   end
   if res then
      return true, res, genDirItem(args.sch, args.schParent, val, args.path, args.env , 1)
   else
      local conf = C.getConfByPath(args.path, args.sch._LOCAL and args.sesNum)
      return false, res==nil and "Error:Incorrect value: Expected: "..err or err, 
                        genDirItem(args.sch, args.schParent, conf, args.path, args.env , 1) or err or ""
   end
end

function executeTask(args)
   locale = args.locale
   return args.executeTaskCommon(args)
end

function newKey(args)
   locale = args.locale
   local path = htmlEncode("root."..table.concat(args.path,"."))
   local ret, err = args.funcCommon(args)
   if ret then
      return ret, nil 
   else
      return ret, err
   end
end

function insertKey(args)
   locale = args.locale
   local path = htmlEncode("root."..table.concat(args.path,"."))
   local ret, err = args.funcCommon(args)
   if ret then
      return ret, nil 
   else
      return ret, err
   end
end

function removeKey(args)
   locale = args.locale
   local ret, err = args.funcCommon(args)
   args.path[#args.path] = nil
   local path = htmlEncode("root."..table.concat(args.path,"."))

   if ret then
      return ret, nil 
   else
      return ret, err
   end
end

function getManual(args)
   return args.man
end
