module(..., package.seeall)
local s = require'scheme'
local usb = require'nsgusbconf'
local evt = require'nsgeventconf'

local oneWireDevName

function setDevName(port)
  oneWireDevName = usb.port2tty[port or "1-wire"]
end

--[[
local arch2oneWireDevName = {
  nsg600 = "/dev/ttyAM1",
  nsg700 = "/dev/nsg/ow1"
}
local oneWireDevName = arch2oneWireDevName[BI.arch]
]]--

local function getDevID(name)
  return name:match"(%x%x%x%x%x%x%x%x%x%x%x%x%x%x%x%x)$"
end

local function getDevType(name)
  return name:match"(%x%x)%x%x%x%x%x%x%x%x%x%x%x%x%x%x$"
end

local devtype2name = {
  ["10"] = 'temp',
  ["29"] = 'swt8',
  ["3A"] = 'swt2'
}
setmetatable(devtype2name, {__index = function() return 'ow' end})

local dummyFunc = function()end
local startEventGenerator = dummyFunc
local stopEventGenerator = dummyFunc
local function xnsgow(com)
  stopEventGenerator()
  local res = s.readpopen(com)
  startEventGenerator()
  return res
end

local function oneWireAutoconfig(path, sch, val)
  path = s.clonePath(path, -1)
  path[#path+1] = 'device'
  if val == 'init' or not C.getConfByPath(path) then
    C.setConfigValue(path, {})
  end
  local devConf = C.getConfByPath(path)
  if not devConf then
    devConf = {}
    C.setConfigValue(path, devConf)
  end
  local str = xnsgow("nsgow -i "..oneWireDevName)
  local devs = {}
  for id in str:gmatch'(%x%x%x%x%x%x%x%x%x%x%x%x%x%x%x%x)%s+' do
    devs[id] = true
  end
  if val == 'update' then
    for k, _ in pairs(devConf) do
      if not devs[getDevID(k)] then
        devConf[k] = nil
      end
    end
  end
  local devConfID = {}
  for k, _ in pairs(devConf) do
    devConfID[getDevID(k)] = true
  end
  for k, _ in pairs(devs) do
    if not devConfID[k] then
      devConf[devtype2name[getDevType(k)]..'-'..k] = {_keep=true}
    end
  end
  return "Find 1-Wire devices:\n"..str
end

local function getCircuitState(oneWireDevName, devID)
  local str = xnsgow(("nsgow %s -d %s -k 2>&1"):format(
  oneWireDevName, devID))
  local circuitState = {}
  if not str:find"[^01%s]" then
    for d in str:gmatch'[01]' do
      circuitState[#circuitState+1] = d
    end
  end
  return circuitState
end

local function getCircuitStateName(state, inverse)
  return state == '1' and (inverse and 'OFF' or 'ON') 
  or (state == '0' and (inverse and 'ON' or 'OFF') or 'UNKNOWN')
end

local function isInverseSPS8(path)
  return not C.getConfByPath(s.clonePath(path, -3, "inverse"))
end

local eventGeneratorScheme = s.newScheme{
  _H = "oneWireDeviceEventgenerator",
  _P = 50,

  enable = s.booleanType{
    _H = "oneWireDeviceEventgeneratorEnable",
    _D = false,
    _P = 100
  },
  ["polling-interval"] = s.integerType{
    _H = "oneWireDeviceEventgeneratorPollinginterval",
    _A = {1,3600},
    _P = 95,
    _D = 10,
  },
  prefix = s.stringType{_H = "oneWireDeviceEventgeneratorPrefix", _P = 90},
  ["off-event"] = s.stringType{_H = "oneWireDeviceEventgeneratorOffname", _P=-100},
  ["on-event"] = s.stringType{_H = "oneWireDeviceEventgeneratorOnname", _P=-100},
}
local eventGeneratorCircuitNameScheme = s.stringType{_H = "oneWireDeviceEventgeneratorCircuitname"}

local circuitScheme = s.newScheme {
  _H = "oneWireDeviceCircuit",
  on = s.newScheme {
    _H = "oneWireDeviceCircuitOn",
    --_T = 'areyousure',
    --_V = areyousureVal,
    --_A = {"Sircuit will switch ON.\n"},
    _X = function(path, sch, val)
      local inverse = isInverseSPS8(path)
      xnsgow(("nsgow %s -d %s -%d %s -k 2>&1"):format(
          oneWireDevName, 
          getDevID(path[#path - 3]), 
          path[#path - 1] - 1,
          inverse and 'open' or 'short'
        )
      )
    end,
    _R = 'user',
    _P = 50,
  },
  off = s.newScheme {
    _H = "oneWireDeviceCircuitOff",
    --_T = 'areyousure',
    --_V = areyousureVal,
    --_A = {"Sircuit will switch OFF.\n"},
    _X = function(path, sch, val)
      local inverse = isInverseSPS8(path)
      xnsgow(("nsgow %s -d %s -%d %s -k 2>&1"):format(
          oneWireDevName, 
          getDevID(path[#path - 3]), 
          path[#path - 1] - 1,
          inverse and 'short' or 'open'
        )
      )
    end,
    _R = 'user',
    _P = 40,
  },
  pulse = s.newScheme {
    _H = "oneWireDeviceCircuitPulse",
    _X = function(path, sch, val, sesNum)
      local inverse = isInverseSPS8(path)
      xnsgow(("nsgow %s -d %s -%d %s -k -r %d 2>&1"):format(
          oneWireDevName, 
          getDevID(path[#path - 3]), 
          path[#path - 1] - 1,
          inverse and 'drop' or 'pulse',
          C.getConfByPath(s.clonePath(path, -1, "reset-delay"), sesNum) or
          1000
        )
      )
    end,
    _R = 'user',
    _P = 20,
  },
  drop = s.newScheme {
    _H = "oneWireDeviceCircuitDrop",
    _X = function(path, sch, val, sesNum)
      local inverse = isInverseSPS8(path)
      xnsgow(("nsgow %s -d %s -%d %s -k -r %d 2>&1"):format(
          oneWireDevName, 
          getDevID(path[#path - 3]), 
          path[#path - 1] - 1,
          inverse and 'pulse' or 'drop',
          C.getConfByPath(s.clonePath(path, -1, "reset-delay"), sesNum) or
          1000
        )
      )
    end,
    _R = 'user',
    _P = 10,
  },
  toggle = s.newScheme {
    _H = "oneWireDeviceCircuitToggle",
    _X = function(path, sch, val)
      xnsgow(("nsgow %s -d %s -%d toggle -k 2>&1"):format(
          oneWireDevName, getDevID(path[#path - 3]), path[#path - 1] - 1))
    end,
    _R = 'user',
    _P = 30,
  },
  ["reset-delay"] = s.integerType {
    _H = "oneWireDeviceCircuitResetDelay",
    _A = {0, 10000},
    _D = 1000,
    _LOCAL = true,
    _R = 'user',
  },
  show = s.newScheme{
    _H = "oneWireShow",
    _X = function(path)
      local circuitState = getCircuitState(oneWireDevName, getDevID(path[#path - 3]))
      local state = circuitState[path[#path - 1]]
      local inverse = isInverseSPS8(path)
      return "Circuit "..getCircuitStateName(state, inverse)
    end,
    _R = 'user',
    _P = -10,
  },
}

local ow29Scheme = s.newScheme{
  _H = "oneWireDeviceName",
  ["type"] = s.newScheme{
    _H = "oneWireDeviceType",
    _T = "choice",
    _A = {
      {"spc8", ""},
    },
    _V = s.choiceVal,
    _DEP = 1,
  },
  show = s.newScheme{
    _H = "oneWireShow",
    _X = function(path)
      local circuitState = getCircuitState(oneWireDevName, getDevID(path[#path - 1]))
      local out = {}
      local inverse = not C.getConfByPath(s.clonePath(path, -1, "inverse"))
      for k = 1,8 do
        out[#out+1] = "Circuit "..tostring(k)..' '..getCircuitStateName(circuitState[k], inverse)
      end
      return table.concat(out, '\n')
    end,
    _R = 'user',
  },
  inverse = s.booleanType{
    _H = "oneWireDeviceInverse",
  },
  circuit = s.newScheme{
    _H = "oneWireDeviceCircuits",
    [1] = circuitScheme,
    [2] = circuitScheme,
    [3] = circuitScheme,
    [4] = circuitScheme,
    [5] = circuitScheme,
    [6] = circuitScheme,
    [7] = circuitScheme,
    [8] = circuitScheme,
  },
  ["event-generator"] = eventGeneratorScheme{
    ['circuit-1'] = eventGeneratorCircuitNameScheme,
    ['circuit-2'] = eventGeneratorCircuitNameScheme,
    ['circuit-3'] = eventGeneratorCircuitNameScheme,
    ['circuit-4'] = eventGeneratorCircuitNameScheme,
    ['circuit-5'] = eventGeneratorCircuitNameScheme,
    ['circuit-6'] = eventGeneratorCircuitNameScheme,
    ['circuit-7'] = eventGeneratorCircuitNameScheme,
    ['circuit-8'] = eventGeneratorCircuitNameScheme,
  }
}

local ow3Adescr = {
  ["IC-2do-220/0,12"] = {},
  ["IC-2do-220/1"] = {},
  ["IC-2di-220"] = {roDev = true},
  ["IC-2dio"] = {},
  ["spc2i"] = {inverse = true, digitNumeration = true},
  ["spc2"] = {digitNumeration = true},
}

local ow3Achoice = {}
for k, _ in pairs(ow3Adescr) do
  ow3Achoice[#ow3Achoice+1] = k
end
table.sort(ow3Achoice)

local digit2letter = {
  [1] = 'B',
  [2] = 'A',
  A = 'A',
  B = 'B',
}


local function isIOdevice(path)
  return not ow3Adescr[C.getConfOrDefault(s.clonePath(path, -3, "type"))].roDev
end
local function isInverseDevice(path, n)
  n = n or -3
  return ow3Adescr[C.getConfOrDefault(s.clonePath(path, n, "type"))].inverse
end

local circuitScheme3A = s.newScheme { -- 3A
  _H = "oneWireDeviceCircuit",
  on = function(_, path)
    return isIOdevice(path) and
    s.newScheme {
      _H = "oneWireDeviceCircuitOn",
      _X = function(path, sch, val)
        xnsgow(("nsgow %s -d %s:b -%s %s -k 2>&1"):format(
            oneWireDevName, 
            getDevID(path[#path - 3]), 
            digit2letter[path[#path - 1]],
            isInverseDevice(path) and 'open' or 'short'
          )
        )
      end,
      _R = 'user',
      _P = 50,
    }
  end,
  off = function(_, path)
    return isIOdevice(path) and
    s.newScheme {
      _H = "oneWireDeviceCircuitOff",
      _X = function(path, sch, val)
        xnsgow(("nsgow %s -d %s:b -%s %s -k 2>&1"):format(
            oneWireDevName, 
            getDevID(path[#path - 3]), 
            digit2letter[path[#path - 1]],
            isInverseDevice(path) and 'short' or 'open'
          )
        )
      end,
      _R = 'user',
      _P = 40,
    }
  end,
  pulse = function(_, path)
    return isIOdevice(path) and
    s.newScheme {
      _H = "oneWireDeviceCircuitPulse",
      _X = function(path, sch, val, sesNum)
        xnsgow(("nsgow %s -d %s:b -%s %s -k -r %d 2>&1"):format(
            oneWireDevName, 
            getDevID(path[#path - 3]), 
            digit2letter[path[#path - 1]],
            isInverseDevice(path) and 'drop' or 'pulse',
            C.getConfByPath(s.clonePath(path, -1, "reset-delay"), sesNum) or
            1000
          )
        )
      end,
      _R = 'user',
      _P = 20,
    }
  end,
  drop = function(_, path)
    return isIOdevice(path) and
    s.newScheme {
      _H = "oneWireDeviceCircuitDrop",
      _X = function(path, sch, val, sesNum)
        xnsgow(("nsgow %s -d %s:b -%s %s -k -r %d 2>&1"):format(
            oneWireDevName, 
            getDevID(path[#path - 3]), 
            digit2letter[path[#path - 1]],
            isInverseDevice(path) and 'pulse' or 'drop',
            C.getConfByPath(s.clonePath(path, -1, "reset-delay"), sesNum) or
            1000
          )
        )
      end,
      _R = 'user',
      _P = 10,
    }
  end,
  toggle = function(_, path)
    return isIOdevice(path) and
    s.newScheme {
      _H = "oneWireDeviceCircuitToggle",
      _X = function(path, sch, val)
        xnsgow(("nsgow %s -d %s:b -%s toggle -k 2>&1"):format(
            oneWireDevName, 
            getDevID(path[#path - 3]), 
            digit2letter[path[#path - 1]])
        )
      end,
      _R = 'user',
      _P = 30,
    }
  end,
  ["reset-delay"] = function(_, path)
    return isIOdevice(path) and
    s.integerType {
      _H = "oneWireDeviceCircuitResetDelay",
      _A = {0, 10000},
      _D = 1000,
      _LOCAL = true,
      _R = 'user',
      _P = -9,
    }
  end,
  show = s.newScheme{
    _H = "oneWireShow",
    _X = function(path)
      local circuitState = getCircuitState(oneWireDevName, getDevID(path[#path - 3]))
      local circuit = path[#path - 1]
      if type(circuit) == 'number' then
        local inverse = isInverseDevice(path)
        return "Circuit "..getCircuitStateName(circuitState[3 - path[#path - 1]], inverse)
      else
        local state = circuitState[circuit == 'A' and 1 or 2]
        return "Circuit "..circuit.." "..(state=='0'and'open'or(state=='1'and'short'or'UNKNOWN'))..'\n'
      end
    end,
    _R = 'user',
    _P = -10,
  },
}

local ow3AScheme = s.newScheme{
  _H = "oneWireDeviceName",
  ["type"] = s.newScheme{
    _H = "oneWireDeviceType",
    _T = "choice",
    _A = ow3Achoice,
    _V = s.choiceVal,
    _D = "IC-2dio",
    _DEP = 1,
    _P = 100,
  },
  show = s.newScheme{
    _H = "oneWireShow",
    _X = function(path)
      local circuitState = getCircuitState(oneWireDevName, getDevID(path[#path - 1]))

      local owType = C.getConfOrDefault(s.clonePath(path, -1, "type"))
      if ow3Adescr[owType].digitNumeration then
        local inverse = isInverseDevice(path, -1)
        return (
          "Circuit 1 "..getCircuitStateName(circuitState[2], inverse)..'\n'..
          "Circuit 2 "..getCircuitStateName(circuitState[1], inverse)..'\n'
        )
      else
	local stA, stB = circuitState[1], circuitState[2]
        return (
          "Circuit A "..(stA=='0'and'open'or(stA=='1'and'short'or'UNKNOWN'))..'\n'..
          "Circuit B "..(stB=='0'and'open'or(stB=='1'and'short'or'UNKNOWN'))..'\n'
        )
      end
    end,
    _R = 'user',
  },
  circuit = function(_, path)
    local owType = C.getConfOrDefault(s.clonePath(path, -1, "type"))
    if ow3Adescr[owType].digitNumeration then
      return s.newScheme{
        _H = "oneWireDeviceCircuits",
        [1] = circuitScheme3A,
        [2] = circuitScheme3A,
      }
    else
      return s.newScheme{
        _H = "oneWireDeviceCircuits",
        ['A'] = circuitScheme3A,
        ['B'] = circuitScheme3A,
      }
    end
  end,
  ["event-generator"] = function(_, path)
    local owType = C.getConfOrDefault(s.clonePath(path, -1, "type"))
    if ow3Adescr[owType].digitNumeration then
      return eventGeneratorScheme{
        ['circuit-1'] = eventGeneratorCircuitNameScheme,
        ['circuit-2'] = eventGeneratorCircuitNameScheme,
      }
    else
      return eventGeneratorScheme{
        ['circuit-A'] = eventGeneratorCircuitNameScheme,
        ['circuit-B'] = eventGeneratorCircuitNameScheme,
      }
    end
  end,
}

local ow10Scheme = s.newScheme{
  _H = "oneWireDeviceName",
  ["type"] = s.newScheme{
    _H = "oneWireDeviceType",
    _T = "choice",
    _A = {
      {"temp", ""},
    },
    _V = s.choiceVal,
  },
  show = s.newScheme{
    _H = "oneWireShow",
    _X = function(path)
      local str = xnsgow(("nsgow %s -d %s -k 2>&1"):format(
          oneWireDevName, getDevID(path[#path - 1])))
      str = str:match"^%s*(.-)%s*$"
      return ("t = %s C"):format(str)
    end,
    _R = 'user',
  },
}

local scriptPrefix = [[
require'nsg'.meta()
local socket = require'socket'
local res,t,s,e
local T = {
]]
local scriptSuffix = [[
}
while true do
  local tim = os.time()
  local min = math.huge
  for _,v in pairs(T) do
    if v[1]<=tim then 
      v[3]()
      v[1] = tim + v[2]
    end
    if v[1]<min then min=v[1]end
  end
  socket.sleep(min-tim)
end
]]

local makeowscript = {
  ["3A"] = function(name, typ, conf)
    local dig = ow3Adescr[typ].digitNumeration
    return
([=[
{0,%d,function()
res=-"nsgow %s -d %s -k"
t={res:match"([01])%%s+([01])%%s+"}
s={%q,%q}
e={['0']=%q,['1']=%q}
for k=1,2 do
  print(s[k], e[t[k]])
end
end},
]=]):format(
  conf["polling-interval"],
  oneWireDevName,
  getDevID(name),
  conf['circuit-A'] or conf['circuit-1'] or (conf.prefix or name)..(dig and '.1' or '.A'),
  conf['circuit-B'] or conf['circuit-2'] or (conf.prefix or name)..(dig and '.2' or '.B'),
  conf['off-event'] or (dig and 'off' or 'open'),
  conf['on-event'] or (dig and 'on' or 'short')
)
  end,

  ["29"] = function(name, typ, conf, inv)
    return
([=[
{0,%d,function()
res=-"nsgow %s -d %s -k"
t={res:match"([01])%%s+([01])%%s+([01])%%s+([01])%%s+([01])%%s+([01])%%s+([01])%%s+([01])%%s+"}
s={%q,%q,%q,%q,%q,%q,%q,%q}
e={['0']=%q,['1']=%q}
for k=1,8 do
  print(s[k], e[t[k]])
end
end},
]=]):format(
  conf["polling-interval"],
  oneWireDevName,
  getDevID(name),
  conf['circuit-1'] or (conf.prefix or name)..'.1',
  conf['circuit-2'] or (conf.prefix or name)..'.2',
  conf['circuit-3'] or (conf.prefix or name)..'.3',
  conf['circuit-4'] or (conf.prefix or name)..'.4',
  conf['circuit-5'] or (conf.prefix or name)..'.5',
  conf['circuit-6'] or (conf.prefix or name)..'.6',
  conf['circuit-7'] or (conf.prefix or name)..'.7',
  conf['circuit-8'] or (conf.prefix or name)..'.8',
  inv and (conf['off-event'] or 'off') or (conf['on-event'] or 'on'),
  inv and (conf['on-event'] or 'on') or (conf['off-event'] or 'off')
)
  end,
}

local function eventGeneratorApply(path, sch, conf, prevConf)
  local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
  local script = ""

  for owdev, owdevconf in pairs(conf) do
    local eg = owdevconf["event-generator"]
    if eg and eg.enable then
      script = script..makeowscript[getDevType(owdev)](owdev, owdevconf.type, eg, owdevconf.inverse)
    end
  end

  if script ~= '' then
    script = scriptPrefix..script..scriptSuffix
    local com = {
      "lua", "-e", script,
    }
    startEventGenerator = function() evt.restartVirtSensor(path, com) end
    startEventGenerator()
    stopEventGenerator = function() C.restartDaemon(path) end
  else
    C.restartDaemon(path)
    startEventGenerator = dummyFunc
    stopEventGenerator = dummyFunc
  end
  return true
end


local devtype2scheme = {
  ["10"] = ow10Scheme,
  ["29"] = ow29Scheme,
  ["3A"] = ow3AScheme,
}
setmetatable(devtype2scheme, {__index = function() return s.newScheme{} end})
local function get1wireDeviceScheme(_, path)
  return devtype2scheme[getDevType(path[#path])]
end

local _oneWireScheme = s.newScheme {
  _H = "oneWirePort",

  autoconfig = s.newScheme{
    _H = "oneWireAutoconfig",
    _T = "choice",
    _A = {
      {"add", "Add new devices to configuration"},
      {"init", "Set configuration with default value"},
      {"update", "Add new devices and remove nonexistent ones"},
    },
    _D = "update",
    _V = s.choiceVal,
    _X = oneWireAutoconfig,
    _DEP = 1,
  },

  device = s.newScheme{
    _H = "oneWireDevice",
    _Z = eventGeneratorApply,
    _NAME = get1wireDeviceScheme,
  },

  show = s.newScheme{
    _H = "oneWireShow",
    _X = function()
      return xnsgow("nsgow -s "..oneWireDevName)
    end,
    _R = 'user',
  },
  ['get-port-parms'] = function(_, path)
    return (C.session and C.session.script) and 
    s.newScheme{
      _X = function(path, sch, val)
        local t = {}
        t.ttyDev = oneWireDevName
        return s.ser(t)
      end,
      _R = 'user',
    }
end,}

oneWireScheme = _oneWireScheme
