module(..., package.seeall)
local s = require'scheme'
local cosmo = require'cosmo'

local pamConfPath = "/etc/pam.d/"
local pamLibPath = "/lib/security/"
local service2name = {
  login = "login",
  ssh = "sshd",
}

local radiusServerConfFilePrefix = "/etc/raddb/server_"
local function pamLib(service, N, facil, conf)
  local parm = {
    deny = function()
      return "pam_deny.so"
    end,
   ["local"] = function()
      return "pam_unix.so shadow nullok"
    end,
    radius = function()
      local fname = radiusServerConfFilePrefix..service..N
      local f, err = io.open(fname, 'w')
      if f then
        f:write(
          ("%s:%d %s %d"):format(
            conf.server, 
            conf.port, 
            tostring(conf.secret), 
            conf.timeout
          )
        )
        
        f:close()
      else
        C.log_error("Can't create radius config file: %s" % err)
      end
      return "pam_radius_auth.so conf="..fname
    end,
    tacacs = function()
      if facil == "password" or facil == "session" then
	return "pam_permit.so"
      end
      
      local out = ("pam_tacplus.so server=%s secret=%s timeout=%d"):format(
        conf.server, tostring(conf.secret), conf.timeout
      )

      if facil == "account" then
	if service == "reverse-telnet" then
	  out = out.." service=raccess protocol=none"
	end
      end

      return out
    end,
  }
  return parm[conf.type]()
end

local template = "$do_cards[[$facil sufficient /lib/security/$lib $parms\n]]"
cards = {
  {facil = "auth"},
  {facil = "account"}, 
  {facil = "password"}, 
  {facil = "session"}, 
}

local function createPamFile(service, conf)
  local fname = pamConfPath..(service2name[service] or service)
  local f = io.open(fname, 'w')
  if not f then return false end

  local pamFile = conf["pam-file"]
  if pamFile and not pamFile:find"^%s*$" then
    f:write(pamFile)
    f:close()
    return true
  end

  local text = {}
  local ufpstr
  local ufp = function(facil)
    if facil == 'auth' then
      if ufpstr then return ufpstr end
      ufpstr = "use_first_pass"
    end
    return''
  end
  for i = 0, #conf do
    if conf[i] then
      text[#text+1] = cosmo.f(template){
        do_cards = function()
          for _, v in ipairs(cards) do
            cosmo.yield{
              facil = v.facil, 
              lib = pamLib(service, i, v.facil, conf[i]), 
              parms = ufp(v.facil)
            }
          end
        end
      }
    end
  end
  f:write(table.concat(text))
  f:close()
  return true
end

local aaaAply = function(path, sch, conf, prevConf)
  local conf = s.addDefault(path, sch, conf, true)
  os.execute("rm "..radiusServerConfFilePrefix..'*')
  for k, _ in sch:_pairs() do
    if k~='default' then
      if not createPamFile(
        k, conf[k] and conf[k][1] and conf[k] or conf.default
      ) then return false end
    end
  end
  return true
end

local function typeRange (path)
   if path[#path - 1] == "ppp" then
      return {"radius", "tacacs", "deny"}, "radius" 
   end
   return {"local", "radius", "tacacs", "deny"}, "local"  -- default
end

local aaaconf = s.newScheme{
  _H = "aaaParms",
  ["type"] = function(_, path)
		local a, d = typeRange(s.clonePath(path, -1))
		return s.newScheme{
		   _H = "aaaParmsType",
		   _T = 'choice',
		   _A = a,
		   _V = s.choiceVal,
		   _D = d,
		   _P = 100,
		   _DEP = 1,
		}
	     end,
		
  server = function(_, path)
    local typepar = C.getConfOrDefault(s.clonePath(path, -1, "type"))
    return (typepar == 'radius' or typepar == 'tacacs') and 
    s.ipAddressType{
      _H = "aaaParmsServer",
      _D = "127.0.0.1",
    }
  end,
  port = function(_, path)
    local typepar = C.getConfOrDefault(s.clonePath(path, -1, "type"))
    return (typepar == 'radius') and 
    s.tcpPortType{
      _H = "aaaParmsPort",
      _D = 1812,
    }
  end,
  secret = function(_, path)
    local typepar = C.getConfOrDefault(s.clonePath(path, -1, "type"))
    return (typepar == 'radius' or typepar == 'tacacs') and 
    s.stringType{
      _H = "aaaParmsSecret",
    }
  end,
  timeout = function(_, path)
    local typepar = C.getConfOrDefault(s.clonePath(path, -1, "type"))
    return (typepar == 'radius' or typepar == 'tacacs') and 
    s.integerType{
      _H = "aaaParmsTimeout",
      _A = {1, 60*60},
      _D = 3,
    }
  end,
}

aaaScheme = s.newScheme{
  _H = "aaa",
  _Z = aaaAply,

  default = s.newScheme{
  _H = "aaaDefault",
    [0] = aaaconf,
    _LIST = aaaconf
  },

  login = s.newScheme{
    _H = "aaaLogin",
    _LIST = aaaconf
  },

  ppp = s.newScheme{
    _H = "aaaPpp",
    _LIST = aaaconf
  },

  ssh = s.newScheme{
    _H = "aaaSsh",
    _LIST = aaaconf
  },

  ["reverse-telnet"] = s.newScheme{
    _H = "aaaReverseTelnet",
    --["pam-file"] = s.stringType{
    --  _h = "aaaPamfile",
    --},

    _LIST = function(_, path)
      local pamFile = C.getConfOrDefault(s.clonePath(path, -1, "pam-file"))
      if not pamFile or pamFile:find"^%s*$" then
	return aaaconf
      end
    end
  },
}
