module(..., package.seeall)
local s = require'scheme'
local iface = require'nsgifaceconf'

local aggBinFile = {
  '/usr/sbin/aggregator'
}

local function restartAggregator(path, sch, conf, prevConf)
  local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
  if conf.enable then
    local aggConfigFilename = os.tmpname()
    local f, err = io.open(aggConfigFilename, 'w')
    if f then
      local res, err = f:write("return", s.ser(conf))
      if not res then
        C.log_error("restartAggregator: Can't write aggregator config file: "..err)
      end
      f:close()
    else
      C.log_error("restartAggregator: Can't create aggregator config file: "..err)
    end

    local aggUtil
    for i = 1, #aggBinFile do
      local f = io.open(aggBinFile[i], 'r')
      if f then
        f:close()
        aggUtil = aggBinFile[i]
        break
      end
    end

    if aggUtil then
      C.restartDaemon(
        path, 
        {aggUtil, '-n', aggConfigFilename }, 
        { 
          noTimestamp = true,
          tmpFileList = {aggConfigFilename}, 
          atExit = C.increasingSleep
        }
      )
    else
      C.restartDaemon(path)
    end
    return true
  end
  C.restartDaemon(path)
  return true
end

local socket = require'socket'
local aggsock, recursion
local function aggcom(com)
  if not aggsock then
    local err
    aggsock, err = socket.connect("127.0.0.1", 50011)
    if not aggsock then
      local msg = "Can't connect to aggregator daemon"
      C.log_error(msg)
      return msg
    end
    aggsock:settimeout(3)
  end
  local res, err = aggsock:send(com..'\n')
  if not res then
    aggsock:close()
    aggsock = nil
    if recursion then
      recursion = false
      return "Can't send request to aggregator daemon"
    else
      recursion = true
      return aggcom(com)
    end
  end

  local buf = ""
  while true do
    local data, err, part = aggsock:receive'*l'
    if not data then
      return "Can't receive answer from aggregator daemon("..err..")"
    end
    buf = buf..(data or part or '')
    if buf:sub(-1) == '\000' then
      return buf:sub(1, -2)
    else
      buf = buf..'\n'
    end
  end
end

aggScheme = s.newScheme{
  _H = "agg",
  _Z = restartAggregator,

  VERSION = s.newScheme{
    _X = function(path)
      return "This is alpha(unstable) version of multilink"
    end,
    _R = 'user',
    _P = 101,
  },
  enable = s.booleanType{
    _H = "aggEnable",
    _P = 100,
  },
  aggregator = s.newScheme{
    _H = "aggAggregator",
    _NAME = s.newScheme{
      _H = "aggAggregatorX",
      --_Z = function() return true end,
      ifname = s.stringType{
	_H = "aggAggregatorIfname",
      },
      ifaddress = iface.ifAddressListScheme{
	_Z = false,
	configurable = function()return nil end,


      },
      tunnel = s.newScheme{
	_H = "aggAggregatorTunnel",
	_NAME = s.newScheme{
	  _H = "aggAggregatorTunnelX",
	  server = s.ipAddressAndPortType{_H = "aggAggregatorTunnelServer"}, 
	  keepalive = s.newScheme{
	    _T = "number", _A = {1, 0x7fffffff}, _V = s.numberVal, 
	    _H = "aggAggregatorTunnelKeepalive",
	    _D = 5,
	  },
	  retry = s.newScheme{
	    _T = "integer", _A = {1, 0x7fffffff}, _V = s.integerVal, 
	    _H = "aggAggregatorTunnelRetry",
	    _D = 3,
	  },
	  enable = s.booleanType{
	    _H = "aggAggregatorTunnelEnable",
	    _P = 100,
	    _D = true,
	  },
	},
      },
      qdisc = s.newScheme{
	_H = "aggAggregatorQdisc",
	_T = 'choice',
	_A = {'roundRobin', 'random', 'queue', 'smart'},
	_V = s.choiceVal,
	_D = 'roundRobin',
      },
      show = s.newScheme{
	_H = "aggAggregatorShow",
	_X = function(path)
	  local aggname = path[#path - 1]
	  return aggcom("stat(%q)"%aggname)
	end,
	_R = 'user',
      },

    },
  },
  listener = s.newScheme{
    _H = "aggListener",
    _LIST = s.newScheme{
      _H = "aggListenerX",
      host = s.ipAddressOrAllType{_H = "aggListenerHost"},
      port = s.tcpPortType{_H = "aggListenerPort"},
    },
  },
  ["log"] =  s.newScheme{
    _H = "aggLog",
    _X = function(path)
      return C.getDaemonLog(s.clonePath(path, -1))
    end,
    _R = 'user',
  },
}
