module(..., package.seeall)
local s = require'scheme'

local auditdConf = {
  log_file = "/var/log/audit/audit.log",
  log_format = "RAW",
  log_group = false,
  priority_boost = 3,
  flush = "incremental",
  freq = 20,
  num_logs = 4,
  dispatcher = false, -- "/sbin/audispd"
  disp_qos = "lossy",
  name_format = false,
  name = false,
  max_log_file = 5,
  max_log_file_action = "rotate",
  space_left = 75,
  space_left_action = "syslog",
  action_mail_acct = "root",
  admin_space_left = 50,
  admin_space_left_action = "suspend",
  disk_full_action = "suspend",
  disk_error_action = "suspend",
}


local function getAuditStatus(key)
  local status = s.readpopen("/sbin/auditctl -s")
  return status:match(key.."=([^%s]+)")
end

local log_file = auditdConf.log_file

local function auditAply(path, sch, conf, prevConf)
  if not s.confCmp(conf, prevConf) or true then
    local auditconf = s.addDefault(path, sch, conf, true) --clone conf with defaults
    if auditconf.enable then
      local acnf = {}
      local confConf = auditconf.conf or {}
      for k, v in pairs(auditdConf) do
	v = confConf[k] or v
	if v then
	  acnf[#acnf+1] = k.." = "..v
	  if k == "log_file" then log_file = v end
	end
      end
      os.execute"mkdir -p /etc/audit >/dev/null 2>&1"
      os.execute(("mkdir -p `dirname %s` >/dev/null 2>&1"):format(log_file))
      local f = io.open("/etc/audit/auditd.conf", 'w')
      if f then
	f:write(table.concat(acnf, '\n'))
	f:close()
      end
      local pid = tonumber(getAuditStatus"pid")
      if pid and pid ~= 0 then
	os.execute("/bin/kill -SIGHUP "..pid)
      else
	os.execute"/sbin/auditd"
      end

      -- rules
      local rules = conf.rules or {}
      local f = io.open("/etc/audit/audit.rules", 'w')
      if f then
	for _, v in ipairs(rules) do
	  if v and v:find"[^%s]+" then
	    f:write(v, '\n')
	  end
	end
	f:close()
      end
      os.execute"/sbin/auditctl -D >/dev/null 2>&1"
      os.execute"/sbin/auditctl -R /etc/audit/audit.rules >/dev/null 2>&1"
    else
      os.execute"/usr/bin/killall auditd >/dev/null 2>&1"
      os.execute"/sbin/auditctl -e0 >/dev/null 2>&1"
    end
  end
  return true
end


auditScheme = s.newScheme{
  _H = "audit",
  _Z = auditAply,

  enable = s.booleanType{
    _H = "auditEnable",
    _P = 100,
  },

  conf = s.newScheme{
    _H = "auditConf",
    log_file = s.stringType{
      _H = "auditConfLogfile",
      _D = "/var/log/audit/audit.log",
    },
    max_log_file = s.integerType{
      _H = "auditConfMaxlogfile",
      _A = {1, 64},
      _D = 1,
    },
    _NAME = s.stringType{
      _H = "auditConfOption",
    },
  },

  rules = s.newScheme{
    _H = "auditRules",
    _LIST = s.stringType{
      _H = "auditRulesN",
      _D = "",
    },
  },

  show = s.newScheme{
    _H = "auditShow",
    conf = s.newScheme{
      _H = "auditShowConf",
      _X = function()
	return '\n'..s.readpopen("cat /etc/audit/auditd.conf 2>&1")
      end,
      _R = 'user',
    },
    status = s.newScheme{
      _H = "auditShowStatus",
      _X = function()
	return '\n'..s.readpopen("/sbin/auditctl -s 2>&1")
      end,
      _R = 'user',
    },
    rules = s.newScheme{
      _H = "auditShowRules",
      _X = function()
	return '\n'..s.readpopen("/sbin/auditctl -l 2>&1")
      end,
      _R = 'user',
    },

    ["log-file"] =  s.newScheme{
      _H = "auditShowLogfile",
      _X = function(path)
        return('\n'..s.readpopen("tail -n100 "..log_file.." 2>&1"))
      end,
      _R = 'user',
    },

    ["report"] =  s.newScheme{
      _H = "auditShowReport",
      _X = function(path, sch, val)
	return('\n'..s.readpopen("/sbin/aureport --input "..log_file.." --"..val.." 2>&1"))
      end,
      _R = 'user',
      _T = "choice",
      _A = {
	{"avc",        "Avc report"},
	{"auth",       "Authentication report"},
	{"config",     "Config change report"},
	{"crypto",     "Crypto report"},
	{"event",      "Event report"},
	{"file",       "File name report"},
	{"failed",     "only failed events in report"},
	{"host",       "Remote Host name report"},
	{"login",      "Login report"},
	{"mods",       "Modification to accounts report"},
	{"mac",        "Mandatory Access Control (MAC) report"},
	{"anomaly",    "aNomaly report"},
	{"pid",        "Pid report"},
	{"response",   "Response to anomaly report"},
	{"syscall",    "Syscall report"},
	{"success",    "only success events in report"},
	{"summary",    "sorted totals for main object in report"},
	{"log",        "Log time range report"},
	{"terminal",   "TerMinal name report"},
	{"user",       "User name report"},
	{"executable", "eXecutable name report"},
      },
      _V = s.choiceVal,
      _D = "summary",
    },
  },
}

