module(..., package.seeall)
local s = require'scheme'
local ip = require'nsgiplib'

local split = function (s,t)
                 local l = {}
                 local f = function (s)
                              if s~= "" then
                                 l[#l + 1] = s
                              end
                              return s
                           end
                 local p = "([^"..t.."]*)"
                 string.gsub(s,p,f)
                 return l
              end

local grepPrefix = [==[grep -oE '([^0-9]|^)(([01]|)[0-9]?[0-9]?|[2]([0-4][0-9]|[5][0-5]))[.](([01]|)[0-9]?[0-9]?|[2]([0-4][0-9]|[5][0-5]))[.](([01]|)[0-9]?[0-9]?|[2]([0-4][0-9]|[5][0-5]))[.](([01]|)[0-9]?[0-9]?|[2]([0-4][0-9]|[5][0-5]))[/]([0-2]?[0-9]|[3][0-2])([^0-9]|$)']==]

local function devPrimaryPattern(val)
   v = tostring(val):gsub("[\n\r]","")
   local errMsg =  "Interface pattern: [\"mask\"] prefix"
   local grep = [==[grep -oE '^[[:space:]]*(["][a-zA-Z0-9.:_*?]+["][[:space:]]*|)[[:digit:]]{1,3}[.][[:digit:]]{1,3}[.][[:digit:]]{1,3}[.][[:digit:]]{1,3}[/][[:digit:]]{1,2}[[:space:]]*$']==]
   if os.execute(grep.." << EOF\n"..v.."\nEOF") ~= 0 then
      return nil, errMsg
   end
   if os.execute(grepPrefix.." << EOF\n"..v.."\nEOF") ~= 0 then
      return nil, "Incorrect prefix:"..v.."\n"..errMsg
   end
   return v
end
local function interfacePattern(val)
   val = (tostring(val)):gsub("[\n\r]","")
   local errMsg =  "Interface pattern: [-] [\"mask\"] [prefix] [, nextPattern [...]]"
   local grep = [==[grep -oE '^[[:space:]]*[-]?[[:space:]]*(["][a-zA-Z0-9.:_*?]+["]|[[:digit:]]{1,3}[.][[:digit:]]{1,3}[.][[:digit:]]{1,3}[.][[:digit:]]{1,3}[/][[:digit:]]{1,2}|["][a-zA-Z0-9.:_*?]+["][[:space:]]*[[:digit:]]{1,3}[.][[:digit:]]{1,3}[.][[:digit:]]{1,3}[.][[:digit:]]{1,3}[/][[:digit:]]{1,2})[[:space:]]*$']==]
   local v_a = split (tostring(val),",")
   for k,v in ipairs(v_a) do
      v=v:gsub("^%s*",""):gsub("%s*$","")
      if os.execute(grep.." << EOF\n"..v.."\nEOF") == 0 then
      elseif os.execute(grep.." << EOF\n\""..v.."\"\nEOF") == 0 then
         v_a[k] = '"'..v..'"'
      else
           return nil, errMsg
      end
      if v:find("[%d][%d]?[%d]?[.][%d][%d]?[%d]?[.][%d][%d]?[%d]?[.][%d][%d]?[%d]?[/][%d][%d]?") then
          if os.execute(grepPrefix.." << EOF\n"..v.."\nEOF") ~= 0 then
             return nil, "Incorrect prefix:"..v.."\n"..errMsg
          end
      end
   end
   return table.concat(v_a,", ")
 end

local function createBirdConfFile(conf, cconf)
   local function addConf(table, prefix, value)
      if value ~= nil then
         table[#table + 1] = prefix..tostring(value)..";"
      end
   end

   local function confStr(str, def)
      return str and ("%q"):format(tostring(str)) or def
   end

   local function indentConf(confTable)
      local indent = 0
      for i,s in ipairs(confTable) do
         if s:find("}") then indent = indent - 1 end
         confTable[i] = ("\t"):rep(indent) .. confTable[i]
         if s:find("{") then indent = indent + 1 end
      end
      return confTable
   end

   local birdConf = {
      "# NSG Bird config",
      "# This file was generated automatically by nsgconfd"
   }

   local function genDebug(conf)
      if conf and conf.debug and conf.debug.protocols then
         if conf.debug.protocols.all then
            return "all;"
         else
            local protoBase = {}
            local protocols = {states=1,routes=1,filters=1,interfaces=1,events=1,packets=1}
            for k,v in  pairs(conf.debug.protocols) do
               if v then
                  protoBase[#protoBase+1] = protocols[k] and k
               end
            end
            if #protoBase>0 then
               return "{"..table.concat(protoBase,",").."};"
            end
         end
      end
      return "off;"
   end

   local function genDefProto(conf)
      if conf then
         addConf(birdConf,"debug ", conf.debug and genDebug(conf))
         birdConf[#birdConf+1] = conf.extra or ""
         addConf(birdConf,"disabled ",conf.disabled and "on" or "off")
         addConf(birdConf,"preference ",conf.preference)
         addConf(birdConf,"description ",conf.description and ("%q"):format(conf.description))
         addConf(birdConf,"import ",conf.import)
         addConf(birdConf,"export ",conf.export)
         addConf(birdConf,"table ",conf.table)
      end
   end
   local debugCmd = {off="0",medium="1",all="2"}
   if conf.debug and conf.debug.commands then
      birdConf[#birdConf+1] = "debug commands "..debugCmd[conf.debug.commands]..";"
   end
   birdConf[#birdConf+1] = conf and conf.debug and conf.debug.protocols and ("debug protocols "..genDebug(conf))
   birdConf[#birdConf+1] = conf.extra or ""
   
   if conf["router-id"] then
      addConf(birdConf,"router id ",conf["router-id"])
   end
   
   --table
   for k,v in pairs(conf.table or {}) do
      if k~= "master" then
         addConf(birdConf,"table ", k )
      end
   end
   
   --rip
   for k,v in ipairs(conf.rip or {}) do
      birdConf[#birdConf+1] = "protocol rip rip"..tostring(k).." {"
      genDefProto(v)
      addConf(birdConf,"authentication ",v.authentication)
      addConf(birdConf,"password ",v.password)
      addConf(birdConf,"honor ",v.honor)
      addConf(birdConf,"port ",v.port)
      addConf(birdConf,"infinity ",v.infinity)
      addConf(birdConf,"period ",v.period)
      addConf(birdConf,"timeout time ",v["timeout-time"])
      addConf(birdConf,"garbage time ",v["garbage-time"])
      for n,c in pairs(v.interface or {}) do
         birdConf[#birdConf+1] = "interface "..n.." {"
         addConf(birdConf,"mode ",c.mode)
         addConf(birdConf,"metric ",c.metric)
         birdConf[#birdConf+1] = "}; #interface "..tostring(n)
      end
      birdConf[#birdConf+1] = "} #protocol rip rip"..tostring(k).."\n"
   end

   --bgp
   
   for k,v in ipairs(conf.bgp or {}) do
      birdConf[#birdConf+1] = "protocol bgp bgp"..k.." {"
      genDefProto(v)
      addConf(birdConf,"local as ",v["local-as"])
      birdConf[#birdConf+1] = "neighbor "..v["neighbor-ip"].." as "..v["neighbor-as"]..";"
      if v.multihop.enable == "on" and v.multihop.number and v.multihop.via then   birdConf[#birdConf+1] = "multihop "..v.multihop.number.." via "..v.multihop.via..";" end
      addConf(birdConf,"next hop self ",v["next-hop-self"]=="on" and "" or nil)
      addConf(birdConf,"missing lladdr ",v["missing-lladdr"])
      addConf(birdConf,"gateway ",v["gateway"])
      addConf(birdConf,"igp table ",v["igp-table"])
      addConf(birdConf,"igp metric ",v["igp-metric"])
      addConf(birdConf,"med metric ",v["igp-metric"])
      addConf(birdConf,"source address ",v["source-address"])
      addConf(birdConf,"password ",confStr(v["password"]))
      addConf(birdConf,"passive ",v["passive"])
      addConf(birdConf,"rr client ", v["rr-client"]=="on" and "" or nil)
      addConf(birdConf,"rr cluster id ",v["rr-cluster-id"])
      addConf(birdConf,"rs client ", v["rs-client"]=="on" and "" or nil)
      addConf(birdConf,"enable route refresh ",v["enable-route-refresh"])
      addConf(birdConf,"interpret communities ",v["interpret-communities"])
      addConf(birdConf,"enable as4 ",v["enable-as4"])
      addConf(birdConf,"capabilities ",v["capabilities"])
      addConf(birdConf,"advertise ipv4 ",v["advertise-ipv4"])
      addConf(birdConf,"route limit ",v["route-limit"])
      addConf(birdConf,"disable after error ",v["disable-after-error"])
      addConf(birdConf,"hold time ",v["hold-time"])
      addConf(birdConf,"startup hold time ",v["startup-hold-time"])
      addConf(birdConf,"keepalive time ",v["keepalive-time"])
      addConf(birdConf,"connect retry time ",v["connect-retry-time"])
      addConf(birdConf,"start delay time ",v["start-delay-time"])
      birdConf[#birdConf+1] = "error wait time "..v["error-wait-time-min"]..","..v["error-wait-time-max"]..";"
      
      addConf(birdConf,"error forget time ",v["error-forget-time"])
      addConf(birdConf,"path metric ",v["path-metric"])
      addConf(birdConf,"prefer older ",v["prefer-older"])
      addConf(birdConf,"default bgp_med ",v["default-bgp-med"])
      addConf(birdConf,"default bgp_local_pref ",v["default-bgp-local-pref"])
      birdConf[#birdConf+1] = "} #protocol bgp bgp "..k.."\n"
   end
   --ospf

   for k,v in ipairs(conf.ospf or {}) do
      birdConf[#birdConf+1] = "protocol ospf ospf"..k.." {"
      genDefProto(v)
      addConf(birdConf,"rfc1583compat ",v.rfc1583compat and "on" or "off")
      addConf(birdConf,"tick ",v.tick)
      addConf(birdConf,"ecmp ",v.ecmp and (v.ecmp and "on" or "off")..(v["ecmp-limit"] and " limit "..v["ecmp-limit"] or ""))
      for areaid,areadata in pairs(v.area or {}) do
         birdConf[#birdConf+1] = "area "..areaid.." {"
         addConf(birdConf,"stub cost ", areadata["stub-cost"])
         birdConf[#birdConf+1] = "networks  {"
         for network,netdata in pairs(areadata.networks or {}) do
            birdConf[#birdConf+1] = network..(netdata.hidden and " hidden;" or ";")
         end
         birdConf[#birdConf+1] = "}; #networks"
         for stubnet, stubdata in pairs(areadata.stubnet or {}) do
            birdConf[#birdConf+1] = "stubnet "..stubnet.." {"
            addConf(birdConf,"hidden ",stubdata.hidden and "on" or "off")
            addConf(birdConf,"summary ",stubdata.summary and "on" or "off")
            addConf(birdConf,"cost ",stubdata.cost)
            birdConf[#birdConf+1] = "}; #stubnet "..stubnet
         end
         for iface, ifdata in pairs(areadata.interface or {}) do
            birdConf[#birdConf+1] = "interface "..iface.." {"
            addConf(birdConf,"cost ",ifdata.cost)
            addConf(birdConf,"stub ",ifdata.stub and "yes" or "no")
            addConf(birdConf,"hello ",ifdata.hello)
            addConf(birdConf,"ecmp weight ",ifdata["ecmp-weight"])
            addConf(birdConf,"poll ",ifdata.poll)
            addConf(birdConf,"retransmit ",ifdata.retransmit)
            addConf(birdConf,"priority ",ifdata.priority)
            addConf(birdConf,"wait ",ifdata.wait)
            addConf(birdConf,"dead count ",ifdata["dead-count"])
            addConf(birdConf,"dead ",ifdata.dead)
            addConf(birdConf,"rx buffer ",ifdata["rx-buffer"])
            addConf(birdConf,"check link ", ifdata["check-link"] and "on" or "off")
            addConf(birdConf,"type ",ifdata.type)
            addConf(birdConf,"strict nonbroadcast ",ifdata["strict-nonbroadcast"] and "on" or "off")
            addConf(birdConf,"authentication ",ifdata.authentication)
            addConf(birdConf,"password ",confStr(ifdata["password"]))
            birdConf[#birdConf+1] = "neighbors  {"
            for neigh,neighdata in pairs(ifdata.neighbors or {}) do
               birdConf[#birdConf+1] = neigh..(neighdata.eligible and " eligible;" or ";")
            end
            birdConf[#birdConf+1] = "}; #neigborhs"
            birdConf[#birdConf+1] = "}; #interface "..iface
         end
         for vil, vildata in pairs(areadata["virtual-link"] or {}) do
            birdConf[#birdConf+1] = "virtual link "..vil.." {"
            addConf(birdConf,"hello ",vildata.hello)
            addConf(birdConf,"retransmit ",vildata.retransmit)
            addConf(birdConf,"wait ",vildata.wait)
            addConf(birdConf,"dead count ",vildata["dead-count"])
            addConf(birdConf,"dead ",vildata.dead)
            addConf(birdConf,"authentication ",vildata.authentication)
            addConf(birdConf,"password ",confStr(vildata.password))
            birdConf[#birdConf+1] = "}; #virt link "..vil
         end
         birdConf[#birdConf+1] = "}; #area id "..areaid
      end
      birdConf[#birdConf+1] = "}; #proto ospf "..k.."\n"
   end

   --device
   
   birdConf[#birdConf+1] = "protocol device {"
   genDefProto(conf.device)
   addConf(birdConf,"scan time ",conf.device["scan-time"])
   for n,c in pairs(conf.device.primary or {}) do
      birdConf[#birdConf+1] = n and "primary "..(c.interface and confStr(c.interface) or "").." "..n..";"
   end
   birdConf[#birdConf+1] = "} #device\n"
   

   --static
   for k,v in ipairs(conf.static or {}) do
      birdConf[#birdConf+1] = "protocol static static"..tostring(k).." {"
      genDefProto(v)
      for n,c in pairs(v.route or {}) do
         if c.action == "via-interface" and c.interface then
            birdConf[#birdConf+1] = "route "..n.." via "..confStr(c.interface,'""')..";"
         elseif c.action == "via-gateway" and c.gateway then
            birdConf[#birdConf+1] = "route "..n.." via "..c.gateway..";"
         elseif c.action == "via-multipath" then
            local routes={}
            for l,m in pairs(c or {}) do
               if l and tostring(l):match("^%d%d?%d?[.]%d%d?%d?[.]%d%d?%d?[.]%d%d?%d?$")
 then
                  routes[#routes+1] = " via "..l..(m.weight and " weight "..m.weight or "")
               end
            end
            if #routes>0 then
               birdConf[#birdConf+1] = "route "..n.." multipath"
               birdConf[#birdConf+1] = table.concat(routes,"\n")..";"
            end
         elseif c.action then
            birdConf[#birdConf+1] = "route "..n.." "..c.action..";"
         end
      end
      birdConf[#birdConf+1] = "}#protocol static"..tostring(k).."\n"
   end
   
   --kernel
   for k,v in ipairs(conf.kernel or {}) do
      birdConf[#birdConf+1] = "protocol kernel kernel"..k.." {"
      genDefProto(v)
      addConf(birdConf,"scan time ",v["scan-time"])
      addConf(birdConf,"learn",v["learn"] and "")
      addConf(birdConf,"kernel table ",v["kernel-table"])
      addConf(birdConf,"device routes ",v["device-routes"])
      birdConf[#birdConf+1] = "}#protocol kernel"..k.."\n"
   end
   

   --direct
   for k,v in ipairs(conf.direct or {}) do
      birdConf[#birdConf+1] = "protocol direct direct"..k.." {"
      genDefProto(v)
      for n,p in pairs(v.interface or {}) do
         addConf(birdConf,"interface ", n and n)
      end
      birdConf[#birdConf+1] = "}# direct"..k.."\n"
   end
   
   --pipe
   for k,v in ipairs(conf.pipe or {}) do
      birdConf[#birdConf+1] = "protocol pipe pipe"..k.." {"
      genDefProto(v)
      addConf(birdConf,"mode ", v.mode)
      addConf(birdConf,"peer table ", v["peer-table"])
      birdConf[#birdConf+1] = "} #protocol pipe"..k.."\n"
   end
   

   birdConf[#birdConf+1] = "\n"
   local f = io.open("/etc/bird.conf", 'w')
   if f then
      f:write(table.concat(indentConf(birdConf),"\n"))
      f:close()
   end
end

local function restartBird(path, sch, conf, prevConf)
   if not s.confCmp(conf, prevConf) then
      if conf and conf.enable then
         local dconf = s.addDefault(path, sch, conf, true) --clone conf with defaults
         createBirdConfFile(dconf, conf)
         if dconf.enable ~= (prevConf and prevConf.enable) then
            os.execute("/usr/bin/killall bird")
            C.restartDaemon(
               path,
               {"/usr/sbin/bird", "-d", "-c", "/etc/bird.conf"}, 
               {noRestart = false, atExit = C.increasingSleep}
            )
         else
            os.execute("/usr/bin/killall -SIGHUP bird")
         end
         local msg = s.readpopen("/usr/sbin/bird -p -c /etc/bird.conf 2>&1")
         if #msg > 1 then
            return false, msg
         else
            return true
         end
      else
         C.restartDaemon(path)
      end
   end
   return true
end

local birtTableScheme = s.newScheme{
         _H = "birdTableName",
         show = s.newScheme{
            _H = "birdTableNShow",
            _X = function(path)
                    local cfg = s.readpopen("echo show route table "..path[#path-1].." all stats | birdc 2>&1 | sed 's/bird>//g'")
                    return cfg or ""
                 end,
            _R = 'user',
         },
      }

local debugScheme = s.newScheme{
   _H = "birdDebug",
   protocols = s.newScheme{
      _H = "birdDebugProtocols",
      _N = function (sch, conf)
              return {
                 [1] = conf and conf.all or nil,
                 all = conf and conf.all or nil,
              }
           end,
      all = s.booleanType{
         _H = "birdDebugProtocolsAll",
         _DEP = 1,
      },
      states = s.booleanType{
         _H = "birdDebugProtocolsStates",
      },
      routes = s.booleanType{
         _H = "birdDebugProtocolsRoutes",
      },
      filters = s.booleanType{
         _H = "birdDebugProtocolsFilters",
      },
      interfaces = s.booleanType{
         _H = "birdDebugProtocolsInterfaces",
      },
      events = s.booleanType{
         _H = "birdDebugProtocolsEvents",
      },
      packets = s.booleanType{
         _H = "birdDebugProtocolsPackets",
      },
   },
}

local protcolScheme = s.newScheme{
   debug = debugScheme{},
   preference = s.integerType{
      _H = "birdProtocolPreference",
      _A = {0, 65535},
   },
   disabled = s.booleanType{_H = "birdProtocolDisabled", _P = 95},
   ["router-id"] = s.quadType{_H = "birdProtocolRouterid"},
   description = s.stringType{_H = "birdProtocolDescription", _P = 100},
   import = s.stringType{_H = "birdProtocolImport", _D="all", _EX={TEXT={},},},
   export = s.stringType{_H = "birdProtocolExport", _D="none", _EX={TEXT={},},},
   table = s.stringType{_H = "birdProtocolTable", _D ="master"},
   extra = s.stringType{_H = "birdProtocolExtra"},
}

birdScheme = s.newScheme{
   _H = "bird",
   _Z = restartBird,
   enable = s.booleanType{ _P = 100, _D = false, _H = "birdEnable" },
   show = s.newScheme{
      _H = "birdShow",
      _P = -100,
      config = s.newScheme{
         _H = "birdShowConfig",
         _X = function()
                 local cfg = s.readpopen("cat /etc/bird.conf 2>&1")
                 local out = {}
                 local i = 0
                 for s in cfg:gmatch("[^\n]*\n") do
                    i = i + 1
                    out[#out +1] = ("%.4d: "):format(i)..s
                 end
                 return('\n'..table.concat(out))
              end,
         _R = 'user',
      },
      log =  s.newScheme{
         _H = "birdShowLog",
         _X = function(path)
                 path = s.clonePath(path, -2)
                 return C.getDaemonLog(path)
              end,
         _R = 'user',
      },
      memory = s.newScheme{
         _H = "birdShowMemory",
         _X = function(path)
                 local cfg = s.readpopen("echo show memory | birdc 2>&1 | sed 's/bird>//g'")
                 return cfg or ""
              end,
         _R = 'user',
      },
      interfaces = s.newScheme{
         _H = "birdShowInterfaces",
         _X = function(path)
                 local cfg = s.readpopen("echo show interfaces | birdc 2>&1 | sed 's/bird>//g'")
                 return cfg or ""
              end,
         _R = 'user',
      },
      ospf = s.newScheme{
         _H = "birdShowOspf",
         _X = function(path)
                 local cfg = s.readpopen("echo show ospf | birdc 2>&1 | sed 's/bird>//g'")
                 return cfg or ""
              end,
         _R = 'user',
      },
      ["ospf-interface"] = s.newScheme{
         _H = "birdShowOspfInterface",
         _X = function(path)
                 local cfg = s.readpopen("echo show ospf interface | birdc 2>&1 | sed 's/bird>//g'")
                 return cfg or ""
              end,
         _R = 'user',
      },
      ["ospf-lsadb"] = s.newScheme{
         _H = "birdShowOspfLsadb",
         _X = function(path)
                 local cfg = s.readpopen("echo show ospf lsadb | birdc 2>&1 | sed 's/bird>//g'")
                 return cfg or ""
              end,
         _R = 'user',
      },
      ["ospf-neighbors"] = s.newScheme{
         _H = "birdShowOspfNeighbors",
         _X = function(path)
                 local cfg = s.readpopen("echo show ospf neighbors | birdc 2>&1 | sed 's/bird>//g'")
                 return cfg or ""
              end,
         _R = 'user',
      },
      ["ospf-state"] = s.newScheme{
         _H = "birdShowOspfState",
         _X = function(path)
                 local cfg = s.readpopen("echo show ospf state | birdc 2>&1 | sed 's/bird>//g'")
                 return cfg or ""
              end,
         _R = 'user',
      },
      ["ospf-topology"] = s.newScheme{
         _H = "birdShowOspfTopology",
         _X = function(path)
                 local cfg = s.readpopen("echo show ospf topology | birdc 2>&1 | sed 's/bird>//g'")
                 return cfg or ""
              end,
         _R = 'user',
      },
      ["protocols"] = s.newScheme{
         _H = "birdShowProtocols",
         _X = function(path)
                 local cfg = s.readpopen("echo show protocols all | birdc 2>&1 |  sed 's/bird>//g'")
                 return cfg or ""
              end,
         _R = 'user',
      },
      ["static"] = s.newScheme{
         _H = "birdShowStatic",
         _X = function(path)
                 local cfg = s.readpopen("echo show static | birdc 2>&1 | sed 's/bird>//g'")
                 return cfg or ""
              end,
         _R = 'user',
      },
      ["status"] = s.newScheme{
         _H = "birdShowStatus",
         _X = function(path)
                 local cfg = s.readpopen("echo show status | birdc 2>&1 | sed 's/bird>//g'")
                 return cfg or ""
              end,
         _R = 'user',
      },
   },
   extra = s.stringType{
       _H = "birdExtra",
   },
   debug = debugScheme{ 
      _H = "birdDebug",
      commands = s.choiceType{
         _H = "birdDebugCommands",
         _A = {
            {"off", ""},
            {"medium", ""},
            {"all", ""},
         },
         _D = "off",
      },
   },
   ["router-id"] = s.quadType{
      _H = "birdRouterid"
   },
   rip = s.newScheme{
      _H = "birdRip",
      _LIST = protcolScheme{
         _H = "birdRipList",
         authentication = s.choiceType{
            _H = "birdRipAuthentication",
            _A = {
               {"none", ""},
               {"plaintext", ""},
               {"md5", ""},
            },
            _D = "none",
            _DEP =1,
         },
         password = function(_, path)
                       return (C.getConfByPath(s.clonePath(path, -1, "authentication")) or "none") ~= "none" and s.stringType{
                          _H = "birdRipPassword",
                       }
                    end,
         honor = s.choiceType{
            _H = "birdRipHonor",
            _A = {
               {"always", ""},
               {"neighbor", ""},
               {"never", ""},
            },
            _D = "never"
         },
         interface = s.newScheme{
            _H = "birdRipInterface",
            _NAME_TYPE = interfacePattern,
            _NAME_H = "birdRipInterfacePrompt",
            _NAME = s.newScheme{
               _H =  "birdRipInterfaceN",
               mode =  s.choiceType{
                  _H = "birdRipInterfaceMode",
                  _A = {
                     {"multicast", ""},
                     {"broadcast", ""},
                     {"quiet", ""},
                     {"nolisten", ""},
                  },
                  _D = "multicast"
               },
               metric = s.integerType{
                  _H = "birdRipInterfaceMetric",
                  _A = {0, 65535},
                  _D = 1
               },
            },
         },
      },
   },
   static = s.newScheme{
      _H = "birdStatic",
      _LIST = protcolScheme{
         _H = "birdStaticList",
         export = s.nilType,
         ["router-id"] = s.nilType,
         route =  s.newScheme{
            _P = 1000,
            _H = "birdStaticRoute",
            _NAME_TYPE = s.ipprefixVal,
            _NAME_H = "birdStaticRoutePrompt",
            _NAME =  s.newScheme{
               _H =  "birdStaticRouteName",
               action  = s.choiceType{
                  _H = "birdStaticRouteAction",
                  _A = {
                     {"via-interface", ""},
                     {"via-gateway", ""},
                     {"via-multipath", ""},
                     {"drop", ""},
                     {"reject", ""},
                     {"prohibit", ""},
                  },
                  _DEP = 1,
                  _P = 1000,
               },
               gateway = function(_, path)
                            return C.getConfByPath(s.clonePath(path, -1, "action"))=="via-gateway" and
                            s.ipAddressType{
                            _H = "birdStaticRouteGateway",
                         }
                      end,
               interface = function(_, path)
                              return C.getConfByPath(s.clonePath(path, -1, "action"))=="via-interface" and
                              s.stringType{
                              _H = "birdStaticRouteInterface",
                           }
                        end,
               _NAME_H =  "birdStaticRouteNPrompt",
               _NAME_TYPE = s.ipaddressVal,
               _NAME = function(_, path)
                          local action = C.getConfByPath(s.clonePath(path, -1, "action"))
                          return action == "via-multipath" and s.newScheme{
                             _H = "birdStaticRouteN",
                             weight = s.integerType{
                                _H = "birdStaticRouteNWeight",
                                _A = {1,256},
                                _D = 1,
                             },
                          }
                    end,
            },
         },
      },
   },
   bgp = s.newScheme{
      _H = "birdBGP",
      _LIST = protcolScheme{
         _H = "birdBGPList",
         ["local-as"] =s.integerType{
            _H = "birdBGPLocalas",
            _A = {0, 4294967295},
            _P = 95,
            _D =0,
         },
         ["neighbor-as"] =s.integerType{
            _H = "birdBGPNeighboras",
            _A = {0, 4294967295},
            _P = 95,
            _D =0,
         },
         ["neighbor-ip"] =s.ipAddressType{
            _H = "birdBGPNeighborip",
            _P = 95,
            _D = "0.0.0.0",
         },
         multihop =  s.newScheme{
            _H = "birdBGPMultihop",
            _N = function (sch, conf)
                    return {
                       [1] = nil,
                       ["number"] = not ( conf and conf.enable == "on") or nil,
                    }
                 end,
            enable = s.choiceType{
               _H = "birdBGPMultihopEnable",
               _A = {
                  {"on",""},
                  {"off",""},
               },
               _D = "off",
               _DEP = 1,
            },
            ["number"] = s.integerType{
               _H = "birdBGPMultihopNumber",
               _A = {1, 65535},
               _D = 1,
            },
         },
         ["next-hop-self"] = s.choiceType{
            _H = "birdBGPNexthopself",
            _A = {
               {"on",""},
               {"off",""},
            },
            _D = "off"
         },
         gateway  =  s.choiceType{
            _H = "birdBGPGateway",
            _A = {
               {"direct",""},
               {"recursive",""},
            },
         },
         ["igp-table"]= s.stringType{
            _H = "birdBGPIgptable",
         },
         ["source-address"] = s.ipAddressType{
            _H = "birdBGPSourceaddress",
         },
         password = s.stringType{
            _H = "birdBGPPassword",
         },
         passive  = s.choiceType{
            _H = "birdBGPPassive",
            _A = {
               {"on",""},
               {"off",""},
            },
            _D = "off"
         },
         ["med-metric"] = s.choiceType{
            _H = "birdBGPMedmetric",
            _A = {
               {"on",""},
               {"off",""},
            },
            _D = "off"
         },
         ["igp-metric"] = s.choiceType{
            _H = "birdBGPIgpmetric",
            _A = {
               {"on",""},
               {"off",""},
            },
            _D = "on"
         },

         ["rr-client"]  = s.choiceType{
            _H = "birdBGPRrclient",
            _A = {
               {"on",""},
               {"off",""},
            },
            _D = "off",
            _DEP = 1,
         },
         ["rr-cluster-id"] =
            function(_, path)
               return C.getConfByPath(s.clonePath(path, -1, "rr-client"))~="on" and s.quadType{_H = "birdBGPRrclusterid",}
            end,
         ["rs-client"]  = s.choiceType{
            _H = "birdBGPRsclient",
            _A = {
               {"on",""},
               {"off",""},
            },
            _D = "off"
         },
         ["enable-route-refresh"]  = s.choiceType{
            _H = "birdBGPEnablerouterefresh",
            _A = {
               {"on",""},
               {"off",""},
            },
            _D = "on"
         },

         ["interpret-communities"]  = s.choiceType{
            _H = "birdBGPInterpretcommunities",
            _A = {
               {"on",""},
               {"off",""},
            },
            _D = "on"
         },
         ["enable-as4"]   = s.choiceType{
            _H = "birdBGPEnableas4",
            _A = {
               {"on",""},
               {"off",""},
            },
            _D = "on"
         },
         capabilities  = s.choiceType{
            _H = "birdBGPCapabilities",
            _A = {
               {"on",""},
               {"off",""},
            },
            _D = "on"
         },
         ["advertise-ipv4"]  = s.choiceType{
            _H = "birdBGPAdvertiseipv4",
            _A = {
               {"on",""},
               {"off",""},
            },
            _D = "on"
         },
         ["route-limit"] = s.integerType{
            _H = "birdBGPRoutelimit",
            _A = {1}
         },
         ["disable-after-error"] = s.choiceType{
            _H = "birdBGPDisableaftererror",
            _A = {
               {"on",""},
               {"off",""},
            },
            _D = "off"
         },
         ["hold-time"] = s.integerType{
            _H = "birdBGPHoldtime",
            _A = {1,65536}
         },
         ["startup-hold-time"] = s.integerType{
            _H = "birdBGPStartholdtime",
            _A = {1,65536},
            _D = 240,
         },
         ["keepalive-time"] = s.integerType{
            _H = "birdBGPKeepalivetime",
            _A = {1,65536}
         },
         ["connect-retry-time"] = s.integerType{
            _H = "birdBGPConnectretrytime",
            _A = {1,65536},
            _D = 120,
         },
         ["start-delay-time"] = s.integerType{
            _H = "birdBGPStartdelaytime",
            _A = {1,65536},
            _D = 5,
         },
         ["error-wait-time-min"] = s.integerType{
            _H = "birdBGPErrorwaittimemin",
            _A = {1,65536},
            _D = 60,
         },
         ["error-wait-time-max"] = s.integerType{
            _H = "birdBGPErrorwaittimemax",
            _A = {1,65536},
            _D = 300,
         },
         ["error-forget-time"] = s.integerType{
            _H = "birdBGPErrorforgettime",
            _A = {1,65536},
            _D = 300,
         },
         ["path-metric"] = s.choiceType{
            _H = "birdBGPPathmetric",
            _A = {
               {"on",""},
               {"off",""},
            },
            _D = "on"
         },
         ["prefer-older"] = s.choiceType{
            _H = "birdBGPPreferolder",
            _A = {
               {"on",""},
               {"off",""},
            },
            _D = "off"
         },
         ["default-bgp-med"] = s.integerType{
            _H = "birdBGPDefaultbgpmed",
            _A = {0, 65535},
            _D = 0,
         },
         ["default-bgp-local-pref"] = s.integerType{
            _H = "birdBGPDefaultbgplocalpref",
            _A = {0, 65535},
            _D = 100,
         },
      },
   },
   ospf = s.newScheme{
      _H = "birdOSPF",
      _LIST = protcolScheme{
         _H = "birdOSPFList",
         rfc1583compat = s.booleanType{ 
            _H = "birdOSPFRfc1583compat",
            _D = false,
         },
         tick = s.integerType{
            _H = "birdOSPFTick",
            _A = {0, 65535},
            _D =1,
         },
         ecmp = s.booleanType{
            _H = "birdOSPFEcmp",
            _D = false,
            _DEP =1,
         },
         ["ecmp-limit"] = function(_, path)
                             return C.getConfByPath(s.clonePath(path, -1, "ecmp"))==true and
                             s.integerType{
                             _H = "birdOSPFEcmplimit",
                             _A = {0, 2^16},
                             _D = 16,
                          }
                       end,
         area = s.newScheme{
            _H = "birdOSPFArea",
            _NAME_H =  "birdOSPFAreaIdPrompt",
            _NAME_TYPE = s.quadVal,
            _NAME =  s.newScheme{
               _H = "birdOSPFAreaId",
               ["stub-cost"] = s.integerType{
                  _H = "birdOSPFStubcost",
                  _A = {1, 65535},
               },
               networks =  s.newScheme{
                  _H = "birdOSPFAreaNetworks",
                  _NAME_TYPE = s.ipprefixVal,
                  _NAME = s.newScheme{
                     _H = "birdOSPFAreaNetworksN",
                     hidden =  s.booleanType{
                        _H = "birdOSPFAreaNetworksHidden",
                        _D = false,
                     },
                  },
               },
               stubnet =  s.newScheme{
                  _H = "birdOSPFAreaStubnet",
                  _NAME_TYPE = s.ipprefixVal,
                  _NAME = s.newScheme{
                     _H = "birdOSPFAreaStubnetPrefix",
                     hidden =  s.booleanType{
                        _H = "birdOSPFAreaStubnetHidden",
                        _D = false,
                     },
                     summary =  s.booleanType{
                        _H = "birdOSPFAreaStubnetSummary",
                        _D = false,
                     },
                  },
               },
               interface =  s.newScheme{
                  _H = "birdOSPFAreaInterface",
                  _NAME_H = "birdOSPFAreaInterfacePrompt",
                  _NAME_TYPE = interfacePattern,
                  _NAME = s.newScheme{
                     _H = "birdOSPFAreaInterfaceN",
                     ["ecmp-weight"] = function(_, path)
                                    return C.getConfByPath(s.clonePath(path, -5, "ecmp"))==true and
                                    s.integerType{
                                    _H = "birdOSPFEcmpweight",
                                    _A = {1, 256},
                                    _D = 1,
                                 }
                              end,
                     cost = s.integerType{
                        _H = "birdOSPFAreaInterfaceCost",
                        _A = {0, 65535},
                        _D = 10,
                     },
                     stub =  s.booleanType{
                        _H = "birdOSPFAreaInterfaceStub",
                        _D = false,
                     },
                     hello = s.integerType{
                        _H = "birdOSPFAreaInterfaceHello",
                        _A = {0, 65535},
                        _D = 10,
                     },
                     retransmit = s.integerType{
                        _H = "birdOSPFAreaInterfaceRetransmit",
                        _A = {0, 65535},
                        _D = 5,
                     },
                     priority = s.integerType{
                        _H = "birdOSPFAreaInterfacePriority",
                        _A = {0, 65535},
                        _D = 1,
                     },
                     wait = s.integerType{
                        _H = "birdOSPFAreaInterfaceWait",
                        _A = {0, 65535},
                        _D = 40,
                     },
                     ["dead-count"] = s.integerType{
                        _H = "birdOSPFAreaInterfaceDeadcount",
                        _A = {0, 65535},
                     },
                     dead = s.integerType{
                        _H = "birdOSPFAreaInterfaceDead",
                        _A = {0, 65535},
                     },
                     ["check-link"] =  s.booleanType{
                        _H = "birdOSPFAreaInterfaceChecklink",
                        _D = false,
                     },
                     ["rx-buffer"] = s.variantType{
                        _H = "birdOSPFAreaInterfaceRxbuffer",
                        _A = {
                           s.integerType{
                              _A = {0, 65535},
                           },
                         --  s.choiceType{
                         --     _A = {"\"NORMAL\"", "\"LARGE\""},
                         --  },
                        },
                        _D = 3200,
                     },
                     type = s.choiceType{
                        _H = "birdOSPFAreaInterfaceType",
                        _A = {
                           {"broadcast", ""},
                           {"pointopoint", ""},
                        },
                     },
                     authentication = s.choiceType{
                        _H = "birdOSPFAreaInterfaceAuthentication",
                        _A = {
                           {"none", ""},
                           {"simple", ""},
                           {"cryptographic", ""},
                        },
                        _D = "none",
                        _DEP = 1,
                     },

                     password = function(_, path)
                                   return (C.getConfByPath(s.clonePath(path, -1, "authentication")) or "none") ~= "none" and  s.stringType{
                                      _H = "birdOSPFAreaInterfacePassword"
                                   }
                                end,
                  },
               },
               ["virtual-link"] =  s.newScheme{
                  _H = "birdOSPFAreaVirtuallink",
                  _NAME_H = "birdOSPFAreaVirtuallinkPrompt",
                  _NAME_TYPE = s.quadVal,
                  _NAME = s.newScheme{
                     _H = "birdOSPFAreaVirtuallinkN",
                     hello = s.integerType{
                        _H = "birdOSPFAreaVirtuallinkHello",
                        _A = {0, 65535},
                        _D = 10,
                     },
                     retransmit = s.integerType{
                        _H = "birdOSPFAreaVirtuallinkRetransmit",
                        _A = {0, 65535},
                        _D = 5,
                     },
                     wait = s.integerType{
                        _H = "birdOSPFAreaVirtuallinkWait",
                        _A = {0, 65535},
                        _D = 40,
                     },
                     ["dead-count"] = s.integerType{
                        _H = "birdOSPFAreaVirtuallinkDeadcount",
                        _A = {0, 65535},
                     },
                     dead = s.integerType{
                        _H = "birdOSPFAreaVirtuallinkDead",
                        _A = {0, 65535},
                     },
                     authentication = s.choiceType{
                        _H = "birdOSPFAreaVirtuallinkAuthentication",
                        _A = {
                           {"none", ""},
                           {"simple", ""},
                           {"cryptographic", ""},
                        },
                        _D = "none",
                        _DEP = 1,
                     },
                     password = function(_, path)
                                   return (C.getConfByPath(s.clonePath(path, -1, "authentication")) or "none") ~= "none" and s.stringType{
                                      _H = "birdOSPFAreaVirtuallinkPassword",
                                   }
                                end,
                  },
               },
            },
         },
      },
   },
   device = s.newScheme{
      _H = "birdDevice",
      disabled = s.booleanType{_H = "birdProtocolDisabled", _P = 95},
      ["scan-time"] = s.integerType{
         _H = "birdDeviceScantime",
         _A = {0, 65535},
         _D = 60,
      },
      primary = s.newScheme{
         _H = "birdDevicePrimary",
         _NAME_TYPE = s.ipprefixOrAddressVal,
         _NAME_H = "birdDevicePrimaryNPrompt",
         _NAME = s.newScheme{
            _H = "birdDevicePrimaryN",
            interface = s.stringType{
                _H = "birdDevicePrimaryNInterface",
            },
         },
      },
   },
   kernel = s.newScheme{
      _H = "birdKernel",
      _LIST = protcolScheme{
         _H = "birdKernelList",
         ["router-id"] = s.nilType,
         ["scan-time"] = s.integerType{
            _H = "birdKernelScantime",
            _A = {0, 65535},
         },
         learn = s.booleanType{ 
            _H = "birdKernelLearn",
         },
         ["device-routes"] = s.booleanType{ 
            _H = "birdKernelDeviceroutes",
         },
         ["kernel-table"] = s.integerType{
            _H = "birdKernelKerneltable",
            _A = {0, 65535},
         },
         ["show-kernel-table"] = s.newScheme{
            _H = "birdKernelShowkerneltable",
            _X = function(path)
                    local kt = C.getConfByPath(s.clonePath(path, -1, "kernel-table")) or "main"
                    kt = tostring(kt)=='0' and "main" or kt
                    local cmd = {
                       ip.ip,
                       (ip.family[path[1]]),
                       "route show",
                       "table "..kt,
                       " 2>&1"
                    }
                    return ('\n'..s.readpopen(table.concat(cmd," ")))
                 end
         }
      },
   },
   direct = s.newScheme{
      _H = "birdDirect",
      _LIST = protcolScheme{
         _H = "birdDirectList",
         export = s.nilType,
         ["router-id"] = s.nilType,
         interface =  s.newScheme{
            _H = "birdDirectInterface",
            _NAME_TYPE = interfacePattern,
            _NAME_H = "birdDirectInterfacePrompt",
            _NAME = s.constantType{
               _H = "birdDirectInterfaceName",
               _D = true,
               _V = function () return true end,
            },
         },
      },
   },
   pipe = s.newScheme{
      _H = "birdPipe",
      _LIST = protcolScheme{
         _H = "birdPipeList",
         ["router-id"] = s.nilType,
         preference = s.nilType,
         ["peer-table"] = s.stringType{
            _H = "birdPipePeertable",
         },
         mode = s.choiceType{
            _H = "birdPipeMode",
            _A = {
               {"opaque", ""},
               {"transparent", ""},
            },
            _D = "opaque",
         },
      },
   },
   table = s.newScheme{
      _H = "birdTable",
      _NAME_H = "birdTablePrompt",
      _NAME = birtTableScheme{
      },
      master = birtTableScheme{
         _H = "birdTableMaster",
      },
   }
}
