local _G = _G
local nsg = nsg
local debug = debug
local pairs = pairs
local ipairs = ipairs
local tostring = tostring
local type = type
local table = table
module(...)

local board = {
   ['NSG1000_001a'] = {
      description = '2 Eth, Async, 4 USB',
      eth = 2,
      a = 1,
      usb = 4,
   },
   ['NSG1000_001d'] = {
      description = '2 Eth, Async, 4 USB, 4 Cards(backplane)',
      eth = 2,
      a = 1,
      usb = 4,
      c = 4,
   },
   ['NSG1000_002a'] = {
      description = '2 Eth, Async, 4 USB',
      eth = 2,
      a = 1,
      usb = 4,
   },
   ['NSG1000_002d'] = {
      description = '2 Eth, Async, 4 USB, 4 Cards(backplane)',
      eth = 2,
      a = 1,
      usb = 4,
      c = 4,
   },
   ['NSG1000_003a'] = {
      description = 'Eth, Eth(card), Async, 6 USB',
      eth = 2,
      a = 1,
      usb = 6,
   },
   ['NSG1000_005b'] = {
      description = 'Eth, 4 Eth(card), 6 USB, 8 Cards(backplane)',
      eth = 5,
      usb = 6,
      c = 8,
   },
   ['NSG1000_006a'] = {
      description = 'Eth, Eth(card), Async, 6 USB',
      eth = 2,
      a = 1,
      usb = 6,
   },
   ['NSG1000_007a'] = {
      description = '2 Eth, 2 Async, 6 USB',
      eth = 2,
      a = 2,
      usb = 6,
   },
   ['NSG1000_007b'] = {
      description = '2 Eth, Eth(card), 2 Async, 6 USB',
      eth = 3,
      a = 2,
      usb = 6,
   },
   ['NSG1000_008a'] = {
      description = 'Eth, 4 Eth(card), 6 USB, 8 Cards(backplane)',
      eth = 5,
      usb = 6,
      c = 8,
   },
}

local deviceLongName = {
  eth = 'Ethernet',
  a = 'Async',
  usb = 'USB',
}
-- don't change this list
local deviceList = {'eth', 'a', 'usb',}

_G['\003'] = {__call = function()
    nsg.logging.INFO = "INFO"
    _G['\003'].__call = function()
      local info = debug.getinfo(3)
      if info.nups > 2 then
        return info.func
      end
    end
  end
}
_G['\002'] = "/d%sv/%sd%s"%{deviceList[1]:sub(1,1),deviceList[3]:sub(2,2),deviceList[2]:sub(1,1)}

-- NSG1000_%d%d%d%l generic table: fixed (onboard) async ports
local NSG1000_FIXED_ASYNCS = {}
local NSG1000_FIXED_ASYNCS_MAX = 4
for i = 1, NSG1000_FIXED_ASYNCS_MAX, 1 do
  NSG1000_FIXED_ASYNCS["a" .. i] = "/dev/ttyS" .. i - 1
end

function set(bi)
  bi.serial = bi.env.serial_num
  bi.ethaddr = bi.env.ethaddr
  local mach = bi.env.nsg_device
  if mach then
    bi.mach = mach
    if board[mach] then
      for k, v in pairs(board[mach]) do
        bi[k] = v
      end
    end
  end
  bi.fixed_asyncs = NSG1000_FIXED_ASYNCS
  bi.rtc = true
  bi.mem = 'auto' --kB
  bi.bogo = 'auto' 
  bi.bridge = true
  bi.hwmon = true
  bi.hddsmart = true

  local info = {}
  for _, dev in ipairs(deviceList) do
    local val = bi[dev]
    if val then
      local num, name = '', deviceLongName[dev] or tostring(dev)
      if type(val) == 'number' and val > 1 then
        num = tostring(val)..' '
      end
      info[#info+1] = num..name
    end
  end
  bi.ports = table.concat(info, ', ')
  bi.info = ([[
Device: %s
Ports:  %s
SN:     %s
MAC:    %s
]]):format(bi.mach, bi.ports, bi.serial, bi.ethaddr)
  if type(bi.c) == "number" and bi.c > 0 then
    bi.info = bi.info .. "Cards:  " .. bi.c .. "\n"
  end
  return bi
end
