local pairs = pairs
local ipairs = ipairs
local tostring = tostring
local type = type
local table = table
module(...)

local board = {
   ['NSG1800'] = {
      description = '1Eth(1), 1Eth(4), 1Eth(opt), s1(usb), s2(usb)',
      eth = 3,
      s1 = true,
      s2 = true,
      m1 = true,
      a = 1, -- console OR rs-232
      mmc = 1, -- micro-sd
   },
   ['NSG1800F'] = {
      description = '1Eth(1), 1Eth(4), 1Eth(opt), s1(usb), s2(usb), WiFi',
      eth = 3,
      s1 = true,
      s2 = true,
      wifi = 1,
      a = 1, -- console OR rs-232
      mmc = 1, -- micro-sd
   },
   ['NSG1800X'] = {
      description = '1Eth(1), 1Eth(4), 1Eth(opt), s1(usb), s2(usb), WiMax',
      eth = 3,
      s1 = true,
      s2 = true,
      wimax = 1,
      a = 1, -- console OR rs-232
      mmc = 1, -- micro-sd
   },
   ['NSG1800L'] = {
      description = '1Eth(1), 1Eth(4), 1Eth(opt), s1(usb), s2(usb), Minicard module',
      eth = 3,
      s1 = true,
      s2 = true,
      a = 1, -- console OR rs-232
      mmc = 1, -- micro-sd
   },
}

local deviceLongName = {
   eth = 'Ethernet(10/100)',
   a = 'Async RS-232',
   s1 = 'Interface module (slot1)',
   s2 = 'Interface module (slot2)',
   wimax = 'WiMAX',
   wifi = 'Wi-Fi',
   m1 = 'Internal minicard module',
   mmc = 'Micro-SD',   -- here is micro-SD
}
local deviceList = {'eth', 'a', 's1', 's2', 'wifi', 'wimax', 'm1', 'mmc', }

-- fixed (onboard) async ports
local NSG1800_FIXED_ASYNCS = {
  a1 = "/dev/ttyS0",
}

function set(bi)
  bi.serial = bi.env.serial_num
  bi.ethaddr = bi.env.ethaddr
  local mach = bi.env.nsg_device
  if mach then
    bi.mach = mach
    if board[mach] then
      for k, v in pairs(board[mach]) do
        bi[k] = v
      end
    end
  end
  bi.fixed_asyncs = NSG1800_FIXED_ASYNCS
  bi.rtc = true
  bi.mem = 'auto' --kB
  bi.bogo = 'auto'
  bi.bridge = true

  local info = {}
  for _, dev in ipairs(deviceList) do
    local val = bi[dev]
    if val then
      local num, name = '', deviceLongName[dev] or tostring(dev)
      if type(val) == 'number' and val > 1 then
        num = tostring(val)..' '
      end
      info[#info+1] = num..name
    end
  end
  bi.ports = table.concat(info, ', ')
  bi.info = ([[
Device: %s
Ports:  %s
SN:     %s
MAC:    %s
]]):format(bi.mach, bi.ports, bi.serial, bi.ethaddr)
  return bi
end
