local pairs = pairs
local ipairs = ipairs
local tostring = tostring
local type = type
local table = table
module(...)

local board = {
   ['NSG1820'] = {
      description = '2Eth(10/100/1000), Minicard module',
      eth = 2,
      m1 = true,
      m2 = true,
      usb = 1,
      a = 1, -- console OR rs-232
      mmc = 1, -- micro-sd
   },
   ['NSG1820H'] = {
      description = '2Eth(10/100/1000), Minicard module, 3G(HSDPA)',
      eth = 2,
      m1 = true,
      ['3g'] = 1,
      usb = 1,
      a = 1, -- console OR rs-232
      mmc = 1, -- micro-sd
   },
   ['NSG1820D'] = {
      description = '2Eth(10/100/1000), Minicard module, CDMA(EVDO)',
      eth = 2,
      m1 = true,
      m2 = true,
      cdma = 1,
      usb = 1,
      a = 1, -- console OR rs-232
      mmc = 1, -- micro-sd
   },
   ['NSG1820HD'] = {
      description = '2Eth(10/100/1000), Minicard module, 3G(HSDPA), CDMA(EVDO)',
      eth = 2,
      m1 = true,
      ['3g'] = 1,
      cdma = 1,
      usb = 1,
      a = 1, -- console OR rs-232
      mmc = 1, -- micro-sd
   },
}

local deviceLongName = {
   eth = 'Ethernet(10/100/1000)',
   a = 'Async RS-232',
   wifi = 'Wi-Fi',
   m1 = 'Internal minicard module(base)',
   m2 = 'Internal minicard module(extended)',
   ['3g'] = '3G',
   cdma = 'CDMA(EVDO)',
   mmc = 'Micro-SD',   -- here is micro-SD
}
local deviceList = {'eth', 'a', 'wifi', 'm1', 'm2', 'cdma', '3g', 'mmc', }

-- fixed (onboard) async ports
local NSG1820_FIXED_ASYNCS = {
  a1 = "/dev/ttyS0",
}

function set(bi)
  bi.serial = bi.env.serial_num
  bi.ethaddr = bi.env.ethaddr
  local mach = bi.env.nsg_device
  if mach then
    bi.mach = mach
    if board[mach] then
      for k, v in pairs(board[mach]) do
        bi[k] = v
      end
    end
  end
  bi.fixed_asyncs = NSG1820_FIXED_ASYNCS
  bi.rtc = true
  bi.mem = 'auto' --kB
  bi.bogo = 'auto'
  bi.bridge = true

  local info = {}
  for _, dev in ipairs(deviceList) do
    local val = bi[dev]
    if val then
      local num, name = '', deviceLongName[dev] or tostring(dev)
      if type(val) == 'number' and val > 1 then
        num = tostring(val)..' '
      end
      info[#info+1] = num..name
    end
  end
  bi.ports = table.concat(info, ', ')
  bi.info = ([[
Device: %s
Ports:  %s
SN:     %s
MAC:    %s
]]):format(bi.mach, bi.ports, bi.serial, bi.ethaddr)
  return bi
end
