local pairs = pairs
local ipairs = ipairs
local tostring = tostring
local type = type
local table = table
module(...)

local board = {
  ['NSGVM'] = {
    description = '4 Eth, 4 Async, USB',
    eth = 4,
    a = 4,
    usb = 1,
  },
}

local deviceLongName = {
  eth = 'Ethernet',
  a = 'Async',
  usb = 'USB',
}
local deviceList = {'eth', 'a', 'usb',}

-- fixed (onboard) async ports
local NSGVM_FIXED_ASYNCS = {}
local NSGVM_FIXED_ASYNCS_MAX = 4
for i = 1, NSGVM_FIXED_ASYNCS_MAX, 1 do
  NSGVM_FIXED_ASYNCS["a" .. i] = "/dev/ttyS" .. i - 1
end

function set(bi)
  bi.serial = "1"
  bi.ethaddr = "00:09:56:86:XX:XX"
  local mach = "NSGVM"
  if mach then
    bi.mach = mach
    if board[mach] then
     for k, v in pairs(board[mach]) do
       bi[k] = v
     end
   end
  end
  bi.fixed_asyncs = NSGVM_FIXED_ASYNCS
  bi.rtc = true
  bi.mem = 'auto'
  bi.bogo = 'auto'
  bi.bridge = true

  local info = {}
  for _, dev in ipairs(deviceList) do
    local val = bi[dev]
    if val then
      local num, name = '', deviceLongName[dev] or tostring(dev)
      if type(val) == 'number' and val > 1 then
        num = tostring(val)..' '
      end
      info[#info+1] = num..name
    end
  end
  bi.ports = table.concat(info, ', ')
  bi.info = ([[
Device: %s
Ports:  %s
SN:     %s
MAC:    %s
]]):format(bi.mach, bi.ports, bi.serial, bi.ethaddr)
  return bi
end
