module(..., package.seeall)
local s = require'scheme'
local nsg = require'nsg'


local function checkName(val)
   if (val==nil) or val=='' then
      val = s.readpopen("date '+%Y%m%d-%H%M%S'"):gsub('[^A-Za-z0-9%-%.]','')
   end
   if  nil == string.match(val, "^[A-Za-z0-9%-%.]+$") or #val > 30 then
      return nil, "Config slot name. Correct symbols: '-' '.' 'a-Z' '0-9'. 30 symbols Max."
   end
   return tostring(val)
end

local function confToTable(str)
   local confTable, res
   local func, err = loadstring('setfenv(1, {}) '..str, "")
   if not func then
      return {_err = err}
   end
   local res, confTable = pcall(func)
   return confTable
end

local function loadConf(si, type, url)
   local confstr
   if type=="etc" then
      local cmd = "transfer "..nsg.shellEnquote(url).." file:///etc/newetc.tar.gz 2>&1 && tar -C /  -zxvf /etc/newetc.tar.gz 2>&1 || rm -f /etc/newetc.tar.gz"
      local res = s.readpopen(cmd)
      if os.execute("[ -f /etc/newetc.tar.gz ]") == 0 then
         local _H = "configMesOkEtcLoaded"
         return res.."\n"..s.localeString(_H, si.locale)
      else
         local _H = "configMesErrEtcNotloaded"
         return res.."\n"..s.localeString(_H, si.locale)
      end
   else
      local err, o1, o2 = s.executeEx("transfer "..nsg.shellEnquote(url).." file://- ")
      if err ~= 0 then
         return o2
      end
      local conf = confToTable(o1) or {_err='Config is empty or wrong!'}
      if conf._err then
         return  conf._err.."\n"..o1
      else
         C.setConfigValue({}, conf or {})
         local _H = "configMesOkNsgconfigLoaded"
         return s.localeString(_H, si.locale)
      end
   end
end

local function validProto(val)
              val = tostring(val or "")
              if val:match("^[a-z]+://.*") then
                 return val
              else
                 return nil, "URL in format [PROTOCOL]://<PROTOCOL_SPECIFIC_DATA>"
              end
           end

local confScheme = s.newScheme{
   _R = 'admin',
   _H = "configN",
   type =  function(_, path)
              local etc = not (C.getConfOrDefault(s.clonePath(path, -1, "url")) or ""):match("^data:") and "etc" or nil
              return s.choiceType{
                 _R = 'admin',
                 _H = 'configNMode',
                 _D = "nsgconfig",
                 _DEP = 1,
                 _P = 90,
                 _A = {"nsgconfig", etc}
              }
           end,
   url = s.stringType{
      _R = 'admin',
      _H = 'configNUrl',
      _D = "data://",
      _P = 90,
      _DEP = 1,
      _V = validProto
   },
   show = s.newScheme{
      _R = 'admin',
      _H = 'configNShow',
      _P = 80,
      _X = function(path,sch,val,sesNum)
              local url = C.getConfOrDefault(s.clonePath(path, -1, "url")) or ""
              local type = C.getConfOrDefault(s.clonePath(path, -1, "type")) or "nsgconfig"
              local confstr
              if type=="etc" then
                 local cmd = "transfer "..nsg.shellEnquote(url).." file://- | tar -tzv 2>&1"
                 confstr = cmd.."\n"..s.readpopen(cmd)
              else
                 local err, o1, o2 = s.executeEx("transfer "..nsg.shellEnquote(url).." file://- ")
                 if err ~= 0 then
                    return o2
                 end
                 local conf = confToTable(o1) or {"_err='Config is empty or wrong!'"}
                 return s.ser(conf,1)
              end
              return confstr
           end,
   },
   store =  function(_, path)
               local url = C.getConfOrDefault(s.clonePath(path, -1, "url")) or ""
               local urlIsData = url:match("^data:") and true or false
               return s.newScheme{
                  _R = 'admin',
                  _H = 'configNStore',
                  _DEP = urlIsData and 1 or nil,
                  _A = {'Store current config.'},
                  _XI = not urlIsData and true or nil,
                  _X = function(path,sch,val,sesNum)
                          local si = sesNum and C.getSessionInfo(sesNum) or {}
                          local _H = "configMesOkStored"
                          local url = C.getConfOrDefault(s.clonePath(path, -1, "url")) or ""
                          local type = C.getConfOrDefault(s.clonePath(path, -1, "type")) or "nsgconfig"
                          if type == "etc" then
                             if url:match("^data://") then
                                _H = "configMesErrCantStoreEtcToConfig"
                                return  s.localeString(_H, si.locale)
                             elseif url:match("^file:") and not url:match("^file:///.*[^/]$") then
                                _H = "configMesErrWrongFilePathName"
                                C.restartDaemon(path,
                                                {"echo", s.localeString(_H, si.locale)},
                                                {sesNum = sesNum, noTimestamp = true, noRestart = true}
                                             )
                             else
                                local exclude
                                if url:match("^file:///") then
                                   exclude="--exclude "..nsg.shellEnquote(nsg.urlDecode(url:match("^file:///(.*)$") or ""))..""
                                end
                                local cmd = "{ tar -cz --exclude etc/newetc.tar.gz "..(exclude or "").." /etc | transfer file://- "..nsg.shellEnquote(url).." ; } 2>&1"
                                C.restartDaemon(path,
                                                {"/bin/sh", "-c", cmd},
                                                {sesNum = sesNum, noTimestamp = true, noRestart = true}
                                             )
                             end
                          else
                             local conf = s.deepcopy(C.getConfByPath({})) or {}
                             if url:match("^data://") then
                                if conf and  conf.system and conf.system.configurations then
                                   conf.system.configurations = {}
                                end
                                C.setConfigValue(s.clonePath(path, -1, "url"),"data://"..nsg.urlEncode("return "..s.ser(conf, 1)))
                                _H = "configMesOkStored"
                                return  s.localeString(_H, si.locale)
                             else
                                local cmd ="transfer data://"..nsg.shellEnquote(nsg.urlEncode("return "..s.ser(conf, 1))).." "..nsg.shellEnquote(url).." 2>&1"
                                C.restartDaemon(path,
                                                {"/bin/sh", "-c", cmd},
                                                {sesNum = sesNum, noTimestamp = true, noRestart = true}
                                             )
                             end
                          end
                       end,
               }
            end,
   clear = s.newScheme{
      _R = 'admin',
      _H = 'configNClear',
      _P = -10,
      _DEP = 1,
      _A= {'Create empty configuration file.'},
      _X = function(path,sch,val,sesNum)
              local si = sesNum and C.getSessionInfo(sesNum) or {}
              local _H = "configMesOkCleared"
              local url = C.getConfOrDefault(s.clonePath(path, -1, "url")) or ""
              local type = C.getConfOrDefault(s.clonePath(path, -1, "type")) or "nsgconfig"
              if url:match("^data://") then
                 C.setConfigValue(s.clonePath(path, -1, "url"),"data://"..nsg.urlEncode("return {}"))
                 return s.localeString(_H, si.locale)
              else
                 local cmd
                 if type == "etc" then
                    cmd = "tar cz --exclude etc /etc | transfer file://- "..nsg.shellEnquote(url)
                 else
                    cmd = "transfer data://"..nsg.urlEncode("return {}").." "..nsg.shellEnquote(url)
                 end
                 local res, stdo, stde = s.executeEx(cmd)
                 if res~=0 then
                    return cmd.."\n"..stde
                 else
                    return s.localeString(_H, si.locale)
                 end
              end
  end
   },
   load = s.newScheme{
      _R = 'admin',
      _H = 'configNLoad',
      _X = function(path,sch,val,sesNum)
              local si = sesNum and C.getSessionInfo(sesNum) or {}
              local url = C.getConfOrDefault(s.clonePath(path, -1, "url")) or ""
              local type = C.getConfOrDefault(s.clonePath(path, -1, "type")) or "nsgconfig"
              return loadConf(si, type,url)
           end,
   },
}


configScheme = s.newScheme{
   _H = "configurations",
   _HTML = "configweb",
   _NAME_TYPE = checkName,
   _NAME = confScheme,
   ["CURRENT"] = confScheme{
      _H = "configCurrent",
      _P = 50,
      type =  s.constantType{
         _R = 'admin',
         _H = 'configNMode',
         _D = "nsgconfig",
         _P = 90,
      },
      url =  s.constantType{
         _R = 'admin',
         _H = 'configNUrl',
         _D = "file:///etc/nsgconfig",
         _P = 90,
      }
   },
   ["nsgconfig-store"] = s.stringType{
      _R = 'admin',
      _H = 'configNsgconfigStore',
      _P = 70,
      _A= {'Copy current config to URL.'},
      _X = function(path,sch,val,sesNum)
              local url, err = validProto(val)
              if not url then
                 return err
              end
              local conf = s.deepcopy(C.getConfByPath({}))
              if url:match("^data://") then
                 _H = "configMesErrInvalidurl"
                 return  s.localeString(_H, si.locale)
              end
              return s.readpopen("transfer data://"..nsg.shellEenquote(nsg.urlEncode("return "..s.ser(conf, 1))).." "..nsg.shellEnquote(url).." 2>&1 1>/dev/null")
           end,
   },
   ["nsgconfig-load"] = s.stringType{
      _P = 70,
      _R = 'admin',
      _H = 'configNsgconfigLoad',
      _X = function(path,sch,val,sesNum)
              local url, err = validProto(val)
              if not url then
                 return err
              end
              local si = sesNum and C.getSessionInfo(sesNum) or {}
              return loadConf(si, "nsgconfig",url)
           end,
      _A= {'Replace current config with "config" value, except node "configurations".'},
      _DEP = 10,
   },

   ["etc-load"] = s.stringType{
      _P = 70,
      _R = 'admin',
      _H = 'configEtcLoad',
      _X = function(path,sch,val,sesNum)
              local url, err = validProto(val)
              if not url then
                 return err
              end
              local si = sesNum and C.getSessionInfo(sesNum) or {}
              return loadConf(si, "etc", url)
           end,
      _A= {'Replace current etc with loaded etc, reboot needed.'},
   },
   ["etc-store"] =s.stringType{
      _R = 'admin',
      _P = 70,
      _H = 'configEtcStore',
      _A = {'Store current etc.'},
      _X = function(path,sch,val,sesNum)
              local url, err = validProto(val)
              if not url then
                 return err
              end
              local si = sesNum and C.getSessionInfo(sesNum) or {}
              local _H = "configMesOkStored"
              local type = "etc"
              if url:match("^data://") then
                 _H = "configMesErrInvalidurl"
                 return  s.localeString(_H, si.locale)
              elseif url:match("^file:") and not url:match("^file:///.*[^/]$") then
                 _H = "configMesErrWrongFilePathName"
                 return s.localeString(_H, si.locale)
              else
                 local exclude
                 if url:match("^file:///") then
                    exclude="--exclude "..nsg.shellEnquote(nsg.urlDecode(url:match("^file:///(.*)$") or ""))..""
                 end
                 local cmd = "{ tar -cz --exclude etc/newetc.tar.gz "..(exclude or "").." /etc | transfer file://- "..nsg.shellEnquote(url).." ; } 2>&1"
                 return  s.readpopen(cmd)
              end
           end
   }
}
