module(..., package.seeall)
local s = require'scheme'
local iface_mod = require'nsgifaceconf'

local restartDnsMasqServerEx
local resetDnsDhcpNeteventKey={}
local resetDnsDhcpAliasKey={}
local dnsDhcpDaemons={}
local dhcpRelayDaemons={}

local resetDnsDhcpNeteventCallBack =
   function(msg)
      if (msg.type == 'newlink' and msg.ifup) then
         for ifName in pairs(dnsDhcpDaemons) do
            if iface_mod.getIfaceName(ifName) == msg.ifname then
               restartDnsMasqServerEx({"services","dns",ifName}, true)
            end
         end
      end
   end

local resetDnsDhcpAliasCallBack =
   function(alias, name)
      if dnsDhcpDaemons[alias] or  dnsDhcpDaemons[name] then
         restartDnsMasqServerEx({"services","dns",dnsDhcpDaemons[name] and name or alias}, true)
      end
   end

function createConfFile(filename, path)
   local needStart = false
   local Changed = true
   local Conf = {}
   local ifName = path[#path]
   local dhcpPath = s.clonePath(path, -2, "dhcp", ifName)
   local dnsPath = s.clonePath(path, -2, "dns", ifName)
   local dhcpConf = C.getConfByPath(dhcpPath)
   local dnsConf = C.getConfByPath(dnsPath)
   dhcpConf = s.addDefault(dhcpPath, C.getSchemeByPath(dhcpPath), dhcpConf, true)
   dnsConf = s.addDefault(dnsPath, C.getSchemeByPath(dnsPath), dnsConf, true)
   local dhcpRelayConf =nil
   if not ((dhcpConf and dhcpConf["adm-state"]=="up") or (dnsConf and dnsConf["adm-state"]=="up")) then
      os.execute("echo > "..filename)
      return needStart, Changed
   end

   Conf[#Conf +1] = "bind-interfaces"
   Conf[#Conf +1] = "except-interface=lo"
   Conf[#Conf +1] = "interface="..iface_mod.getIfaceName(ifName)
   if dhcpConf and dhcpConf["adm-state"]=="up" and dhcpConf["mode"]~="relay"  then
      Conf[#Conf +1] = table.concat{
         "dhcp-range=interface:",
         iface_mod.getIfaceName(ifName),
         ",set:",
         iface_mod.getIfaceName(ifName),
         ",",
         dhcpConf["ip-address-pool"].from,
         ",",
         dhcpConf["ip-address-pool"].to,
         ",",
         dhcpConf.mask,
         ",",
         dhcpConf["lease-time"]
      }

      Conf[#Conf +1] = (dhcpConf.dns1 or dhcpConf.dns2) and
         table.concat{
         "dhcp-option=tag:",
         iface_mod.getIfaceName(ifName),
         ",",
         "6",
         dhcpConf.dns1 and (","..dhcpConf.dns1) ,
         dhcpConf.dns2 and (","..dhcpConf.dns2),
      }
      Conf[#Conf +1] = dhcpConf.gateway
         and "dhcp-option=tag:"..iface_mod.getIfaceName(ifName)..",".."3,"..dhcpConf.gateway
      Conf[#Conf +1] = dhcpConf.extra
      Conf[#Conf +1] = dhcpConf["max-leases"] and "dhcp-lease-max="..dhcpConf["max-leases"]
   else
      Conf[#Conf +1] = "no-dhcp-interface="..iface_mod.getIfaceName(ifName)
   end

   if dhcpConf and dhcpConf["adm-state"]=="up" and dhcpConf["mode"]=="relay" and dhcpConf["server-interface"] then
      dhcpRelayConf = dhcpConf
   end

   if dnsConf and dnsConf["adm-state"]=="up" then
      Conf[#Conf +1] = dnsConf.dns1 and "server="..dnsConf.dns1
      Conf[#Conf +1] = dnsConf.dns2 and "server="..dnsConf.dns2
      Conf[#Conf +1] = dnsConf.extra
   else
      Conf[#Conf +1] = "port=0"
   end

   Conf = table.concat(Conf, '\n')..'\n'
   Changed = s.readpopen("cat "..filename.." 2>/dev/null") ~= Conf
   needStart = true
   local f = io.open(filename, 'w')
   if f then
      f:write(Conf)
      f:close()
   end
   return needStart, Changed, dhcpRelayConf
end

local function killRemovedDaemons()
   local conf=C.getConfByPath({"services"})
   for ifName in pairs(dnsDhcpDaemons) do
      if not (conf and (conf.dhcp and conf.dhcp[ifName] or conf["dns"] and conf["dns"][ifName])) then
         C.restartDaemon({"services", "dns", ifName})
         dnsDhcpDaemons[ifName] = nil
      end
   end
end

restartDnsMasqServerEx = function(path, restart)
   local ifName = path[#path]
   local cPath = s.clonePath(path,-2,"dns",ifName)
   local rPath = s.clonePath(path,-2,"dhcp",ifName)
   local confFile = C.getOption(cPath,"confFile")
   local leaseFile = C.getOption(cPath,"leaseFile")
   
   if not confFile then
      confFile = os.tmpname()
      C.setOption(cPath, "confFile", confFile)
   end
   if not leaseFile then
      leaseFile = os.tmpname()
      C.setOption(cPath, "leaseFile", leaseFile)
   end

   local needStart, Changed, rConf = createConfFile(confFile, cPath)
   C.restartDaemon(rPath,
                   rConf and {"/usr/sbin/dhcprelay", ifName, rConf["server-interface"],rConf["server-address"]}
                )


   if not needStart then
      C.restartDaemon(cPath)
      dnsDhcpDaemons[ifName] = nil
   end

   if (needStart and (Changed or restart))  then
      dnsDhcpDaemons[ifName] = true
      C.restartDaemon(cPath,
                      {"/usr/sbin/dnsmasq", "--no-daemon", "--conf-file="..confFile,"--dhcp-leasefile="..leaseFile},
                      {
                         noRestart = false,
                         atExit = C.increasingSleep,
                         atKill = function(oper)
                                      oper.daemon:kill(2)
                                  end,
                      }
                   )
      C.updateNeteventCallback( -- try set on dev up
         resetDnsDhcpNeteventKey,
         resetDnsDhcpNeteventCallBack
      )
      iface_mod.setAliasHook(resetDnsDhcpAliasKey, resetDnsDhcpAliasCallBack)
      return true
   else
      return true
   end
end


local function restartDnsMasqServer(path)
   return restartDnsMasqServerEx(path, false)
end



DhcpScheme = s.newScheme {
   _Z = function(path, sch, conf, prevConf)
           killRemovedDaemons()
           if not s.confCmp(conf, prevConf) then
        --      os.execute("killall -SIGINT dnsmasq")
           end
           return true
        end,
   _H = "Dhcp",
   _NAME = function (_, path)
              local base = s.newScheme {
                 _H = "DhcpN",
                 _Z = restartDnsMasqServer,
                 description = s.stringType{_H = "DhcpServerNDescription", _P = 100},
                 ["adm-state"] = s.newScheme{
                    _T = "choice",
                    _A = {"up", "down"},
                    _V = s.choiceVal,
                    _H = "DnsMasqDhcpAdmState",
                    _D = "down",
                    _P =95,
                 },
                 mode = s.choiceType{
                    _T = "choice",
                    _A = {"server","relay"},
                    _H = "DnsMasqDhcpMode",
                    _D = "server",
                    _P = 90,
                    _DEP = 1,
                 }
              }
              local mode = C.getConfByPath(s.clonePath(path, nil,"mode"))
              if mode~="relay" then
                 return base {
                    ["max-leases"] = s.integerType{_H = "DhcpMaxLeases", _A = {1, 65535}, _D = 254, },
                    extra = s.stringType{_H = "DhcpServerExtra", _P = -100},
                    show = s.newScheme{
                       _H = "DnsMasqShow",
                       ["log"] =  s.newScheme{
                          _H = "DnsMasqShowLog",
                          _X = function(path)
                                  path = s.clonePath(path, -2)
                                  path[#path-1]= "dns"
                                  return C.getDaemonLog(path)
                               end,
                          _R = 'user',
                       },
                       ["leases"] =  s.newScheme{
                          _H = "DnsMasqShowLeases",
                          _X = function(path)
                                  path = s.clonePath(path, -2)
                                  path[#path-1]= "dns"
                                  local fname = C.getOption(path,"leaseFile")
                                  if fname then
                                     local leases = s.readpopen("cat "..fname.." 2>&1")
                                     return leases
                                  else
                                     return "Do apply and try again."
                                  end
                               end,
                          _R = 'user',
                       },
                       ["configuration"] =  s.newScheme{
                          _H = "DnsMasqShowConfiguration",
                          _X = function(path)
                                  path = s.clonePath(path, -2)
                                  path[#path-1]= "dns"
                                  local fname = C.getOption(path,"confFile")
                                  if fname then
                                     local leases = s.readpopen("cat "..fname.." 2>&1")
                                     return leases
                                  else
                                     return "Do apply and try again."
                                  end
                               end,
                          _R = 'user',
                       },
                    },
                    gateway = s.ipAddressType{_H = "DhcpGateway",},
                    dns1 = s.ipAddressType{_H = "DhcpDns1", },
                    dns2 = s.ipAddressType{_H = "DhcpDns2", },
                    ["ip-address-pool"] = s.IpAddressPool,
                    ["mask"] = s.ipAddressType{_H = "DhcpMask", _D = "255.255.0.0", },
                    ["lease-time"] = s.integerType{_H = "DhcpLeaseTime", _A = {1, 4294967295}, _D = 84600, },
                 }
              else
                 return base {
                    ["server-interface"] = s.stringType{
                       _P =50,
                       _H = "DhcpRelayInterface",
                       _DEP = 1,
                    },
                    ["server-address"] =  function(_, path)
                                             return C.getConfByPath(s.clonePath(path, -1, "server-interface")) and
                                             s.ipAddressType{
                                             _H = "DhcpRelayAddress",
                                          }
                                       end,
                    ["show-log"] = s.newScheme{
                       _H = "DnsRelayShowLog",
                       _X = function(path)
                               path = s.clonePath(path, -1)
                               return C.getDaemonLog(path)
                            end,
                       _R = 'user',
                    }
                 }
              end
           end,
}

DnsServerScheme = s.newScheme {
   _H = "DnsMasqServer",
   _Z = function(path, sch, conf, prevConf)
           killRemovedDaemons()
           if not s.confCmp(conf, prevConf) then
           --   os.execute("killall -SIGINT dnsmasq")
           end
           return true
        end,
   _NAME = s.newScheme {
      _H = "DnsMasqServerN",
      _Z = restartDnsMasqServer,
      show = s.newScheme{
         _H = "DnsMasqShow",
         ["log"] =  s.newScheme{
            _H = "DnsMasqShowLog",
            _X = function(path)
                    path = s.clonePath(path, -2)
                    return C.getDaemonLog(path)
                 end,
            _R = 'user',
         },
         ["configuration"] =  s.newScheme{
            _H = "DnsMasqShowConfiguration",
            _X = function(path)
                    path = s.clonePath(path, -2)
                    local fname = C.getOption(path,"confFile")
                    if fname then
                       local leases = s.readpopen("cat "..fname.." 2>&1")
                       return leases
                    else
                       return "Do apply and try again."
                    end
                 end,
            _R = 'user',
         },
      },
      description = s.stringType{_H = "DnsMasqServerNDescription", _P = 100},
      ["adm-state"] = s.newScheme{
         _T = "choice",
         _A = {"up", "down"},
         _V = s.choiceVal,
         _H = "DnsMasqDnsAdmState",
         _D = "down",
      },
      dns1 = s.ipAddressType{_H = "DnsMasqDns1", },
      dns2 = s.ipAddressType{_H = "DnsMasqDns2", },
      extra = s.stringType{_H = "DnsMasqExtra", _P = -100},
   },

}