module(..., package.seeall)
local s = require'scheme'
local dynDnsDaemons={}

local function killRemovedDaemons(path)
   local conf = C.getConfByPath(path)
   for name in pairs(dynDnsDaemons) do
      if not (conf and conf[name]) then
         C.restartDaemon(s.clonePath(path,0,name))
         dynDnsDaemons[name] = nil
      end
   end
end

local restartDynDns =
   function(path, sch, conf, prevConf)
      if not s.confCmp(conf, prevConf) then
         conf = s.addDefault(path, sch, conf, true)
         local cmd
         if conf["adm-state"]=="up" then
            dynDnsDaemons[path[#path]] = true
            cmd = {}
            cmd[#cmd + 1] = "/usr/sbin/inadyn"
            if conf["user-name"] then
               cmd[#cmd + 1] = "-u"
               cmd[#cmd + 1] = conf["user-name"]
               cmd[#cmd + 1] = "-p"
               cmd[#cmd + 1] = conf["user-password"]
            end
            if conf["host-name"] then
               cmd[#cmd + 1] = "-a"
               cmd[#cmd + 1] = conf["host-name"]
            end
            if conf["ip-server"] and conf["ip-server"]~="auto" then
               cmd[#cmd + 1] = "--ip_server_name"
               cmd[#cmd + 1] = conf["ip-server"]
               cmd[#cmd + 1] = conf["ip-server-script"]
            end
            if conf["dyndns-server"] and conf["dyndns-server"]~="auto" then
               cmd[#cmd + 1] = "--dyndns_server_name"
               cmd[#cmd + 1] = conf["dyndns-server"]
               cmd[#cmd + 1] = "--dyndns_server_url"
               cmd[#cmd + 1] = conf["dyndns-server-script"]
            end
            cmd[#cmd + 1] = "--dyndns_system"
            cmd[#cmd + 1] = conf["dyndns-system"]
            cmd[#cmd + 1] = "--update_period_sec"
            cmd[#cmd + 1] = tostring(conf["update-period"])
            cmd[#cmd + 1] = "--forced_update_period"
            cmd[#cmd + 1] = tostring(conf["forced-update-period"])
         else
            dynDnsDaemons[path[#path]] = nil
         end
         C.restartDaemon(path,
                         cmd,
                         {
                            noRestart = false,
                            atExit = C.increasingSleep,
                         }
                      )
         return true
      else
         return true
      end
   end

DynDnsServerScheme = s.newScheme {
   _H = "DynDns",
   _Z = function(path, sch, conf, prevConf)
           killRemovedDaemons()
           if not s.confCmp(conf, prevConf) then
           --   os.execute("killall -SIGINT inadyn")
           end
           return true
        end,
   _NAME = s.newScheme {
      _H = "DynDnsN",
      _Z = restartDynDns,
      ["show-log"] =  s.newScheme{
         _P = -100,
         _H = "DynDnsShowLog",
         _X = function(path)
                 path = s.clonePath(path, -1)
                 return C.getDaemonLog(path)
              end,
      },
      description = s.stringType{_H = "DynDnsNDescription", _P = 100},
      ["adm-state"] = s.choiceType{
         _A = {"up", "down"},
         _H = "DynDnsAdmState",
         _D = "down",
         _P = 90,
      },
      ["dyndns-system"] = s.choiceType{
         _P = 80,
         _A = {
            "custom@http_svr_basic_auth",
            "default@dyndns.org",
            "customdns@dyndns.org",
            "dyndns@dyndns.org",
            "default@freedns.afraid.org",
            "default@no-ip.com",
            "default@zoneedit.com",
            },
         _H = "DynDnsSystem",
         _D = "default@dyndns.org",
      },
      ["dyndns-server"] = s.stringType{_DEP = 1 , _H = "DynDnsServer", _D = "auto", _P= 70},
      ["dyndns-server-script"] =
         function(_, path)
            local serv = C.getConfByPath(s.clonePath(path, -1,"dyndns-server")) or "auto"
            if serv ~= "auto" then
               return s.stringType{
                  _H = "DynDnsServerScript",
                  _D = "/",
                  _P = 65,
               }
            end
            return nil
         end, 
      ["ip-server"] = s.stringType{_DEP = 1, _H = "DynDnsIpserver", _D = "auto", _P = 60},
      ["ip-server-script"] =
         function(_, path)
            local serv = C.getConfByPath(s.clonePath(path, -1,"ip-server")) or "auto"
            if serv ~= "auto" then
               return s.stringType{
                  _H = "DynDnsIpserverScript",
                  _D = "/",
                  _P = 55,
               }
            end
            return nil
         end,
      ["host-name"] =  s.stringType{_H = "DynDnsHostname", _P = 50},
      ["user-name"] =  s.stringType{_H = "DynDnsUsername", _P = 45},
      ["user-password"] =  s.stringType{_H = "DynDnsUserpassword", _P = 40},
      ["update-period"] = s.integerType{
         _H = "DynDnsUpdateperiod",
         _A = {10, 864000},
         _D = 180,
         _P = 35,
      },
      ["forced-update-period"] = s.integerType{
         _H = "DynDnsForcedupdateperiod",
         _A = {10, 864000},
         _D = 86400,
         _P = 30,
      },

   },

}