module(..., package.seeall)
local s = require'scheme'
local iface = require'nsgifaceconf'
local nsg = require'nsg'
require'bit'
local bit = bit

local ip = "/usr/sbin/ip"
local resetBridgeNeteventKey = {}
local resetBondNeteventKey = {}

local function getNetDevices()
  local procNetDev, err = io.open"/proc/net/dev"
  if not procNetDev then
    error(err)
  end
  local s = procNetDev:read'*a'
  local t = {}
  for dev in s:gmatch"\n%s*([^%s]*):" do
    t[#t+1] = dev
    t[dev] = true
  end
  return t
end

local function badIfaceForBridgeGroup(newiface, brif, invtab)
  for iface in pairs(brif) do
    for _, set in pairs(invtab) do
      if set[newiface] and set[iface] then
        return iface
      end
    end
  end
end

local function bridgeVal(val)
  val = (tostring(val)):match("^br(%d+)$") or val
  local n, err = s.integerVal(tonumber(val), 1, 255)
  if n then
    local brname = "br"..tostring(n)
    return brname
  else
    return nil, err
  end
end

local function bondVal(val)
  val = (tostring(val)):match("^bn(%d+)$") or val
  local n, err = s.integerVal(tonumber(val), 1, 255)
  if n then
    local bnname = "bn"..tostring(n)
    return bnname
  else
    return nil, err
  end
end

local function getBridgeVal(ifaceName)
  return function(val)
    local res, err = bridgeVal(val)
    if res then
      local brIf = C.getOption({"ethernet","bridge", res}, "brIf")
      if brIf then
        if BI.invalidBridgeGroup then
          local conflict = badIfaceForBridgeGroup(ifaceName, brIf, BI.invalidBridgeGroup)
          if conflict then
            return nil, "other bridge-group  because of "..ifaceName..
            " can't be in the same bridge group with "..conflict
          end
        end
      end
    end
    return res, err
  end
end

local function applyBridgeGroup(path, bridge, ifaceName)
  local oldBridge = C.getOption(path, "bridge-group")
  local test = (bridge and (os.execute("brctl showstp "..bridge.." | grep -oE '^[[:alnum:]._]+' | grep -oE '^"..ifaceName:gsub("[._]","[%0]").."$'") == 0)) or (not bridge and true)

  if bridge then
     resetBridgeNeteventKey[iface.getIfaceName(ifaceName)] = s.clonePath(path)
     resetBondNeteventKey[iface.getIfaceName(ifaceName)] = nil
  else
     resetBridgeNeteventKey[iface.getIfaceName(ifaceName)] = nil
  end

  if (oldBridge ~= bridge) or (not test) then
    if oldBridge then
       os.execute("/sbin/brctl delif "..oldBridge..' '..iface.getIfaceName(ifaceName).." >/dev/null 2>&1")
      local brIf = C.getOption({"ethernet", "bridge", oldBridge}, "brIf")
      if brIf then
        brIf[ifaceName] = nil
      end
    end

    local brIf = C.getOption({"ethernet", "bridge", bridge}, "brIf")
    if bridge then
      if brIf then
        if BI.invalidBridgeGroup then
          local res = badIfaceForBridgeGroup(ifaceName, brIf, BI.invalidBridgeGroup)
          if res then
            return false, ifaceName.." can't be in the same bridge group with "..res
          end
        end
        brIf[ifaceName] = true
      else
        C.setOption({"ethernet", "bridge", bridge}, "brIf", {[ifaceName] = true})
      end
      os.execute("/sbin/brctl addif "..bridge..' '..iface.getIfaceName(ifaceName).." >/dev/null 2>&1")
    else
      if brIf then brIf[ifaceName] = nil end
    end
    C.setOption(path, "bridge-group", bridge)
  end
  return true
end

local function applyBondGroup(path, bond, ifaceName)
   local oldBond = C.getOption(path, "bond-group")
   local test = (bond and (os.execute("cat /proc/net/bonding/"..bond.." | grep -E '^Slave Interface:' | grep -E '[[:space:]]"..ifaceName:gsub("[._]","[%0]").."$'") == 0)) or (not bond and true)

   if bond then
      resetBondNeteventKey[iface.getIfaceName(ifaceName)] = s.clonePath(path)
      resetBridgeNeteventKey[iface.getIfaceName(ifaceName)] = nil
   else
      resetBondNeteventKey[iface.getIfaceName(ifaceName)] = nil
   end

   if (oldBond ~= bond) or (not test) then
      if oldBond then
         os.execute("echo  -"..iface.getIfaceName(ifaceName).." > /sys/class/net/"..oldBond.."/bonding/slaves")
         local bnIf = C.getOption({"ethernet", "bond", oldBond}, "bnIf")
         if bnIf then
            bnIf[ifaceName] = nil
         end
      end

      local bnIf = C.getOption({"ethernet", "bond", bond}, "bnIf")
      if bond then
         if bnIf then
            bnIf[ifaceName] = true
         else
            C.setOption({"ethernet", "bond", bond}, "bnIf", {[ifaceName] = true})
         end

         os.execute(ip.." link set dev "..iface.getIfaceName(ifaceName).." down;echo  +"..iface.getIfaceName(ifaceName).." > /sys/class/net/"..bond.."/bonding/slaves")
      else
         if bnIf then bnIf[ifaceName] = nil end
      end
      C.setOption(path, "bond-group", bond)
   end
   return true
end

local  resetBBNeteventCallBack = function(msg)
  if (msg.type == 'newlink') then
     local path = resetBridgeNeteventKey[msg.ifname]
     if path then
        local conf = C.getConfByPath(path)
        applyBridgeGroup(path, (conf or {})["bridge-group"] ,msg.ifname)
     end
     local path = resetBondNeteventKey[msg.ifname]
     if path then
        local conf = C.getConfByPath(path)
        applyBondGroup(path, (conf or {})["bond-group"] ,msg.ifname)
     end
  end
end
applyEthScheme = function(path, sch, conf)
           local ifaceName = path[#path]
           -- iface.addIfaceAlias(ifaceName, ifaceName)
           local resB, errB = applyBridgeGroup(
              path,
              (conf or {})["bridge-group"],
              iface.getIfaceName(ifaceName)
           )
           local resC, errC = applyBondGroup(
              path,
              (conf or {})["bond-group"],
              iface.getIfaceName(ifaceName)
           )

           C.updateNeteventCallback( -- try set on dev creation
              resetBridgeNeteventKey,
              resetBBNeteventCallBack
           )
           local res, err = iface.interfaceScheme._Z(path, sch, conf)
           return resC and resB and res, table.concat({errC or "", errB or "", err or ""}, '\n')
        end

ethScheme = iface.interfaceScheme{
   _Z = applyEthScheme,
   ["bridge-group"] =
      BI.bridge and function(_, path)
                       local conf = C.getConfByPath(s.clonePath(path,-1))
                       if conf and ((conf["bond-group"])) then
                          return nil
                       end
                       return s.newScheme{
                          _H = "ethBridgegroup",
                          _T = "string",
                          _V = getBridgeVal(path[#path - 1]),
                          _DEP = 1,
                       }
                    end,
   ["bond-group"] =
      function(_, path)
         local conf = C.getConfByPath(s.clonePath(path,-1))
         if conf and ((conf["bridge-group"])) then
            return nil
         end
         return s.stringType{
            _H = "ethBondgroup",
            _V = bondVal,
            _DEP = 1,
         }
      end
   ,
}

local prioClassVal = function (val)
  local u = s.integerVal(tonumber(val),  0, 4294967295)
  local hi,lo = (tostring(val)):match("^([%d]*):([%d]*)$")
  if hi and lo then
    hi = s.integerVal(tonumber(hi),  0, 65535)
    lo = s.integerVal(tonumber(lo),  0, 65535)
  end

  if hi and lo then
    return tostring(hi)..":"..tostring(lo)
  elseif u then
    return u
  else
    return nil, "integer in range 0..4294967295 or classname [0..65535]:[0..65535]"
  end
end

local normPrioClassVal = function(val)
  local u = s.integerVal(tonumber(val),  0, 4294967295)
  local hi,lo = (tostring(val)):match("^([%d]*):([%d]*)$")
  if hi and lo then
    u = hi * 65536 + lo
  end
  return u
end

local vlanScheme = ethScheme{
   _H = "ethVlanN",
   _Z = function(path, sch, conf)
           local devname = iface.getIfaceName(path[#path])
           local ethN, vlanID = devname:match"^(.*)%.(%d+)$"
           if not getNetDevices()[devname] then
              os.execute("/sbin/vconfig add "..ethN.." "..vlanID.." >/dev/null 2>&1")
              for k,v in pairs(conf and conf["out-Class2CoS-map"] or {}) do
                 if prioClassVal(k) then
                    local key = normPrioClassVal(k)
                    os.execute("/sbin/vconfig set_egress_map "..devname.." "..key.." "..v.." >/dev/null 2>&1")
                 end
              end
              for k,v in pairs(conf and conf["in-CoS2Class-map"] or {}) do
                 if s.integerVal(k, 0, 7) then
                    local val = normPrioClassVal(v)
                    os.execute("/sbin/vconfig set_ingress_map "..devname.." "..val.." "..k.." >/dev/null 2>&1")
                 end
              end
           end
           return ethScheme._Z(path, sch, conf)
        end,
   ["out-Class2CoS-map"] = s.newScheme{
      _H = "vlanOutclass2cosmap",
      _NAME_TYPE = prioClassVal,
      _NAME = s.integerType{
         _H = "vlanClass2cosmap",
         _A = {0,7}
      },
   },
   ["in-CoS2Class-map"] = s.newScheme{
      _H = "vlanIncos2classmap",
      _NAME_TYPE = function(val)
                      return s.integerVal(tonumber(val), 0, 7)
                   end,
      _NAME = s.integerType{
         _H = "vlanCos2classmap",
         _V = prioClassVal,
      },
   },
}

ethScheme.vlan =
   function(_, path)
      local conf = C.getConfByPath(s.clonePath(path,-1))
      if conf and ((conf["bridge-group"]) or (conf["bond-group"])) then
         return nil
      end

      return s.newScheme{
         _H = "ethVlan",
         _Z = function(path, sch, conf)
                 local netdevs = getNetDevices()
                 local baseIfaceName = path[#path - 1]
                 for i = 1, #netdevs do
                    local dev = netdevs[i]
                    if ( dev:match("^"..baseIfaceName.."%.%d+$") and 
                      not (conf or {})[dev]
                ) then
                       os.execute("/sbin/vconfig rem "..dev.." >/dev/null 2>&1")
                    end
                 end
                 return true
              end,
         _NAME = vlanScheme,
         _NAME_TYPE = function(val)
                         local portName = path[#path - 1]
                         val = (tostring(val)):match("^"..portName.."%.(%d+)$") or val
                         local n, err = s.integerVal(tonumber(val), 1, 4094)
                         if n then
                            return portName..'.'..tostring(n)
                         else
                            return nil, err
                         end
                      end,
      }
end

--------------------------------------------------------------------------------
-- vrrp
--

local vrrpScheme =  s.newScheme{ 
  _H = "vrrpN",
  _Z = function(path, sch, conf)
    local IfaceName = path[#path - 2]
    local vrid = tostring(path[#path])
    local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults

    if conf["adm-state"]=="up" and conf.ifAddress and conf.ifAddress[1] then
      local appl = {
        "/usr/sbin/vrrpd", "-F",
         "-i", iface.getIfaceName(IfaceName),
        "-v", vrid,
        "-p", tostring(conf.prio),
        "-d", tostring(conf.delay),
      }
      if conf["update-mac"] ~= "yes" then
        appl[#appl + 1] = "-n"
      end
      for ik, ip in ipairs(conf.ifAddress) do
        appl[#appl + 1] = ip
      end
      C.restartDaemon(path,
        appl,
        {noRestart = false, atExit = C.increasingSleep}
      )
    else
      C.restartDaemon(path)
    end
    return true
  end,
  log =  s.newScheme{
    _H = "vrrpNLog",
    _X = function(path)
      return C.getDaemonLog( s.clonePath(path, -1))
    end,
    _R = 'user',
  },
  ["adm-state"] =  s.newScheme{
    _T = "choice",
    _A = {"up", "down"},
    _V = s.choiceVal,
    _H = "vrrpNAdm",
    _D = "up"
  },
  delay = s.integerType{
    _H = "vrrpNDelay",
    _A = {1, 255},
    _D = 1,
  },
  prio = s.integerType{
    _H = "vrrpNPrio",
    _A = {1, 255},
    _D = 100,
  },
  ["update-mac"] = s.newScheme{
    _T = "choice",
    _A = {"yes", "no"},
    _V = s.choiceVal,
    _H = "vrrpNUpdateMac",
    _D = "yes",
  },
  ifAddress = s.newScheme{
    _H = "vrrpNifAddress",
    _LIST = s.ipAddressType{_H = "vrrpNifAddress",},
  }
}

ethScheme.vrrp =
   function(_, path)
      local conf = C.getConfByPath(s.clonePath(path,-1))
      if conf and ((conf["bridge-group"]) or (conf["bond-group"])) then
         return nil
      end
      return s.newScheme{
         _H = "ethVRRP",
         _NAME = vrrpScheme,
         _NAME_TYPE = function(val)
                         return s.integerVal(tonumber(val), 1, 255)
                      end,
      }
end

--------------------------------------------------------------------------------
-- bridge stuff
--

local par2brctl = {
  ["aging-time"] = "setageing",
  ["forward-time"] = "setfd",
  ["hello-time"] = "sethello",
  ["max-age"] = "setmaxage", 
  ["priority"] = "setbridgeprio",
  ["stp"] = "stp",
}

local function setBridgeParams(devname, conf)
  for k, v in pairs(par2brctl) do
    if conf[k] then
      os.execute("/sbin/brctl "..v.." "..devname.." "..conf[k].." >/dev/null 2>&1")
      print("/sbin/brctl "..v.." "..devname.." "..conf[k].." >/dev/null 2>&1")
    end
  end
end

bridgeNScheme = ethScheme{
  _H = "bridgeN",

  _Z = function(path, sch, conf) 
    local devname = path[#path]
    if not getNetDevices()[devname] then
      os.execute("/sbin/brctl addbr "..devname.." >/dev/null 2>&1")
    end
    local brIf = C.getOption({"ethernet", "bridge", devname}, "brIf") or{}
    for ifaceName in pairs(brIf) do
       os.execute("/sbin/brctl addif "..devname..' '..iface.getIfaceName(ifaceName).." >/dev/null 2>&1")
    end
    conf = s.addDefault(path, sch, conf or {}, true) --clone conf with defaults
    setBridgeParams(devname, conf)
    return ethScheme._Z(path, sch, conf)
  end,

  ["bridge-group"] = s.nilType,

  ["aging-time"] = s.integerType{
    _H = "bridgeAgingtime",
    _A = {10, 1000000},
    _D = 300,
  },
  ["forward-time"] = s.integerType{
    _H = "bridgeForwardtime",
    _A = {4, 200},
    _D = 15,
  },
  ["hello-time"] = s.integerType{
    _H = "bridgeHellotime",
    _A = {1, 10},
    _D = 2,
  },
  ["max-age"] = s.integerType{
    _H = "bridgeMaxage",
    _A = {6, 200},
    _D = 20,
  },
  ["priority"] = s.integerType{
    _H = "bridgePriority",
    _A = {0, 65535},
    _D = 32768,
  },
  ["stp"] =  s.newScheme{
    _H = "bridgeStp",
    _T = "choice",
    _A = {"on", "off"},
    _V = s.choiceVal,
    _D = "on"
  },

  showmacs = s.newScheme{
    _H = "bridgeShowmacs",
    _R = 'user',
    _X = function(path)
      return -("/sbin/brctl showmacs "..path[#path].." 2>&1")
    end,
  },

  show = s.newScheme{
    _H = "bridgeShow",
    _R = 'user',
    _X = function(path)
      local dev = path[#path - 1]
      local stp = -("/sbin/brctl showstp "..dev.." 2>&1")
      local ifstate = iface.showInterfaceStat(dev)
      return stp..ifstate
    end,
  },
}

bridgeScheme = s.newScheme{
  _H = "bridge",
  _Z = function(path, sch, conf)
    local netdevs = getNetDevices()
    for i = 1, #netdevs do
      local dev = netdevs[i]
      if ( dev:match("^br%d+$") and 
        not (conf or {})[dev]
      ) then
        os.execute(
          ip.." link set "..dev.." down;"..
          "/sbin/brctl delbr "..dev.." >/dev/null 2>&1"
        )
      end
    end
    return true
  end,
  _NAME = bridgeNScheme,
  _NAME_TYPE = bridgeVal,

  show = s.newScheme{
    _H = "bridgeShow",
    _R = 'user',
    _X = function()
      return -"/sbin/brctl show 2>&1"
    end,
  },
}
--------------------------------------------------------------------------------
-- start BONDING

local par2bond = {
  ["ad-select"] = "ad_select",
  ["arp-interval"] = "arp_interval",
  ["arp-validate"] = "arp_validate",
  ["down-delay"] = "downdelay",
  ["up-delay"] = "updelay",
  ["fail-over-mac"] = "fail_over_mac",
  ["lacp-rate"] = "lacp_rate",
  ["miimon"] = "miimon",
  ["mode"] = "mode",
  ["num-grat-arp"] = "num_grat_arp",
  ["num-unsol-na"] = "num_unsol_na",
  ["primary"] = "primary",
  ["use-carrier"] = "use_carrier",
  ["xmit-hash-policy"] = "xmit_hash_policy",
  ["arp-ip-target"] = "arp_ip_target",
}

local function setBondParams(devname, conf)
  for k, v in pairs(par2bond) do
    if conf[k] then
       if type(conf[k]) == "table" then
          for _,x in ipairs(conf[k]) do
             os.execute("echo +"..x.." >/sys/class/net/"..devname.."/bonding/"..v)
          end
       elseif  type(conf[k]) == "boolean" then
          os.execute("echo "..tostring(conf[k] and 1 or 0).." >/sys/class/net/"..devname.."/bonding/"..v)
       else
          os.execute("echo "..conf[k].." >/sys/class/net/"..devname.."/bonding/"..v)
       end
    end
  end
end

bondNScheme = ethScheme{
   _H = "bondN",
   _Z = function(path, sch, conf)
           local devname = path[#path]
           os.execute("modprobe bonding")
           if getNetDevices()[devname] then
              os.execute("echo -"..devname.." > /sys/class/net/bonding_masters")
           end
           os.execute("echo +"..devname.." > /sys/class/net/bonding_masters")

           conf = s.addDefault(path, sch, conf or {}, true) --clone conf with defaults
           setBondParams(devname, conf)

           local bnIf = C.getOption({"ethernet", "bond", devname}, "bnIf") or{}
           for ifaceName in pairs(bnIf) do
              os.execute("echo  +"..iface.getIfaceName(ifaceName).." > /sys/class/net/"..devname.."/bonding/slaves")
           end
           return ethScheme._Z(path, sch, conf)
        end,

   ["bond-group"] = s.nilType,

   ["ad-select"] = function(_, path)
                      if C.getConfByPath(s.clonePath(path,-1,"mode")) ~= "802.3ad" then
                         return nil
                      end
                      return s.choiceType{
                         _H = "bondNAdselect",
                         _A = {"stable", "bandwidth","count"},
                         _D = "stable",
                      }
                   end,

   ["arp-interval"] = s.integerType{
      _H = "bondNArpinterval",
      _A = {0, 2^32 -1},
      _D = 0,
      _DEP =1,
   },


   ["arp-ip-target"] = function (_, path )
                          local i =  C.getConfByPath(s.clonePath(path,-1,"arp-interval")) or 0
                          if i == 0 then
                             return nil
                          end
                          return s.newScheme{
                             _H = "bondNArpiptarget",
                             _LIST = s.ipAddressType{
                                _H = "bondNArpiptargetIP",
                             },
                          }
                       end,


   ["arp-validate"] = function (_, path )
                         if C.getConfByPath(s.clonePath(path,-1,"mode")) ~= "active-backup" then
                            return nil
                         end
                         return s.choiceType{
                            _H = "bondNArpvalidate",
                            _A = {"none", "active","backup","all"},
                            _D = "none",
                         }
                      end,

   ["down-delay"] = function (_, path )
                      local miimon = C.getConfByPath(s.clonePath(path,-1,"miimon")) or 0
                      if miimon == 0 then
                         return nil
                      end
                      return s.integerType{
                         _H = "bondNDowndelay",
                         _A = {0, 2^32 -1},
                         _D = 0,
                      }
                   end,

   ["up-delay"] =  function (_, path )
                      local miimon = C.getConfByPath(s.clonePath(path,-1,"miimon")) or 0
                      if miimon == 0 then
                         return nil
                      end
                      return s.integerType{
                         _H = "bondNUpdelay",
                         _A = {0, 2^32 -1},
                         _D = 0,
                      }
                   end,

   ["fail-over-mac"] = function (_, path )
                          if C.getConfByPath(s.clonePath(path,-1,"mode")) ~= "active-backup" then
                             return nil
                          end
                          return s.choiceType{
                             _H = "bondNFailovermac",
                             _A = {"none", "active","follow"},
                             _D = "none",
                          }
                       end,

   ["lacp-rate"] =
      function(_, path)
         if C.getConfByPath(s.clonePath(path,-1,"mode")) ~= "802.3ad" then
            return nil
         end
         return s.choiceType{
            _H = "bondNLacprate",
            _A = {"slow", "fast"},
            _D = "slow",
         }
      end,

   ["miimon"] = s.integerType{
      _H = "bondNMiimon",
      _A = {0, 2^32 -1},
      _D = 0,
      _DEP =1,
   },

   ["mode"] = s.choiceType{
      _H = "bondNMode",
      _A = {"balance-rr", "active-backup", "balance-xor",
            "broadcast","802.3ad","balance-tlb", "balance-alb"},
      _D = "balance-rr",
      _DEP = 1,
   },

   ["num-grat-arp"] =
      function(_, path)
         if C.getConfByPath(s.clonePath(path,-1,"mode")) ~= "active-backup" then
            return nil
         end
         return s.integerType{
            _H = "bondNNumgratarp",
            _A = {0, 255},
            _D = 1,
         }
      end,
--[[  IPv6  ONLY !!!
   ["num-unsol-na"] =
      function(_, path)
         if C.getConfByPath(s.clonePath(path,-1,"mode")) ~= "active-backup" then
            return nil
         end
         return s.integerType{
            _H = "bondNNumunsolna",
            _A = {0, 255},
            _D = 1,
         }
      end,
   --]]

   ["primary"] =
      function(_, path)
         if C.getConfByPath(s.clonePath(path,-1,"mode")) ~= "active-backup" then
            return nil
         end
         return s.stringType{
            _H = "bondNPrimary",
         }
      end,

   ["use-carrier"] = s.booleanType{
      _H = "bondNUsecarrier",
      _D = true,
   },

   ["xmit-hash-policy"] = function(_, path)
                             local mode = C.getConfByPath(s.clonePath(path,-1,"mode"))
                             if (mode ~= "balance-xor") and (mode ~= "802.3ad") then
                                return nil
                             end
                             return s.choiceType{
                                _H = "bondNXmithashpolicy",
                                _A = {"layer2", "layer2+3", "layer3+4"},
                                _D = "layer2",
                             }
                          end,

   show = s.newScheme{
      _H = "bondNShow",
      _R = 'user',
      _X = function(path)
              local dev = path[#path - 1]
              local bn = s.readpopen("cat /proc/net/bonding/"..dev.." 2>&1")
              local ifstate = iface.showInterfaceStat(dev)
              return bn..ifstate
           end,
   },
}

bondScheme = s.newScheme{
   _H = "bond",
   _Z = function(path, sch, conf)
           local netdevs = getNetDevices()
           for i = 1, #netdevs do
              local dev = netdevs[i]
              if ( dev:match("^bn%d+$") and
                not (conf or {})[dev]
          ) then
                 os.execute(
                    ip.." link set "..dev.." down;"..
                       os.execute("echo -"..dev.." > /sys/class/net/bonding_masters")
                 )
              end
           end
           return true
        end,
  _NAME = bondNScheme,
  _NAME_TYPE = bondVal,
}

-- end BONDING
--------------------------------------------------------------------------------
local ethPhyScheme = s.integerType{
  _H = "ethernetSwitchPhy",
  _A = {0,4},
}

local swvlan2nsg = {
  [0] = 4,
  [1] = 3,
  [2] = 2,
  [3] = 1,
  [4] = 0,
}

local swport2nsg = {
  ['0'] = 4,
  ['1'] = 3,
  ['2'] = 2,
  ['3'] = 1,
  ['4'] = 0,
}

ethernetSwitchScheme = s.newScheme{
  _H = "ethernetSwitch",
  _Z = function(path, sch, conf)
    conf = s.addDefault(path, sch, conf or {}, true) --clone conf with defaults
    local matrix = {
      [0] = {},
      [1] = {},
      [2] = {},
      [3] = {},
      [4] = {},
    }
    for k, v in pairs(conf) do
      local phy = k:match"^phy(%d)$"
      if phy then
        matrix[v][#matrix[v]+1] = phy
      end
    end
    for i = 0, 4 do
      local s = table.concat(matrix[i]):gsub('%d', swport2nsg)
      if #s > 0 then s = s..'c' end
      local com = '/sbin/admswconfig eth'..i..' "'..s..'"'
      os.execute(com.." >/dev/null 2>&1")
    end
    return true
  end,
  phy0 = ethPhyScheme{_D = 0},
  phy1 = ethPhyScheme{_D = 1},
  phy2 = ethPhyScheme{_D = 2},
  phy3 = ethPhyScheme{_D = 3},
  phy4 = ethPhyScheme{_D = 4},
  show = s.newScheme{
    _H = "ethernetSwitchShow",
    _X = function(path)
      local info = s.readpopen("/sbin/admswconfig")
      local matrix = {}
      for vlan, p4, p3, p2, p1, p0, cpu in 
      info:gmatch"\n(%d)\t(1?)\t(1?)\t(1?)\t(1?)\t(1?)\t(1?)" do
        matrix[swvlan2nsg[tonumber(vlan)]] = {
          [0] = p0 == '1' and true, 
          [1] = p1 == '1' and true, 
          [2] = p2 == '1' and true, 
          [3] = p3 == '1' and true, 
          [4] = p4 == '1' and true,
        }
      end
      local out = {'\n      phy0 phy1 phy2 phy3 phy4'}
      for i = 0, 4 do
        local vlan = {}
        vlan[#vlan+1] = "eth"..i
        for j = 0, 4 do
          vlan[#vlan+1] = matrix[i][j] and 'X' or ' '
        end
        out[#out+1] = table.concat(vlan, '    ')
      end

      return table.concat(out, '\n')..'\n'
    end,
  },
}

local p2name
if BI.arch == 'nsg700' then
  p2name = {
    [0] = 'eth0',
    [1] = 'e1',
    [2] = 'e2',
    [3] = 'e3',
    [4] = 's1',
  }
else
  p2name = {
    [0] = 'eth0',
    [1] = 'e1',
    [2] = 'e2',
    [3] = 'e3',
    [4] = 'e4',
  }
end


local state2str = {
  [0] = "10 HalfDuplex" ,
  [1] = "100 HalfDuplex" ,
  [2] = "10 FullDuplex",
  [3] = "100 FullDuplex",
}

local VLAN_TAG_MODE_REMOVE = 1
local VLAN_TAG_MODE_ADD = 2
local ETHERNET_SWITCH_MB_NUM = 16
local COMMON_VID = 4095

local rtlPhyScheme = s.newScheme{
  _H = "ethernetSwitchRtlPhy",
  ["vlan-tagged"] = s.booleanType{
    _H = "ethernetSwitchRtlPhyVlantagged",
    _DEP = 1,
  },

  ["vlan-group"] = function(_, path)
    return not C.getConfOrDefault(s.clonePath(path, -1, "vlan-tagged")) and
    s.integerType {
      _H = "ethernetSwitchRtlPhyVlangroup",
      _A = {1, 4094},
      _D = COMMON_VID
    }
  end,

  ["vlan-groups"] = function(_, path)
    return C.getConfOrDefault(s.clonePath(path, -1, "vlan-tagged")) and
    s.newScheme {
      _H = "ethernetSwitchRtlPhyVlangroups",
      _LIST = s.integerType {
        _H = "ethernetSwitchRtlPhyVlangroup",
        _A = {1, 4094},
      },
    }
  end,

}

local function createVlanGroup(sw, port, vlan, add_port) 
  local i
  for j = 0, ETHERNET_SWITCH_MB_NUM do
    if sw.vlan[j] then
      if sw.vlan[j].vid == vlan then
        i = j
        break
      end
    else
      if j >= ETHERNET_SWITCH_MB_NUM then
        return false
      end
      i = j
      sw.vlan[i] = {vid = vlan, mb = 0}
      break
    end
  end

  if add_port then
    sw.vlan[i].mb = bit.bor(sw.vlan[i].mb, 2^port)
  end
  return i
end

local function setPhyParam(sw, phyConf, realPhy)
  --print('#####', realPhy, s.ser(phyConf))
  local idx
  sw.phy[realPhy] = {}
  if phyConf["vlan-tagged"] then
    sw.phy[realPhy]['tag-mode'] = VLAN_TAG_MODE_ADD
    for _, v in ipairs(phyConf['vlan-groups']or{}) do
      if v then
        idx = createVlanGroup(sw, realPhy, v, true);
        if not idx then 
          return false
        end
      end
    end
    idx = createVlanGroup(sw, realPhy, COMMON_VID, false)
    if idx then
      sw.phy[realPhy].index = idx
      --if (phy->permit_other) {
      --  sw->vlan[idx].mb |= (1 << real_port);
      --}
    end
  else -- no vlan-tagged
    local vid = phyConf['vlan-group'] or COMMON_VID
    sw.phy[realPhy]['tag-mode'] = VLAN_TAG_MODE_REMOVE
    idx = createVlanGroup(sw, realPhy, vid, vid ~= COMMON_VID);
    if idx then
      sw.phy[realPhy].index = idx;
    end
  end
  --print('*****', s.ser(sw))
end

local function getSwConf(conf)
  local sw = {
    vlan = {},
    phy = {}
  }
  local portmap, err = nsg.rtlswconf''
  if not portmap then
    if err == 'Invalid argument' then
      return true 
    else
      return false, err
    end
  end
  local realPhy = {}
  for i = 1, 5 do
    local port = portmap:byte(2 * i - 1)
    realPhy[port] = i - 1
  end
  for k, v in pairs(conf) do
    local phy = k:match"^phy(%d)$"
    if phy then
      setPhyParam(sw, v, realPhy[tonumber(phy)])
    end
  end
  return sw
end

local function applyEthSwitch(conf)
  if conf["vlan-mode"] then
    local res, err = nsg.rtlswconf(getSwConf(conf))
    if not res and err ~= 'Invalid argument' then
      return false, err
    end
  else -- no vlan-mode
    local res, err = nsg.rtlswconf()
    if not res and err ~= 'Invalid argument' then
      return false, err
    end
  end
  return true
end

local ethSwitchNeteventKey = {}

ethernetSwitchRtlScheme = s.newScheme{
  _H = "ethernetSwitchRtl",

  _Z = function(path, sch, conf)
    conf = s.addDefault(path, sch, conf or {}, true) --clone conf with defaults
    C.updateNeteventCallback(
      ethSwitchNeteventKey,
      function(msg)
        if (msg.ifname == 'eth0' and msg.type == 'newlink' and msg.ifup) then
          return applyEthSwitch(conf)
        end
      end
    )
    return applyEthSwitch(conf)
  end,

  ["vlan-mode"] = s.booleanType{
    _H = "ethernetSwitchRtlVlanmode",
    _DEP = 1,
  },

  phy0 = function(_, path)
    return C.getConfOrDefault(s.clonePath(path, -1, "vlan-mode")) and
    rtlPhyScheme
  end,

  phy1 = function(_, path)
    return C.getConfOrDefault(s.clonePath(path, -1, "vlan-mode")) and
    rtlPhyScheme
  end,

  phy2 = function(_, path)
    return C.getConfOrDefault(s.clonePath(path, -1, "vlan-mode")) and
    rtlPhyScheme
  end,

  phy3 = function(_, path)
    return C.getConfOrDefault(s.clonePath(path, -1, "vlan-mode")) and
    rtlPhyScheme
  end,

  phy4 = function(_, path)
    return C.getConfOrDefault(s.clonePath(path, -1, "vlan-mode")) and
    rtlPhyScheme
  end,

  show = s.newScheme{
    _H = "ethernetSwitchRtlShow",
    _X = function(path)
      local str = nsg.rtlswconf''
      local res = {}
      for i = 1, 5 do
        local stateStr
        local port = str:byte(2 * i - 1)
        local state = str:byte(2 * i)
        if state == 0 then
          stateStr = 'DOWN'
        elseif port == 0 then
          stateStr = 'UP'
        else
          stateStr = state2str[state % 4]
        end
	res[#res+1] = {port, stateStr}
      end
      table.sort(res, function(a,b)return a[1]<b[1]end)
      local statusShow = ''
      for _, v in pairs(res) do
	statusShow = statusShow..("%-6s %s\n"%{p2name[v[1]], v[2]})
      end

      local swconf = C.getConfWithDefault(s.clonePath(path, -1))
      local swt
      if swconf["vlan-mode"] then
        local sw = {
          vlan = {},
          phy = {}
        }
        for k, v in pairs(swconf) do
          local phy = k:match"^phy(%d)$"
          if phy then
            setPhyParam(sw, v, tonumber(phy))
          end
        end

        print(s.ser(sw, 0))

        swt = {
          "\n",
          "      %s  %s  %s  %s  %s"%{
            sw.phy[0]["tag-mode"] == 2 and 'v' or ' ',
            sw.phy[1]["tag-mode"] == 2 and 'v' or ' ',
            sw.phy[2]["tag-mode"] == 2 and 'v' or ' ',
            sw.phy[3]["tag-mode"] == 2 and 'v' or ' ',
            sw.phy[4]["tag-mode"] == 2 and 'v' or ' ',
          },
          "      P0 P1 P2 P3 P4"
        } 

        for i = 0, 15 do
          local vlan = sw.vlan[i]
          if not vlan then break end
          if vlan.vid ~= COMMON_VID then
            swt[#swt+1] = "%4d  %s  %s  %s  %s  %s"%{vlan.vid, 
              bit.band(vlan.mb, 1) == 0 and ' ' or '*',
              bit.band(vlan.mb, 2) == 0 and ' ' or '*',
              bit.band(vlan.mb, 4) == 0 and ' ' or '*',
              bit.band(vlan.mb, 8) == 0 and ' ' or '*',
              bit.band(vlan.mb, 16) == 0 and ' ' or '*',
            }
          end
        end
        swt = table.concat(swt, '\n')
      end

      return statusShow..(swt or '')
    end
  },
}
