module(..., package.seeall)
local s = require'scheme'
local sock = require'socket'
local i = require'nsgifaceconf'
local e = require'nsgethconf'

local clonePath = s.clonePath

local function splitReceive(udp)
   udp:settimeout(2)
   local res, ip, port = udp:receivefrom()
   if res then
      local out={}
      local i = 1
      local k, n, buf =  res:match("^([%d]+):([%d]+):(.*)$")
      k = tonumber(k)
      n = tonumber(n)
      if k then
         out[k] = buf
         while i < n do
            res = udp:receive()
            if res then
               k, buf =  res:match("^([%d]+):[%d]+:(.*)$")
               k = tonumber(k)
               out[k] = buf
               i = i + 1
            else
               i = n
            end
         end
      end
      return table.concat(out), ip, port
   else
      return nil
   end
end

local function clearConfForGre(conf)
   conf = s.deepcopy(conf) or {}
   local allowKey = {
      encapsulation = 1,
      source = 1,
      device = 1,
      destination = 1,
      key = 1,
      ttl = 1,
      tos = 1,
      checksum = 1,
      keepalive = 1,
      ["keepalive-without-key"] = 1,
      ["keepalive-interval"] = 1,
      ["keepalive-retry"] = 1,
      ["adm-state"] = 1,
      ["sequence-datagrams"] = 1,
      ["reoder-buffer"] = 1,
      ["reoder-timeout"] = 1,
      ["sync-buffer"] = 1,
      ["sync-timeout"] = 1,
      ["sync-integrity"] = 1,
   }
   for k, v in pairs(conf) do
      if nil == allowKey[k]
      then
         conf[k] = nil
      end
   end
   conf.device = i.getIfaceName(conf.device)
   local vS = s.ip4addressVal(conf.source) and 4 or s.ip6addressVal(conf.source) and 6 or nil
   local vD = s.ip4addressVal(conf.destination) and 4 or s.ip6addressVal(conf.destination) and 6 or nil
   conf.ipver = ((vS or vD) == (vD or vS)) and (vS or vD) or nil
   return conf
end


local greDevConfigs = {

}

local greIpConfigs = {

}

local greNeteventKey = {}
local greAliasHookKey = {}

local greAliasHook =
   function()
      --C.log_debug("gre alias hook")
      local newDevConfigs = {}
      for k,v in pairs(greDevConfigs) do
         newDevConfigs[i.getIfaceName(v[(next(v))].device or 'lo') or 'lo'] = v
      end
      local sock=require"socket"
      local udp=sock:udp()
      if udp then
         for k,v in pairs(greDevConfigs) do
            if not newDevConfigs[k] then
               local cmd
               for g,_ in pairs(v or {}) do
                  cmd = 'delTunnel("'..tostring(g)..'")'
                  C.log_debug("send cmd:"..cmd)
                  udp:sendto(cmd,"127.0.0.1",50009)
               end
            end
         end
         for k,v in pairs(newDevConfigs) do
            if not greDevConfigs[k] then
               local cmd
               for g, gv in pairs(v) do
                  cmd = 'addTunnel("'..tostring(g)..'",'..s.ser(clearConfForGre(gv) or {})..')'
                  C.log_debug("send cmd:"..cmd)
                  udp:sendto(cmd,"127.0.0.1",50009)
               end
            end
         end
         udp:close()
      end
      greDevConfigs = newDevConfigs
   end

local greUpdateTunnelState =
   function(msg)
      --C.log_debug("gre netevent hook")
      if msg and (msg.type=='dellink' or  msg.type=='newlink') and msg.ifname and greDevConfigs[msg.ifname] then
         local sock=require"socket"
         local udp=sock:udp()
         if udp then
            for k,v in pairs(greDevConfigs[msg.ifname] or {}) do
               local cmd
               if msg.type=='newlink' and msg.ifup then
                  cmd = 'addTunnel("'..tostring(k)..'",'..s.ser(clearConfForGre(v) or {})..')'
               elseif msg.type=='dellink' or (msg.type=='newlink' and not msg.ifup)  then
                  cmd = 'delTunnel("'..tostring(k)..'")'
               end
               C.log_debug("send cmd:"..cmd)
               udp:sendto(cmd,"127.0.0.1",50009)
            end
            udp:close()
         end
      elseif msg and  (msg.type=='newaddr' or msg.type=='deladdr') and msg.address then
         local sock=require"socket"
         local udp=sock:udp()
         if udp then
            for k,v in pairs(greIpConfigs[msg.address] or {}) do
               local cmd
               if msg.type=='newaddr' then
                  cmd = 'addTunnel("'..tostring(k)..'",'..s.ser(clearConfForGre(v) or {})..')'
               elseif msg.type=='deladdr' then
                  cmd = 'delTunnel("'..tostring(k)..'")'
               end
               C.log_debug("send cmd:"..cmd)
               udp:sendto(cmd,"127.0.0.1",50009)
            end
            udp:close()
         end
      end
   end

local function updateGreDeviceTable(greName, conf)
   if conf.device and (conf["adm-state"] == 'up') then
      local ifname = i.getIfaceName(conf.device)
      greDevConfigs[ifname] = greDevConfigs[ifname] or {}
      greDevConfigs[ifname][greName] = conf
   end
   if conf.source and conf["adm-state"] == 'up' then
      greIpConfigs[conf.source] = greIpConfigs[conf.source] or {}
      greIpConfigs[conf.source][greName] = conf
   end
end

local function createGreConf(conf)
   --C.log_warn(s.ser(conf))
   greDevConfigs = {}
   greIpConfigs = {}
   local greConf
   for k,v in pairs(conf or {}) do
      if(tostring(k):find("^gre%d+$")) then
         if v["adm-state"] == "up" and v.destination and v.source then
            local clear = clearConfForGre(v) or {}
            greConf = (greConf or "")..tostring(k).."="..s.ser(clear)..",\n"
            updateGreDeviceTable(k, clear)
         end
      end
   end
   if greConf then
      greConf = "return {\n"..greConf.."}\n"
   end
   return greConf
end

local function startGre(path)
   local sch = C.getSchemeByPath(path)
   local conf =  C.getConfByPath(path)
   local defConf = s.addDefault(path, sch, conf, true)
   local greConf, notEmpty = createGreConf(defConf)
   local grePath = s.clonePath(path,nil,"gre")
   local fname = os.tmpname()
   local f = io.open(fname, 'w')
   if f then
      f:write(greConf or "")
      f:close()
      C.restartDaemon(
         grePath,
         {"gre", fname},
         {noRestart = false,
          usePopen = false,
          atExit = C.increasingSleep,
          tmpFileList = {fname},
       }
      )
      C.setOption(path, "run", true)
      C.updateNeteventCallback(
         greNeteventKey,
         greUpdateTunnelState
      )
      i.setAliasHook(
         greAliasHookKey,
         greAliasHook
      )
   end
end

local function tryGre(path)
   if  C.getOption(path, "run") ~= true then
      startGre(path)
      return true
   end
   return false
end

local greTunnels = {}

local function restartGre(path, sch, conf, prevConf)

   for k,v in pairs(greTunnels) do
      if not conf[k] then
         os.execute("ip tunnel del "..k)
      end
   end

   local defConf = s.addDefault(path, sch, conf, true)
   C.restartDaemon(s.clonePath(path, nil, "gre"))
   C.setOption(path, "run", nil)
   local greConf = createGreConf(defConf)
   if greConf and not s.confCmp(defConf, prevConf)
   then
      startGre(path)
   end
   if not greConf then
      C.updateNeteventCallback(
         greNeteventKey,
         nil
      )
      i.setAliasHook(
         greAliasHookKey,
         nil
      )
   end
   return true
end

local greTunnelScheme =
   function (_,path)
      local encapsulation = C.getConfByPath(clonePath(path, nil, "encapsulation")) or 'ip'
      local seqd = C.getConfByPath(clonePath(path, nil, "sequence-datagrams")) or 'no'
      local greSchemes = {
         eth = e.ethScheme,
         ip = i.interfaceScheme,
         x3 = s.newScheme,
      }

      local encapScheme =  greSchemes[encapsulation or 'x3']
      
      seqd = seqd=='yes' and s.newScheme{
          ["reoder-buffer"] = s.integerType{
            _H = "greTunReoderbuffer",
            _A = {0, 2^16},
            _D = 0,
            _P = -85,
         },
         ["reoder-timeout"] = s.integerType{
            _H = "greTunReodertimeout",
            _A = {10, 100000},
            _D = 200,
            _P = -85,
         },
         ["sync-buffer"] = s.integerType{
            _H = "greTunSyncbuffer",
            _A = {0, 2^16},
            _D = 0,
            _P = -85,
         },
         ["sync-timeout"] = s.integerType{
            _H = "greTunSynctimeout",
            _A = {100, 100000},
            _D = 10000,
            _P = -85,
         },
         ["sync-integrity"] = s.integerType{
            _H = "greTunSyncintegrity",
            _A = {70, 100},
            _D = 95,
            _P = -85,
         },
      } or s.newScheme{}

      local greScheme ={
         _Z = function(path, sch, conf, prevConf)
                 greTunnels[path[#path]] = true
                 local grePath = s.clonePath(path, -1)
                 if tryGre(grePath) then
                    -- gre will started with new config
                    if type(encapScheme._Z)=='function' then
                       return  encapScheme._Z(path, sch, conf, prevConf)
                    end
                    return true
                 end
                 -- gre already started so send him new conf
                 if not s.confCmp(conf, prevConf) then
                    local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
                    local sock=require"socket"
                    local udp=sock:udp()
                    local name = tostring(path[#path])
                    if udp then
                       udp:settimeout(1)
                       cmd = 'delTunnel("'..name..'")'
                       C.log_debug("send cmd:"..cmd)
                       udp:sendto(cmd,"127.0.0.1",50009)
                       local clear = clearConfForGre(conf) or {}
                       updateGreDeviceTable(name, clear)
                       cmd = 'addTunnel("'..name..'",'..s.ser(clear)..')'
                       C.log_debug("send cmd:"..cmd)
                       udp:sendto(cmd,"127.0.0.1",50009)
                       udp:close()
                    end
                    if type(encapScheme._Z) == 'function' then
                       return  encapScheme._Z(path, sch, conf, prevConf)
                    end
                 end
                 return true
              end,
         _H = "greTun",
         ["adm-state"] =  s.newScheme{
            _T = "choice",
            _A = {"up", "down"},
            _V = s.choiceVal,
            _H = "ifLinkAdm",
            _D = "up",
            _DEP = 10,
         },
         encapsulation = s.choiceType{
            _H = "greTunEncapsulation",
            _A = {
               {"ip", ""},
               {"eth", ""},
            },
            _D = "ip",
            _DEP = 1,
            _P = 0,
         },
         show = s.newScheme{
            _H = "ifShow",
            _X = function(path)
                    local out = {}
                    out[#out + 1] = "Interface stats:\n"
                    out[#out + 1] = i.showInterfaceStat(tostring((path[#path-1])))
                    out[#out + 1] = "Tunnel info:\n"

                    local sock=require"socket"
                    local udp=sock:udp()
                    cmd = 'statistic("'..tostring((path[#path-1]))..'")'
                    if udp then
                       udp:settimeout(2)
                       udp:sendto(cmd,"127.0.0.1",50009)
                       local res,ip,port = splitReceive(udp)
                       if res then
                          if res:find("^Error") then
                             C.log_warn(res)
                          else
                             C.log_debug(res)
                          end
                       end
                       udp:close()
                       out[#out + 1] = res
                    end
                    return table.concat(out)
                 end,
            _R = 'user',
         },
--         description = s.stringType{_H = "greTunDescription", _P = 0},
         source = function (_,path)
                     local dest = C.getConfByPath(clonePath(path, -1, "destination"))
                     return s.ip46AddressType{
                        _H = "greTunSource",
                        _D = s.ip4addressVal(dest) and "0.0.0.0" or "::",
                        _P = -50,
                        _DEP =1,
                     }
                  end,
         destination =  function (_,path)
                     local source = C.getConfByPath(clonePath(path, -1, "source"))
                     return s.ip46AddressType{
                        _H = "greTunDestination",
                        _D = s.ip4addressVal(source) and "0.0.0.0" or "::",
                        _P = -55,
                        _DEP =1,
                     }
                  end,
         device = s.stringType{
            _H = "greTunDev",
--            _D = "lo", --lo is always active device
            _P = -60,
         },
         key = s.integerType{
            _H = "greTunKey",
            _A = {0, 2^32 -1},
            _P = -65,
            _DEP =1,
         },
         tos = function (_,path)
                  local encap = C.getConfByPath(clonePath(path, -1, "encapsulation")) or 'ip'
                  if encap == 'ip' then
                     return  s.variantType{
                        _H = "greTunTos",
                        _A = {
                           s.integerType{
                              _A = {0, 255},
                           },
                           s.stringType{
                              _V = function(val)
                                      if  val=="inherit" then
                                         return val
                                      else
                                         return nil, "string 'inherit'"
                                      end
                                   end
                           },
                        },
                        _P = -65,
                        _D = "inherit",
                     }
                  else
                     return  s.integerType{
                        _H = "greTunTos",
                        _A = {0, 255},
                        _P = -65,
                     }
                  end
               end,
         ttl = s.integerType{
            _H = "greTunTtl",
            _A =  {1, 255},
            _D = 64,
            _P = -65,
         },
         checksum = s.choiceType{
            _H = "greTunChecksum",
            _A = {
               {"yes", ""},
               {"no", ""},
            },
            _D = "no",
            _P = -70,
         },
         keepalive =
            function(_, path)
               local dest = C.getConfByPath(s.clonePath(path, -1, "destination" )) or "0.0.0.0"
               return (dest ~= "0.0.0.0") and s.choiceType{
                  _H = "greTunKeepalive",
                  _A = {
                     {"yes", ""},
                     {"no", ""},
                  },
                  _D = "no",
                  _DEP = 10,
                  _P = -75,
               }
            end,
         ["keepalive-without-key"] =
            function(_, path)
               local keep = C.getConfByPath(s.clonePath(path, -1, "keepalive" ))
               local key = C.getConfByPath(s.clonePath(path, -1, "key" ))
               return (keep == "yes") and (key) and s.choiceType{
                  _H = "greTunKeepalivewithoutkey",
                  _A = {
                     {"yes", ""},
                     {"yes-strict", ""},
                     {"no", ""},
                     {"no-strict", ""},
                  },
                  _D = "no",
                  _P = -77,
               }
            end,
         ["keepalive-interval"] =
            function(_, path)
               local keep = C.getConfByPath(s.clonePath(path, -1, "keepalive" ))
               return (keep == "yes") and s.integerType{
                  _H = "greTunKAInterval",
                  _A = {1, 2^32 - 1},
                  _D = 5,
                  _P = -80,
               }
            end,
         ["keepalive-retry"] =
            function(_, path)
               local keep = C.getConfByPath(s.clonePath(path, -1, "keepalive" ))
               return (keep == "yes") and s.integerType{
                  _H = "greTunKARetry",
                  _A = {1, 2^32 - 1},
                  _D = 3,
                  _P = -85,
               }
            end,
         ["sequence-datagrams"] = s.choiceType{
            _H = "greTunSequencedatagrams",
            _A = {
               {"yes", ""},
               {"no", ""},
            },
            _D = "no",
            _DEP = 1,
            _P = -75,
         },
      }

      return encapScheme(greScheme)(seqd)
   end

local function greTunnelVal(val)
   val = (tostring(val)):match("^gre(%d+)$") or val
   local n, err = s.integerVal(tonumber(val), 1, 255)
   if n then
      return "gre"..tostring(n)
   else
      return nil, err
   end
end

greScheme = s.newScheme{
   _H = "gre",
   _Z = restartGre,
   _NAME = greTunnelScheme,
   _NAME_TYPE = greTunnelVal,
   show =  s.newScheme{
      _H = "greShow",
      statistics = s.newScheme{
         _H = "ifShow",
         _X = function(path)
                 local out = {}
                 out[#out + 1] = "GRE stats\n"
                 local sock=require"socket"
                 local udp=sock:udp()
                 cmd = 'statistic()'
                 if udp then
                    udp:settimeout(2)
                    udp:sendto(cmd,"127.0.0.1",50009)
                    local res,ip,port = splitReceive(udp)
                    if res then
                       if res:find("^Error") then
                          C.log_warn(res)
                       else
                          C.log_debug(res)
                       end
                    end
                    udp:close()
                    out[#out + 1] = res
                 end
                 return table.concat(out)
              end,
            _R = 'user',
         },
      log =  s.newScheme{
         _H = "greShowLog",
         _X = function(path)
                 path = s.clonePath(path, -2, "gre")
                 return C.getDaemonLog(path)
              end,
         _R = 'user',
      },
      ["actual-config"] = s.newScheme{
         _H = "greShowConfig",
         _X = function(path)
                 local sock=require"socket"
                 local udp=sock:udp()
                 cmd = 'config()'
                 if udp then
                    udp:settimeout(2)
                    udp:sendto(cmd,"127.0.0.1",50009)
                    local res,ip,port = splitReceive(udp)
                    if res then
                       if res:find("^Error") then
                          C.log_warn(res)
                       else
                          C.log_debug(res)
                       end
                    end
                    udp:close()
                    return res
                 end
              end,
      },
   },
}

