module(..., package.seeall)
local s = require'scheme'
local ip = require'nsgiplib'
local iface_mod = require'nsgifaceconf'
local nf = require'nsgnfconf'
local bird = require'nsgbird'
local rule = require'nsgruleconf'

local function ipRouteApplyAux(path, conf, flush, rebuild)
   if flush then
      os.execute(ip.ip..ip.family[path[1]].." route flush proto nsg  >/dev/null 2>&1")
   elseif not rebuild then
      os.execute("if [ -f /var/run/staticroute"..ip.familyNum[path[1]].."d ] ; then kill -USR1 `cat /var/run/staticroute"..ip.familyNum[path[1]].."d`; fi")
      return
   end
  local script = {}
  for _, route in ipairs(conf) do
     if route.network then
        local type = route.type and route.type or ''
        local gateway = route.gateway and "via "..route.gateway or ''
        local device = route.device and "dev "..iface_mod.getIfaceName(route.device) or ''
        local metric = route.metric and "metric "..tostring(route.metric) or ''
        local com = (ip.ip.." route add %s %s %s %s %s proto nsg  >/dev/null 2>&1 ;"):
        format(
           type, ip.prefixToNet(route.network)  or 'default', gateway, device, metric
        )
        script[#script+1] = com
     end
  end
  if #script > 0 then
    local scriptName = os.tmpname()
    local f = io.open(scriptName, 'w')
    f:write(
      [[#!/bin/sh
        trap 'n=1' USR1
            if [ -f /var/run/staticroute]]..ip.familyNum[path[1]]..[[d ] ; then
          kill `cat /var/run/staticroute]]..ip.familyNum[path[1]]..[[d`
        fi;
        echo $$ > /var/run/staticroute]]..ip.familyNum[path[1]]..[[d
        n=1
        route() {
      ]]..
      table.concat(script, '\n')..
      [[
        }
        while true; do
           sleep 1;
           n=$(($n-1));
           if [ "0" == "$n" ]; then
              n=300;
              route;
           fi;
        done;
      ]]
    )
    f:close()
    C.restartDaemon(path,
      {"sh", scriptName},
      {tmpFileList = {scriptName,'/var/run/staticroute'..ip.familyNum[path[1]]..'d'},}
    )
  else
    C.restartDaemon(path)
  end
end

local ipRouteAliasHookKey = {["6"]={}, ["4"]={}}
local ipNeteventKey = {["6"]={}, ["4"]={}}
local function ipRouteApply(path, sch, conf, prevConf)
  if not s.confCmp(conf, prevConf) then
    local path = s.clonePath(path)
    local conf = s.addDefault(path, sch, conf or {}, true)
    iface_mod.setAliasHook(ipRouteAliasHookKey[tostring(ip.familyNum[path[1]])],
      function(alias, name)
        return  ipRouteApplyAux(path, conf, false, true)
      end
    )
    C.updateNeteventCallback(
       ipNeteventKey[tostring(ip.familyNum[path[1]])],
      function(msg)
        if msg.type == 'newaddr' then
           return  ipRouteApplyAux(path, conf, false)
        end
      end
    )
    ipRouteApplyAux(path, conf, true)
  end
  return true
end
local routeNScheme =
   function (_, path)
      return s.newScheme{
         _H = "ipRoute",
         type = s.choiceType{_H = "ipRouteType", _D = 'unicast', _P = 110, _A = {'unicast', 'unreachable', 'prohibit', 'blackhole'}, _DEP=1},
         network = s[ip.ipPrefixType[path[1]]]{_H = "ipRouteNet", _D = ip.default[path[1]], _P = 100},
         gateway = function (_, path)
                      return (C.getConfByPath(s.clonePath(path, -1, "type")) or "unicast")=='unicast' and s.ip4AddressType{_H = "ipRouteVia", _P = 80} or nil
                   end,
         device = function (_, path)
                     return (C.getConfByPath(s.clonePath(path, -1, "type")) or "unicast")=='unicast' and s.stringType{_H = "ipRouteDev", _P = 60} or nil
                  end,
         metric = s.integerType{
            _H = "ipRouteMetric",
            _A = {0, 0x7fffffff},
            _D = 0,
            _P = 40
         }
      }
   end

local routeScheme = s.newScheme{
  _H = "ipRouteList",
  _Z = ipRouteApply,
  _LIST = routeNScheme,
  show = s.newScheme{
    _H = "ipRouteShow",
    _X = function(path)
            local cmd = {
               ip.ip,
               (ip.family[path[1]]),
               "route show",
               (path[#path-1] ~= "route") and "table "..path[#path-1] or "",
               " 2>&1"
            }
            return ('\n'..s.readpopen(table.concat(cmd," ")))
    end
  }
}

ipScheme = s.newScheme{
  _H = "ip",
  _Z = function() return true end,
  rule = rule.ruleScheme,
  route = routeScheme{
     _P = 100,
  },
  mangle = nf.mangleScheme{_P = 30},
  nat = function (_, path)
           return path[1]=="ip" and nf.natScheme{_P = 20} or nil
        end,
  filter = nf.filterScheme{_P = 10},
  ["dynamic-routing"] = function (_, path)
           return path[1]=="ip" and  bird.birdScheme{_P = 90} or nil
        end,
  show = s.newScheme{
     _H = "ipShow",
     _X = function (path)
             local out = {}
             out[#out + 1] = "\nInterfaces:"
             out[#out + 1] = s.readpopen(ip.ip..(ip.family[path[1]]).." -s -s -d -d link ls 2>&1")
             out[#out + 1] = "\nAddresses:"
             out[#out + 1] = s.readpopen(ip.ip..(ip.family[path[1]]).." -s -s -d -d addr ls 2>&1")
             out[#out + 1] = "\nDefault routing table:"
             out[#out + 1] = s.readpopen(ip.ip..(ip.family[path[1]]).." route ls 2>&1")
             out[#out + 1] = "\nIptables rules:"
             out[#out + 1] = s.readpopen(ip["iptables-save"][path[1]].." 2>&1")
             return table.concat(out,"\n")
          end,
  },
}
