module(..., package.seeall)
local nsg = require'nsg'

ip = "/usr/sbin/ip"

family = {
   ip = " -4 ",
   ipv6 = " -6 ",
}

ipPrefixType = {
   ip = "ip4PrefixType",
   ipv6 = "ip6PrefixType"
}

familyNum ={
   ip = "4",
   ipv6 = "6",
}

default = {
   ip = "0.0.0.0/0",
   ipv6 = "::/0",
}

_G["iptables-save"] = {
   ip = "/usr/local/sbin/iptables-save",
   ipv6 = "/usr/local/sbin/ip6tables-save",
}

iptables = {
   ip = "/usr/local/sbin/iptables",
   ipv6 = "/usr/local/sbin/ip6tables",
}

function idiv(a,b)
   return math.floor(a/b), a%b
end

function intToIp(n, p)
   n, a4 = idiv(n,256)
   n, a3 = idiv(n,256)
   n, a2 = idiv(n,256)
   n, a1 = idiv(n,256)
   return tostring(a1)..'.'..tostring(a2)..'.'..tostring(a3)..'.'..tostring(a4)..
(p and '/'..tostring(p) or '')
end

function ipToInt(prefix)
   local n
   local a1, a2, a3, a4, p =
      string.match(prefix,
                   "^(%d%d?%d?)%.(%d%d?%d?)%.(%d%d?%d?)%.(%d%d?%d?)(/?%d?%d?)$")
   if a1 then
      a1 = tonumber(a1)
      a2 = tonumber(a2)
      a3 = tonumber(a3)
      a4 = tonumber(a4)
      if a1 <= 255 and a2 <= 255 and a3 <= 255 and a4 <= 255 then
         n = a1 * 256 * 256 * 256 +
            a2 * 256 * 256 +
            a3 * 256 +
            a4
      else
         return nil
      end
      if p == '' then
         return n, 32
      end
      if #p >= 2 and p:sub(1,1) == '/' then
         p = tonumber(p:sub(2))
         if p <= 32 then
            return n, p
         end
      end
   end
   return nil
end

function _prefixToNet(prefix)
   local n, p = ipToInt(prefix)
   if n then
      local pp = 2^(32-p)
      local net = math.floor(n/pp) * pp
      return intToIp(net, p)
   end
   return nil
end


function prefixToNet(prefix)
   local ip, net = (tostring(prefix)):match("^([0-9a-fA-F%.%:]*)/([0-9]*)$")
   if ip and net then
      ip, net = nsg.prefixToNet(ip, net)
      return ip and  tostring(ip)..'/'..tostring(net) or nil
   end
   return nil
end

