local M = {}
local nsg = require'nsg'
local s = require'scheme'
local usb = require'nsgusbconf'
local sens = require'nsgsensconf'
local evt = require'nsgeventconf'

local ser = nsg.serialize

local function eventGeneratorApply(path, sch, conf, prevConf)
  local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
  if not conf.enable then
    C.restartDaemon(path)
    return true
  end

  local event = {}
--[[
event = {
  U = {
    ['1'] = {{nil,215,"low$(VAL)"},{230, nil, "H"},{[3]="norm $(VAL)"}, name = "Напряг1"},
    ['2'] = {{nil,215,"low$(VAL)"},{230, nil, "H"},{[3]="norm $(VAL)"}},
    ['3'] = {{nil,215,"low$(VAL)"},{230, nil, "H"},{[3]="norm $(VAL)"}},
  },
  N = {{{nil,nil,"$(VAL)"}}},
}]]
  for k, v in pairs(conf) do
    if type(v) == 'table' then
      local par, phase = k:match"^([^%-]*)%-?(.*)$"
      if not event[par] then event[par] = {} end
      local t = {name = v["sensor-name"]}
      for i = 1, #v do
	t[i] = {v[i].from, v[i].to, v[i]["event-name"]}
      end
      event[par][phase] = t
    end
  end

  event = ser(event)
  usb.probeUSBDeviceScheme(nil, s.clonePath(path, - 1))
  local addr = C.getConfOrDefault(s.clonePath(path, -1, "addr"))
  local com = {
    "mercury230", "-q", "-p", usb.port2tty[path[#path - 1]],
    "--event", event,
    "--prefix", conf.prefix or "",
    "--interval", conf.interval or 5,
    "--addr", addr or 0,
  }
  evt.restartVirtSensor(path, com)
  return true
end


M.mercury230scheme = s.newScheme{
  addr = s.integerType{
    _H = "mercury230Addr",
    _A = {0, 0xFE},
    _D = 0,
  },
  show = s.newScheme{
    _H = "mercury230Show",
    _X = function(path, sch, conf)
      local enableEvent = C.getConfOrDefault(s.clonePath(path, -1,
        "event-generator", "enable")
      )
      if enableEvent then
        C.setConfigValue(s.clonePath(path, -1, "event-generator", "enable", false))
        C.applyConf(s.clonePath(path, -1, "event-generator"))
      end
      usb.probeUSBDeviceScheme(nil, s.clonePath(path, -1))
      local addr = C.getConfOrDefault(s.clonePath(path, -1, "addr"))
      local res = -("mercury230 -q -a%d -p%s"%{addr, usb.port2tty[path[#path - 1]]})
      if enableEvent then
        C.setConfigValue(s.clonePath(path, -1, "event-generator", "enable", true))
        C.applyConf(s.clonePath(path, -1, "event-generator"))
      end
      return res
    end,
    _R = 'user',
  },

  ['get-port-parms'] = function(_, path)
    return (C.session and C.session.script) and
    s.newScheme{
      _X = function(path, sch, val)
	usb.probeUSBDeviceScheme(nil, s.clonePath(path, - 1))
        local t = {}
        t.ttyDev = usb.port2tty[path[#path-1]]
        return s.ser(t)
      end,
      _R = 'user',
    }
  end,

  ["event-generator"] = s.newScheme{
    _H = "mercury230Eventgenerator",
    _Z = eventGeneratorApply,

    enable = s.booleanType{
      _H = "eventgeneratorEnable",
      _D = false,
      _P = 100,
    },
    interval = s.integerType{
      _H = "eventgeneratorInterval",
      _A = {1, 60*60},
      _D = 5,
      _P = 90,
    },
    prefix = s.stringType{
      _H = "eventgeneratorPrefix",
      _D = "mercury230.",
      _P = 80,
    },
    ["U-1"] = sens.eventgeneratorNumScheme,
    ["U-2"] = sens.eventgeneratorNumScheme,
    ["U-3"] = sens.eventgeneratorNumScheme,
    ["I-1"] = sens.eventgeneratorNumScheme,
    ["I-2"] = sens.eventgeneratorNumScheme,
    ["I-3"] = sens.eventgeneratorNumScheme,
    ["P-1"] = sens.eventgeneratorNumScheme,
    ["P-2"] = sens.eventgeneratorNumScheme,
    ["P-3"] = sens.eventgeneratorNumScheme,
    ["P-t"] = sens.eventgeneratorNumScheme,
    ["Q-1"] = sens.eventgeneratorNumScheme,
    ["Q-2"] = sens.eventgeneratorNumScheme,
    ["Q-3"] = sens.eventgeneratorNumScheme,
    ["Q-t"] = sens.eventgeneratorNumScheme,
    ["S-1"] = sens.eventgeneratorNumScheme,
    ["S-2"] = sens.eventgeneratorNumScheme,
    ["S-3"] = sens.eventgeneratorNumScheme,
    ["S-t"] = sens.eventgeneratorNumScheme,
    ["COS-1"] = sens.eventgeneratorNumScheme,
    ["COS-2"] = sens.eventgeneratorNumScheme,
    ["COS-3"] = sens.eventgeneratorNumScheme,
    ["COS-t"] = sens.eventgeneratorNumScheme,
    ["FU-12"] = sens.eventgeneratorNumScheme,
    ["FU-23"] = sens.eventgeneratorNumScheme,
    ["FU-31"] = sens.eventgeneratorNumScheme,
    ["F"] = sens.eventgeneratorNumScheme,
    ["E"] = sens.eventgeneratorNumScheme,
  }
}

return M
