module(..., package.seeall)
local s = require'scheme'
local i = require'nsgifaceconf'
local nsg = require'nsg'
----------------------------------------
netflows = {}

local netflowAliasHookFunction =
   function(alias, name)
      for sifPath in pairs(netflows) do
         ifPath = loadstring("return "..sifPath)()
         if ifPath[#ifPath] == alias then
            restartNetflow(sifPath)
         end
      end
      return true
   end

local netflowAliasHookKey ={}


function restartNetflow(sifPath)
   i.setAliasHook(netflowAliasHookKey, netflowAliasHookFunction)
   ifPath = loadstring("return "..sifPath)()
   local nf = (C.getConfByPath(ifPath) or {} ).netflow or -1

   local sch = C.getSchemeByPath({"services","netflow", nf})
   local conf =  C.getConfByPath({"services","netflow", nf})
   local nc = sch and s.addDefault({"services","netflow", nf}, sch, conf, true) or nil

   if nc==nil or nc["adm-state"] == "down" then
      C.restartDaemon(s.clonePath(ifPath,nil,"netflow"))
   else
      local param = {
         "fprobe",
         "-v4",
         "-l2",
         "-i", C.getConfOrDefault(s.clonePath(ifPath, nil, "ifName")) or i.getIfaceName(ifPath[#ifPath]),
         "-n", nc.version or 5,}
      param[#param+1] = nc.source and "-a" or nil
      param[#param+1] = nc.source and nc.source or nil
      param[#param+1] = nc.filter and "-f" or nil
      param[#param+1] = nc.filter and (nc.filter) or nil
      param[#param+1] = (nc.promiscuous==false) and "-p" or nil
      for k,v in ipairs(nc.destination or {}) do
          param[#param+1] = v and v or nil
      end
      C.restartDaemon(
         s.clonePath(ifPath,nil,"netflow"),
         param,
         {noRestart = false,
          usePopen = false,
          atExit = C.increasingSleep,
       }
      )
   end
end

local restartNetflowDaemon =
   function(path, sch, conf, prevConf)
      for sifPath in pairs(netflows) do
         if netflows[sifPath] == path[#path] then
            restartNetflow(sifPath)
         end
      end
      return true
   end

local restartNetflowDaemons =
   function(path, sch, conf, prevConf)
      for sifPath in pairs(netflows) do
         restartNetflow(sifPath)
      end
      return true
   end

NetflowScheme = s.newScheme {
   _H = "Netflow",
   _Z = restartNetflowDaemons,
   _NAME_TYPE= function(val)
                  if val == nil or val=="" then
                     local conf = C.getConfByPath({"services","netflow"}) or {}
                     val = table.maxn(conf)<255 and table.maxn(conf) + 1 or val
                  end
                  return s.integerVal(tonumber(val), 1, 255)
               end,
   _NAME = s.newScheme {
      _H = "NetflowN",
      _Z = restartNetflowDaemon,
      description = s.stringType{_H = "NetflowNDescription", _P = 100},
      ["adm-state"] = s.choiceType{
         _A = {"up", "down"},
         _H = "NetflowNState",
         _D = "up",
         _P = 110,
      },
      version = s.choiceType{
            _H = "NetflowNVersion",
            _A = {
               {1, ""},
               {5, ""},
               {7, ""},
            },
            _D = 5,
            _P = 0,
         },
      source = s.ip46AddressType{
         _H = "NetflowNSource",
      },
      destination = s.newScheme{
         _H = "NetflowNDestination",
         _LIST = s.hostnameAndPortType{
            _H = "NetflowNDestinationM",
         }
      },
      filter = s.stringType{
          _H = "NetflowNFilter",
      },
      promiscuous = s.booleanType{
         _H = "NetflowNPromiscuous",
         _D = true
      }
   },
}