module(..., package.seeall)
local s = require'scheme'
local i = require'nsgifaceconf'
local e = require'nsgethconf'
local nsg = require'nsg'

local clonePath = s.clonePath

local ovpns = {

}

local function createOvpnConf(name, conf)
   local out = {}
   conf = conf or {}
   out[#out+1] = conf["verb"] and "verb "..conf["verb"] or nil
   out[#out+1] = "writepid /var/run/openvpn/"..name
   out[#out+1] = "suppress-timestamps"
   out[#out+1] = "mode "..(conf.mode or "p2p")
   out[#out+1] = "dev "..(conf["dev-type"] ~= "null" and name or "null")
   out[#out+1] = "dev-type "..(conf["dev-type"] or "tun")
   out[#out+1] = "proto "..(conf["proto"] or "udp")
   out[#out+1] = conf["connect-retry"] and "connect-retry "..conf["connect-retry"] or nil
   out[#out+1] = conf["local"] and "local "..conf["local"] or nil
   out[#out+1] = conf["lport"] and "lport "..conf["lport"] or nil
   if conf.remote then
      for i,v in ipairs(conf.remote or {}) do
         out[#out+1] = v and "remote "..v or nil
      end
      out[#out+1] = conf.remote["rport"] and "rport "..conf.remote["rport"] or nil
      out[#out+1] = conf.remote["remote-random"]=='yes' and "remote-random" or nil
      out[#out+1] = conf.remote["float"]=='yes' and "float" or nil
   end
   out[#out+1] = conf["ping"] and "ping "..conf["ping"] or nil
   out[#out+1] = conf["secret"] and "secret "..conf["secret"] or nil
   out[#out+1] = (conf.security == "tls-server" or conf.security == "tls-client" ) and conf.security or nil
   out[#out+1] = conf.dh and "dh "..conf.dh or nil
   out[#out+1] = conf.ca and "ca "..conf.ca or nil
   out[#out+1] = conf.cert and "cert "..conf.cert or nil
   out[#out+1] = conf.key and "key "..conf.key or nil
   out[#out+1] = "script-security 2"
   out[#out+1] = 'up /tmp/openvpn/'..name..'_up.sh'
   out[#out+1] = conf["duplicate-cn"]=='yes' and "duplicate-cn" or nil
   out[#out+1] = conf['comp-lzo'] and "comp-lzo "..conf['comp-lzo'] or nil
   out[#out+1] = conf['auth'] and "auth "..conf['auth'] or nil
   out[#out+1] = conf['cipher'] and "cipher "..conf['cipher'] or nil
   out[#out+1] = conf["extra"]
-- out[#out+1] = "management /tmp/openvpn/"..name..'.unix unix' 
-- out[#out+1] = "management tunnel 50015"
   os.execute("mkdir -p /tmp/openvpn/")
   os.execute("echo -e '#!/bin/sh\\nifconfig $1 up\\n' >/tmp/openvpn/"..name.."_up.sh; chmod +x /tmp/openvpn/"..name.."_up.sh")
   os.execute("mkdir -p /var/run/openvpn/")
   local f = io.open("/tmp/openvpn/"..name, "w")
   f:write(table.concat(out,"\n"))
   f:close()
   return true
end


local function restartOvpns(path, sch, conf, prevConf)
   for k in pairs(ovpns) do
      if not conf[k] then
         C.restartDaemon(clonePath(path, nil, {k,"openvpn"}))
      end
   end
   return true
end

local ovpnTunnelScheme =
   function (_,path)
      local devtype = C.getConfByPath(clonePath(path, nil, "dev-type")) or 'tun'
      local proto = C.getConfByPath(clonePath(path, nil, "proto")) or 'udp'
      local mode = C.getConfByPath(clonePath(path, nil, "mode")) or 'p2p'
      local security = C.getConfByPath(clonePath(path, nil, "security")) or (mode == 'p2p') and "none" or "tls-server"
      if proto and not ({ p2p = {
                           udp=1,
                           ["tcp-client"]=1,
                           ["tcp-server"]=1
                        },
                     server  = { udp=1,tcp=1}
                  })[mode][proto]
      then
         C.setConfigValue(clonePath(path, nil, "proto"))
         proto = 'udp'
      end

      if security and not ({ p2p = {
                           none=1,
                           ["preshared-key"]=1,
                           ["tls-server"]=1,
                           ["tls-client"]=1
                        },
                             server  = { ["tls-server"]=1}
                  })[mode][security]
      then
         C.setConfigValue(clonePath(path, nil, "security"))
         security = (mode == 'p2p') and "none" or "tls-server"
      end

      local ovpnSchemes = {
         tap = e.ethScheme,
         tun = i.interfaceScheme,
         x3 = s.newScheme,
         null = s.newScheme
      }

      local encapScheme =  ovpnSchemes[devtype or 'tun']

      local ovpnScheme ={
         _Z = function(path, sch, conf, prevConf)
                 if type(encapScheme._Z)=='function' then
                    encapScheme._Z(path, sch, conf, prevConf)
                 end
                 if not s.confCmp(conf, prevConf) then
                    local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
                    s.confValidate(path, sch, conf, true) --clear conf
                    if conf["adm-state"] == "up" then
                       createOvpnConf(path[#path], conf)
                       local scr = "[ -f /var/run/openvpn/"..path[#path].." ] && ps w | grep `cat /var/run/openvpn/zzz | grep -o '[0-9]*'` | grep openvpn | grep -oE '^[[:space:]]*[0-9]+' | grep -oE '[0-9]+' && kill `cat /var/run/openvpn/"..path[#path].."` 1>&2 2>/dev/null"
                       if ovpns[path[#path]] then
                          os.execute(scr)
                       else
                          os.execute(scr)
                          C.restartDaemon(
                             clonePath(path, nil, "openvpn"),
                             {"openvpn", "/tmp/openvpn/"..path[#path]},
                             {noRestart = false,
                              atExit = C.increasingSleep,
                              tmpFileList = {"/tmp/openvpn/"..path[#path]},
                           }
                          )
                          ovpns[path[#path]] = true
                       end
                    else
                       C.restartDaemon(clonePath(path, nil, "openvpn"))
                       ovpns[path[#path]] = nil
                    end
                 end
                 return true
              end,
         _H = "ovpnTun",
         mode = s.choiceType{
            _H = "ovpnTunMode",
            _A = {
               {"p2p", ""},
               {"server", ""},
            },
            _D = "p2p",
            _DEP = 1,
            _P = 100
         },
         ['dev-type'] = s.choiceType{
            _H = "ovpnTunDevtype",
            _A = {
               {"tun", ""},
               {"tap", ""},
               {"null", ""},
            },
            _D = "tun",
            _DEP = 1,
            _P = 90
         },
         proto = s.choiceType{
            _H = "ovpnTunProto",
            _A = mode == "p2p" and {
               {"udp", ""},
               {"tcp-client", ""},
               {"tcp-server", ""},
            } or {
               {"udp", ""},
               {"tcp", ""}
            },
            _D = "udp",
            _P = 85,
            _DEP = 1 ,
         },
         ['connect-retry'] = (proto == 'tcp-client') and s.integerType{
            _H = "ovpnTunConnectRetry",
            _A = {1, 65535},
            _D =5,
            _P = 84,
         },
         ['local'] = s.ip46AddressType{
            _H = "ovpnTunLocal",
            _P = 70
         },
         ['lport'] = s.integerType{
            _H = "ovpnTunLport",
            _P = 68,
            _A = {1,65535},
            _D = 1194,
         },
         ['remote'] = mode == "p2p" and s.newScheme {
            _P = 65,
            _H = "ovpnTunRemote",
            _LIST = s.hostnameOrHostnameAndPortType{
               _H = "ovpnTunRemoteX",
               _P = 0,
               _D = "127.0.0.1"
            },
            ['rport'] = s.integerType{
               _H = "ovpnTunRport",
               _P = 0,
               _A = {1,65535},
               _D = 1194,
            },

            ["remote-random"] = s.choiceType{
               _H = "ovpnTunRemoterandom",
               _A = {
                  {"yes", ""},
                  {"no", ""},
               },
               _D = "no",
               _P = -10,
            },
            float = s.choiceType{
               _H = "ovpnTunFloat",
               _A = {
                  {"yes", ""},
                  {"no", ""},
               },
               _D = "no",
               _P = -20,
            },
         } or nil,
         ["max-clients"] = mode == "server" and s.integerType{
               _H = "ovpnTunMaxclients",
               _P = 0,
               _A = {1,65535},
               _P = 60
            } or nil,

         ["duplicate-cn"] =  mode == "server" and s.choiceType{
               _H = "ovpnTunDuplicatecn",
               _A = {
                  {"yes", ""},
                  {"no", ""},
               },
               _D = "no",
               _P = 55,
            } or nil,
         ["extra"] = s.stringType{
            _H = "ovpnTunExtra",
            _P = 0,
            },
         ['ping'] = s.integerType{
               _H = "ovpnTunPing",
               _P = 0,
               _A = {1,65535},
            },
         ['security'] = s.choiceType{
            _H = "ovpnTunSecurity",
            _A = (mode == 'p2p') and {
               {"none", ""},
               {"preshared-key", ""},
               {"tls-client", ""},
               {"tls-server", ""}
            } or
            {{"tls-server", ""}},
            _D = (mode == 'p2p') and "none" or "tls-server",
            _DEP = 1,
            _P = -30,
         },
--preshared -key
         secret = (security == "preshared-key") and s.stringType{
            _H = "ovpnTunSecret",
            _P = -33,
            _D = '/etc/openvpn/'..path[#path]..'/secret'
         },
--tls-server
         dh = (security == "tls-server") and s.stringType{
            _H = "ovpnTunDh",
            _P = -36,
            _D = '/etc/openvpn/'..path[#path]..'/dh.pem'
         },
--tls-client/server
         ca = (security == "tls-server" or security == "tls-client" ) and s.stringType{
            _H = "ovpnTunCa",
            _P = -39,
            _D = '/etc/openvpn/'..path[#path]..'/ca.crt'
         },
         cert = (security == "tls-server" or security == "tls-client" ) and s.stringType{
            _H = "ovpnTunCert",
            _P = -42,
            _D = '/etc/openvpn/'..path[#path]..'/'..(security:gsub("tls%-",''))..'.crt'
         },
         key = (security == "tls-server" or security == "tls-client" ) and s.stringType{
            _H = "ovpnTunKey",
            _P = -45,
            _D = '/etc/openvpn/'..path[#path]..'/'..(security:gsub("tls%-",''))..'.key'
         },

         cipher = function (_,path)
                     local ciphers = s.readpopen("openvpn --show-ciphers | grep -A1000 -E '^$' | grep -oE '^[^[:space:]]+' | sort -f")
                     local ct = loadstring('return {"'..((ciphers or ""):gsub('\n$','') or ""):gsub('\n','","')..'"}')() or {}
                     ct[#ct + 1] = "none"
                     return s.choiceType{
                         _H = "ovpnTunCipher",
                        _A = ct,
                        _D = "BF-CBC",
                        _P = 0,
                     }
                  end,
         auth = function (_,path)
                     local digests = s.readpopen("openvpn --show-digests | grep -A1000 -E '^$' | grep -oE '^[^[:space:]]+' | sort -f")
                     local dt = loadstring('return {"'..((digests or ""):gsub('\n$','') or ""):gsub('\n','","')..'"}')() or {}
                     dt[#dt + 1] = "none"
                     return s.choiceType{
                         _H = "ovpnTunAuth",
                        _A = dt,
                        _D = "SHA1",
                        _P = 0,
                     }
                  end,
         ['comp-lzo'] = s.choiceType{
            _H = "ovpnTunCompression",
            _A = {
               {"yes", ""},
               {"no", ""},
               {"adaptive", ""},
            },
            _D = "adaptive",
            _P = 0,
         },

         show = s.newScheme{
            _H = "ifShow",
            _X = function(path)
                    local out = {}
                    out[#out + 1] = "Interface stats:\n"
                    out[#out + 1] = i.showInterfaceStat(tostring((path[#path-1])))
                    return table.concat(out)
                 end,
            _R = 'user',
            _P = -150,
         },
         ['verb'] = s.integerType{
               _H = "ovpnTunVerb",
               _P = 0,
               _A = {0,11},
               _D = 1
            },
         log =  s.newScheme{
            _H = "ovpnShowLog",
            _X = function(path)
                    return C.getDaemonLog(s.clonePath(path, -1, "openvpn"))
                 end,
            _R = 'admin',
            _P = -150,
         },
         ["actual-config"] = s.newScheme{
            _H = "ovpnShowConfig",
            _X = function(path)
                    return s.readpopen("cat /tmp/openvpn/"..(path[#path - 1] or ""))
                 end,
            _R = 'user',
            _P = -150,
         },
      }

      return encapScheme(ovpnScheme)
   end

local function ovpnTunnelVal(val)
   val = (tostring(val)):match("^ovpn(%d+)$") or val
   local n, err = s.integerVal(tonumber(val), 1, 255)
   if n then
      return "ovpn"..tostring(n)
   else
      return nil, err
   end
end

ovpnScheme = s.newScheme{
   _H = "ovpn",
   _Z = restartOvpns,
   _NAME = ovpnTunnelScheme,
   _NAME_TYPE = ovpnTunnelVal
}

