module(..., package.seeall)
local s = require'scheme'
local ppp = require'nsgpppconf'
local sms = require'nsgsmsconf'
local usb = require'nsgusbconf'

local function restartPort(path, sch, conf, prevConf)
  if not s.confCmp(conf, prevConf) then
    conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
    if conf["adm-state"] ~= 'down' then
      if conf.encapsulation == 'ppp' then
        ppp.restartPPPDaemon(path, sch, conf)
        return true
      elseif conf.encapsulation == 'sms-handler' then
        sms.restartSMSDaemon(path, sch, conf)
        return true
      end
    end
    C.restartDaemon(path)
  end
  return true
end

local baudrateScheme = s.newScheme{
  _T = 'choice',
  _A = {
    50,
    75,
    110,
    134,
    150,
    200,
    300,
    600,
    1200,
    1800,
    2400,
    4800,
    9600,
    19200,
    38400,
    57600,
    115200,
    230400,
  },
  _V = s.choiceVal,
  _H = "asyncBaudrate",
  _D = 115200,
}


portScheme = s.newScheme{
  --[[
  _N = function(sch, conf)
    local need = {
      true,
      encapsulation = true,
      baudrate = true,
      ["adm-state"] = true,
      ["raw-access"] = true,
    }
    if conf and conf.encapsulation then
      need[conf.encapsulation] = true
      if conf.encapsulation == 'ppp' then
        need['ppp-aux'] = true
        need['ppp-priority'] = true
      end
    end
    return need
  end,
  --]]
  _H = "port",
  _Z = restartPort,
  encapsulation = function(_, path)
    local smsType = C.getOption(s.clonePath(path, -1), "smsType")
    return s.newScheme{
      _T = "choice",
      _A = {"ppp", "none", smsType and "sms-handler"},
      _V = s.choiceVal,
      _H = "portEncapsulation",
      _D = "ppp",
      _DEP = 1,
    }
  end,
  --baudrate = baudrateScheme,
  ["adm-state"] = s.newScheme{
    _T = "choice",
    _A = {"up", "down"},
    _V = s.choiceVal,
    _H = "portAdmState",
    _D = "down",
    _DEP = 1,
  },
  ["raw-access"] = function(_, path)
    local enc = C.getConfOrDefault(s.clonePath(path, -1, "encapsulation"))
    local adm = C.getConfOrDefault(s.clonePath(path, -1, "adm-state"))
    return (enc == 'none' or adm == 'down') and ppp.rawAccessScheme
  end,

  ppp = function(_, path)
    local enc = C.getConfOrDefault(s.clonePath(path, -1, "encapsulation"))
    return enc == 'ppp' and ppp.pppScheme
  end,

  ["sms-handler"] = function(_, path)
    local enc = C.getConfOrDefault(s.clonePath(path, -1, "encapsulation"))
    local port = path[#path - 1]
    local smsType = C.getOption(s.clonePath(path, -1), "smsType")
    return (enc == 'sms-handler' or
      (enc == 'ppp' and type(smsType) == "string")) and sms.smsScheme
  end,

  ["sms-send"] = function(_, path)
    local wirelessType = C.getOption(s.clonePath(path, -1), "wirelessType")
    if wirelessType == "edge" or  wirelessType == "3g" then
      return s.newScheme{
	_H = "portSendsms",
	send = s.newScheme{
	  _H = "portSendsmsSend",
	  _R = 'user',
	  _P = 100,
	  _X = function(path, sch, val, sesnum)
	    local conf = C.getConfByPath(s.clonePath(path, -1), sesnum) or {}
	    if not conf.phone or not conf.text then
	      return 'no phone or text parameter'
	    end
	    local dev = usb.port2tty[path[#path-2] .. "-ctl"]

	    if C.getConfOrDefault(
	      s.clonePath(path, -2, "sms-handler", "ppp-cooperation")
	      ) then
	      return -("at2 --tcpport=50000 sms "..conf.phone..' '..conf.text)
	    elseif dev and dev ~= "" then
	      return -("at2 --dev=" .. dev .. " sms "..conf.phone..' '..conf.text)
	    else
	      return "No control port, try via \"sms-handler.ppp-cooperation\""
	    end
	  end
	},
	phone = s.stringType{
	  _H = "portSendsmsPhone", 
	  _LOCAL = true,
	  _R = "user"
	},
	text = s.stringType{
	  _H = "portSendsmsText",
	  _LOCAL = true,
	  _R = "user"
	},
      }
    end
  end,

  restart = RootOrAdminScheme(
    s.newScheme{
      _H = "portRestart",
      _T = 'choice',
      _A = {'main', 'aux'},
      _X = function(path, sch, val)
        local path = s.clonePath(path, -1)
        local enc = C.getConfOrDefault(s.clonePath(path, nil, "encapsulation"))
        if enc == 'ppp' then
          if val == 'main' then
            C.setConfigValue(s.clonePath(path, nil, 'ppp', 'main', 'attempts'), 1)
            C.setConfigValue(s.clonePath(path, nil, 'ppp', 'aux', 'attempts'), 0)
          elseif val == 'aux' then
            C.setConfigValue(s.clonePath(path, nil, 'ppp', 'main', 'attempts'), 0)
            C.setConfigValue(s.clonePath(path, nil, 'ppp', 'aux', 'attempts'), 1)
          end
        end
        local res, err = C.applyConf(path, nil, true)
        return res and "Port restarted" or (err or "Port not restarted")
      end,
      _R = 'user',
      _P = -90,
    }
  ),

  ["log"] =  s.newScheme{
    _H = "portLog",
    _X = function(path)
      path = s.clonePath(path, -1)
      return C.getDaemonLog(path)
    end,
    _R = 'user',
    _P = -100,
  },

  ["csq-check"] = function(_, path)
    local wirelessType = C.getOption(s.clonePath(path, -1), "wirelessType")
    if wirelessType == "edge" or  wirelessType == "3g" then
      return s.newScheme{
        _H = "portCsqcheck",
        _R = 'user',
        _P = -98,
        _X = function(path, sch, val)
          local ppath = s.clonePath(path, -1)
          local dev = usb.port2tty[ppath[#ppath] .. "-ctl"]
	  
          if C.getConfOrDefault(
            s.clonePath(path, -1, "sms-handler", "ppp-cooperation")
          ) then
            return -("at2 --tcpport=50000 csq")
          elseif dev and dev ~= "" then
            return -("at2 --dev=" .. dev .. " csq")
          else
            return "No control port, try via \"sms-handler.ppp-cooperation\""
          end

        end
      }
    end
  end,

  ["radio-info"] = function(_, path)
    local wirelessType = C.getOption(s.clonePath(path, -1), "wirelessType")
    local device = usb.FindDevice(path[#path - 1])
    if wirelessType == "edge" or  wirelessType == "3g" then
      return s.newScheme{
        _H = "portRadioinfo",
        _R = 'user',
        _P = -99,
        _X = function(path, sch, val)
          local ppath = s.clonePath(path, -1)
          local dev = usb.port2tty[ppath[#ppath] .. "-ctl"]

          if C.getConfOrDefault(
            s.clonePath(path, -1, "sms-handler", "ppp-cooperation")
          ) then
            return -("at2 --tcpport=50000 rinfo")
          elseif dev and dev ~= "" then
            return -("at2 --dev=" .. dev .. " rinfo")
          else
            return "No control port, try via \"sms-handler.ppp-cooperation\""
          end

        end
      }
    elseif device and ((device.vidpid == "16d8/6803") or 
		       (device.vidpid == "05c6/3197") or 
		       (device.vidpid == "15eb/0101")) then
      return s.newScheme{
        _H = "portRadioinfo",
        _R = 'user',
        _P = -99,
        _X = function(path, sch, val)
          local ppath = s.clonePath(path, -1)
          local dev = usb.port2tty[ppath[#ppath] .. "-ctl"]
          if dev and dev ~= "" then
	     if (device.vidpid == "15eb/0101") then
		return -("at3 --dev=" .. dev) -- ATEL (via ctl-port) 
	     else
		return -("dm --dev=" .. dev)  -- CMoTech (DebugMonitor)
	     end
          else
            return "Oops! No control port."
          end
        end,
      }
    end
  end,

  ['get-port-parms'] = function(_, path)
    return (C.session and C.session.script) and
    s.newScheme{
      _X = function(path, sch, val)
        local t = {}
        t.ttyDev = usb.port2tty[path[#path-1]]
        t.ttyDevCtl =  usb.port2tty[path[#path-1] .. "-ctl"]
        t.smsType = C.getOption(s.clonePath(path, -1), "smsType")
        return s.ser(t)
      end,
      _R = 'user',
    }
  end,

  ['connect-time'] = function(_, path)
    return (C.session and C.session.script) and
    s.newScheme{
      _H = "hiddenConnectTime",
      _T = 'integer',
      _V = s.integerVal,
      _X = function(path, sch, val)
        local path = s.clonePath(path, -1)
        C.setOption(path, 'connectTime', val)
        return true
      end,
      _R = 'user',
    }
  end,

  ["switch-to-usb-serial-mode"] = function(_, path)
    local enc = C.getConfOrDefault(s.clonePath(path, -1, "encapsulation"))
    local adm = C.getConfOrDefault(s.clonePath(path, -1, "adm-state"))
    local path = s.clonePath(path, -1)  -- Up to port-name
    local wirelessType = C.getOption(path, "wirelessType")
    local device = usb.FindDevice(path[#path])
    local vidpid = device and device.vidpid
    if wirelessType == "cdma" and vidpid == "16d8/680a" and (enc == "none" or adm == "down") then
      return s.newScheme{
        _H = "portSwitchToUsbSerialMode",
        _R = 'user',
        _P = -98,
        _X = function(path, sch, val)
          local dev = usb.port2tty[path[#path -1]]
          if dev then
            C.restartDaemon(s.clonePath(path, -1))  -- kill ppp daemon if it is
            local a = -("at2 --dev=" .. dev .. " newmode")
            if a:find("OK!") then
              usb.port2tty[path[#path -1]] = nil   -- module will have another tty
              usb.port2tty[path[#path -1] .. "-ctl"] = nil -- and
              return "Module mode changed!"
            else
              return (a .. "\nTry again....")
            end
          else
            return "No port"
          end
        end,
        _DEP = 1,
      }
    end
  end,
}

