module(..., package.seeall)
local s = require'scheme'
local ppp = require'nsgpppconf'
local iface = require'nsgifaceconf'
--------------------------
-- common prio (high)
local PPPOECOMMONADMSTATEPRIO = 100 
local PPPOECOMMONDESCRIPTIONPRIO = 95
local PPPOECOMMONMODEPRIO     = 90
local PPPOECOMMONNAMEPRIO     = 80
local PPPOECOMMONIFACEPRIO    = 70
--------------------------
local pppoeDiscoveryScan = 
   function(path, sch, conf, sesNum) 
      local dev =  iface.getIfaceName(C.getConfOrDefault(s.clonePath(path, -2, "iface")) or "eth0")
      local com = {"/usr/sbin/pppoe", "-A", "-I", dev, }

      C.restartDaemon(path, com, {sesNum = sesNum, noTimestamp = true, noRestart = true, }) 
   end


local pppoeDiscovery = s.newScheme {
   _H = "pppoeDiscovery",
   _A = {"\nSearching PPPoE-servers in the LAN segment.\n".. "It takes a few time...\n"},
   _X = pppoeDiscoveryScan,
   _XI = true,
}

local pppoeOneShow = s.newScheme {
   _H = "pppoeOneShow",

   ["log"] =  s.newScheme{
      _H = "pppoeOneShowLog",
      _X = function(path)
	      path = s.clonePath(path, -2)
	      return C.getDaemonLog(path)
	   end,
      _R = 'user',
   },  

   discovery = function(_, path)
		  local mode = C.getConfOrDefault(s.clonePath(path, -2, "mode"))
		  return mode == 'client' and pppoeDiscovery
	       end,
}
--------------------------
local function AddOptions(target, opt, value) 
   if value then
      target[#target + 1] = opt
      target[#target + 1] = value
   end
end

local function restartPppoe(path, sch, conf, prevConf, first)
   ppp.addIpUpDownScripts()
   local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
   local ApCmdStart = {}
   --clone PPP-conf with defaults (only MAIN)
   local pppconf = s.addDefault(path, sch, conf, true) 
   local mainCom = ppp.getPPPParams(path, pppconf, 'main', {})
   local ConfText = table.concat(mainCom, '\n')..'\n'
   local filename = os.tmpname()
   local f = io.open(filename, 'w')
   if f then
      f:write(ConfText)
      f:close()
   end
  

   if conf.mode and conf.mode == "server" then
      ApCmdStart = {'/usr/sbin/pppoe-server', '-s', '-k', '-F',  }

      AddOptions(ApCmdStart, '-I', iface.getIfaceName(conf.iface))
      AddOptions(ApCmdStart, '-C', conf.name)
      AddOptions(ApCmdStart, '-N', conf.limit)
      AddOptions(ApCmdStart, '-L', conf.ppp.main["ip-address"])
      AddOptions(ApCmdStart, '-R', conf.ppp.main["peer-ip-address"])
      AddOptions(ApCmdStart, '-T', conf.idle)
      AddOptions(ApCmdStart, '-O', filename)
   else
      if not conf.name then
	 C.restartDaemon(path) -- kill the daemon (if present)
	 return first or false, "Server NAME is needed to start!" -- No error if first boot
      end
      ApCmdStart = {'/usr/sbin/pppd', 'plugin', '/etc/ppp/plugins/rp-pppoe.so',
	 iface.getIfaceName(conf.iface),
	 'nodetach', 'rp_pppoe_ac', conf.name, 'file', filename }
   end

   if conf["adm-state"] == 'up' then
      C.restartDaemon(path, ApCmdStart, { tmpFileList = {filename}, atExit = C.increasingSleep})
      return true
   end
   C.restartDaemon(path)
   return true
end
--------------------------
local pppoeOne = s.newScheme {

   _Z = restartPppoe,

   _H = "pppoeOne",

   mode = s.newScheme {
      _H = "pppoxOneMode",
      _T = 'choice',
      _A = { "client", "server"},
      _V = s.choiceVal,
      _D = "client",
      _DEP = 1,
      _P = PPPOECOMMONMODEPRIO,
   },

   ["adm-state"] = s.newScheme{
      _T = "choice",
      _A = {"up", "down"},
      _V = s.choiceVal,
      _H = "pppoxOneAdmState",
      _D = "down",
      _P = PPPOECOMMONADMSTATEPRIO,
   },

   description = s.stringType{ _H = "pppoxOneDescription", _P = PPPOECOMMONDESCRIPTIONPRIO},

   iface = s.stringType { _H = "pppoeOneIface", _D = "eth0", _P = PPPOECOMMONIFACEPRIO, },

   name = s.stringType { _H = "pppoeOneName", _P = PPPOECOMMONNAMEPRIO, },

   ppp = function (_, path)
	    return ppp.pppScheme { 
	       show = s.newScheme{
		  _H = "pppShow",
		  _X = function(path)
			  local path = s.clonePath(path, -2)
			  local out = "\n"
			  local unit = C.getOption(path, 'pppUnit')
			  if unit then
			     out = out..'\n'..iface.showInterfaceStat('ppp'..unit)
			  else
			     out = out.."No PPP network interface\n"
			  end
			  return out
		       end,
	       }
	    }
	 end,

   show = pppoeOneShow,

-- PPPoE-server params
   
   
   limit = function(_, path)
	      local mode = C.getConfOrDefault(s.clonePath(path, -1, "mode"))
	      return (mode == "server") and s.integerType { _H = "pppoeOneLimit", _A = {1, 100}, _D = 10, }
	   end,
   
   idle = function(_, path)
	     local mode = C.getConfOrDefault(s.clonePath(path, -1, "mode"))
	     return (mode == "server") and s.integerType { _H = "pppoeOneIdle", _A = {30, 3600}, _D = 600, }
	  end, 

-- PPPoE-client params

}
local function pppoeTunnelVal(val)
   val = (tostring(val)):match("^pppoe(%d+)$") or val
   local n, err = s.integerVal(tonumber(val), 1, 255)
   if n then
      return "pppoe"..tostring(n)
   else
      return nil, err
   end
end

pppoeScheme = s.newScheme {
   _H = "pppoeScheme",

   _NAME = pppoeOne,
   _NAME_TYPE = pppoeTunnelVal,
}
