module(..., package.seeall)
local s = require'scheme'
local ppp = require'nsgpppconf'
--------------------------
-- common prio (high)
local PPTPCOMMONADMSTATEPRIO = 100 
local PPTPCOMMONDESCRIPTIONPRIO = 95
local PPTPCOMMONMODEPRIO     = 90

-- client prio
local PPTPCLIENTDESTINATIONPRIO = 50 
local PPTPCLIENTSOURCEIPPRIO    = 40
local PPTPCLIENTKEEPALIVEPRIO   = 30 
local PPTPCLIENTRETRYPRIO       = 20 


-- server prio
local PPTPSERVERLIMITPRIO           = 40
local PPTPSERVERLISTENPRIO          = 30

--common prio (low)
local PPTPCOMMONPPPPRIO      = 10
local PPTPCOMMONSHOWPRIO     = 5
--------------------------
local pptpOneShow = s.newScheme {
   _H = "pptpOneShow",

   ["log"] =  s.newScheme{
      _H = "pptpOneShowLog",
      _X = function(path)
	      path = s.clonePath(path, -2)
	      return C.getDaemonLog(path)
	   end,
      _R = 'user',
   },  

}
--------------------------
local function  pptpClientAddRoute(oper, conf)
   oper.log:write("++ trying set temporary route to destination (" .. conf.destination .. ")\n")
   local get = -("ip route get " .. conf.destination .. " 2>&1") 
   if string.find(get, "unreachable") then
      oper.log:write("*** Error: Destination unreachable, service can't work ***")
   else
      local ucast = string.match(get, " via.-\n") -- may be Ethernet-like route (via host) 
      local ptp = string.match(get, " dev.-\n")   -- may be pointopoint (only dev)
      local route = ucast or ptp -- the order is essential !!!
      if route then 
	 local set = "ip route add " .. conf.destination .. "/32" .. route
	 local setroute = string.gsub(set, "\n", " 2>&1")
	 local ret = -(setroute)

	 if string.find(ret, "exists") then
	    oper.log:write("++ permanent route already exist. Continue...")
	    oper.extra.permanent = true
	 else
	    oper.log:write("++ temporary route to " .. conf.destination .. " is " .. route)
	 end
      else
	 oper.log:write("*** Error: Can't set route (undefined syntax) ***")
      end
   end
end
local function  pptpClientDelRoute(oper, conf)
   if not oper.extra.permanent then
      oper.extra.permanent = nil
      local ret = -("ip route del " .. conf.destination .. "/32")
      oper.log:write("++ temporary route to " .. conf.destination .. " deleted\n\n")
   end
   C.increasingSleep(oper)
end
--------------------------
local function AddOptions(target, opt, value) 
   if value then
      target[#target + 1] = opt
      target[#target + 1] = value
   end
end

local function restartPptp(path, sch, conf, prevConf, first)
   local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
   local ApCmdStart = {}
   local TmpFileList = {}
   --clone PPP-conf with defaults (only MAIN)
   local pppconf = s.addDefault(path, sch, conf, true) 
   local mainCom = ppp.getPPPParams(path, pppconf, 'main', {})
   local ConfText = table.concat(mainCom, '\n')..'\n'
   local filename = os.tmpname()
   local f = io.open(filename, 'w')
   if f then
      f:write(ConfText)
      f:close()
   end
   TmpFileList[#TmpFileList+1] = filename   --- ppp-parameters file
   
   if conf.mode == "server" then
      local pptpfilename = os.tmpname()
      local f
      if not conf.listen or (conf.listen == "127.0.0.1") then
	 return false, "Local listener address is required!"
      end
      f = io.open(pptpfilename, 'w')
      if f then
	 if conf.ppp.main["ip-address"] then
	    f:write("localip " .. conf.ppp.main["ip-address"] ..'\n')
	 end
	 if conf.ppp.main["peer-ip-address"] then
	    f:write("remoteip " .. conf.ppp.main["peer-ip-address"] ..'-254\n')
	 end
	 f:close()
      end      
      TmpFileList[#TmpFileList+1] = pptpfilename   --- pptpd-parameters file
      
      ApCmdStart = {'/sbin/pptpd', '-f',  }
      AddOptions(ApCmdStart, '-l', conf.listen)
      AddOptions(ApCmdStart, '-C', conf.limit)
      AddOptions(ApCmdStart, '-c', pptpfilename)
      AddOptions(ApCmdStart, '-o', filename)
   else
      if not conf.destination then
	 return first or false, "Destination address needed to start!" -- No error if first boot 
      end
      ApCmdStart = {'/usr/sbin/pptp', '--debug', '--sync', conf.destination, 'nodetach', 'sync', 'file', filename }
      AddOptions(ApCmdStart, '--localbind', conf["source-ip"])
      AddOptions(ApCmdStart, '--idle-wait', conf.keepalive)
      AddOptions(ApCmdStart, '--max-echo-wait', conf.keepalive and conf.retry and (conf.keepalive * conf.retry))
   end
   
   
   if conf["adm-state"] == 'up' then
      C.restartDaemon(path, ApCmdStart, { 
			 tmpFileList = TmpFileList, 
			 atStart = (conf.mode == "client") and 
			    function(oper) 
			       pptpClientAddRoute(oper, conf) 
			    end or nil,
			 atExit = (conf.mode == "client") and 
			    function(oper) 
			       pptpClientDelRoute(oper, conf) 
			    end or C.increasingSleep,
		      })
      return true
   end
   C.restartDaemon(path)
   return true
end

--------------------------
local pptpOne = s.newScheme {
   
   _Z = restartPptp,
   
   _H = "pptpOne",
   
   -- Common Lines 
   
   description = s.stringType{ _H = "pppoxOneDescription", _P = PPTPCOMMONDESCRIPTIONPRIO, },

   mode = s.newScheme {
      _H = "pppoxOneMode",
      _T = 'choice',
      _A = {"client", "server"},
      _V = s.choiceVal,
      _D = "client",
      _DEP = 1,
      _P = PPTPCOMMONMODEPRIO,
   },
   
   ["adm-state"] = s.newScheme{
      _T = "choice",
      _A = {"up", "down"},
      _V = s.choiceVal,
      _H = "pppoxOneAdmState",
      _D = "down",
      _P = PPTPCOMMONADMSTATEPRIO,
   },

   ppp = ppp.pppScheme{_P = PPTPCOMMONPPPPRIO, },

   show = pptpOneShow{_P = PPTPCOMMONSHOWPRIO, },

-- Pptp-server params
   
   listen = function(_, path)
	       local mode = C.getConfOrDefault(s.clonePath(path, -1, "mode"))
	       return (mode == "server") and s.ipAddressType { _H = "pptpOneListen", _P = PPTPSERVERLISTENPRIO, }
	    end,
   
   limit = function(_, path)
	      local mode = C.getConfOrDefault(s.clonePath(path, -1, "mode"))
	      return (mode == "server") and s.integerType { _H = "pptpOneLimit", _A = {1, 100}, _D = 10, _P = PPTPSERVERLIMITPRIO, }
	   end,

-- Pptp-client params

   destination = function(_, path)
		    local mode = C.getConfOrDefault(s.clonePath(path, -1, "mode"))
		    return (mode == "client") and s.ipAddressType { _H = "pptpOneDestination", _P = PPTPCLIENTDESTINATIONPRIO, }
		 end,

   ["source-ip"] = function(_, path)
		      local mode = C.getConfOrDefault(s.clonePath(path, -1, "mode"))
		      return (mode == "client") and s.ipAddressType { _H = "pptpOneSourceIp", _P = PPTPCLIENTSOURCEIPPRIO, }
		   end,

   keepalive = function(_, path)
		  local mode = C.getConfOrDefault(s.clonePath(path, -1, "mode"))
		  return (mode == "client") and s.integerType{
		     _H = "pptpOneKeepalive",
		     _A = {1, 3600},
		     _P = PPTPCLIENTKEEPALIVEPRIO,
		     _DEP = 1,
		  }
	       end,

   retry = function(_, path)
	      local keepalive = C.getConfOrDefault(s.clonePath(path, -1, "keepalive"))
	      local mode = C.getConfOrDefault(s.clonePath(path, -1, "mode"))
	      return (mode == "client") and keepalive and s.integerType{ 
		 _H = "pptpOneRetry", 
		 _A = {1, 100}, 
		 _P = PPTPCLIENTRETRYPRIO,
	      }
	   end,   
}

local function pptpTunnelVal(val)
   val = (tostring(val)):match("^pptp(%d+)$") or val
   local n, err = s.integerVal(tonumber(val), 1, 255)
   if n then
      return "pptp"..tostring(n)
   else
      return nil, err
   end
end

pptpScheme = s.newScheme {
   _H = "pptpScheme",

   _NAME = pptpOne,
   _NAME_TYPE = pptpTunnelVal,
}
