module(..., package.seeall)
local s = require'scheme'
local usb = require'nsgusbconf'

-- priority 
local PRINTERTCPPORTPRIO       = 50
local PRINTERADMSTATEPRIO      = 40
local PRINTERBINDIPPRIO          = 30
local PRINTERBIDIRECTIONALPRIO = 20
local PRINTERINFOPRIO          = 10



local function restartPrinter(path, sch, conf, prevConf)
   local tcpport = conf and conf["tcp-port"]       --save user value (mandatory!)
   local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
   local CmdStart = {'/usr/sbin/p910nd', '-n', '-f', usb.port2tty[path[#path]], }

   if tcpport then
      CmdStart[#CmdStart + 1] = tcpport - 9100
   else
      if conf["adm-state"] == 'up' then
	 return false, "TCP-port is mandatory!"
      end
   end
   if conf.bidirectional then
      CmdStart[#CmdStart + 1] = "-b"
   end
   if conf["bind-ip"] then
      CmdStart[#CmdStart + 1] = "-i"
      CmdStart[#CmdStart + 1] = conf["bind-ip"]
   end

   if conf["adm-state"] == 'up' then
      C.restartDaemon(path, CmdStart, { atExit = C.increasingSleep, })
      return true
   end
   C.restartDaemon(path)
   return true
end
--------------------------
local printerUsedPort = {} -- 
--------------------------
printerScheme = s.newScheme {
   _Z = restartPrinter,

   ["adm-state"] = s.newScheme{
      _T = "choice",
      _A = {"up", "down"},
      _V = s.choiceVal,
      _H = "printerAdmState",
      _D = "down",
      _P = PRINTERADMSTATEPRIO,
   },

   ["tcp-port"] = function(_, path)
		     return s.choiceType{
			_A = {9100, 9101, 9102},
			_V = function (...)
				local val =  s.choiceVal(...)
				local path = s.clonePath(path, -1)
				for i, v in pairs(printerUsedPort) do 
				   if v == val and i ~= path[#path] then
				      return nil, "unused (!) TCP-port. (check port \"" .. i .. "\")" 
				   end
				end
				printerUsedPort[path[#path]] = val
				return s.choiceVal(...)
			     end,
			_H = "printerTcpPort",
			_DEP = 1,
			_P = PRINTERTCPPORTPRIO,
		     }
		  end,

   bidirectional = function(_, path)
		      local tcpport = C.getConfByPath(s.clonePath(path, -1, "tcp-port"))
		      return tcpport and s.booleanType{
			 _H = "printerBidirectional", 
			 _D = true, 
			 _P = PRINTERBIDIRECTIONALPRIO, }
		   end,

   ["bind-ip"] = function(_, path)
	     local tcpport = C.getConfByPath(s.clonePath(path, -1, "tcp-port"))
	     return tcpport and s.ipAddressType{_H = "printerBindIP", _P = PRINTERBINDIPPRIO, }
	  end,

   info = s.newScheme{
      _H = "printerInfo",
      _X = function(path)
	      local path = s.clonePath(path, -1)
	      local type = usb.probeUSBDeviceScheme(nil, path)
	      local out = "Printer:\t" 
	      
	      if type == "printer" then
		 out = out .. "CONNECTED (" .. usb.port2tty[path[#path]]..")\n"
		 local device = usb.FindDevice(path[#path])
		 out = out .. "Manufacturer:\t" .. (device.manufacturer or "UNKNOWN") .. "\n"
		 out = out .. "Model:\t\t" .. (device.product or "UNKNOWN") .. "\n"
	      else
		 out = out .. "NOT CONNECTED\n"
	      end
	      return out
	   end,
      _P = PRINTERINFOPRIO,
   },

}

