module(..., package.seeall)
local s = require'scheme'
local ppp = require'nsgpppconf'
local usb = require'nsgusbconf'
local ser = require'nsgserconf'
local oneWire = require'nsg1wireconf'
local ox = require'posix'

local function getAsyncFormat(portConf)
  return
  (tostring(portConf.databits or "8"))..
  ((portConf.parity or 'N'):sub(1,1):upper())..
  (tostring(portConf.stopbits or "1"))
end

local function restartRawTcpDaemon(path, sch, conf)
  local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
  local rt = conf["raw-tcp"] or {}
  local ttyDev = usb.port2tty[path[#path]] 
  if (not ttyDev) or (ttyDev == '') then 
     ttyDev = usb.probeFixDevices(path) and usb.port2tty[path[#path]] -- lookup "Fixed" port if not present
  end
  local com = {
    "dcdstarter", ttyDev,
    "-s", tostring(conf.baudrate) or "9600",
    "-t", (
      (tostring(conf.databits) or "8")..
      ((conf.parity or 'N'):sub(1,1):upper())..
      (tostring(conf.stopbits) or "1")
    ),
    "-i", rt["ip-address"] or "127.0.0.1",
    "-p", rt["tcp-port"] or 50002,
    "-k", rt["dcd-inquiry-time"] or 1000,
    "-c", rt["reconnect-delay"] or 30,
    "-r", rt["hangup-time"] or 2000,
  }
  if rt["server-mode"] then
    com[#com+1] = "-a"
  end
  local dl = rt["debug-level"]
  if dl > 0 then
    com[#com+1] = "-lstdout"
    if dl > 1 then
      com[#com+1] = "-"..("v"):rep(dl-1)
    end
  end
  C.restartDaemon(path, com, {noTimestamp = true, atExit = C.increasingSleep}) 
end

local function restartReverseTelnet(path, sch, conf)
  C.restartDaemon(path) -- remove previos daemon
  local ttyDev = usb.port2tty[path[#path]]
  os.execute("/usr/bin/fuser -k "..ttyDev) -- kill processes using this tty
  os.execute("/usr/bin/fuser -9 -k "..ttyDev) -- kill processes using this tty with SIGKILL
  local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
  local rt = conf["reverse-telnet"] or {}
  local inetdApplyFunc = C.getOption(path, "inetdApplyFunc")
  if not inetdApplyFunc then
    inetdApplyFunc = (function()
        local reverseTelnetKey = {}
        return ser.getInetdApplyFunc(
          function(inetdConf, conf)
            inetdConf[reverseTelnetKey] = 
            (conf.encapsulation == 'reverse-telnet' and conf["adm-state"] == 'up')
            and (function()
                local ttyDev = usb.port2tty[path[#path]]
		if (not ttyDev) or (ttyDev == '') then 
		   ttyDev = usb.probeFixDevices(path) and usb.port2tty[path[#path]] 
		end
		local nsgcuParms = ("%s  -s %d -t %s -E '%s' --break '%s' --exit '%s'"):
		format(
		  ttyDev, conf.baudrate, getAsyncFormat(conf), 
		  conf["reverse-telnet"]["escape-char"] or '',
		  conf["reverse-telnet"]["escape-break"] or '',
		  conf["reverse-telnet"]["escape-exit"] or ''
		)
		local loginProgName = '/tmp/'..table.concat(path,'-')..
			'-reverse-telnet-login-script'
                local f = io.open(loginProgName, 'w')
                if f then
		  local content
		  if conf["reverse-telnet"]["authentication"] then
		    content = ([[
#!/usr/bin/lua
local nsg = require"nsg"
io.write("Username: "); io.flush()
local name = io.read()
io.write("Password: "); io.flush()
os.execute("stty -echo")
local passw = io.read()
os.execute("stty echo")
local res, err = nsg.pamAuth('reverse-telnet', name, passw)
print()
if res then
  print"You are online"
  os.execute("/bin/nsgcu %s")
else
  print(err)
end
]]                  ):format(nsgcuParms)
		  else
		    content = ([[
#!/bin/sh
/bin/nsgcu %s
]]                  ):format(nsgcuParms)
		  end
		  f:write(content)
                  f:close()
		  ox.chmod(loginProgName, 'a+x')
		end
                return
                ("%d stream tcp nowait root /usr/sbin/telnetd telnetd -l %s"):
                format(conf["reverse-telnet"]["tcp-port"] or 50010, loginProgName)
              end
            )()
            or nil
          end
        )
    end)()
    C.setOption(path, "inetdApplyFunc", inetdApplyFunc)
  end
  inetdApplyFunc(path, sch, conf)
end

local function restartLoginDaemon(path, sch, conf)
  local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
  local ttyDev = usb.port2tty[path[#path]] 
  if (not ttyDev) or (ttyDev == '') then 
     ttyDev = usb.probeFixDevices(path) and usb.port2tty[path[#path]] -- lookup "Fixed" port if not present
  end
  local com = {
    "sh", "-c",
    "/sbin/getty -L "..(tostring(conf.baudrate) or "9600")..' '..ttyDev,
  }
  C.restartDaemon(
    path, com, 
    {
      atExit = C.increasingSleep,
      atKill = function(oper)
	oper.daemon:kill(9)
      end,
    }
    ) 
  end


local function restartAsyncPort(path, sch, conf, prevConf)
  if not s.confCmp(conf, prevConf) then
    conf = s.addDefault(path, sch, conf, true)
    C.restartDaemon(path)
    restartReverseTelnet(path, sch, conf)
    if conf and conf["adm-state"] ~= 'down' then
      local encap = conf.encapsulation
      if encap == "raw-tcp" then
        restartRawTcpDaemon(path, sch, conf)
        return true
      elseif encap == 'ppp' then
        ppp.restartPPPDaemon(path, sch, conf)
        return true
      elseif encap == 'login' then
       	restartLoginDaemon(path, sch, conf)
      end
    end
  end
  return true
end

local rawTcpScheme = s.newScheme{
  _H = "rs232Rawtcp",
  ["dcd-inquiry-time"] = s.integerType{
    _H = "rs232RawtcpDcdinquirytime",
    _A = {0, 60000},
    _D = 1000,
  }, 
  ["debug-level"] = s.integerType{
    _H = "rs232RawtcpDebuglevel",
    _A = {0, 4},
    _D = 1,
  }, 
  ["hangup-time"] = s.integerType{
    _H = "rs232RawtcpHanguptime",
    _A = {-1, 60000},
    _D = 2000,
  }, 
  ["ip-address"] = s.ipAddressType{
    _H = "rs232RawtcpIpaddress",
    _D = "127.0.0.1"
  },
  ["reconnect-delay"]  = s.integerType{
    _H = "rs232RawtcpReconnectdelay",
    _A = {1, 86400},
    _D = 30,
  }, 
  ["server-mode"] = s.booleanType{_H = "rs232RawtcpServer"},
  ["tcp-port"] = s.tcpPortType{
    _H = "rs232RawtcpTcpport",
    _D = 50002,
  },
}

local escapeCharScheme = s.newScheme{
  _T = "string",
  _V = function(val)
    if (
      type(val) == 'string' and 
      ( #val <= 1 or 
        ( #val == 2 and val:sub(1,1) == '^' and
          val:byte(2) >= ('@'):byte() and val:byte(2) <= ('_'):byte()
        )
      )
    ) then return val end
    return nil, "Empty string or any single character or ^@ ^A ... ^Z ^[ ^\\ ^] ^^ ^_ (max 2 characters)"
  end,
}

local reverseTelnetScheme = s.newScheme{
  _H = "rs232Reversetelnet",
  ["authentication"] = s.booleanType{
    _H = "rs232ReversetelnetAuthentication"
  },
  ["tcp-port"] = s.tcpPortType{
    _H = "rs232ReversetelnetTcpport",
    _D = 50010,
  },
  ["escape-break"] = escapeCharScheme{
    _H = "rs232ReversetelnetEscapebreak",
  },
  ["escape-char"] = escapeCharScheme{
    _H = "rs232ReversetelnetEscapechar",
  },
  ["escape-terminate"] = escapeCharScheme{
    _H = "rs232ReversetelnetEscapeterminate",
  },
}

local baudrateScheme = s.newScheme{
  _T = 'choice',
  _A = {
    --50,
    --75,
    --110,
    --134,
    --150,
    --200,
    300,
    600,
    1200,
    --1800,
    2400,
    4800,
    9600,
    19200,
    38400,
    57600,
    115200,
    230400,
  },
  _V = s.choiceVal,
  _H = "rs232Baudrate",
  _D = 9600,
}

local startRawAccess = function(path, sch, conf, sesNum)
  local ppath = s.clonePath(path, -2)  --- parent port path 
  local ttyDev = usb.port2tty[ppath[#ppath]]
  if (not ttyDev) or (ttyDev == '') then 
     ttyDev = usb.probeFixDevices(ppath, true) and usb.port2tty[ppath[#ppath]] 
  end

  C.restartDaemon(ppath) -- kill parent port daemon (i.e. pppd)

  local portConf = C.getConfWithDefault(ppath)
  C.restartDaemon(
    path, {'nsgcu', ttyDev, 
      "-s", tostring(portConf.baudrate) or "9600",
      "-t", getAsyncFormat(portConf),
    },
    {
      sesNum = sesNum, noTimestamp = true, noRestart = true,
    }
  ) 
end

-- physycal param  priority
local RS232ADMSTATEPRIO    = 30
local RS232ENCAPSULPRIO    = 25
local RS232BAUDRATEPRIO    = 20
local RS232DATABITSPRIO    = 19
local RS232PARITYPRIO      = 18
local RS232STOPBITSPRIO    = 17
local RS232RAWACCESSPRIO   = 10

local rawAccessScheme = s.newScheme{
  _H = "portRawAccessNode",
  _P = RS232RAWACCESSPRIO,
  launch = s.newScheme{
    _H = "portRawAccess",
    _X = startRawAccess,
    _XI = true,
  },
}


rs232Scheme = s.newScheme{
  _H = "rs232",
  _Z = restartAsyncPort,

  ["adm-state"] = s.newScheme{
    _T = "choice",
    _A = {"up", "down"},
    _V = s.choiceVal,
    _H = "rs232AdmState",
    _D = "up",
    _DEP = 1,
    _P = RS232ADMSTATEPRIO,
  },
  encapsulation = s.newScheme{
    _T = "choice",
    _A = {"login", "ppp", "raw-tcp", "reverse-telnet", "1-wire", "none"},
    _V = s.choiceVal,
    _H = "rs232Encapsulation",
    _D = "none",
    _DEP = 1,
    _P = RS232ENCAPSULPRIO,
  },
  baudrate = function(_, path)
    local enc = C.getConfOrDefault(s.clonePath(path, -1, "encapsulation"))
    return enc ~= '1-wire' and 
    baudrateScheme{_P = RS232BAUDRATEPRIO, }
  end,
  databits = function(_, path)
    local enc = C.getConfOrDefault(s.clonePath(path, -1, "encapsulation"))
    return enc ~= 'login' and enc ~= 'ppp' and enc ~= '1-wire' and s.newScheme{
      _T = 'choice',
      _A = {
        5,
        6,
        7,
        8,
      },
      _V = s.choiceVal,
      _H = "rs232Databits",
      _D = 8,
      _P = RS232DATABITSPRIO,
    }
  end,
  stopbits = function(_, path)
    local enc = C.getConfOrDefault(s.clonePath(path, -1, "encapsulation"))
    return enc ~= 'login' and enc ~= 'ppp' and enc ~= '1-wire' and s.newScheme{
      _T = 'choice',
      _A = {
        1,
        2,
      },
      _V = s.choiceVal,
      _H = "rs232Stopbits",
      _D = 1,
      _P = RS232STOPBITSPRIO,
    }
  end,
  parity = function(_, path)
    local enc = C.getConfOrDefault(s.clonePath(path, -1, "encapsulation"))
    return enc ~= 'login' and enc ~= 'ppp' and enc ~= '1-wire' and s.newScheme{
      _T = 'choice',
      _A = {
        "none",
        "even",
        "odd",
      },
      _V = s.choiceVal,
      _H = "rs232Parity",
      _D = "none",
      _P = RS232PARITYPRIO,
    }
  end,

  ["raw-tcp"] = function(_, path)
    local enc = C.getConfOrDefault(s.clonePath(path, -1, "encapsulation"))
    return enc == 'raw-tcp' and rawTcpScheme{_P = -90}
  end,

  ["reverse-telnet"] = function(_, path)
    local enc = C.getConfOrDefault(s.clonePath(path, -1, "encapsulation"))
    return enc == 'reverse-telnet' and reverseTelnetScheme{_P = -90}
  end,

  ppp = function(_, path)
    local enc = C.getConfOrDefault(s.clonePath(path, -1, "encapsulation"))
    return enc == 'ppp' and ppp.pppScheme{_P = -90}
  end,

  ["1-wire"] = function(_, path)
    local enc = C.getConfOrDefault(s.clonePath(path, -1, "encapsulation"))
    --usb.probeUSBDeviceScheme(nil, s.clonePath(path, -1))
    oneWire.setDevName(path[#path-1])
    return enc == '1-wire' and oneWire.oneWireScheme{_P = -90}
  end,

  ["log"] =  s.newScheme{
    _H = "rs232Log",
    _X = function(path)
      path = s.clonePath(path, -1)
      return C.getDaemonLog(path)
    end,
    _R = 'user',
    _P = -100,
  },

  ["raw-access"] = function(_, path)
    local enc = C.getConfOrDefault(s.clonePath(path, -1, "encapsulation"))
    local adm = C.getConfOrDefault(s.clonePath(path, -1, "adm-state"))
    return (enc == 'none' or adm == 'down') and rawAccessScheme
  end,
}


do -- private for asyncConsoleScheme
--
local function restartAsyncConsolePort(path, sch, conf, prevConf)
  if not conf then
    conf = {} -- avoid nil config for startup applying
  end
  for _, v in ipairs{"encapsulation", "baudrate",} do
    -- constants are defaults forever
    conf[v] = nil
    C.setConfigValue(s.clonePath(path, nil, v), nil)
  end
  return restartAsyncPort(path, sch, conf, prevConf) -- apply config
end

asyncConsoleScheme = s.newScheme{
  _H = "asyncConsole",
  _Z = restartAsyncConsolePort,

  ["adm-state"] = s.newScheme{
    _T = "choice",
    _A = {"up", "down"},
    _H = "asyncConsoleAdmState",
    _D = "up",
    _P = RS232ADMSTATEPRIO,
  },
  encapsulation = s.constantType{
    _A = {"login",},
    _D = "login",
    _H = "asyncConsoleEncapsulation",
    _P = RS232ENCAPSULPRIO,
  },
  baudrate = function(_, path)
    local dev = usb.port2tty[path[#path - 1]] or ""
    local dflt = tonumber(BI.kcmdline:match("console=" .. (dev:match("^/dev/(.*)") or dev) .. ",(%d*)")) or 115200
    return s.constantType{
      _A = {dflt,},
      _D = dflt,
      _H = "asyncConsoleBaudrate",
      _P = RS232BAUDRATEPRIO,
    }
  end,

  ["log"] =  s.newScheme{
    _H = "asyncConsoleLog",
    _X = function(path)
      path = s.clonePath(path, -1)
      return C.getDaemonLog(path)
    end,
    _R = "user",
    _P = -100,
  },
}
--
end -- private for asyncConsoleScheme
