module(..., package.seeall)
local s = require'scheme'
local clonePath = s.clonePath

local family = {
   ip = "-4",
   ipv6 = "-6",
}

local ipRuleNodes = {
   ip = {},
   ipv6 = {}
}

local removeOldNodes =
   function(family)
      local conf = C.getConfByPath({family, "rule"}) or {}
      for k,v in pairs(ipRuleNodes[family] or {}) do
         if type(tonumber(k)) == "number" and  not conf[k] then
            os.execute("ip tule del priority "..tostring(k))
         end
      end
   end

ruleNScheme = s.newScheme{
   _Z =  function(path, sch, conf, prevConf)
            ipRuleNodes[path[1]][path[#path]] = true
            local cmd = {"ip"}
            cmd[#cmd + 1] = family[path[1]]
            cmd[#cmd + 1] = "rule add"
            conf = conf or {}
            cmd[#cmd + 1] = "priority "..path[#path]
            cmd[#cmd + 1] = conf.from and "from "..conf.from
            cmd[#cmd + 1] = conf.to and "to "..conf.to
            cmd[#cmd + 1] = conf.iif and "iif "..conf.iif
            cmd[#cmd + 1] = conf.tos and "tos "..conf.tos
            cmd[#cmd + 1] = conf.fwmark and "fwmark "..conf.fwmark
            cmd[#cmd + 1] = "type "..(conf.type or "unicast")

            if conf.type or 'unicast' == 'unicast' then
               cmd[#cmd + 1] = "table "..tostring(conf.table or 'main')
            end

            C.log_debug("ip "..family[path[1]].." rule  del priority "..path[#path])
            C.log_debug(s.readpopen("ip "..family[path[1]].." rule  del priority "..path[#path]))
            C.log_debug( table.concat(cmd," "))
            C.log_debug( s.readpopen(table.concat(cmd," ")))
         --   os.execute("ip rule flush")
            return true
         end,
   _H = "ipRuleN",
   type = s.choiceType{
      _P = -10,
      _DEP = 1,
      _H = "ipRuleNType",
      _A = {"unicast", "blackhole", "unreachable", "prohibit"},
      _D = "unicast",
   },
   from = s.ipPrefixType{
      _H = "ipRuleNFrom",
   },
   to = s.ipPrefixType{
      _H = "ipRuleNTo",
   },
   iif =  s.stringType{
      _H = "ipRuleNIif",
   },
   tos  =  s.integerType{
      _H = "ipRuleNTos",
   },
   fwmark =  s.integerType{
      _H = "ipRuleNFwmark",
   },
   table =  function (_, path)
               return (C.getConfByPath(s.clonePath(path, -1, "type")) or "unicast")=='unicast' and s.variantType{
                  _H = "ipRuleNTable",
                  _P = -20,
                  _D = 'main',
                  _A = {s.integerType{
                           _A ={1, 4294967295}
                        },
                        s.choiceType{
                           _A = {'local', 'main', 'default'}
                        }
                     }
               }
               or nil
         end
}


ruleScheme = function(_, path)
        local s = s.newScheme{
           _Z = function(path, sch, conf, prevConf)
                   removeOldNodes(path[1])
                   return true
                end,
           _H = "ipRule",
           _NAME = ruleNScheme,
           _NAME_TYPE = s.integerVal,
           _A  = {0,65535},
           [0] = ruleNScheme{_Z = false},
           [32766] = ruleNScheme{_Z = false},
           show = s.newScheme{
              _H = "UpnpShowLog",
              _X = function(path)
                      return s.readpopen("ip "..family[path[1]].." rule show")
                   end
           }
        }
        if path[1]== "ip" then
           s = s{[32767] = ruleNScheme}
        end
        return s
     end