module(..., package.seeall)
local s = require'scheme'
local usb = require'nsgusbconf'
local evt = require'nsgeventconf'
local nsg = require'nsg'
nsg.meta()

local function asIs(...)
  return ...
end

local ms6Sensors = setmetatable(
  {
    TEMP = {1, asIs},
    HUMI = {2, asIs},
    LUMI = {3, asIs},
    PYRO = {4, asIs},
    MAGN = {5, asIs},
    ACCL = {6, asIs},
  },
  {__index = function() return {0, asIs} end }
)


local function ms6show(dev)
  os.execute("stty -F %s raw -echo" % dev)
  local ms, err = io.open(dev, 'r')
  if ms then
    local res = {'','','','','',''}
    for i = 1, 6 do
      local str = ms:read()
      local sens, val = str:match(":NFC;([%w_]*);(.*)\r") 

      if sens then
        res[ms6Sensors[sens][1]] = sens..': '..ms6Sensors[sens][2](val)
      end
    end
    ms:close()
    return '\n'..table.concat(res, '\n')..'\n'
  else
    return err
  end
end

local scriptPattern = [[
local prefix = %q
local dev = %q
os.execute(("stty -F %%s raw -echo >/dev/null 2>&1"):format(dev))
local ms, err = io.open(dev, 'r')
if ms then
  while true do
    local str = ms:read()
    if not str then
      ms:close()
      return
    end
    local sens, val = str:match(":NFC;([%%w_]*);(.*)\r") 

    if sens then
      if sens == "ACCL" then
        --
      else
        local s = sensor[sens]
        if s then
          local state = s.f(val)
	  if state ~= s.state then
	    print(s.snam or prefix..sens, state)
	    s.state = state
	  end
        end
      end
    end
  end
end
]]

local function createScriptEventTable(name, conf)
  conf = conf or {}
  local sname = conf["sensor-name"]
  local scr = "%s = {snam=%s,f = function(val)local v=tonumber(val)\n"%
    {name, (sname and sname~='') and '%q'%tostring(sname) or 'nil'}
  for _, v in ipairs(conf) do
    local retstate = v['event-name']
    local num
    retstate, num = retstate:gsub('%%', '%%%%'):gsub("%$%(VAL%)", "%%s")
    retstate = ("%q%s"):format(retstate, (",tostring(val)"):rep(num))

    scr = scr..(
      'if v>=%s and v<=%s then return string.format(%s) end\n' % 
      {tostring(tonumber(v.from) or "-math.huge"), 
      tostring(tonumber(v.to) or "math.huge"), 
      retstate}
    )
  end
  scr = scr..('return "UNKNOWN" end},\n' % name)
  return scr
end

local function getStates(conf)
  local t = {}
  for _, v in ipairs(conf or {}) do
      t[#t+1] = v['event-name']
  end
  return t
end

local function eventGeneratorApply(path, sch, conf, prevConf)
  local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
  if not conf.enable then
    C.restartDaemon(path)
    evt.unregisterVirtSensor(conf.prefix..'TEMP')
    evt.unregisterVirtSensor(conf.prefix..'HUMI')
    return true
  end

  local scriptData = 'local sensor = {\n'
  scriptData = scriptData..createScriptEventTable('TEMP', conf['TEMP-states'])
  scriptData = scriptData..createScriptEventTable('HUMI', conf['HUMI-states'])
  scriptData = scriptData..'}\n'

  usb.probeUSBDeviceScheme(nil, s.clonePath(path, - 1))
  local script = scriptPattern:format(conf.prefix, usb.port2tty[path[#path - 1]])
  script = scriptData..script
  local com = {
    "lua", "-e", script,
  }
  evt.restartVirtSensor(path, com)
  --evt.registerVirtSensor(conf.prefix..'TEMP', getStates(conf['TEMP-states']))
  --evt.registerVirtSensor(conf.prefix..'HUMI', getStates(conf['HUMI-states']))
  return true
end


local eventListScheme = s.newScheme{
  _H = "ms6EventgeneratorEventlist",
  from = s.newScheme{
    _H = "ms6EventgeneratorEventlistFrom",
    _T = "number",
    _V = s.isNumber,
    _P = 10,
  },
  to = s.newScheme{
    _H = "ms6EventgeneratorEventlistTo",
    _T = "number",
    _V = s.isNumber,
    _P = 5,
  },
  ["event-name"] = s.stringType{
    _H = "ms6EventgeneratorEventlistName",
    _D = "$(VAL)"
  },
}
eventgeneratorNumScheme = s.newScheme{
  _H = "eventgeneratorNumEvents",
  ["sensor-name"] = s.stringType{_H = "EventgeneratorSensorname"},
  _LIST = eventListScheme,
}

ms6Scheme = s.newScheme{
  show = s.newScheme{
    _H = "ms6Show",
    _X = function(path, sch, conf)
      local enableEvent = C.getConfOrDefault(s.clonePath(path, -1,
        "event-generator", "enable")
      )
      if enableEvent then
        C.setConfigValue(s.clonePath(path, -1, "event-generator", "enable", false))
        C.applyConf(s.clonePath(path, -1, "event-generator"))
      end
      usb.probeUSBDeviceScheme(nil, s.clonePath(path, -1))
      local res = ms6show(usb.port2tty[path[#path - 1]])
      if enableEvent then
        C.setConfigValue(s.clonePath(path, -1, "event-generator", "enable", true))
        C.applyConf(s.clonePath(path, -1, "event-generator"))
      end
      return res
    end,
    _R = 'user',
  },
  ["event-generator"] = s.newScheme{
    _H = "ms6Eventgenerator",
    _Z = eventGeneratorApply,

    enable = s.booleanType{
      _H = "ms6EventgeneratorEnable",
      _D = false,
      _P = 100
    },
    prefix = s.stringType{
      _H = "ms6EventgeneratorPrefix",
      _D = "ms6.",
    },
    ["TEMP-states"] = eventgeneratorNumScheme{
      _H = "ms6EventgeneratorTempevents",
    },
    ["HUMI-states"] = eventgeneratorNumScheme{
      _H = "ms6EventgeneratorHumievents",
    },

  },
}
