module(..., package.seeall)
local s = require'scheme'
local destTable = {["output"] = "o", ["error"] = "e", ["syslog"] = "s", ["file"] = "f", } 
----------------------------------------
local SNMPADMSTATEPRIO       = 100
local SNMPSYSTEMINFOPRIO     =  90
local SNMPCOMMUNITYMODELPRIO =  80
local SNMPUSMMODELPRIO       =  70
local SNMPLOGPRIO            =  60
----------------------------------------

local function insConfLine(target, prefix, object, suffix)
   if object and string.len(object) > 0 then
      target[#target+1] = prefix .. object .. suffix
   else
      target[#target+1] = "# Empty field! >>" .. prefix 
   end
end

local function mibVal(val)
   if type(val) == 'string' then
      local mib  = string.match(val, "^[%d%.]+$") --- only digit and dots
      if mib and not string.match(val, '%.%.') then --- NOT multiple dots !!
	 return val
      end
   end
   return nil, "OID path value (i.e. 1.3.6.1.2.44)"
end
---
local function modelVal(val)
   if type(val) == 'string' then
      if val == "v1" or val == "v2c" or val == "usm" then
	 return val
      end
   end
   return nil, "must be on of these (v1, v2c, usm)"
end
---
local function snmpCreateConfFile(conf, fname)
   local snmpConf = { "####### snmpd configuration file ########\n", }
   local f = io.open(fname, 'w')

   if not (f and conf) then
      print("ERROR: SNMP can't create temporary configuration file. Stoped.")
      return false
   end 

-- systems group 
   if conf["system-info"] then
      insConfLine(snmpConf, "syslocation ", conf["system-info"].location, "")
      insConfLine(snmpConf, "syscontact ", conf["system-info"].contact, "")
   end
   -- communities section (only v1)
   snmpConf[#snmpConf + 1] = "\n### Community section (only v1/v2c) ###\n"

   if conf["community-model"] then
      for i, v in pairs(conf["community-model"].community or {}) do
	 snmpConf[#snmpConf + 1] = "\n#community \"" .. i .. "\""
	 if  v.managers then
	    for _, c in pairs (v.managers or {}) do
	       snmpConf[#snmpConf + 1] =  ("com2sec %s %s %s"):format(i, c, i)
	    end
	 else
	    snmpConf[#snmpConf + 1] =  ("com2sec %s default %s"):format(i, i)
	 end
	 local tmpstr = "_" .. ((v.view and i) or "all_") 
	 snmpConf[#snmpConf + 1] = ("group %s v1 %s"):format(i, i)
	 snmpConf[#snmpConf + 1] = ("group %s v2c %s"):format(i, i)
	 snmpConf[#snmpConf + 1] = ("view %s included %s"):format(tmpstr, (v.view or ".1"))
	 snmpConf[#snmpConf + 1] = ("access %s \"\" any noauth exact %s %s _none_"):
	 format(i, tmpstr, ((v.access == "read-write" and tmpstr) or "_none_"))
      end
   end
   if conf["usm-model"] then
      -- view (SNMPv3)
      snmpConf[#snmpConf + 1] = "\n# ACCESS CONTROL (SNMPv3)\n"
      for i, v in pairs(conf["usm-model"].view or {}) do
	  snmpConf[#snmpConf + 1] = "\n### view \"" .. i .. "\" ###"
	  for oid, access in pairs(v["oid-path"]) do
	    snmpConf[#snmpConf + 1] = "view " .. i .. ' ' .. access .. ' ' .. oid   
	 end
      end
      
      -- group 
      snmpConf[#snmpConf + 1] = "\n# GROUPs and VIEWs (SNMPv3)\n"
      for gname, gvalue in pairs(conf["usm-model"].group or {}) do
	 for gmodel, v in pairs(gvalue.model) do
	    snmpConf[#snmpConf + 1] = ("access %s \"\" %s %s exact %s %s %s"):
	    format(gname,  gmodel, 
		   v.level or "noauth",
		   v.read or "none", 
		   v.write or "none",
		   v.notify or "none")
	 end
      end
      for i, v in pairs(conf["usm-model"].user or {}) do
	 snmpConf[#snmpConf + 1] = ("group %s %s %s"):format(v.group, v.model, i)
      end
      -- user 
      snmpConf[#snmpConf + 1] = "\n# USERs\n"
      for i, v in pairs(conf["usm-model"].user or {}) do
	 if v.model == "usm" then 
	    local user  = ("createUser %s "):format(i, i)
	    if v.auth and v.auth ~= "none" then
	       user = user .. string.upper(v.auth) .. ' ' .. v.authpass .. ' '
	    end
	    if v.priv and v.priv ~= "none" then
	       user = user .. string.upper(v.priv) .. ' ' .. v.privpass .. ' '
	    end
	    snmpConf[#snmpConf + 1] = user	 
	 else
	    snmpConf[#snmpConf + 1] = ("com2sec %s default %s"):format(i, i)
	 end
      end
   end

   local snmpConfText = table.concat(snmpConf, '\n')..'\n'   
   f:write(snmpConfText)
   f:close()
   return true
end

local function restartSnmp(path, sch, conf, prevConf)
   local snmpCmdStart = {}
   local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
   local fname = os.tmpname()
   local logfname = os.tmpname()

   if snmpCreateConfFile(conf, fname) then

      snmpCmdStart = { '/usr/sbin/snmpd', '-f', '-c', fname, '-L', }

       snmpCmdStart[#snmpCmdStart] = 
	  snmpCmdStart[#snmpCmdStart] .. destTable[conf.log.destination]

      if conf.log.destination == "file" then
	 snmpCmdStart[#snmpCmdStart+1] = conf.log.file or logfile
      end

      if conf.log.hexdebug then
	 snmpCmdStart[#snmpCmdStart+1] =  "-d"
      end
      if conf.log.tokens then
	 snmpCmdStart[#snmpCmdStart+1] =  "-D" .. conf.log.tokens
      end

      if conf["adm-state"] ~= 'down'then
	 C.restartDaemon(path, snmpCmdStart, 
			 {atExit = C.increasingSleep, tmpFileList = {fname, logfname},})
	 return true
      end
   end
   C.restartDaemon(path)
   return true
end
----------------------------------------
snmpSysteminfo = s.newScheme {
   _H = "snmpSysteminfo",
   _P = SNMPSYSTEMINFOPRIO,

   location = s.stringType {
      _H = "snmpSysteminfoLocation",
      _D = "",
   },
   contact = s.stringType {
      _H = "snmpSysteminfoContact",
      _D = "",
   },
   engineID = s.stringType{
      _H = "snmpSysteminfoEngineID",
      _D = "",
   },
}
----------------------------------------

snmpScheme = s.newScheme {
   _H = "snmp",
   _Z = restartSnmp,

   ["adm-state"] = s.newScheme{
      _T = "choice",
      _A = {"up", "down"},
      _V = s.choiceVal,
      _H = "snmpAdmState",
      _D = "down",
      _P = SNMPADMSTATEPRIO,
   },
   
  log = s.newScheme {
     _H = "snmpLog",
     _P = SNMPLOGPRIO,
     _N = function(sch, conf)
	     local need = {
		true,
		hexdebug = true,
		destination = true,
		show = true,
		tokens = true,
	     }
	     if conf and conf.destination == "file" then
		need["file"] = true
	     else
		need["file"] = false
	     end
	     return need
	  end,

     hexdebug = s.booleanType{_H = "snmpLogHexDebug", },

     destination =  s.newScheme{
	_H = "snmpLogDestination",
	_T = "choice",
	_A = {
	   {"output"},
	   {"error"},
	   {"syslog"},
	   {"file"},
	},
	_V = s.choiceVal,
	_DEP = 1,
	_D = "output",
     },

     file = s.stringType{_H = "snmpLogFile", },

     show =  s.newScheme{
	_H = "snmpLogShow",
	_X = function(path)
		path = s.clonePath(path, -2)
		return C.getDaemonLog(path)
	     end,
     },
     tokens = s.stringType{_H = "snmpLogTokens", },
  },

  ["system-info"] = snmpSysteminfo,

  ["community-model"] = s.newScheme {
     _H = "snmpCommunityModel",
     _P = SNMPCOMMUNITYMODELPRIO,
     community = s.newScheme{
	_H = "snmpCommunity",
	_NAME = s.newScheme{
	   _H = "snmpCommunityName",
	   managers = s.newScheme { 
	      _H = "snmpCommunityManagers",
	      _LIST = s.ipPrefixType{
		 _H = "snmpCommunityManagersPrefix", 
		 _D = "127.0.0.1/32", },
	   },
	   
	   access = s.newScheme{
	      _T = "choice",
	      _A = {"read-only", "read-write"},
	      _V = s.choiceVal,
	      _H = "snmpCommunityAccess",
	      _D = "read-only",
	   },
	   
	   view =  s.stringType{_H = "snmpCommunityView", },
	},
     },
  },
  ["usm-model"] = s.newScheme {
     --- view (v1, v2c, v3)
     _H = "snmpUsmModel",
     _P = SNMPUSMMODELPRIO,
     view = s.newScheme{
	_H = "snmpView",
	_NAME = s.newScheme{
	   _H = "snmpViewName",
	   ["oid-path"] = s.newScheme{
	      _H = "snmpViewNameOidPath",
	      _NAME = s.newScheme{
		 _H = "snmpViewNameOidPathAccess",
		 _T = "choice",
		 _A = {"included", "excluded"},
		 _V = s.choiceVal,
		 _D = "included",
	      },
	      _NAME_TYPE = mibVal,
	   },
	},
     }, 
     
     --- groups (v1, v2c, v3)
     group = s.newScheme{
	_H = "snmpGroup",
	_NAME = s.newScheme{
	   _H = "snmpGroupName",
	   model = s.newScheme{
	      _H = "snmpGroupModel",
	      _NAME = function(_, path)
			 if path[#path] == "usm"  then
			    return s.newScheme {
			       level = s.newScheme{
				  _H = "snmpGroupLevel",
				  _T = "choice",
				  _A = {
				     {"auth", "group using the authNoPriv Security Level"},
				     {"priv", "group using SNMPv3 authPriv security level"},
				     {"noauth", "group using the noAuthNoPriv Security Level"},
				  },
				  _V = s.choiceVal,
				  _D = "noauth",
				  _P = 15,
			       }, 
			       -- look priority _P below
			       read = s.stringType{_H = "snmpGroupRead", _D = "none", _P = 12, },
			       write = s.stringType{_H = "snmpGroupWrite", _D = "none", _P = 11, },
			       notify = s.stringType{_H = "snmpGroupNotify", _D = "none", _P = 10, },
			    }
			 else
			    return  s.newScheme {}
			 end
		      end,
	      _NAME_TYPE = modelVal,
--	      _DEP = 1,
	   },
	},
     },
     
     --- users (v1, v2c, v3)
     user = s.newScheme{
	_H = "snmpUser",
	_NAME = s.newScheme{
	   _H = "snmpUserName",
	   _N = function(sch, conf)
		   local need = {
		      true,
		      group = true,
		      model = true,
		      auth      = false,
		      authpass  = false,
		      priv      = false,
		      privpass  = false,
		   }
		   if conf and conf.model and conf.model == "usm" then
		      need["auth"] = true
		      if conf.auth and conf.auth ~= "none" then  
			 need["authpass"] = true
			 need["priv"] = true
			 if conf.priv and conf.priv ~= "none" then 
			    need["privpass"] = true
			 end
		      end
		   end
		   return need
		end,
	   
	   group = s.stringType{_H = "snmpUserGroup", _D = "noname", _P = 40, },
	   
	   model = s.newScheme{
	      _H = "snmpUserModel",
	      _T = "choice",
	      _A = {
		 {"v1",  "using the v1 security model"},
		 {"v2c", "using the v2c security model"},
		 {"usm", "using the User Security Model (SNMPv3)"},
	      },
	      _V = s.choiceVal,
	      _D = "v1",
	      _DEP = 1,
	      _P = 30,
	   },
	   auth = s.newScheme{
	      _T = "choice",
	      _A = {"none", "md5", "sha"},
	      _D = "none",
	      _V = s.choiceVal,
	      _H = "snmpUserAuth",
	      _DEP = 1,
	      _P = 20,
	   },
	   
	   authpass = s.stringType{_H = "snmpUserAuthpass", _D = "", _P = 18, },
	   
	   priv = s.newScheme{
	      _T = "choice",
	      _A = {"none", "des", "aes"},
	      _D = "none",
	      _V = s.choiceVal,
	      _H = "snmpUserPriv",
	      _P = 16,
	      _DEP = 1,
	   },
	   privpass = s.stringType{_H = "snmpUserPrivpass", _D = "", _P = 14, },
	   
	},
     },  
  },
}

