module(..., package.seeall)
local s = require'scheme'

softwareScheme = s.newScheme{
  _H = "soft",
}

local function getParent(path, sesNum)
  local path = s.clonePath(path, -1)
  local sch = C.getSchemeByPath(path)
  local conf = C.getConfByPath(path) or {}
  local locConf = C.getConfByPath(path, sesNum) or {}
  for k, v in pairs(locConf) do
    conf[k] = v
  end
  conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
  return sch, conf
end

--[[
local nsgupdateOptionsMap = {
  protocol = "--proto=",
  user = "--user=",
  password = "--passwd=",
  server = "--server=",
  port = "--port=",
  file = "--file=",
}
--]]

local function launchUpdate(path, sch, val, sesNum)
  local sch, conf = getParent(path, sesNum)
  local com = {"nsgupdate", "-i"}
  if  (path[#path]=="launch") then
     --FULL firmware update needed
     com[#com+1] = "-f"
  end
  if conf.url then
    com[#com+1] = conf.url
    --[[  else
    for par,val in pairs(conf) do
      local option = nsgupdateOptionsMap[par]
      if option then
        com[#com+1] = option..val
      end
  end
--]]
  end
  C.restartDaemon(path, com,
    {sesNum = sesNum, noTimestamp = true, noRestart = true}) 
end

softwareScheme["service-pack"] = s.newScheme{
  _H = "softServicePack",
  install = s.newScheme{
    _T = "string",
    _H = 'softServicePackInstall',
    _V = s.areyousureVal,
    _X = launchUpdate,
    _A = {"Service pack will be installed.\n"},
    _EX = { subtype = 'areyousure'},
    _XI = true,
    _P = 0,
  },
  url = s.stringType{
    _H = "softServicePackUrl",
    _P= 10,
  },
}

softwareScheme["full-update"] = s.newScheme{
  _H = "softUpdate",
  launch = s.newScheme{
    _T = "string",
    _H = 'softUpdateLaunch',
    _V = s.areyousureVal,
    _X = launchUpdate,
    _A = {"Full system update will be performed.\n"},
    _EX = { subtype = 'areyousure'},
    _XI = true,
    _P = 0,
  },
  url = s.stringType{
    _H = "softUpdateUrl",
    _P= 10,
  },
}

-- check update
-- _H = "updMsg0"
-- _H = "updMsg1"
-- _H = "updMsg2"
-- _H = "updMsg2no"
-- _H = "updConclRequired"
-- _H = "updConclDowngrade"
-- _H = "updConclRecommended"
-- _H = "updConclPossible"
-- _H = "updConclFail


local checkUpdateMessage
local function checkUpdateLaunch(path, sch, conf, sesNum)
  local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
  local myLog = ""
  C.restartDaemon(
    path, 
    {"wget", conf.url, "-qO-"}, 
    {
      noRestart = true,
      atRead = function(oper)
	myLog = myLog..oper.log:read(-1)
	local stable, experimental, message = myLog:match(
	  "Current%s+stable%s+release:%s+build%s+([^<%s]*)%s*</"..
	  ".-Current%s+experimental%s+release:%s+build%s+([^<%s]*)%s*</"..
	  ".-Message:%s+(.-)</"
	)
	if stable then
	  experimental = experimental ~= '' and "2.0."..experimental or ''
	  checkUpdateMessage = 
	      ("<%s><%s><%s><%s>"):
	      format(os.date'%F %T', "2.0."..stable, experimental, message)
	else
	  checkUpdateMessage = nil
	end
      end
    }
  )
  return true
end

local function v2b(ver)
  local vt = {ver:match"^(%d+)%.(%d+)%.(%d+)[_%.]?(%a*)(%d*)$"}
  local res = ""
  for i = 1, 5 do
    if i == 4 then
      res=res..(({z=0,alpha=2,beta=5,rc=8,['']=9})[vt[i]or'z']or 0)
    else
      res=res..('00'..(vt[i]or'')):sub(-2)
    end
  end
  return res
end
local function isStable(vBin)
  return vBin:sub(-3, -3) == '9'
end
local function isRc(vBin)
  return vBin:sub(-3, -3) == '8'
end


local function showCheckUpdate(path, sch, conf, sesNum)
  local locale = (C.getSessionInfo(sesNum) or {}).locale
  local hlp = function(h) return s.localeString(h, locale) end
  local currentVersion = C.getVersion()

  local mes = {}
  if checkUpdateMessage then
    local date, stable, experimental, message =
          checkUpdateMessage:match"<([^<>]*)><([^<>]*)><([^<>]*)><([^<>]*)>"

    local cur = v2b(currentVersion)
    local stab = v2b(stable)
    local exp = experimental ~= "" and v2b(experimental)

    mes[#mes+1] = date
    mes[#mes+1] = hlp"updMsg0":format(currentVersion)
    mes[#mes+1] = hlp"updMsg1":format(stable)
    if exp then
      mes[#mes+1] = hlp"updMsg2":format(experimental)
    else
      mes[#mes+1] = hlp"updMsg2no"
    end

    mes[#mes+1] = '\n'..message..'\n'

    local mescount = 2
    if cur < stab then
      mes[#mes+1] = hlp"updConclRequired":format(stable)
    elseif (isStable(cur) or not exp) and cur > stab then
      mes[#mes+1] = hlp"updConclDowngrade":format(stable, currentVersion)
    else
      mescount = mescount - 1
    end

    if exp then
      if cur < exp then
	if isRc(exp) then
	  mes[#mes+1] = hlp"updConclRecommended":format(experimental)
	else
	  mes[#mes+1] = hlp"updConclPossible":format(experimental)
	end
      elseif not isStable(cur) and cur > exp then
	mes[#mes+1] = hlp"updConclDowngrade":format(experimental, currentVersion)
      else
	mescount = mescount - 1
      end
    end

    if mescount == 0 then
      mes[#mes+1] = hlp"updConclSame"
    end

  else -- no checkUpdateMessage
    mes[#mes+1] = hlp"updConclFail"
    mes[#mes+1] = C.getDaemonLog(s.clonePath(path, -1))
  end
  return table.concat(mes, '\n')
end

local schedEvent = "CHECK-UPDATE"
local function getSchedulerCheckUpdate()
  local event = {
    ["virt-sensor"] = "scheduler",
    state = schedEvent,
    action = ".system.software.check-update.launch",
    _keep = true,
  }
  local eventPath = {'services', 'event-handler'}
  local eventList = C.getConfByPath(eventPath)
  if eventList then
    for k, v in ipairs(eventList) do
      if s.confCmp(event, v) then
        return true, eventPath, k, event
      end
    end
    return false, eventPath, #eventList + 1, event
  else
    return false, eventPath, 1, event
  end
end

local function replaceScheduler(val)
  local schedPath = {'services', 'event-handler', 'event-generators', 'scheduler'}
  local schedList = C.getConfByPath(schedPath)
  local i = 1
  if schedList then
    while schedList[i] do
      if schedList[i]['event-name'] == schedEvent then
	C.removeConfigValue(s.clonePath(schedPath, nil, i))
      else
	i = i + 1
      end
    end
  end

  if val then
    local sched = {
      ['event-name'] = schedEvent,
      min = 0,
      hour = 0,
      day = 1,
      month = '*',
      wday = '*',
      _keep = true,
    }
    if val == "daily" then
      sched.day = '*'
    elseif val == "weekly" then
      sched.wday = 1
      sched.day = '*'
    elseif val == "yearly" then
      sched.month = 2
    end

    C.setConfigValue(s.clonePath(schedPath, nil, i), sched)
  end
  C.applyConf(schedPath)
  return s.clonePath(schedPath, nil, i)
end

softwareScheme["check-update"] = s.newScheme{
  _H = "softCheckupdate",
  --_Z = checkUpdateLaunch,
  launch = s.newScheme{
    _H = "softCheckupdateLaunch",
    _X = function(path, sch, conf, sesNum)
      sch, conf = getParent(path, sesNum)
      path[#path] = nil
      checkUpdateLaunch(path, sch, conf, sesNum)
      return
    end,
    _R = "user",
  },
  url = s.stringType{
    _H = "softCheckupdateUrl",
    _D = "http://www.nsg.ru/nsg-linux/2.0/update_check.php",
  },
  show = s.newScheme{
    _H = "softCheckupdateShow",
    _X = showCheckUpdate,
    _R = "user",
  },
  --[[
  log =  s.newScheme{
    _H = "softCheckupdateLog",
    _X = function(path)
      path = s.clonePath(path, -1)
      return C.getDaemonLog(path)
    end,
    _R = 'user',
  },
  ]]

  ["add-to-scheduler"] = function(_, path)
    return not getSchedulerCheckUpdate(path) and
    s.newScheme{
      _H = "softCheckupdateAddtoscheduler",
      _T = "choice",
      _V = s.choiceVal,
      _A = {"daily", "weekly", "monthly", "yearly"},
      _D = "monthly",
      _X = function(path,sch,val,sesNum)
	local si = sesNum and C.getSessionInfo(sesNum) or {}
	local res, eventPath, num, event = getSchedulerCheckUpdate()
	if res then
	  local _H = "softCheckupdateMesEventAllreadyExists_S_"
	  return s.localeString(_H, si.locale):format(table.concat(eventPath, '.')..'.'..num)
	else
	  C.setConfigValue(s.clonePath(eventPath, nil, num), event)
	  C.applyConf(eventPath)
	  local schedPath = replaceScheduler(val)
	  local _H = "softCheckupdateMesEventAddTo_S_SchedulerTask_S_"
	  return s.localeString(_H, si.locale):
	         format(table.concat(eventPath, '.')..'.'..num, table.concat(schedPath, '.'))
	end
      end,
      _DEP = 4,
      _P = -100,
    }
  end,

  ["rm-from-scheduler"] = function(_, path)
    return getSchedulerCheckUpdate(path) and
    s.newScheme{
      _H = "softCheckupdateRmfromscheduler",
      _X = function(path,sch,val,sesNum)
	local si = sesNum and C.getSessionInfo(sesNum) or {}
	local res, eventPath, num, event = getSchedulerCheckUpdate(path)
	replaceScheduler()
	if res then
	  C.removeConfigValue(s.clonePath(eventPath, nil, num))
	  C.applyConf(eventPath)
	  local _H = "softCheckupdateMesEventDeleted_S_"
	  return s.localeString(_H, si.locale):format(table.concat(eventPath, '.')..'.'..num)
	else
	  local _H = "softCheckupdateMesNoEventToDeleteIn_S_"
	  return s.localeString(_H, si.locale):format(table.concat(eventPath, '.'))
	end
      end,
      _DEP = 4,
      _P = -100,
    }
  end,



}
