module(..., package.seeall)
local s = require'scheme'
local usb = require'nsgusbconf'

------------------
local mounts = {}
------------------

local storageMountVolume = 
   function (path)
      if not mounts[path[#path]] then
	 local mountpoint = "/var/mounts/" .. path[#path] .. "/"
	 local parts = usb.getPartitionName(path)
	 if parts and parts[1] then
	    mounts[path[#path]] = mountpoint
	    os.execute("mkdir -p " .. mountpoint)
	    return s.readpopen("mount /dev/" .. parts[1] .. " " .. mountpoint.. " && echo OK")
	 else
	    return ("No partition on media. Look \"info\" for detail.")
	 end
      end
      return "Already mounted!"
   end			    

-- concatinate the paths as a FULL-path file name
local getFullPath = function(path)
		       local cdr = ""
		       local pl = #path
		       while ((pl > 0) and (path[pl] ~= "//")) do
			  cdr = "'" .. path[pl]:gsub("'", "\\'") .. "'" .. cdr		 
			  pl = pl - 1
		       end
		       return (mounts[path[pl - 1]] or "") .. cdr
		    end
------------------

local moveCopyScheme = s.choiceType{
   _T = "choice",
   _A = {
      "/etc/uitcp/certs/",
      "/etc/stunnel/",
      "/etc/ipsec.d/certs/",
---      "/etc/tftp/",
      "user-defined",
   },
   _V = s.choiceVal,
   _D = "user-defined",
   _X = function(path, sch, val)
	   local act = path[#path]
	   local udf = C.getConfByPath(s.clonePath(path, -1, "user-defined-path"))
	   local path = s.clonePath(path, -1)
	   local cmd = (((act == "copy-to") or (act == "copy-all-to")) and "cp ") or "mv "
	   local sfx = ((act == "copy-all-to") and "*") or "" -- let "*" for all files
	   if val ~= "user-defined" then
	      return s.readpopen(cmd .. getFullPath(path) .. sfx .. " " .. val .. " 2>&1 && echo OK")  
	   else
	      if udf then
		 return s.readpopen(cmd .. getFullPath(path) .. sfx .. " " .. udf .. " && echo OK")  
	      else
		 return "set \"user-defined-path\" !"
	      end
	   end
	end,
   _LOCAL = true,
}

local deleteUmountScheme = s.choiceType{
   _T = "choice",
   _A = {"no", "yes"},
   _V = s.choiceVal,
   _D = "no",
   _X = function(path, sch, val)
	   if val == "yes" then
	      local action = path[#path]
	      local path = s.clonePath(path, -1)   -- go to object (dir or file)
	      if action == "mount" then
		 return storageMountVolume(path) 
	      else
		 if action == "umount" then
		    local ret = s.readpopen("umount " .. mounts[path[#path]] .. " && echo OK")
		    mounts[path[#path]] = nil -- delete mountpoint
		    return ret
		 else
		    if ((action == "delete") or (action == "delete-all")) then
		       local sfx = ((action == "delete-all") and "*") or ""
		       return s.readpopen("rm '" .. getFullPath(path) .. sfx .. "' && echo OK")
		    else
		       return action .. " " .. getFullPath(path) .. sfx
		    end
		 end
	      end
	   else
	      return "OPERATION NOT CONFIRMED\n" .. "(set \"yes\" to confirm and try again)"
	   end
	end,
   _LOCAL = true,
}


------------------
local ONEFILEINFOPRIO   = 90
local ONEFILECOPYTOPRIO = 80
local ONEFILEMOVETOPRIO = 70
local ONEFILEUDPPRIO    = 60
local ONEFILEDELETEPRIO = 50

local oneFile = s.newScheme {
   _H = "storageOneFile",
   
   ["copy-to"] = moveCopyScheme { _H = "storageOneFileCopyTo", _P = ONEFILECOPYTOPRIO, },
   
   ["move-to"] = moveCopyScheme { _H = "storageOneFileMoveTo", _P = ONEFILEMOVETOPRIO, _DEP = 2,},

   ["delete"] = deleteUmountScheme{ _H = "storageOneFileDelete", _P = ONEFILEDELETEPRIO, _DEP = 2, },

   ["user-defined-path"] = s.stringType{_H = "storageOneFilePath",_P = ONEFILEUDPPRIO, },
   
   info = s.newScheme{
      _H = "storageOneFileInfo", 
      _X = function(path)
	      local path = s.clonePath(path, -1)  -- follow to file node
	      return s.readpopen("ls -l " .. getFullPath(path) .. " 2>&1")
	   end,
      _LOCAL = true,
      _P = ONEFILEINFOPRIO,
   },
   
}

------------------
local DIRROOTPRIO      = 91
local DIRDELETEALLPRIO = 93  
local DIRUDPPRIO       = 95
local DIRCOPYALLPRIO   = 96  

local Groot

Groot = function(_, path)

	   if C.UNDER_APPLY or C.UNDER_SEARCH then 
	      return nil
	   end
	   if path[#path] == "//" and (not mounts[path[#path - 1]]) then
	      return nil  -- not mounted
	   end
	   local dir = s.newScheme{ _P = DIRROOTPRIO, _H = "storageOneDir", }
	   local currdir = getFullPath(path)
	   -- local get all names except for "directories"
	   local content = s.readpopen("cd " .. currdir .. " 2>/dev/null && find -maxdepth 1 ! -type d 2>/dev/null")

	   -- then extract filenames 
	   for fname in content:gmatch("%./([^\n]+)") do
	      dir[fname] = oneFile  --  clear filename (w/o "./")
	   end
	   --- Group operations
	   dir["copy-all-to"] = moveCopyScheme {_H = "storageCopyAllTo", _P = DIRCOPYALLPRIO, }
	   dir["delete-all"] = deleteUmountScheme {_H = "storageDeleteAll", _P = DIRDELETEALLPRIO,}
	   dir["user-defined-path"] = s.stringType{_H = "storageUdpPath",_P = DIRUDPPRIO,}
	   
	   --- SubDirs 
	   content = s.readpopen("cd " .. currdir .. " 2>/dev/null && find -maxdepth 1 -type d 2>/dev/null")
	   -- then choose only a files ("/" at the end)
	   for fname in content:gmatch("\n%./([^\n]+)") do
	      dir[fname .. "/"] = Groot  -- "/" to end dir name
	   end
	   return dir
	   
	end
------------------


local DIRFORMATPRIO = 80
local DIRMOUNTPRIO  = 70
local DIRUMOUNTPRIO = 70
local DIRINFOPRIO   = 60

storageScheme = s.newScheme {
   _Z = function(path)
	   local parts = usb.getPartitionName(path) 
	   if mounts[path[#path]] and (not parts) then -- "mounted" but "not present"!
	      local ret = s.readpopen("umount " .. mounts[path[#path]] .. " 2>/dev/null")
	      mounts[path[#path]] = nil                -- may be incorectly removed ?
	      return false
	   end
	   return storageMountVolume(path)
	end,

   ["//"] = Groot,

--   format = deleteUmountScheme{ _P = DIRFORMATPRIO, },

   mount = function(_, path)
	      return (not mounts[path[#path - 1]]) and deleteUmountScheme{ _P = DIRMOUNTPRIO, _DEP = 1, }
	    end,
   umount = function(_, path)
	       return mounts[path[#path - 1]] and deleteUmountScheme{ _P = DIRUMOUNTPRIO, _DEP = 1, }
	    end,
   info = s.newScheme{
      _H = "storageInfo",
      _P = DIRINFOPRIO,
      _X = function(path)
	      local path = s.clonePath(path, -1)
	      local type = usb.probeUSBDeviceScheme(nil, path)
	      local out = {} 
	      if type == "storage" then
		 local device = usb.FindDevice(path[#path])
		 local parts = usb.getPartitionName(path)
		 out[#out + 1] = "MASS Storage:\t\tPRESENT"
		 out[#out + 1] = "Manufacturer:\t\t" .. (device.manufacturer or "UNKNOWN")
		 out[#out + 1] = "Model:\t\t\t" .. (device.product or "UNKNOWN")
		 out[#out + 1] = "Partition name:\t\t" .. ((parts and table.concat(parts, ', ')) or "NO")
		 out[#out + 1] = "Mount point:\t\t" .. ((mounts[path[#path]] and mounts[path[#path]]) or "NO")

		 -- check existence module "usb-storage" if No parttitions
		 if (not parts) or (#parts == 0) then
		    local ret = os.execute("lsmod | grep usb_storage > /dev/null")
		    if tonumber(ret) ~= 0 then
		       out[#out + 1] = "\n\t\t\t*** WARNING ***"
		       out[#out + 1] = "\tThis device (usb-storage) can't be used normally."
		       out[#out + 1] = "Appropriate kernel-module was unloaded due to system conflict."
		       out[#out + 1] = "Set CDMA module(s) to \"usb-serial\" mode and reboot the system."
		    end
		 end
	      elseif path[#path] == "micro-SD" then
		 local parts = usb.getPartitionName(path)
		 out[#out + 1] = "MASS Storage:\t\tMicro-SD"
		 out[#out + 1] = "Partition name:\t\t" .. ((parts and table.concat(parts, ', ')) or "NO")
		 out[#out + 1] = "Mount point:\t\t" .. ((mounts[path[#path]] and mounts[path[#path]]) or "NO")
	      else
		 out[#out + 1] = "No storage device"
	      end
	      return table.concat(out, '\n')
	   end,
   },
}
