module(..., package.seeall)
local s = require'scheme'

local clonePath = s.clonePath
local function getParent(path, sesNum)
  local path = clonePath(path, -1)
  local sch = C.getSchemeByPath(path)
  local conf = C.getConfByPath(path) or {}
  local locConf = C.getConfByPath(path, sesNum) or {}
  for k, v in pairs(locConf) do
    conf[k] = v
  end
  conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
  return sch, conf
end

local launchPing = function(path,sch, conf, sesNum)
  local sch, conf = getParent(path, sesNum)
  local com = {
    "ping",
    conf.host,
    '-c', tostring(conf.count),
    '-s', tostring(conf.size),
  }
  if conf["source-ip"] then
    com[#com+1] = '-I'
    com[#com+1] = conf["source-ip"]
  end
  C.restartDaemon(path, com,
    {sesNum = sesNum, noTimestamp = true, noRestart = true}) 
end

local launchTraceroute = function(path,sch, conf, sesNum)
  local sch, conf = getParent(path, sesNum)
  local com = {
    "traceroute",
    conf.host,
    '-w', tostring(conf.wait),
  }
  if conf["source-ip"] then
    com[#com+1] = '-s'
    com[#com+1] = conf["source-ip"]
  end
  C.restartDaemon(path, com, 
    {sesNum = sesNum, noTimestamp = true, noRestart = true}) 
end

local launchTelnet = function(path, sch, conf, sesNum)
  local sch, conf = getParent(path, sesNum)
  local com = {
    "telnet",
    conf.host,
    tostring(conf.port),
  }
  if conf["escapechar"] then
    com[#com+1] = '-e'
    com[#com+1] = conf["escapechar"]
  end
  C.restartDaemon(path, com, 
    {sesNum = sesNum, noTimestamp = true, noRestart = true}) 
end

local function getUserName(sesNum)
  local si = C.getSessionInfo(sesNum)
  if si and si.user then
    return si.user..'@'
  else
    return ''
  end
end
local launchSSH = function(path, sch, conf, sesNum)
  local sch, conf = getParent(path, sesNum)
  local com = {
    "ssh",
    (conf.user and conf.user ..'@' or getUserName(sesNum))..conf.host,
    '-p', tostring(conf.port),
  }
  if conf["escapechar"] then
    com[#com+1] = '-e'
    com[#com+1] = conf["escapechar"]
  end
  C.restartDaemon(path, com, 
    {sesNum = sesNum, noTimestamp = true, noRestart = true}) 
end

local launchTcpdump = function(path,sch, conf, sesNum)
  local sch, conf = getParent(path, sesNum)
  local com = {
    "tcpdump",
    "-x",
  }
  if conf["write-to-file"] and conf["write-to-file"] ~= "no" then
    com[#com+1] = '-w'
    com[#com+1] = conf["write-to-file"]
  end
  if conf.interface and conf.interface:match'^%s*$' then
    com[#com+1] = '-i'
    com[#com+1] = conf.interface
  end
  if conf.expression then
    com[#com+1] = conf.expression
  end
  C.restartDaemon(path, com, 
    {sesNum = sesNum, noTimestamp = true, noRestart = true}) 
end

local launchMail = function(path,sch, conf, sesNum)
  local sch, conf = getParent(path, sesNum)
  local com = {
    "nsgsmtp", '-q',
  }
  if conf.server then
    com[#com+1] = '--server'
    com[#com+1] = conf.server
  end
  if conf.from then
    com[#com+1] = '--from'
    com[#com+1] = conf.from
  end
  if conf.to then
    com[#com+1] = '--rcpt'
    com[#com+1] = conf.to
  end
  if conf.domain then
    com[#com+1] = '--domain'
    com[#com+1] = conf.domain
  end
  if conf.user then
    com[#com+1] = '--user'
    com[#com+1] = conf.user
  end
  if conf.password then
    com[#com+1] = '--password'
    com[#com+1] = conf.password
  end
  if conf.subject then
    com[#com+1] = '--subject'
    com[#com+1] = conf.subject
  end
  if conf.body then
    com[#com+1] = '--body'
    com[#com+1] = conf.body
  end
  if conf.port then
    com[#com+1] = '--port'
    com[#com+1] = tostring(conf.port)
  end

  C.restartDaemon(path, com, 
    {sesNum = sesNum, noTimestamp = true, noRestart = true}) 
end

local launchScheme = s.newScheme{
  _H = "toolsLaunch",
  _XI = true,
  _P = 100,
}

local localStringType = s.stringType{
  _LOCAL = true,
  _R = 'user',
}


local writeToFile = (require'nsgconflib'.writeToFile)
require'nsg'.meta()
local function ledControlScheme(num, switch)
  return s.newScheme{
    _H = "toolsLedSwitch",
    _R = 'user',
    _X = function(path,sch, conf, sesNum)
      local res = writeToFile('/sys/class/gpio/gpio%s/value'%num, switch)
      if not res then
        writeToFile('/sys/class/gpio/export', num)
        writeToFile('/sys/class/gpio/gpio%s/direction'%num, 'out')
        writeToFile('/sys/class/gpio/gpio%s/value'%num, switch)
      end
      return 'OK'
    end,
  }
end

toolsScheme = s.newScheme{
  _H = "tools",


  ping = s.newScheme{
    _H = "toolsPing",
    launch = launchScheme{_X = launchPing, _R = 'user'},

    host = s.ipAddressType{
      _H = "toolsPingHost",
      _D = "127.0.0.1",
      _LOCAL = true,
      _R = 'user'
    },
    ["source-ip"] = s.ipAddressType{
      _H = "toolsPingSourceIP",
      _LOCAL = true,
      _R = 'user'
    },
    count = s.integerType{
      _H = "toolsPingCount",
      _A = {0, math.huge},
      _LOCAL = true,
      _R = 'user',
      _D = 0,
    },
    size = s.integerType{
      _H = "toolsPingSize",
      _A = {0, 65507},
      _D = 56,
      _LOCAL = true,
      _R = 'user',
    },
  },

  traceroute = s.newScheme{
    _H = "toolsTraceroute",
    launch = launchScheme{_X = launchTraceroute, _R = 'user'},

    host = s.ipAddressType{
      _H = "toolsTracerouteHost",
      _D = "127.0.0.1",
      _LOCAL = true,
      _R = 'user',
    },
    ["source-ip"] = s.ipAddressType{
      _H = "toolsTracerouteSourceIP",
      _LOCAL = true,
      _R = 'user',
    },
    wait = s.integerType{
      _H = "toolsTracerouteWait",
      _A = {2, 86400},
      _D = 3,
      _LOCAL = true,
      _R = 'user',
    },
  },

  telnet = s.newScheme{
    _H = "toolsTelnet",
    launch = launchScheme{_X = launchTelnet, _R = 'user'},

    host = s.ipAddressType{
      _H = "toolsTelnetHost",
      _D = "127.0.0.1",
      _LOCAL = true,
      _R = 'user',
    },
    port = s.tcpPortType{
      _H = "toolsTelnetPort",
      _D = 23,
      _LOCAL = true,
      _R = 'user',
    },
    escapechar = s.stringType{
      _H = "toolsTelnetEscapechar",
      _LOCAL = true,
      _R = 'user',
    }
  },

  ssh = s.newScheme{
    _H = "toolsSSH",
    launch = launchScheme{_X = launchSSH, _R = 'user'},

    host = s.ipAddressType{
      _H = "toolsSSHHost",
      _D = "127.0.0.1",
      _LOCAL = true,
      _R = 'user',
    },
    port = s.tcpPortType{
      _H = "toolsSSHPort",
      _D = 22,
      _LOCAL = true,
      _R = 'user',
    },
    user = s.stringType{
      _H = "toolsSSHUser",
      _LOCAL = true,
      _R = 'user',
    },
    escapechar = s.stringType{
      _H = "toolsSSHEscapechar",
      _LOCAL = true,
      _R = 'user',
    },
  },

  tcpdump = s.newScheme{
    _H = "toolsTcpdump",
    launch = launchScheme{_X = launchTcpdump},

    interface = s.stringType{
      _H = "toolsTcpdumpInterface",
      _LOCAL = true,
    },
    expression = s.stringType{
      _H = "toolsTcpdumpExpression",
      _LOCAL = true,
    },
    ["write-to-file"] = s.newScheme{
      _H = "toolsTcpdumpWritetofile",
      _T = 'choice',
      _A = {"no", "/tmp/tcpdump.cap"},
      _V = s.choiceVal,
    },
  },

  mail = s.newScheme{
    _H = "toolsMail",
    send = launchScheme{_X = launchMail, _R = 'user'},

    server = localStringType{_H = "toolsMailServer", _P = 80},
    port = s.tcpPortType{
      _H = "toolsMailPort",
      _D = 25,
      _LOCAL = true,
      _R = 'user', 
      _P = 70,
    },
    domain = localStringType{_H = "toolsMailDomain", _P = 60},
    user = localStringType{_H = "toolsMailUser", _P = 50},
    password= localStringType{_H = "toolsMailPassword", _P = 40},
    from = localStringType{_H = "toolsMailFrom", _P = 30},
    to = localStringType{_H = "toolsMailRcpt", _P = 20},
    subject = localStringType{_H = "toolsMailSubject", _P = 10},
    body = localStringType{_H = "toolsMailBody", _P = 0},
  },

  led = BI.arch == 'nsg1800' and
  s.newScheme{
    _H = "toolsLedOneled",
    red = s.newScheme{
      _H = "toolsLedRed",
      on = ledControlScheme('196', '0'),
      off = ledControlScheme('196', '1'),
    },
    green = s.newScheme{
      _H = "toolsLedGreen",
      on = ledControlScheme('197', '0'),
      off = ledControlScheme('197', '1'),
    },
  }
  or (BI.arch == 'nsg700' and
    s.newScheme{
      _H = "toolsLed",
      l1 = s.newScheme{
        _H = "toolsLedOneled",
        red = s.newScheme{
          _H = "toolsLedRed",
          on = ledControlScheme('96', '1'),
          off = ledControlScheme('96', '0'),
        },
        green = s.newScheme{
          _H = "toolsLedGreen",
          on = ledControlScheme('97', '1'),
          off = ledControlScheme('97', '0'),
        },
      },
      l2 = s.newScheme{
        _H = "toolsLedOneled",
        red = s.newScheme{
          _H = "toolsLedRed",
          on = ledControlScheme('98', '1'),
          off = ledControlScheme('98', '0'),
        },
        green = s.newScheme{
          _H = "toolsLedGreen",
          on = ledControlScheme('99', '1'),
          off = ledControlScheme('99', '0'),
        },
      },
  })
}
