module(..., package.seeall)
local s = require'scheme'
local iface = require'nsgifaceconf'

local tpBinFile = {
  '/usr/sbin/tcpproxy'
}

local function restartTcpproxy(path, sch, conf, prevConf)
  local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
  if conf.enable then
    local tpConfigFilename = os.tmpname()
    local f, err = io.open(tpConfigFilename, 'w')
    if f then
      local res, err = f:write("return", s.ser(conf))
      if not res then
        C.log_error("restartTcpproxy: Can't write tcpproxy config file: "..err)
      end
      f:close()
    else
      C.log_error("restartTcpproxy: Can't create tcpproxy config file: "..err)
    end

    local tpUtil
    for i = 1, #tpBinFile do
      local f = io.open(tpBinFile[i], 'r')
      if f then
        f:close()
        tpUtil = tpBinFile[i]
        break
      end
    end

    if tpUtil then
      C.restartDaemon(
        path, 
        {tpUtil, '-n', tpConfigFilename }, 
        { 
          noTimestamp = true,
          tmpFileList = {tpConfigFilename}, 
          atExit = C.increasingSleep
        }
      )
    else
      C.restartDaemon(path)
    end
    return true
  end
  C.restartDaemon(path)
  return true
end

local socket = require'socket'
local tpsock, recursion
local function tpcom(com)
  if not tpsock then
    local err
    tpsock, err = socket.connect("127.0.0.1", 50012)
    if not tpsock then
      local msg = "Can't connect to tcpproxy daemon"
      C.log_error(msg)
      return msg
    end
    tpsock:settimeout(3)
  end

  local res, err = tpsock:send(com..'\n')
  if not res then
    tpsock:close()
    tpsock = nil
    if recursion then
      recursion = false
      return "Can't send request to tcpproxy daemon"
    else
      recursion = true
      return tpcom(com)
    end
  end

  local buf = ""
  while true do
    local data, err, part = tpsock:receive'*l'
    if not data then
      return "Can't receive answer from tcpproxy daemon("..err..")"
    end
    buf = buf..(data or part or '')
    if buf:sub(-1) == '\000' then
      return buf:sub(1, -2)
    else
      buf = buf..'\n'
    end
  end
end

function tcpproxySend(tpname, val) -- TODO
  return tpcom("send(%q,%q)"%{tpname, tostring(val)})
end

tcpproxyScheme = s.newScheme{
  _H = "servicesTcpproxy",
  _Z = restartTcpproxy,  

  enable = s.booleanType{
    _H = "servicesTcpproxyEnable",
    _P = 100,
  },
  ["log"] =  s.newScheme{
    _H = "servicesTcpproxyLog",
    _X = function(path)
      return C.getDaemonLog(s.clonePath(path, -1))
    end,
    _R = 'user',
  },
 
  _NAME = s.newScheme{
    _H = "servicesTcpproxyX",
    enable = s.booleanType{
      _H = "servicesTcpproxyXEnable",
      _P = 100,
      _D = true,
    },
    incoming = s.newScheme{
      _H = "servicesTcpproxyIn",
      host = s.ipAddressOrAllType{
	_H = "servicesTcpproxyInHost",
	_D = '*',
	_P = 100
      },
      port = s.tcpPortType{
	_H = "servicesTcpproxyInPort",
	_P = 90,
      },
      keepalive = s.newScheme{
	_T = "number", _A = {0, 0x7fffffff}, _V = s.numberVal,
	_H = "servicesTcpproxyInKeepalive",
	_D = 5,
      },
      retry = s.newScheme{
	_T = "integer", _A = {1, 0x7fffffff}, _V = s.integerVal, 
	_H = "servicesTcpproxyInRetry",
	_D = 3,
      },
    },
    outgoing = s.newScheme{
      _H = "servicesTcpproxyOut",
      server = s.ipAddressAndPortType{
	_H = "servicesTcpproxyOutServer",
	_P = 100,
      }, 
      srcAddr = s.ipAddressType{
	_H = "servicesTcpproxyOutSrcAddr",
        _P = 90,
      },
      keepalive = s.newScheme{
	_T = "number", _A = {0, 0x7fffffff}, _V = s.numberVal,
	_H = "servicesTcpproxyOutKeepalive",
	_D = 5,
      },
      retry = s.newScheme{
	_T = "integer", _A = {1, 0x7fffffff}, _V = s.integerVal, 
	_H = "servicesTcpproxyOutRetry",
	_D = 3,
      },
    },
  },
}

