module(..., package.seeall)
local s = require'scheme'
local iface = require'nsgifaceconf'

local tsBinFile = {
  '/usr/sbin/tcpsender'
}

local function restartTcpsender(path, sch, conf, prevConf)
  local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
  if conf.enable then
    local tsConfigFilename = os.tmpname()
    local f, err = io.open(tsConfigFilename, 'w')
    if f then
      local res, err = f:write("return", s.ser(conf))
      if not res then
        C.log_error("restartTcpsender: Can't write tcpsender config file: "..err)
      end
      f:close()
    else
      C.log_error("restartTcpsender: Can't create tcpsender config file: "..err)
    end

    local tsUtil
    for i = 1, #tsBinFile do
      local f = io.open(tsBinFile[i], 'r')
      if f then
        f:close()
        tsUtil = tsBinFile[i]
        break
      end
    end

    if tsUtil then
      C.restartDaemon(
        path, 
        {tsUtil, '-n', tsConfigFilename }, 
        { 
          noTimestamp = true,
          tmpFileList = {tsConfigFilename}, 
          atExit = C.increasingSleep
        }
      )
    else
      C.restartDaemon(path)
    end
    return true
  end
  C.restartDaemon(path)
  return true
end

local socket = require'socket'
local tssock, recursion
local function tscom(com)
  if not tssock then
    local err
    tssock, err = socket.connect("127.0.0.1", 50013)
    if not tssock then
      local msg = "Can't connect to tcpsender daemon"
      C.log_error(msg)
      return msg
    end
    tssock:settimeout(3)
  end

  local res, err = tssock:send(com..'\n')
  if not res then
    tssock:close()
    tssock = nil
    if recursion then
      recursion = false
      return "Can't send request to tcpsender daemon"
    else
      recursion = true
      return tscom(com)
    end
  end

  local buf = ""
  while true do
    local data, err, part = tssock:receive'*l'
    if not data then
      return "Can't receive answer from tcpsender daemon("..err..")"
    end
    buf = buf..(data or part or '')
    if buf:sub(-1) == '\000' then
      return buf:sub(1, -2)
    else
      buf = buf..'\n'
    end
  end
end

function tcpsenderSend(tsname, val)
  return tscom("send(%q,%q)"%{tsname, tostring(val)})
end

tcpsender = s.newScheme{
  _H = "eventEventactionsTcpsender",
  _Z = restartTcpsender,  

  enable = s.booleanType{
    _H = "eventEventactionsTcpsenderEnable",
    _P = 100,
  },
  ["log"] =  s.newScheme{
    _H = "eventEventactionsTcpsenderLog",
    _X = function(path)
      return C.getDaemonLog(s.clonePath(path, -1))
    end,
    _P = -100,
    _R = 'user',
  },
 
  _NAME = s.newScheme{
    _H = "eventEventactionsTcpsenderX",
    enable = s.booleanType{
      _H = "eventEventactionsTcpsenderXEnable",
      _P = 100,
      _D = true,
    },
    description = s.stringType{
      _H = eventEventactionsTcpsenderDescription,
      _P = 90,
    },
    server = s.ipAddressAndPortType{
      _H = "eventEventactionsTcpsenderServer",
      _D = "localhost:50015"
    }, 
    keepalive = s.newScheme{
      _T = "number", _A = {1, 0x7fffffff}, _V = s.numberVal, 
      _H = "eventEventactionsTcpsenderKeepalive",
      _D = 5,
    },
    retry = s.newScheme{
      _T = "integer", _A = {1, 0x7fffffff}, _V = s.integerVal, 
      _H = "eventEventactionsTcpsenderRetry",
      _D = 3,
    },
    sendtest = s.stringType{
      _H = "eventEventactionsTcpsenderSendtest",
      _X = function(path, sch, val)
      	local tsname = path[#path - 1]
	return tscom("send(%q,%q)"%{tsname, tostring(val)})
      end,
    },
  },
}

