module(..., package.seeall)
local s = require'scheme'
local nsg = require'nsg'
local ox = require'posix'

local timezone do
  local eet = "EET-2EEST"
  local fet = "FET-3"
  timezone  = {
    Kaliningrad = fet,
    Moscow = "MSK-4",
    Yekaterinburg = "YEKT-6",
    Omsk = "OMST-7",
    Krasnoyarsk = "KRAT-8",
    Irkutsk = "IRKT-9",
    Yakutsk = "YAKT-10",
    Vladivostok = "VLAT-11",
    Magadan = "MAGT-12",
    Minsk = fet,
    Kiev = eet,
    Chisinau = eet,
    Tallinn = eet,
    Riga = eet,
    Vilnius = eet,
    Baku = "AZT-4AZST",
    Yerevan = "AMT-4",
    Tbilisi = "GET-4",
    Almaty = "ALMT-6",
    Aqtobe = "AQTT-5",
    Ashkhabad = "TMT-5",
    Dushanbe = "TJT-5",
    Tashkent = "UZT-5",
    Bishkek = "KGT-6",
    UTC = "UTC",
  }
end

local cityList = {}
for k in pairs(timezone) do
  cityList[#cityList+1] = k
end
table.sort(cityList)
cityList[#cityList+1] = "other"


timeZoneScheme = s.newScheme{
  _H = "tz",
  _Z = function(path, sch, conf, prevConf)
    if not s.confCmp(conf, prevConf) then
      local conf = s.addDefault(path, sch, conf, true)
      local TZ = conf.location == 'other' and conf.TZ or
                 (timezone[conf.location] or 'UTC')
      ox.setenv("TZ", TZ, true)
      os.execute("export TZ="..TZ..";echo "..TZ.." > /etc/TZ")
    end
    return true
  end,

  location = s.choiceType{
    _H = "tzLocation",
    _A = cityList,
    _D = "UTC",
    _DEP = 1,
  },

  TZ = function(_, path)
    return C.getConfOrDefault(s.clonePath(path, -1, "location")) == "other" and
    s.stringType{
      _H = "tzTZ",
      _D = "CET-1CEST",
    }
  end,
}
