module(..., package.seeall)
local s = require'scheme'

local UPnPDaemons={}

local function killRemovedDaemons(path)
   local conf = C.getConfByPath(path)
   for name in pairs(UPnPDaemons) do
      if not (conf and conf[name]) then
         C.restartDaemon(s.clonePath(path,0,name))
         UPnPDaemons[name] = nil
      end
   end
end

local restartUPnP =
   function(path, sch, conf, prevConf)
      local confFile = C.getOption(path,"confFile")
      if not confFile then
         confFile = os.tmpname()
         C.setOption(path, "confFile", confFile)
      end

      if not s.confCmp(conf, prevConf) then
         conf = conf or {}
         local cfg = {}
         local cmd
         conf = s.addDefault(path, sch, conf, true)

         if conf["adm-state"]=="up" then

            if conf["wan-ifname"] then
               cfg[#cfg + 1] = "ext_ifname=" .. conf["wan-ifname"]
            end
            if conf["wan-ip"] then
               cfg[#cfg + 1] = "ext_ip=" .. conf["wan-ip"]
            end

            for k, v in ipairs(conf["lan-ip"] or {}) do
               cfg[#cfg + 1] = v and "listening_ip="..v or nil
            end

            cfg[#cfg + 1] = conf["natpmp"] and "enable_natpmp="..conf["natpmp"]
            cfg[#cfg + 1] = conf["secure-mode"] and "secure_mode="..conf["secure-mode"]

            cfg[#cfg + 1] = "upnp_forward_chain=UPnP"
            cfg[#cfg + 1] = "upnp_nat_chain=UPnP"
            cfg[#cfg + 1] = "uuid="..BI.uuid
            cfg[#cfg + 1] = "serial="..BI.serial
            cfg[#cfg + 1] = "model_number="..BI.mach

            for k,v in ipairs(conf["permissions"] or {}) do
               cfg[#cfg + 1] = v["action"].." "..v["external-ports"]:gsub("[%:]","-").." "..v["internal-net"].." "..v["internal-ports"]:gsub("[%:]","-")
            end
            cfg[#cfg + 1] = conf["permissions"]["default-action"].." 0-65535 0.0.0.0/0 0-65535"
            cfg[#cfg + 1] = "\n"

            local f = io.open(confFile,"w")
            if f then
               f:write(table.concat(cfg,"\n"))
               f:close()
            end

            UPnPDaemons[path[#path]] = true
            cmd = {}
            cmd[#cmd + 1] = "/sbin/miniupnpd"
            cmd[#cmd + 1] = "-d"
            cmd[#cmd + 1] = "-f"
            cmd[#cmd + 1] = confFile
         else
            os.execute('rm -f '..confFile)
            C.setOption(path, "confFile", nil)
            UPnPDaemons[path[#path]] = nil
         end
         C.restartDaemon(path,
                         cmd,
                         {
                            noRestart = false,
                            atExit = C.increasingSleep,
                         }
                      )
         return true
      else
         return true
      end
   end

local portsVal =
   function(val)
      local help = ""
      local lval  = tostring(val):gsub("[^%-%:0-9]","")
      local p1, s, p2 = lval:match("^([0-9]*)([%:]?)([0-9]*)$")
      p1, s, p2 = (p1 ~= '' and p1 or nil), (s ~= '' and s or nil), (p2 ~= '' and p2 or nil)
      p1 = tonumber(p1)
      p2 = tonumber(p2)
      p1 = s and (p1 or 0) or p1
      p2 = s and (p2 or 65535) or nil
      p1, p2 = (((p1 and p2) and (p1<p2 and p1 or p2))  or p1), (((p1 and p2) and (p1<p2 and p2 or p1)) or p2)
      if (p1 and (p1<0 or p1>65535)) or (p2 and (p2<0 or p2>65535)) or (not p1 and not p2) then
         return nil, "Port range or single port, e.g. '1024:20000' or '35000'"
      end
      p2 = (p1~=p2) and p2 or nil
      return tostring(p1..( p2 and ":"..p2 or ""))
   end

UPnPServerScheme = s.newScheme {
   _H = "Upnp",
--[[   _Z = function(path, sch, conf, prevConf)
           killRemovedDaemons()
           return true
        end,
        --]]
  -- _NAME = s.newScheme {
   --   _H = "UpnpN",
        _Z = restartUPnP,
        ["show"]= s.newScheme{
           _H = "UpnpShow",
           ["log"] =  s.newScheme{
              _H = "UpnpShowLog",
              _X = function(path)
                      path = s.clonePath(path, -2)
                      return C.getDaemonLog(path)
                   end,
           },
           ["configuration"] =  s.newScheme{
              _H = "UpnpShowConfig",
              _X = function(path)
                      path = s.clonePath(path, -2)
                      local confFile = C.getOption(path,"confFile")
                      return s.readpopen("cat "..confFile.." 2>&1")
                   end,
           },
        },
   
      ["adm-state"] = s.choiceType{
         _A = {"up", "down"},
         _H = "UpnpAdmstate",
         _D = "down",
         _P = 90,
         _DEP = 3,
      },
      ["wan-ifname"] = s.stringType{ _H = "UpnpWanifname", _P= 70},
      ["wan-ip"] = s.ip46AddressType{ _H = "UpnpWanip", _P= 70},
      ["lan-ip"] = s.newScheme{
         _H = "UpnpLanip",
         _LIST = s.ipPrefixOrAddressType{
            _H = "UpnpLanipNPrefix",
         },
      },
      ["natpmp"] = s.choiceType{
          _H = "UpnpNatpmp",
         _A = {"yes", "no"},
         _D = "yes"
      },
      ["secure-mode"] = s.choiceType{
          _H = "UpnpSecuremode",
         _A = {"yes", "no"},
         _D = "no"
      },
      ["presentation-url"] = s.stringType{_H = "UpnpPresentationurl"},
      ["notify-interval"] = s.integerType{
         _H = "UpnpNotifyinterval",
         _A = {1,65535},
         _D = 30,
      },
      permissions = s.newScheme{
         _H = "UpnpPermissions",
         _LIST = s.newScheme{
            _H = "UpnpPermissionsN",
            action = s.choiceType{
               _H = "UpnpPermissionsNAction",
               _P = 10,
               _A = {"deny","allow"},
               _D = "allow"
            },
            ["external-ports"] = s.stringType{
               _H = "UpnpPermissionsNExtports",
               _V = portsVal,
               _D = "1024:65535",
            },
            ["internal-net"] = s.ipPrefixType{
               _H = "UpnpPermissionsNIntnet",
               _D = "0.0.0.0/0",
            },
            ["internal-ports"] = s.stringType{
               _H = "UpnpPermissionsNIntports",
               _V = portsVal,
               _D = "1024:65535",
            },
         },
         ["default-action"] = s.choiceType{
            _H = "UpnpPermissionsDefaultaction",
            _P = -10,
            _A = {"deny","allow"},
            _D = "allow"
         }
      }
      --  },

}