module(..., package.seeall)
local s = require'scheme'
local eth = require'nsgethconf'
local port = require'nsgportconf'
local iface = require'nsgifaceconf'
local rs232 = require'nsgrs232conf'
local nsg = require'nsg'
local WiFi = require'nsgwifi'
local sens = require'nsgsensconf'
local prn = require'nsgprinter'
local stor = require'nsgstorage'
local wmxd = require'nsgwimax'
local eth703 = require'nsgeth703'
local lte = require'nsglteconf'
local mercury230 = require'nsgmercury230conf'

local lfs = require "lfs"
local socket = require "socket"

local BASE_DIR = "/sys/bus/usb/devices"

local allTypes = {"eth", "cdma", "edge", "edge-3a", "3g", "wifi", "2com", "rs-232", "multisensor", "printer", "storage", 
  "wimax", "eth-703", "gsm-2sim", "lte", "empty", }

-- NSG-1000 onboard USB ports
local NSG1000USB_BLACKLIST = {}
for _,v in ipairs(allTypes) do
  if v ~= "empty"
    and v ~= "rs-232"
    and v ~= "printer"
    and v ~= "storage"
  then
    NSG1000USB_BLACKLIST[#NSG1000USB_BLACKLIST + 1] = v
  end
end

-- USB ports (slots) on KRO-1000-USB backplain for NSG-1000
local KRO1000USB_BLACKLIST = {}
for _,v in ipairs(allTypes) do
  if v ~= "empty"
    and v ~= "eth"
    and v ~= "eth-703"
  then
    KRO1000USB_BLACKLIST[#KRO1000USB_BLACKLIST + 1] = v
  end
end

local modules = { 
  ["0b05/1706"] = { descr = "Wifi-Dongle", type = "wifi", }, 
  ["0b05/1742"] = { descr = "Wifi-Module RaLink2770", type = "wifi", }, 
  ["0bda/8189"] = { descr = "WiFi-Module RTL", type = "wifi", },
  ["18e8/6229"] = { descr = "WiFi-Module RaLink", type = "wifi", },
  ["07a6/1986"] = { descr = "um-et100",    type = "eth", },
  ["0b95/772a"] = { descr = "um-et100-v2/me-eth", type = "eth", },
--  ["0b95/1790"] = { descr = "AX88179 GbE", type = "eth", },
  ["16d8/5543"] = { descr = "uim-evdo",    type = "cdma", intf = {[1] = "", }, },
  ["16d8/680a"] = { descr = "uim-evdo",    type = "cdma", intf = {[1] = "", }, },
  ["16d8/6803"] = { descr = "uim-evdo",    type = "cdma", intf = {[1] = "", [2] = "-ctl",}, },
  ["15eb/0101"] = { descr = "cdma(EP45)",  type = "cdma", intf = {[2] = "", [3] = "-ctl", }, },
  ["0471/1234"] = { descr = "uim-edge-v3", type = "edge", intf = {[1] = "", }, },
  ["0471/0020"] = { descr = "uim-edge-v3a",type = "edge-3a", intf = {[1] = "", [2] = "-ctl", }, },
--  ["05c6/6000"] = { descr = "uim-3g",      type = "3g",   intf = {[1] = "", [3] = "-ctl", }, },
  ["05c6/6000"] = { descr = "khd-450",     type = "cdma",   intf = {[1] = "", [2] = "-ctl", }, },
  ["05c6/9000"] = { descr = "um-3g",       type = "3g",   intf = {[3] = "", [4] = "-ctl", }, },
  ["1199/683c"] = { descr = "SW MC",       type = "3g",   intf = {[4] = "", [5] = "-ctl", }, },
--  ["1199/68a3"] = { descr = "SW MC",       type = "3g",   intf = {[4] = "", [5] = "-ctl", }, },
  ["1199/68a3"] = { descr = "SW MC",       type = "lte",   intf = {[4] = "", [5] = "-ctl", }, },
  ["216f/0041"] = { descr = "ATEL LP-16",  type = "lte",   intf = {[1] = "", }, },
  ["05c6/3197"] = { descr = "um-proba",    type = "cdma", intf = {[1] = "", [2] = "-ctl",}, },
  ["0403/6010"] = { descr = "um-2COM",     type = "2com", intf = {[1] = ".0", [2] = ".1", }, },
  ["0483/fffc"] = { descr = "ms-6",        type = "multisensor", intf = {[1] = "", },},
  ["0403/6001"] = { descr = "mercury-230", type = "mercury", intf = {[1] = "", },},
  ["0483/fff7"] = { descr = "me-2m",       type = "eth-703", },
  ["0483/fff4"] = { descr = "me-2m-v4",    type = "eth-703", },
  ["04e8/6761"] = { descr = "Wimax",       type = "wimax", },
  ["067b/2303"] = { descr = "Prolific",    type = "rs-232", intf = {[1] = "", }, },
-- Class/SubClass
  ["07/01"]     = { descr = "USB-Printer", type = "printer", intf = {[1] = "", }, },
  ["08/06"]     = { descr = "USB-Storage", type = "storage", },
}
-------------------------------------------------
local writeToFile = (require'nsgconflib'.writeToFile)
--[[
local function writeToFile(fname, str)
  local f, err = io.open(fname, 'w')
  if f then
    f:write(str) 
    f:close()
  else
    --log_error("Can't open "..fname.." "..err)
  end

end
]]
------------------------------------
local NSG1800_USB_3 = { 
    HS_path = "1-3",
    FS_path = "2-3",
    powerOn = function () 
      writeToFile('/sys/class/gpio/gpio134/value', '0')
    end,
    powerOff = function () 
      writeToFile('/sys/class/gpio/gpio134/value', '1')
    end,
    setGpio = function ()
      writeToFile('/sys/class/gpio/export', '134')   -- power control pin (PA6)
      writeToFile('/sys/class/gpio/gpio134/direction', 'out')
      writeToFile('/sys/class/gpio/export', '133')   -- SIM control pin (PA5)
      writeToFile('/sys/class/gpio/gpio133/direction', 'out')
    end,
    setSim = function (sim)
      writeToFile('/sys/class/gpio/gpio133/value', sim) 
    end,
    blacklist = {"rs-232", "multisensor", "2com", "eth", "printer",  "wimax", "eth-703", },
  }
local NSG1800_Platform = {   
  ["s1"] = {
    HS_path = "1-2",
    FS_path = "2-2",
    powerOn = function ()
      writeToFile('/sys/class/gpio/gpio217/value', '0')
    end,
    powerOff = function ()
      writeToFile('/sys/class/gpio/gpio217/value', '1')
    end,
    setGpio = function ()
      writeToFile('/sys/class/gpio/export', '217')   -- DTR s1
      writeToFile('/sys/class/gpio/gpio217/direction', 'out')
      writeToFile('/sys/class/gpio/export', '223')   -- "RTS" s1
      writeToFile('/sys/class/gpio/gpio223/direction', 'out')
    end,
    setSim = function (sim)
      writeToFile('/sys/class/gpio/gpio223/value', sim) 
    end,
    blacklist = {"rs-232", },
  },
  ["s2"] = {
    HS_path = "1-1",
    FS_path = "2-1",
    powerOn = function ()
      writeToFile('/sys/class/gpio/gpio137/value', '0')
    end,
    powerOff = function ()
      writeToFile('/sys/class/gpio/gpio137/value', '1')
    end,
    setGpio = function ()
      writeToFile('/sys/class/gpio/export', '137')   -- DTR s2
      writeToFile('/sys/class/gpio/gpio137/direction', 'out')
      writeToFile('/sys/class/gpio/export', '140')   -- "RTS" s2
      writeToFile('/sys/class/gpio/gpio140/direction', 'out')
    end,
    setSim = function (sim)
      writeToFile('/sys/class/gpio/gpio140/value', sim) 
    end,
    blacklist = {"rs-232", },
  },
  ["m1"] = NSG1800_USB_3,
  ["wlan0"] = NSG1800_USB_3,
}
------------------------------------
local NSG1820_USB_1_3 = { 
    HS_path = "1-1.3",
    FS_path = "1-1.3",
    powerOn = function () 
      nsg.hubcontrol("/dev/hub1", 3, "ON")
    end,
    powerOff = function () 
      nsg.hubcontrol("/dev/hub1", 3, "OFF")
    end,
    setGpio = function ()
      writeToFile('/sys/class/gpio/export', '239')  
      writeToFile('/sys/class/gpio/gpio239/direction', 'out')
    end,
    setSim = function (sim)
      writeToFile('/sys/class/gpio/gpio239/value', sim == mainSimNumber and '0' or '1')
    end,
    blacklist = {"rs-232", "multisensor", "2com", "eth", "printer",  "wimax", "eth-703", },
  }

local NSG1820_Platform = {   
  ["m1"] = {
    HS_path = "1-1.1",
    FS_path = "1-1.1",
    -- PowerON-pin(233) can NOT be used due to PCIe conflict!
    powerOn = function ()
      nsg.hubcontrol("/dev/hub1", 1, "ON")
    end,
    powerOff = function ()
      nsg.hubcontrol("/dev/hub1", 1, "OFF")
    end,
    setGpio = function ()
      writeToFile('/sys/class/gpio/export', '239')  
      writeToFile('/sys/class/gpio/gpio239/direction', 'out')
    end,
    setSim = function (sim)
		-- sim = '1' - main (pin=0) 
		-- sim = '0' - aux  (pin=1)
      writeToFile('/sys/class/gpio/gpio239/value', sim == mainSimNumber and '0' or '1')
    end,
    blacklist = {"rs-232", "multisensor", "2com", "eth", "printer",  "wimax", "eth-703", },
  },
  ["usb1"] = {
    HS_path = "1-1.2",
    FS_path = "1-1.2",
    powerOn = function () 
      nsg.hubcontrol("/dev/hub1", 2, "ON")
    end,
    powerOff = function () 
      nsg.hubcontrol("/dev/hub1", 2, "OFF")
    end,
    setGpio = function ()
    end,
--  setSim - unimplemented function
  },
  ["m2"] = NSG1820_USB_1_3,
  ["3g"] = NSG1820_USB_1_3,
  ["cdma"] = {
    HS_path = "1-1.4",
    FS_path = "1-1.4",
    powerOn = function () 
      nsg.hubcontrol("/dev/hub1", 4, "ON")
    end,
    powerOff = function () 
      nsg.hubcontrol("/dev/hub1", 4, "OFF")
    end,
    setGpio = function () end,
--  setSim - unimplemented function
  },
}
------------------------------------
local NSG600_USB_2 = {
    FS_path = "1-2",
    powerOn = function () 
      writeToFile('/sys/class/gpio/gpio9/value', '1')
    end,
    powerOff = function () 
      writeToFile('/sys/class/gpio/gpio9/value', '0')
    end,
    setGpio = function ()
      writeToFile('/sys/class/gpio/export', '9')
      writeToFile('/sys/class/gpio/export', '12')
      writeToFile('/sys/class/gpio/gpio9/direction', 'out')
      writeToFile('/sys/class/gpio/gpio12/direction', 'out')
    end,
    setSim = function (sim)
      writeToFile('/sys/class/gpio/gpio12/value', sim)
    end,
  }

local NSG600_Platform = {   
   ["usb1"] = {
    FS_path = "1-1",
    powerOn = function () 
      writeToFile('/sys/class/gpio/gpio15/value', '0')
    end,
    powerOff = function () 
      writeToFile('/sys/class/gpio/gpio15/value', '1')
    end,
    setGpio = function ()
      writeToFile('/sys/class/gpio/export', '15')
      writeToFile('/sys/class/gpio/gpio15/direction', 'out')
    end,
--  setSim - unimplemented function
  },
  ["3g"] = NSG600_USB_2, 
  ["edge"] = NSG600_USB_2,
  ["edge-3a"] = NSG600_USB_2,
  ["cdma"] = NSG600_USB_2,
}
------------------------------------
-- Save local variable (SIM-value) to use during "start"
local NSG700_s2_SIM = 0 
-- "s1" control pins DTR - power; RTS - sim-selector
-- kernel 2.6.32  DTR(gpio66), RTS(gpio53) (due to PIN_BASE = 32)
-- kernel 3.4.x   DTR(gpio34), RTS(gpio21) (due to PIN_BASE is abcent)

local NSG700_USB_1_4 = { -- dual-interface device (#0, #1)
    FS_path = "1-1.4",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    --  setSim - unimplemented function
}

local NSG700_Platform = {   
  ["s1"] = {
    FS_path = "1-1.1",
    powerOn = function ()
      writeToFile('/sys/class/gpio/gpio34/value', '0')
    end,
    powerOff = function ()
      writeToFile('/sys/class/gpio/gpio34/value', '1')
    end,
    setGpio = function ()
      writeToFile('/sys/class/gpio/export', '34')   -- PIO_PB2
      writeToFile('/sys/class/gpio/gpio34/direction', 'out')
      writeToFile('/sys/class/gpio/export', '21')   -- PIO_PA21
      writeToFile('/sys/class/gpio/gpio21/direction', 'out')
    end,
    setSim = function (sim)
      writeToFile('/sys/class/gpio/gpio21/value', sim)   -- ???
    end,
  },
  ["usb1"] = {
    FS_path = "1-1.2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
--  setSim - unimplemented function
  },
  ["s2"] = {
    FS_path = "1-1.3",
    powerOn = function () 
		 nsg.modem_pair(port2tty["s2-aux"], 1, (1 - NSG700_s2_SIM))
    end,
    powerOff = function () 
      nsg.modem(port2tty["s2-aux"], "DTR", 0)
    end,
    setGpio = function ()
      local path = {"s2-aux", } -- path emulation
      probeUSBDeviceScheme(nil, path, 1) -- first iface ! (set real tty to table)
    end,
    setSim = function (sim) 
		-- sim = '1' - main (then RTS=Off (0)) 
		-- sim = '0' - aux  (then RTS=On  (1))
		NSG700_s2_SIM = sim  -- save value and to do nothing
    end,
  },
  ["1-wire"] = NSG700_USB_1_4,  -- using interface #0
  ["s2-aux"] = NSG700_USB_1_4,  -- using interface #1
}
local NSG700_s2_SIM = nil -- not accepted more

------------------------------------------
------  NSG1000_001a/NSG1000_002a  -------
------------------------------------------
local NSG1000_001a_Platform = {
  ["usb1"] = { 
    HS_path = "1-2",
    FS_path = "2-2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb2"] = {
    HS_path = "1-1",
    FS_path = "2-1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb3"] = {
    HS_path = "1-4",
    FS_path = "3-2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb4"] = {
    HS_path = "1-3",
    FS_path = "3-1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
}

local NSG1000_002a_Platform = NSG1000_001a_Platform

------------------------------------------
------  NSG1000_001d/NSG1000_002d  -------
------------------------------------------
local NSG1000_001d_Platform = {
  ["usb1"] = { 
    HS_path = "1-2",
    FS_path = "2-2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb2"] = {
    HS_path = "1-1",
    FS_path = "2-1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb3"] = {
    HS_path = "1-4",
    FS_path = "3-2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb4"] = {
    HS_path = "1-3",
    FS_path = "3-1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["c1-1"] = {
    HS_path = "1-6.1",
    FS_path = "1-6.1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c1-2"] = {
    HS_path = "1-5.1",
    FS_path = "1-5.1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c2-1"] = {
    HS_path = "1-6.2",
    FS_path = "1-6.2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c2-2"] = {
    HS_path = "1-5.2",
    FS_path = "1-5.2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c3-1"] = {
    HS_path = "1-6.3",
    FS_path = "1-6.3",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c3-2"] = {
    HS_path = "1-5.3",
    FS_path = "1-5.3",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c4-1"] = {
    HS_path = "1-6.4",
    FS_path = "1-6.4",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c4-2"] = {
    HS_path = "1-5.4",
    FS_path = "1-5.4",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
}

local NSG1000_002d_Platform = NSG1000_001d_Platform

-----------------------------------
-------  NSG1000_003a -------------
-----------------------------------
local NSG1000_003a_Platform = {
  ["usb1"] = {
    HS_path = "1-1",
    FS_path = "2-1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb2"] = {
    HS_path = "1-2",
    FS_path = "2-2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb3"] = {
    HS_path = "1-4",
    FS_path = "3-2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb4"] = {
    HS_path = "1-3",
    FS_path = "3-1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb5"] = {
    HS_path = "1-8",
    FS_path = "5-2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb6"] = {
    HS_path = "1-7",
    FS_path = "5-1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
}

-----------------------------------
-------  NSG1000_005b -------------
-----------------------------------
local NSG1000_005b_Platform = {
  ["usb1"] = {
    HS_path = "1-5",
    FS_path = "4-2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb2"] = {
    HS_path = "1-6",
    FS_path = "4-3",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb3"] = {
    HS_path = "1-3",
    FS_path = "3-3",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb4"] = {
    HS_path = "1-4",
    FS_path = "4-1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb5"] = {
    HS_path = "1-1",
    FS_path = "3-1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb6"] = {
    HS_path = "1-2",
    FS_path = "3-2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },

  ["c1-1"] = {
    HS_path = "2-2.1",
    FS_path = "2-2.1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c1-2"] = {
    HS_path = "2-1.1",
    FS_path = "2-1.1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c2-1"] = {
    HS_path = "2-2.2",
    FS_path = "2-2.2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c2-2"] = {
    HS_path = "2-1.2",
    FS_path = "2-1.2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c3-1"] = {
    HS_path = "2-2.3",
    FS_path = "2-2.3",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c3-2"] = {
    HS_path = "2-1.3",
    FS_path = "2-1.3",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c4-1"] = {
    HS_path = "2-2.4",
    FS_path = "2-2.4",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c4-2"] = {
    HS_path = "2-1.4",
    FS_path = "2-1.4",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c5-1"] = {
    HS_path = "2-4.1",
    FS_path = "2-4.1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c5-2"] = {
    HS_path = "2-3.1",
    FS_path = "2-3.1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c6-1"] = {
    HS_path = "2-4.2",
    FS_path = "2-4.2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c6-2"] = {
    HS_path = "2-3.2",
    FS_path = "2-3.2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c7-1"] = {
    HS_path = "2-4.3",
    FS_path = "2-4.3",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c7-2"] = {
    HS_path = "2-3.3",
    FS_path = "2-3.3",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c8-1"] = {
    HS_path = "2-4.4",
    FS_path = "2-4.4",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c8-2"] = {
    HS_path = "2-3.4",
    FS_path = "2-3.4",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
}

-----------------------------------
-------  NSG1000_006a -------------
-----------------------------------
local NSG1000_006a_Platform = {
  ["usb1"] = {
    HS_path = "1-8",
    FS_path = "5-2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb2"] = {
    HS_path = "1-7",
    FS_path = "5-1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb3"] = {
    HS_path = "1-3",
    FS_path = "3-1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb4"] = {
    HS_path = "1-4",
    FS_path = "3-2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb5"] = {
    HS_path = "1-5",
    FS_path = "4-1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb6"] = {
    HS_path = "1-6",
    FS_path = "4-2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
}

------------------------------------------------
-------  NSG1000_007a/NSG1000_007b -------------
------------------------------------------------
local NSG1000_007a_Platform = {
  ["usb1"] = {
    HS_path = "1-3",
    FS_path = "3-1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb2"] = {
    HS_path = "1-4",
    FS_path = "3-2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb3"] = {
    HS_path = "1-5",
    FS_path = "4-1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb4"] = {
    HS_path = "1-6",
    FS_path = "4-2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb5"] = {
    HS_path = "1-8",
    FS_path = "5-2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb6"] = {
    HS_path = "1-7",
    FS_path = "5-1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
}

local NSG1000_007b_Platform = NSG1000_007a_Platform

-----------------------------------
-------  NSG1000_008a -------------
-----------------------------------
local NSG1000_008a_Platform = {
  ["usb1"] = {
    SS_path = "4-4",
    HS_path = "3-4",
    FS_path = "3-4",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb2"] = {
    SS_path = "4-3",
    HS_path = "3-3",
    FS_path = "3-3",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb3"] = {
    HS_path = "2-1.7",
    FS_path = "2-1.7",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb4"] = {
    HS_path = "1-1.5",
    FS_path = "1-1.5",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb5"] = {
    HS_path = "1-1.1",
    FS_path = "1-1.1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },
  ["usb6"] = {
    HS_path = "1-1.2",
    FS_path = "1-1.2",
    powerOff = function () end,
    setGpio = function () end,
    blacklist = NSG1000USB_BLACKLIST,
  },

  ["c1-1"] = {
    HS_path = "2-1.5.1",
    FS_path = "2-1.5.1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c1-2"] = {
    HS_path = "2-1.6.1",
    FS_path = "2-1.6.1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c2-1"] = {
    HS_path = "2-1.5.2",
    FS_path = "2-1.5.2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c2-2"] = {
    HS_path = "2-1.6.2",
    FS_path = "2-1.6.2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c3-1"] = {
    HS_path = "2-1.5.3",
    FS_path = "2-1.5.3",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c3-2"] = {
    HS_path = "2-1.6.3",
    FS_path = "2-1.6.3",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c4-1"] = {
    HS_path = "2-1.5.4",
    FS_path = "2-1.5.4",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c4-2"] = {
    HS_path = "2-1.6.4",
    FS_path = "2-1.6.4",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c5-1"] = {
    HS_path = "1-1.3.1",
    FS_path = "1-1.3.1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c5-2"] = {
    HS_path = "1-1.4.1",
    FS_path = "1-1.4.1",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c6-1"] = {
    HS_path = "1-1.3.2",
    FS_path = "1-1.3.2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c6-2"] = {
    HS_path = "1-1.4.2",
    FS_path = "1-1.4.2",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c7-1"] = {
    HS_path = "1-1.3.3",
    FS_path = "1-1.3.3",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c7-2"] = {
    HS_path = "1-1.4.3",
    FS_path = "1-1.4.3",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c8-1"] = {
    HS_path = "1-1.3.4",
    FS_path = "1-1.3.4",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
  ["c8-2"] = {
    HS_path = "1-1.4.4",
    FS_path = "1-1.4.4",
    powerOn = function () end,
    powerOff = function () end,
    setGpio = function () end,
    blacklist = KRO1000USB_BLACKLIST,
  },
}

-------------------------------------------------

local  Platform =
((BI.arch == 'nsg600') and NSG600_Platform) or
((BI.arch == 'nsg700') and NSG700_Platform) or
((BI.arch == 'nsg1800') and NSG1800_Platform) or
((BI.arch == 'nsg1820') and NSG1820_Platform) or
((BI.env.nsg_device == 'NSG1000_001a') and NSG1000_001a_Platform) or
((BI.env.nsg_device == 'NSG1000_001d') and NSG1000_001d_Platform) or
((BI.env.nsg_device == 'NSG1000_002a') and NSG1000_002a_Platform) or
((BI.env.nsg_device == 'NSG1000_002d') and NSG1000_002d_Platform) or
((BI.env.nsg_device == 'NSG1000_003a') and NSG1000_003a_Platform) or
((BI.env.nsg_device == 'NSG1000_005b') and NSG1000_005b_Platform) or
((BI.env.nsg_device == 'NSG1000_006a') and NSG1000_006a_Platform) or
((BI.env.nsg_device == 'NSG1000_007a') and NSG1000_007a_Platform) or
((BI.env.nsg_device == 'NSG1000_007b') and NSG1000_007b_Platform) or
((BI.env.nsg_device == 'NSG1000_008a') and NSG1000_008a_Platform) or
{} -- at the end of comparison


-------------------------------------------------
isWirelessPort = {
  ["3g"] = true,
  ["edge"] = true,
  ["edge-3a"] = true,
  ["cdma"] = true,
}

port2tty = {
  ["3g"] = "/dev/ttyUSB0",
  ["edge"] = "/dev/ttyACM0",
  ["edge-3a"] = "/dev/ttyUSB0",
  ["cdma"] = "/dev/ttyACM0",
  ["rs-232"] = "/dev/ttyAM0",
  ["1-wire"] =  "/dev/ttyAM1",   --++ static for NSG-600 (not autodetected, overwrite in NSG-700, )
}
setmetatable(port2tty, {__index = function()return''end})

port2netdev = {
  ["wlan0"] = "wlan0",  -- compatibility to NSG-600 (port "wlan0")
-- compatibility to NSG-1820 (port "wlan0")
--  ["m1"] =    "wlan0",  
}
-------------------------------------

local function IsDevice(name) 
  if name == "." or 
  name == ".." or 
  string.find(name, "usb") or
  string.find(name, ":") 
  then
    return false
  end
  return true
end

local function GetValue (dir, name)
  local f = io.open(dir .. name)
  if f then
     local s = tostring(f:read())
     f:close()
     return s or "NONE"
  end
  return "NONE"
end
-------------------------------------
-- kind  : type of devices (only "tty", "usb", "net")
-- iface : full path to usb-interface dir (i.e. "/sys/bus/usb/devices/1-2:1.0")
-- fname : instace of "file name" in the interface-dir 
-- return: "real" node name (ttyUSB0) or nil    

local function FindNodeName(fname, iface, kind)
   if fname == kind then -- new style sysfs, look dir
      for dev in lfs.dir(iface .. "/" .. kind) do
	 if dev ~= "." and dev ~= ".." then
	    return dev
	 end
      end
   else  -- old style sysfs (i.e. tty:ttyUSB0)
      return string.gsub(fname, "^" .. kind .. ":", "") 
   end
   return nil
end
-------------------------------------
local function SetFeature(module, name)
  local head = name .. ":" 
  module.netdevice = nil
  module.ttydevice = {}  
  for entry in lfs.dir(BASE_DIR) do
    if (string.find(entry, head)) then
      local iface = BASE_DIR .. "/" .. entry
      for fname in lfs.dir(iface) do
        if (string.find(fname, "tty")) then
	   module.ttydevice[#module.ttydevice + 1] = FindNodeName(fname, iface, "tty") -- wireless (char-type)
	end
        if (string.find(fname, "usbmisc")) then
	   module.ttydevice[#module.ttydevice + 1] = FindNodeName(fname, iface, "usbmisc") -- i.e. printers
        end
        if (string.find(fname, "net")) then  
	   module.netdevice = FindNodeName(fname, iface, "net") -- i.e. "wifi", "LTE", "USB-Netcard"
        end
      end
    end
  end
end
-------------------------------------
function FindDevice(port)
   if not (port and Platform[port]) then
      return nil  -- item not present
   end
  for de in lfs.dir(BASE_DIR) do
    local name = de
    if IsDevice(name) then
      local item = Platform[port]
      if (name == item.FS_path) or (name == item.HS_path) or (name == item.SS_path) then
        local place = BASE_DIR .. "/" .. name
	local devdir =  place .. "/"
	local ifdir = place .. ":1.0/"

	local vidpid =  GetValue(devdir, "idVendor") .. "/" .. GetValue(devdir, "idProduct")
	local clsubcl =  GetValue(ifdir, "bInterfaceClass") .. "/" .. GetValue(ifdir, "bInterfaceSubClass")
	local obj = s.deepcopy(modules[vidpid] or modules[clsubcl])

	-- check CDMA(mode=5) & mass-storage conflict 
	if vidpid == "16d8/680a" then
	   os.execute("rmmod usb-storage")
	end
	--
	if obj then
	  obj.manufacturer = GetValue(devdir, "manufacturer")
	  obj.product = GetValue(devdir, "product")
	  obj.ifdir = ifdir
	  obj.vidpid = vidpid
          SetFeature(obj, name)
          return obj
        else
          return {} -- to return "unknown" device {} when we haven't this vid/pid
        end
      end
    end
  end
  -- to return "nil"  when NO device in the slot 
end
------------------------------------
local um2comScheme = function(_, path)
  local name = path[#path]
  return s.newScheme {
     _Z =  function() return true end,
    [name..'.0'] = rs232.rs232Scheme,
    [name..'.1'] = rs232.rs232Scheme,
  }			
end

------------------------------------
local type2scheme = {
  ["eth"] = { 
     scheme = eth.ethScheme{
	_Z = function(path, sch, conf) 
		probeUSBDeviceScheme(nil, path)
		return eth.ethScheme._Z(path, sch, conf) -- iface.interfaceMainApply(path, sch, conf) 
	     end, 
     },
  },
  ["lte"] = { 
     scheme = lte.LTEScheme,
  },
  ["edge"] = {
     scheme = port.portScheme,
     options = { ['wirelessType'] = "edge", ['simNumber'] = "2", ["smsType"]= "+++",},
  },
  ["edge-3a"] = {
     scheme = port.portScheme,
     options = { ['wirelessType'] = "edge", ['simNumber'] = "2", ["smsType"]= "aux",},
  },
  ["cdma"] = { 
     scheme = port.portScheme,
     options = { ['wirelessType'] = "cdma", ['simNumber']= "1", },
  },
  ["3g"] = {
     scheme = port.portScheme,
     options = { ['wirelessType'] = "3g", ['simNumber'] = "2", ["smsType"]= "aux",},
  }, 
  ["2com"] = {
     scheme = um2comScheme,
  },

  ["wifi"] = {
     scheme = WiFi.WiFiScheme,
  },
  
  ["rs-232"] = {
     scheme = function(_, path)
		 local dev = port2tty[path[#path]] or ""
		 if BI.kcmdline:find("console=" .. (dev:match("^/dev/(.*)") or dev) .. ",") then
		    return rs232.asyncConsoleScheme
		 else
		    return rs232.rs232Scheme
		 end
	      end,
  },

  ["gsm-2sim"] = {
     scheme = rs232.rs232Scheme{
	_Z = function(path, sch, conf) 
		C.setOption(path, 'controlledPower', nil)
		C.setOption(path, 'simNumber', "2")
		return rs232.rs232Scheme._Z(path, sch, conf) 
	     end,
     },
  },

  ["multisensor"] = {
     scheme = sens.ms6Scheme,
  },

  ["printer"] = {
     scheme = prn.printerScheme,
  },

  ["storage"] = {
     scheme = stor.storageScheme {_DEP = 0, },
  },

  ["wimax"] = {
     scheme = wmxd.wimaxScheme,
  },

  ["eth-703"] = {
     scheme = eth703.eth703Scheme,
  },

  ["mercury"] = {
     scheme = mercury230.mercury230scheme,
  },

  ["empty"] = {
     scheme = s.newScheme {},
  },
}

------------------------------------
function probeUSBDeviceScheme(_, path, interface)
  local portName = path[#path]
  local device = FindDevice(portName)

  if device then
    if(device.netdevice) then
      iface.addIfaceAlias( portName, device.netdevice)
      port2netdev[portName] = device.netdevice
    end
    if(device.ttydevice) then
       if interface then -- get only this interface 
	  if device.ttydevice[interface] then -- (if present) and NO use a "suffix" 
	     port2tty[portName] = '/dev/' .. device.ttydevice[interface]
	     return device.type
	  end
       else
	  for i, v in pairs(device.intf or {}) do -- get all "childs" and use "suffix"
	     if device.ttydevice[i] then
		port2tty[portName .. device.intf[i]] = '/dev/' .. device.ttydevice[i]
	     end
	  end
       end
       isWirelessPort[portName] = true
    else
      port2tty[portName] = nil
      isWirelessPort[portName] = nil
    end
    return device.type
  end
end
------------------------------------
local function SetPrivateOptions(path, type)
   for opt, val in pairs(type2scheme[type or "empty"].options or {}) do
      C.setOption(path, opt, val)
   end
end
------------------------------------
local delayedApply
delayedApply = function(path, sch, conf, delay)
			local type = probeUSBDeviceScheme(nil, path)
			if type then
			   SetPrivateOptions(path, type)
			   return C.applyConf(path, nil, true) -- Apply with recursive!
			else
			   local delay = 3
			   C.log_info("*** Device (%s) not ready, trying after %d sec ", path[#path], delay)   
			   C.timer:add(delay, delayedApply, path, sch, conf, delay) -- Apply after
			   return true
			end
		     end
------------------------------------
local function allowModules(path) 
   local item = Platform[path[#path]]
   local allowTypes = {} 
   if item then
      for _, type in pairs(allTypes) do
	 local allow = true
	 for _, v in pairs(item.blacklist or {}) do
	    if v == type then
	       allow = nil
	       break
	    end
	 end
	 allowTypes[#allowTypes + 1] = allow and type 
      end
   else
      allowTypes[#allowTypes + 1] = "empty"
   end
   return allowTypes
end
------------------------------------
do -- private for probeFixDevices()
--
local NSG700_Async_Slots = {
  ["s1"] = function() return "/dev/ttyAT1" end,
  ["s2"] = function() return probeUSBDeviceScheme(nil, {"s2-aux"}, 1) and port2tty["s2-aux"] end,
}

local getAsyncDevAndTag = function (devs)
  if type(devs) == "string" then
    devs = {devs,}
  end
  if type(devs) ~= "table" then
    return nil
  end
  local dev, tag
  for _, v in ipairs(devs) do
    if type(v) == "string" then
      dev = v
      if BI.kcmdline:find("console=" .. (dev:match("^/dev/(.*)") or dev) .. ",") then
        tag = "async-console"
        break
      end
    end
  end
  return dev, dev and (tag or "rs-232") -- return: async device name, async scheme tag
end

probeFixDevices = function (path, fixed)
  local portName = path[#path]
  if type(portName) ~= "string" then
    return nil
  end
  local conf = C.getConfByPath(path) or {}
  if (BI.arch == "nsg700") and ((conf.type == "rs-232") or fixed) and portName:find("^s[12]$") then
    port2tty[portName] = NSG700_Async_Slots[portName] and NSG700_Async_Slots[portName]()
    return (port2tty[portName] ~= "") and (fixed or conf.type) -- return the type if port present
  end
  if fixed and type(BI.fixed_asyncs) == "table" and portName:find("^a%d+$") then
    local tag
    port2tty[portName], tag = getAsyncDevAndTag(BI.fixed_asyncs[portName])
    return tag -- return: async scheme tag / nil
  end
  return nil
end
--
end -- private for probeFixDevices()
------------------------------------
function portTemplateFunction(_, path)
  local type = C.getConfByPath(s.clonePath(path, nil, "type")) or 'empty'

  C.setOption(path, 'controlledPower', "pin")

  local portTemplate = {
     _H = "portTemplate",

     _Z = function(path, sch, conf) 
	     if conf and conf.type ~= "empty" then 
		local realtype = probeUSBDeviceScheme(nil, path) or probeFixDevices(path)
		if realtype == conf.type then
		   SetPrivateOptions(path, realtype)
		   return  (type2scheme[type].scheme(nil, path)._Z or 
		   (function()return true end))(path, sch, conf)  --- Apply real
		else
		   local item = Platform[path[#path]]  
		   if item then 
		      item.setGpio()
		      item.powerOn()  --- PowerON
		      C.timer:add(3, delayedApply, path, sch, conf, 3) -- Apply after
		   end
		end
	     end
	     return true
	  end,

     _P = -10, -- autodetected (removable) ports are below "fixed" in menu
     type = function(_, path)
	       return s.choiceType{
		  _H = "portType",
		  _A = allowModules(s.clonePath(path, -1)),
		  _D = "empty",
		  _P = 99,
		  _V = function (...)
			  local type = s.choiceVal(...)
			  if type and type2scheme[type] then
			     local path = s.clonePath(path, -1)
			     SetPrivateOptions(path, type)
			  end
			  return  s.choiceVal(...)
		       end,
		  _DEP = 1, 
	       }
	    end,

    update = s.newScheme{
      _H = "portUpdate",
      _X = function(path, sch, val)
        local path = s.clonePath(path, -1)
        local item = Platform[path[#path]]
        if item then 
          item.setGpio()
          -- C.threadSleep(1)
          item.powerOn()
        end
        local newtype = probeUSBDeviceScheme(nil, path) or 'empty'
        C.setConfigValue(path, { type = newtype, } )
	SetPrivateOptions(path, newtype)
        C.cleanDaemonLog(path)
        return
      end,
      _P = -999, -- Below all params (to prevent erroneous "update" instead of "apply")
      _DEP = 1,
    },

  }

  return type2scheme[type or 'empty'].scheme(nil, path)(portTemplate)
end
------------------------------------
function getOptionRecursive(path, option)
  local opt = C.getOption(path, option)
  local path = path
  if opt then
    return opt
  end
  path = s.clonePath(path, -1)
  if #path == 0 then
    return nil
  else
    return getOptionRecursive(path, option)
  end
end
------------------------------------

setGPIO = 
function(path) 
  local item = Platform[path[#path]]
  return item and item.setGpio()
end
------------------------------------

togglePower = 
function(path)
  local item = Platform[path[#path]]
  if item then
    item.powerOff()
    C.threadSleep(1)
    item.powerOn()
  end
end 
------------------------------------
mainSimNumber = '1'
auxSimNumber = '0'

setSim = 
function (path, n)
  local item = Platform[path[#path]]
  C.log_info("Set SIM %s(%d)", n == mainSimNumber and "MAIN" or "AUX", n)
  return item and item.setSim and item.setSim(n)
end
------------------------------------
powerOff = 
function(path)
  local item = Platform[path[#path]]
  if item then
    item.powerOff()
  end
end
------------------------------------
initialPowerOnAllSlots =  function()
			     C.log_info("initialPowerOnAllSlots")
			     for _, item in pairs(Platform) do    -- for all ports 
				if item.setGpio and item.powerOn then
				   item.setGpio()
				   item.powerOn()
				end
			     end
			  end
------------------------------------
getPartitionName = function(path)
  local device = FindDevice(path[#path])
  local parts = {}
  if device then
     local content = s.readpopen("find " .. device.ifdir .. " -name block:sd* 2>/dev/null")

     local _, _, disk = content:find(".+block:(.+)\n")                       -- find a name of disk (i.e. "sda")

     if not disk then -- try find in new style (dir)
	local cnt = s.readpopen("find " .. device.ifdir .. " -name block 2>/dev/null")
	cnt = string.gsub(cnt, "\n", "/") 
	for fname in lfs.dir(cnt) do
	   if fname ~= "." and fname ~= ".." then
	      disk = fname
	   end
	end
     end
     if disk then 
	content = s.readpopen("echo /sys/block/" .. disk .. "/" .. disk .. "*") -- out all partitions ("sda1", ...)

	for w in content:gmatch("[%S]+/([%S]+)%s") do
	   parts[#parts + 1] = w       -- store name of the partitions setparately
	end	
     return parts
     end
  else
     -- NOT usb-storage partitions (i.e. MMC, etc.)
     for fname in lfs.dir("/dev") do
	local card, part = fname:match("^mmcblk(%d)p(%d)")  -- check the name i.e. "mmcblk0p1"
	if card then
	   local b =  s.readpopen("stat -c %F /dev/" .. fname) -- check block attribute
	   if b:match("^block .+") then   
	      local maj = string.gsub(s.readpopen("stat -c %t /dev/" .. fname), "\n", "")
	      local min = string.gsub(s.readpopen("stat -c %T /dev/" .. fname), "\n", "")
	      if (maj == "b3") and (min == part) then   -- maj=179 (mmc) min = part_num 
		 parts[#parts + 1] = fname
	      end
	   end
	end
     end
     return parts
  end
  return nil
end

showAliasScheme = s.newScheme{
   _H = "Show system's aliases (nodes, interfaces, etc.) ",
   port_to_tty = s.newScheme {
      _X = function()
	      local out = ("%-15s\t%-15s\n\n"):format("PORT","TTY-NODE")
	      for i,v in pairs(port2tty) do
		 out = out .. ("%-15s\t%-15s\n"):format(i, v) 
	      end
	      return '\n'..out 
	   end,
   },
   interface_to_alias = s.newScheme {
      _X = function()
	      local out = ("%-20s\t%-20s\n\n"):format("ALIAS","INTERFACE")
	      for i,v in pairs(iface.ifaceAliasesGlobal) do
		 out = out .. ("%-20s\t%-20s\n"):format(i, v) 
	      end
	      return '\n'..out
	   end,
   },
}
