module(..., package.seeall)
local s = require'scheme'
local usb = require'nsgusbconf'
local eth = require'nsgethconf'
local iface = require'nsgifaceconf'

-- priority 
local WIMAXADMSTATEPRIO      = 40


local function restartWimax(path, sch, conf, prevConf)
   local conf = s.addDefault(path, sch, conf, true) --clone conf with defaults
   local CmdStart = {'/sbin/madwimax', '-n', path[#path], }  

   if conf["debug-level"] > 0 then
      CmdStart[#CmdStart+1] =  '-'..("v"):rep(conf["debug-level"]) 
   end

   if conf["adm-state"] == 'up' then
      C.restartDaemon(path, CmdStart, { atExit = C.increasingSleep, })
      return eth.ethScheme._Z(path, sch, conf) 
   end
   C.restartDaemon(path)
   return eth.ethScheme._Z(path, sch, conf)
end
--------------------------
local WIMAXSHOWINFOPRIO = 50
local WIMAXSHOWLOGRIO   = 40

local wimaxShow = s.newScheme {
   _H = "wimaxShow",

   ["log"] =  s.newScheme{
      _H = "wimaxShowLog",
      _X = function(path)
	      local path = s.clonePath(path, -2)
	      return C.getDaemonLog(path)
	   end,
      _P =  WIMAXSHOWLOGRIO,
      _R = 'user',
   },  

   ["info"] = s.newScheme{
      _H = "wimaxShowInfo",
      _X = function(path)
	      local path = s.clonePath(path, -2)
	      local type = usb.probeUSBDeviceScheme(nil, path)
	      local out = "Device:\t\t" 
	      
	      if type == "wimax" then
		 out = out .. "CONNECTED\n"
		 local device = usb.FindDevice(path[#path])
		 out = out .. "Manufacturer:\t" .. (device.manufacturer or "UNKNOWN") .. "\n"
		 out = out .. "Model:\t\t" .. (device.product or "UNKNOWN") .. "\n"
	      else
		 out = out .. "NOT CONNECTED\n"
	      end
	      return out
	   end,
      _P = WIMAXSHOWINFOPRIO,
   },
   ["interface"] = s.newScheme{
      _H = "wimaxShowInterface",
      _R = 'user',
      _X = function(path)
              return iface.showInterfaceStat(path[#path - 2])
           end,
   },
}

--------------------------

wimaxScheme = eth.ethScheme {
   _Z = restartWimax,

   ["adm-state"] = s.newScheme{
      _T = "choice",
      _A = {"up", "down"},
      _V = s.choiceVal,
      _H = "portAdmState",
      _D = "down",
      _P = WIMAXADMSTATEPRIO,
   },

   ["show"] = wimaxShow,

   ["debug-level"] = s.newScheme{
      _H = "debugLevel",
      _T = "number",
      _A = {0, 2},
      _V = s.integerVal,
      _D = 0,
      _P = 5,
   },

}

