local uita = require'uita'

local getServersFromConfig = function() return false end -- TODO

local getServersFromFile = function()
  local res, f = pcall(loadfile, "/etc/uitcpservers")
  if res then res, f = pcall(f) end
  if res then return f end
end

local servers = getServersFromConfig() or getServersFromFile()

if not servers then
  return uita
end

local M = {}
local nsg = require'nsg'
nsg.meta()

local readpopen = function(prog)
  local f = io.popen(prog)
  local s
  if f then
    s = f:read("*a")
    f:close()
  end
  return s
end

local evaluate = function (str)
  if not str then return end
  local f = loadstring("return"..str)
  if not f then return end
  local res, conf = pcall(f)
  if not res then return end
  return conf
end

local get = function(str)
  return (evaluate(readpopen(str)) or {})
end

local SSHCOM = "ssh -o ConnectTimeout=3 root@%s 'uigetstat %s'"

--[[
local tunlist = function() -- TODO
  local t
  local tuns = {}
  for ser, host in pairs(servers) do
    if host == 'local' then
      t = uita.tun()
    else
      t = get(SSHCOM%{host, '-l'})
    end
    for i = 1, #t do
      tuns[#tuns+1] = t[i]..'@'..ser
    end
  end
  return tuns
end
--]]

local stateIdx 
do
  local _ = {
    dead = 0,
    suspended = 1,
    normal = 2,
  }
  stateIdx = function(state)
    return _[state]
  end 
end

local split = function(tun)
  return tun:match"^(.-)@?([^@]*)$"
end

local con = function(tun)
  local t
  local tuns = {}
  local bestServer = {}
  if tun then
    tun = split(tun)
  end
  for ser, host in pairs(servers) do
    if host == 'local' then
      t = uita.con(tun)
    else 
      t = get(SSHCOM%{host, '-c '..(tun and "-t"..tun or "")})
    end
    for tun, stat in pairs(t) do
      local oldstat = tuns[tun] and tuns[tun].currState or "dead"
      if stateIdx(stat.currState) > stateIdx(oldstat) then
	tuns[tun] = stat
	bestServer[tun] = ser
      end
    end
  end
  local cons = {}
  for tun, stat in pairs(tuns) do
    cons[tun..'@'..bestServer[tun]] = stat
  end
  return cons
end
M.con = con

M.tun = function()
  local besttun = con()
  local all = {}
  for k, _ in pairs(besttun) do
    all[split(k)] = true
  end

  for ser, host in pairs(servers) do
    if host == 'local' then
      t = uita.tun()
    else
      t = get(SSHCOM%{host, '-l'})
    end
    for _, v in ipairs(t) do
      if not all[v] then
	all[v] = true
	besttun[v..'@'..ser] = true
      end
    end
  end

  local res = {}
  for k, _ in pairs(besttun) do
    res[#res+1] = k
  end
  return res
end

M.stat = function(tun)
  local t
  if tun then
    local tunname, server = split(tun)
    if servers[server] == 'local' then
      t = uita.stat(tunname)
    else
      t = get(SSHCOM%{servers[server], '-s -t'..tunname})
    end
    t.tun[tun] = t.tun[tunname]
    t.tun[tunname] = nil
    return t
  else -- get all stats
    local stats = {}
    local common = {tun=stats}
    for ser, host in pairs(servers) do
      if host == 'local' then
	t = uita.stat()
	for k, v in pairs(t) do
	  if k ~= 'tun' then
	    common[k] = v
	  end
	end
      else
	t = get(SSHCOM%{host, '-s'})
      end
      for tun, stat in pairs(t.tun) do
	stats[tun..'@'..ser] = stat
      end
    end
    return common
  end
end

return M
