module(..., package.seeall)

--[[
_H = "uitcp_current"
_H = "uitcp_previous"
_H = "uitcp_last24Hours"
_H = "uitcp_fromStart"
_H = "uitcp_interval"
_H = "uitcp_currentInt"
_H = "uitcp_recv"
_H = "uitcp_send"
_H = "uitcp_bytes"
_H = "uitcp_pack"
_H = "uitcp_err"
_H = "uitcp_noSuchObject"
_H = "uitcp_peerHost"
_H = "uitcp_peerPort"
_H = "uitcp_direction"
_H = "uitcp_host"
_H = "uitcp_port"
_H = "uitcp_systemUsage"
_H = "uitcp_tunnelStates"
_H = "uitcp_total"
_H = "uitcp_normal"
_H = "uitcp_dead"
_H = "uitcp_suspended"
_H = "uitcp_summaryDataTransferStats"
_H = "uitcp_dataTransferStats"
_H = "uitcp_ifaceUsageStats"
_H = "uitcp_tunnelStatesStats"
_H = "uitcp_link"
_H = "uitcp_duration"
_H = "uitcp_quantity"
_H = "uitcp_none"
_H = "uitcp_tunnelParameters"
_H = "uitcp_state"
_H = "uitcp_peer"
_H = "uitcp_currStateTime"
_H = "uitcp_time"
_H = "uitcp_csq"
_H = "uitcp_from"
_H = "uitcp_to"
--]]

local utf8 = require"utf8"

readpopen = function (prog)
  local f = io.popen(prog)
  local s
  if f then
    s = f:read("*a")
    f:close()
  end
  return s
end

local version = readpopen("{ cat /version 2>/dev/null || date; } | sed  -r 's/[^[:alnum:]._-]+//g' | tr -d '\\n'") or tostring(math.random())
local replaceNil
local gen_interval_stats
local htmlEncode

local fixCRLF =
   function (s)
      return s~=nil and  tostring(s):gsub("\n( *)(\t?)", function(s,t)
                                                            return'\n'..(t~=''and('\t'):rep(#s+1)or'')
                                                         end or ""
                                       )
   end

local splitString =
   function (s)
      s = s or ""
      local out = {}
      local i
      while #s>0 do
         i = utf8.find(s,"%s",70) or 80
         out[#out+1] = s:sub(1,i)
         s = s:sub(i + 1)
      end
      return out
   end

local escapeForScript =
   function (s)
      s = splitString(s)
      for k,v in ipairs(s) do
         s[k] = '"'..((((((tostring(v or "" )):gsub("\\","\\\\")):gsub("\r","\\r")):gsub("\n","\\n")):gsub('"','\\"')):gsub("'","\\'") or "")..'"'
      end
      return table.concat(s,"+")
   end

local function locale(s)
   return '<span class="locale">uitcp_'..htmlEncode(tostring(s))..'</span>'
end

local alertScript =
   function (s)
      local len = utf8.len(s or "")
      local max=2500
      return s~=nil and script('(oldAlert || alert)('..escapeForScript(fixCRLF(len>max and s:sub(1,utf8.find(s,"%s",max/2) or max/2)..'...\n---- truncated ----\n...'..s:sub(utf8.find(s,"%s",-max/2) or -max/2) or s))..');') or ""
   end

calcValue =
   function (valString, raw)
      if not raw then
         valString = valString:match"^%s*(.-)%s*$" or ''
      end
      if valString == '' or valString == 'nil' then
         valString = nil
      else
         local res, value = pcall(loadstring('setfenv(1, {})return '..valString))
         if not res or value == nil then
            res, valString = pcall(loadstring('setfenv(1, {})return '..
                                              string.format('%q', valString)))
         else
            valString = value
         end
         if not res then valString = nil end
      end
      return valString
   end

replaceNil =
   function (a, ...)
      if(#{...} == 0) then
         return a
      else
         return (a or ""), replaceNil(...)
      end
   end

local fstr =
   function (...)
      local ret, msg = pcall(table.concat,{replaceNil(...)})
      if not ret then
         msg = ""
         for _,v in ipairs({replaceNil(...)}) do
            msg = msg .. tostring(v)
         end
      end
      return msg
   end

local calcValue =
   function (valString, raw)
      if not raw then
         valString = valString:match"^%s*(.-)%s*$" or ''
      end
      if valString == '' or valString == 'nil' then
         valString = nil
      else
         local res, value = pcall(loadstring('setfenv(1, {})return '..valString))
         if not res or value == nil then
            res, valString = pcall(loadstring('setfenv(1, {})return '..
                                              string.format('%q', valString)))
         else
            valString = value
         end
         if not res then valString = nil end
      end
      return valString
   end

local urlDecode =
   function(s)
      return (string.gsub (string.gsub (tostring(s or ""), "+", " "),
                           "%%(%x%x)",
                           function (str)
                              return string.char (tonumber (str, 16))
                           end )
           )
   end

htmlEncode =
   function (s)
      return string.gsub( string.gsub (string.gsub (string.gsub (tostring(s or ""), "&", "&amp;"),
                                                    "\"",
                                                    "&quot;"),
                                       "<",
                                       "&lt;"),
                          ">",
                          "&gt;")
   end
local gen_channel_stat =
   function (mod_uita, tunnel, channel)
      local subc;
      uita_con  = mod_uita.con(tunnel)
      if((type(uita_con) ~= 'nil') and (type(uita_con[tunnel]) ~= 'nil')and (type(uita_con[tunnel][tonumber(channel)]) ~= 'nil')) then
         subc=uita_con[tunnel];
      else
         return locale("noSuchObject");
      end

      return fstr("<center><div class=\"ui_tun_name\">",tunnel,":",channel,"</div>",
                  "<table border=1 align=center  class=\"ui_chan_table\">",
                  "<tr><td>",locale("peerHost"),"</td><td>", subc[tonumber(channel)]['peerHost'],"</td></tr>",
                  "<tr><td>",locale("peerPort"),"</td><td>", subc[tonumber(channel)]['peerPort'],"</td></tr>",
                  "<tr><td>",locale("direction"),"</td><td>", subc[tonumber(channel)]['direction'],"</td></tr>",
                  "<tr><td>",locale("host"),"</td><td>", subc[tonumber(channel)]['host'],"</td></tr>",
                  "<tr><td>",locale("port"),"</td><td>", subc[tonumber(channel)]['port'],"</td></tr>",
                  "</table></center>");
   end

local gen_summary_stat =
   function (mod_uita, htmlEnv, tuns, con)
      local get_prev_interval =
         function (i)
            if (i==1)
            then
               return 96
            else
               return i-1
            end
         end
      local addStat =
         function(to, from)
            local values = {
               'recvPack',
               'recvBytes',
               'sendPack',
               'sendBytes',
               'sendErr',
            }

            -- from start
            to = to or {}
            if from then
               for _, k in ipairs(values) do
                  to[k] = (to[k] or 0) + (from[k] or 0)
               end
               -- 24
               to.day = to.day or {}
               if from.day and from.currentTimeInterval and from[from.currentTimeInterval] then
                  for _, k in ipairs(values) do
                     to.day[k] = (to.day[k] or 0) + (from.day[k] or 0)
                  end
                  -- current
                  to.current = to.current or {}
                  local current = from.currentTimeInterval
                  for _, k in ipairs(values) do
                     to.current[k] = (to.current[k] or 0) + (from[current] and from[current][k] or 0)
                  end
                  -- previous
                  to.previous = to.previous or {}
                  local previous = get_prev_interval(from.currentTimeInterval);
                  for _, k in ipairs(values) do
                     to.previous[k] = (to.previous[k] or 0) + (from[previous] and from[previous][k] or 0)
                  end
                  to.currentTimeInterval = current
                  to[current] = to.current
                  to[previous] = to.previous
               end
            end
            return to
         end

      local out={}
      out[#out+1] = fstr("<table class=\"ui_state_table\"><caption>",locale("systemUsage"),"</caption><tr><td><pre>",
                         (string.gsub(readpopen("top -b -n1 | head -n $((`top -b -n1 | grep -nEm1  PI[D] | grep -oE '^[[:digit:]]*'`-1))") or "","\n","<br>")),
                         "</pre></td></tr></table>")

      local dead = 0
      local normal = 0
      local suspended = 0
      
      for _,k in ipairs(tuns) do
         dead = dead + ((not con[k] or not con[k].currState or (con[k].currState=="dead")) and 1 or 0)
         normal = normal + (con[k] and con[k].currState=="normal" and 1 or 0)
         suspended = suspended + (con[k] and con[k].currState=="suspended" and 1 or 0)
      end
      local sum =  mod_uita.stat(true)
      

      -- ////////  USE   sum as summary :)
      out[#out+1] = table.concat(gen_interval_stats(sum, sum.tun.SUMMARY, locale('summaryDataTransferStats')))

      return "<center>"..table.concat(out).."</center>"
   end

local make_diff_time =
   function (t)
      local ret="";
      t=tonumber(t)
      if(type(t)~='number') then
         return "Error"
      end
      ret =  (math.floor(t / (60*60*24)) >0 and tostring(math.floor(t / (60*60*24))).."&nbsp;" or "") .. os.date("!%H:%M:%S",t)
      return ret
   end

local  make_time =
   function (t)
      local ret="";
      t=tonumber(t)
      if(type(t)~='number') then
         return "Error"
      end
      ret = os.date("!%H:%M:%S",t)
      if ( t >= 60*60*24 ) then
         ret=os.date("!%d&nbsp;",t)..ret;
      end;
      if ( t >= 30*60*60*24 ) then
         ret=os.date("!%m.",t)..ret;
      end;
      if ( t >= 365*60*60*24 ) then
         ret=os.date("!%Y.",t)..ret;
      end;
      return ret
   end

gen_interval_stats =
   function (uita_stat, tun_stat, caption)
      local get_next_interval =
         function (i)
            if (i==96)
            then
               return 1
            else
               return i and (i+1) or nil
            end
         end

      local get_prev_interval =
         function (i)
            if (i==1)
            then
               return 96
            else
               return i and (i-1) or nil
            end
         end

      local int_stat = {}
      int_stat[#int_stat+1] = fstr('<table class="ui_int_table"><caption colspan=7>',
                                   (caption or locale('dataTransferStats')),
                                    '</caption>')

      int_stat[#int_stat+1] = fstr("<tr><th colspan=2>",locale("interval"),"</th><th>",
                                   locale("recv"),'<br/>',locale("pack"),
                                   "</th><th>",
                                   locale("recv"),'<br/>',locale("bytes"),
                                   "</th><th>",
                                   locale("send"),'<br/>',locale("pack"),
                                   "</th><th>",locale("send"),'<br/>',locale("bytes"),
                                   "</th><th>",locale("send"),'<br/>',locale("err"),
                                   "</th></tr>")

 	    local i_time = math.floor(uita_stat['time']/(15*60)) * (15*60);
 	    --
	    local k = tun_stat['currentTimeInterval'];
      if(k and type(tun_stat[k]) == 'table') then
	       int_stat[#int_stat+1] = fstr(
            "<tr><td>",locale("currentInt"),"</td><td>",
            make_time(i_time),"<br/>",
            make_time(uita_stat['time']),"<br>",
            make_diff_time(uita_stat['time'] - i_time),
            "<td>",tun_stat[k]['recvPack'] or '0',"</td>",
			      "<td>",tun_stat[k]['recvBytes'] or '0',"</td>",
			      "<td>",tun_stat[k]['sendPack'] or '0',"</td>",
			      "<td>",tun_stat[k]['sendBytes'] or '0',"</td>",
			      "<td>",tun_stat[k]['sendErr'] or '0',"</td>",
            "</tr>"
         )
	    end
      k=get_prev_interval(k);
      if(k and type(tun_stat[k]) == 'table') then
         int_stat[#int_stat+1] = fstr(
            "<tr><td>",locale("previous"),"</td><td>",
            make_time(i_time  -  15*60),"<br/>",
            make_time(i_time),"<br>",
            make_diff_time( 15*60),
            "<td>",tun_stat[k]['recvPack'] or '0',"</td>",
            "<td>",tun_stat[k]['recvBytes'] or '0',"</td>",
            "<td>",tun_stat[k]['sendPack'] or '0',"</td>",
            "<td>",tun_stat[k]['sendBytes'] or '0',"</td>",
            "<td>",tun_stat[k]['sendErr'] or '0',"</td>",
            "</tr>"
         )
      end
      -----------

      if tun_stat and tun_stat["day"] then
         int_stat[#int_stat+1] = fstr(
            "<tr><td>",locale("last24Hours"),"</td><td>",
            make_time(i_time - 3600*24 + 15*60),
            "<br/>",
            make_time(uita_stat['time']),"<br>",
            make_diff_time(uita_stat['time']-(i_time - 3600*24 + 15*60)),
            "<td>",tun_stat["day"]['recvPack'] or '0',"</td>",
            "<td>",tun_stat["day"]['recvBytes'] or '0',"</td>",
            "<td>",tun_stat["day"]['sendPack'] or '0',"</td>",
            "<td>",tun_stat["day"]['sendBytes'] or '0',"</td>",
            "<td>",tun_stat["day"]['sendErr'] or '0',"</td>",
            "</tr>"
         )
      end

      --
      int_stat[#int_stat+1] = fstr(
         "<tr><td>",locale("fromStart"),"</td><td>",
         make_time(uita_stat['startTime']),
         "<br/>",
         make_time(uita_stat['time']),"<br>",
         make_diff_time(uita_stat['time'] - uita_stat['startTime']),
         "<td>",tun_stat['recvPack'] or '0',"</td>",
         "<td>",tun_stat['recvBytes'] or '0',"</td>",
         "<td>",tun_stat['sendPack'] or '0',"</td>",
         "<td>",tun_stat['sendBytes'] or '0',"</td>",
         "<td>",tun_stat['sendErr'] or '0',"</td>",
         "</tr>"
      )

      int_stat[#int_stat+1] = "</table>"

	    ----------------------------
	    -----------table stat
      return int_stat

   end

local function gen_tunnel_stat(mod_uita, tun, htmlEnv)
   local   function get_color_from_state( state)
           return (tostring(state) == "dead" and "red" or
                         (tostring(state) == "normal" and "green" or
                                (tostring(state) == "suspended" and "blue" or "grey")))
   end

   local stat={};
   local int_stat={};
   local links_stat={"<table class=\"ui_link_table\"><caption colspan=3>",locale("ifaceUsageStats"),"</caption>"};
   local state_stat={"<table class=\"ui_state_table\"><caption colspan=3>",locale("tunnelStatesStats"),"</caption>"};

   local tun_stat = mod_uita.stat(tun)
   local uita_stat = tun_stat
   if(type(tun)=="string") then
      tun_stat=tun_stat and tun_stat['tun'] and tun_stat['tun'][tun];
      if(type(tun_stat)=='table') then
         htmlEnv.data["interval"] = 88;
         ----table  link
         local links=tun_stat['links'];
         links_stat[#links_stat+1] = fstr("<tr><th>",locale("link"),"</th><th>",locale("duration"),"</th><th>",locale("quantity"),"</th></tr>")
         if(type(links)=='table') then
            if (type(links[tun_stat['link']]) ~= 'nil') then
               links[tun_stat['link']]['duration'] = links[tun_stat['link']]['duration'] + uita_stat['time'] -tun_stat['currStateTime'];
               links[tun_stat['link']]['quantity'] = links[tun_stat['link']]['quantity'] + 1;
            else
               if(type(tun_stat['link']) == 'nil') then
                  tun_stat['link']= locale('none')
               end;
               links[tun_stat['link']]={};
               links[tun_stat['link']]['duration'] = uita_stat['time'] - tun_stat['currStateTime'];
               links[tun_stat['link']]['quantity'] =  1;
            end;


            for i,v in pairs(links) do
               if(type(v)=='table') then
                  links_stat[#links_stat+1] = fstr(
                     "<tr><td>",
                     tostring(i),
                     tostring(i) == (tun_stat['link'] or locale('none')) and  ("("..locale("current")..")") or "",
                     "</td><td>",
                     make_diff_time(v['duration'] or '0'),
                     "</td><td>",
                     tostring(v['quantity'] or '0'),
                     "</td></tr>"
                  )
               end
            end
            end
            links_stat[#links_stat+1] = "</table>"

            ------------
            ----table state
            local state=tun_stat['state'];
            state_stat[#state_stat+1] = fstr("<tr><th>",locale("state"),"</th><th>",locale("duration"),"</th><th>",locale("quantity"),"</th><tr>")

            if(type(state)=='table') then


	       if (type(state[tun_stat['currState']]) ~= 'nil') then
            state[tun_stat['currState']]['duration'] = state[tun_stat['currState']]['duration'] + uita_stat['time'] -tun_stat['currStateTime'];
            state[tun_stat['currState']]['quantity'] = state[tun_stat['currState']]['quantity'] + 1;
	       else
            state[tun_stat['currState']]={};
            state[tun_stat['currState']]['duration'] = uita_stat['time'] - tun_stat['currStateTime'];
            state[tun_stat['currState']]['quantity'] =  1;
	       end;

	       for i,v in pairs(state) do
            if(type(v)=='table') then
               state_stat[#state_stat+1] = fstr(
                  "<tr><td>",
                  locale(tostring(i)),
                  tostring(i) == tun_stat['currState'] and "("..locale("current")..")" or "",
                  "</td><td>",
                  make_diff_time(v['duration']),
                  "</td><td>",
                  tostring(v['quantity']),
                  "</td></tr>"
               )
            end
         end
      end
      state_stat[#state_stat+1] = "</table>"

      ----------------
      -------table intervals
      int_stat = gen_interval_stats(uita_stat, tun_stat)

      ----------------------------
	    -----------table stat

       stat[#stat+1] = fstr('<table class="ui_stat_table"><caption colspan=3>',
                            locale("tunnelParameters"),
                            '</caption><tr><td>',locale('link'),'</td>',
                            "<td>",tun_stat['link'] or locale('none'),"</td></tr>",
                            "<tr><td>",locale("peer"),"</td><td>",
                            tun_stat['peer'] or locale('none'),"</td></tr>",
                            "<tr><td>",locale("state"),"</td><td><font ",
                         "color=\"", get_color_from_state(tun_stat['currState']),"\">",
                         locale(tun_stat['currState']),"</font></td></tr>",
                         "<tr><td>",locale("currStateTime"),"</td><td>",
                         make_diff_time(uita_stat['time']-tun_stat['currStateTime']),
                         "</td></tr>",
                         "<tr><td>",locale("time"),"</td><td>",make_time(uita_stat['time']),"</td></tr>",
                         "<tr><td>",locale("csq"),"</td><td>",(tun_stat.peerData and tun_stat.peerData.csq) or locale('none'),
                         "</td></tr></table>")

	    ----------------------------------

  end
end
      return  fstr("<center><div class=\"ui_center\"><div class=\"ui_tun_name\">",
                   tun,
                   (tun_stat.description and " - " or ""),
                   (tun_stat.description or ""),"</div>",
                   table.concat(stat),
                   table.concat(links_stat),
                   table.concat(state_stat),
                   --gen_stat_buttons(),
                   table.concat(int_stat),"</div></center>"
                );
   end;



_L = {
   start = function (htmlEnv)
              return true, fstr([[<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
                                <html>
                                <head>
                                <title>NSG uiTCP Client/Server Viewer</title>
                                       <meta http-equiv="X-UA-Compatible" content="IE=7" />
                                <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
                                <script src="/jquery.js?]]..version..[[" type="text/javascript" ></script>
                                <script src="/form.js?]]..version..[[" type="text/javascript" ></script>
                                <script src="/tinysort.js?]]..version..[[" type="text/javascript" ></script>
                                <script src="/boxover.js?]]..version..[[" type="text/javascript" ></script>
                                <link rel="stylesheet" type="text/css" href="/style.css?]]..version..[[">
                                <link rel="stylesheet" type="text/css" href="/uitcpstyle.css?]]..version..[[">
                                <link rel="icon" type="image/x-icon" href="/nsg.ico">
                                <link rel="shortcut icon" type="image/x-icon" href="/nsg.ico">
                                </head>]],
                       "\n<script type=\"text/javascript\">\nvar NSGVERSION=\"",version:gsub("[^%w]*",""),"\";var PATH=",
                       ("%q"):format(htmlEnv.data.path),
                       ";\nvar LOGIN=\"",
                       htmlEnv.data.login,
                       "\";\nvar PASS=\"",
                       htmlEnv.data.pass,
                       "\";\nvar LOCALE=\"",
                       htmlEnv.data.locale,
                       "\";\nvar MODULE=\"",
                       htmlEnv.data.module,
                       "\";\nvar AUTH=\"locale=\"+LOCALE+\"&module=\"+MODULE+\"&login=\"+LOGIN+ \"&pass=\" + PASS +\"&path=\" + PATH + \"&maxRights=user&\";\nvar PAGENAME=\""..htmlEnv.data.pagename.."\";\nvar CONTENT=\"\";\nvar TIMER=null;\nvar REQUEST=null;\n</script>\n",
                       "<script src=\"/nsghelp.js?"..version.."\"   type=\"text/javascript\"></script>\n",
                       "<script src=\"/script.js?"..version.."\"   type=\"text/javascript\"></script>\n",
                       "<script src=\"/uitcpweb.js?"..version.."\" type=\"text/javascript\"></script>\n",
                       [[<frameset rows="26,*" cols="*" onload="onLoad();" bordercolor="silver" border="0" framespacing="0" >
                                <frame id="top" name="top" scrolling="no" noresize src="/frame.html?"..version.."" frameborder="no" >
                                <frameset rows="*" cols="20%,*"  border="4" framespacing="4"  bordercolor="silver">
                                 <frame id="left" name="left"  scrolling="auto" src="/frame.html?"..version.."" >
                                 <frame id="right" name="right" scrolling="auto" src="/frame.html?"..version..""  >
                                </frameset>
                              </frameset>
]]
                    )
        end,

  update = function (htmlEnv)
               local oldMenu = calcValue(urlDecode(htmlEnv.data.menu or "{}"))
               local out = {}
	       local res, mod_uita = pcall(require, "uitar")
	       if not res then 
		 mod_uita = require"uita"
	       end
               local con = mod_uita.con()
               local tuns = mod_uita.tun()
               local sort = {};
               if not tuns then
                  return true,'(oldAlert || alert)("Can`t get stat. May be uiTCP is not started yet?");'
               end


               if type(htmlEnv.data.content)=='string'then
                  local content = calcValue(htmlEnv.data.content) or {}
                  if type(content) == 'table' then

                     if #content == 0 then
                        out[#out+1] = fstr("NSG.right.body.innerHTML = ",
                                           escapeForScript(gen_summary_stat(mod_uita, htmlEnv, tuns, con)),
                                           ";")
                     elseif #content == 1 then
                        out[#out+1] = fstr("NSG.right.body.innerHTML = " ,
                                           escapeForScript(gen_tunnel_stat(mod_uita,content[1], htmlEnv)),
                                           ";")
                     elseif #content == 2 then
                        out[#out+1] = fstr("NSG.right.body.innerHTML =  " ,
                                          escapeForScript(gen_channel_stat(mod_uita, content[1],content[2])),
                        ";")
                     end
                  else
                     out[#out+1] =  "parent.frames['right'].document.body.innerHTML =  "..("%q"):format(htmlEnv.data.content)..";"
                  end
                 -- out[#out+1] =  "$(\"strong\")[0].innerHTML = "..("%q"):format(htmlEnv.data.content..":"..tostring(content).."#".. tostring(#content))..";"
               end

               for _,k in ipairs(tuns) do
                  sort[#sort + 1] = tostring(k)
                  con[k] = con[k] or {currState="dead"}
               end
               table.sort(sort);
               for k,tun in ipairs(sort) do
                  local tunName = escapeForScript(tun)
                  oldMenu[tun]=oldMenu[tun] or {}
                  if con[tun]["currState"]~= oldMenu[tun]["currState"] then
                     out[#out+1] = fstr('updTun(',tunName,',"',
                                        tostring(con[tun]["currState"]),'");')
                  end
                  for k,ch in pairs(con[tun]) do
                     if type(ch) == "table" then
                        if oldMenu[tun][k] == nil or
                           ch.host    ~=oldMenu[tun][k].host or
                           ch.port    ~=oldMenu[tun][k].port or
                           ch.peerPort~=oldMenu[tun][k].peerPort or
                           ch.peerHost~=oldMenu[tun][k].peerHost or
                           ch.direction~=oldMenu[tun][k].direction
                        then
                           out[#out+1] =  fstr('updTunCh(',tunName,',"',
                                               tostring(k),'","',
                                               tostring(ch.host),'","',
                                               tostring(ch.port),'","',
                                               tostring(ch.peerHost),'","',
                                               tostring(ch.peerPort),'","',
                                               tostring(ch.direction=="to" and ">" or "<"),'");')
                        end
                        oldMenu[tun][k]= false
                     end
                  end
                  for k,ch in pairs(oldMenu[tun]) do
                     if type(ch) == "table" then
                        out[#out+1] =  fstr('delTunCh(',tunName,',"',
                                            tostring(k),'");')
                     end
                  end
                  oldMenu[tun] = nil
               end
               for tun in pairs(oldMenu) do
                  out[#out+1] =  'delTun('..escapeForScript(tun)..');'
               end
               return true, table.concat(out,"\n")
            end,

}


-- Entry point for call from web

function getHTML(htmlEnv)
   if _L[htmlEnv.data.subcmd]  then
      htmlEnv.data.mode = 'html'
      return (_L[htmlEnv.data.subcmd])(htmlEnv)
   else
      return nil, "Can`t find function `"..htmlEnv.data.subcmd.."` in module `uitcpweb`."
   end
end


