module(..., package.seeall)

--- Return count of printable symbols include spaces
-- @param s utf-8 string
len = function(s)
                   local _, len = s:gsub("[^\128-\193]", "")
                   return len
                end

--- Return reversed version of input string
-- @param s utf-8 string
reverse = function(s)
                       local rev=""
                       for uchar in s:gfind("([%z\1-\127\194-\244][\128-\191]*)") do
                          rev=uchar..rev
                       end
                       return rev
                    end

--- Return substring of input string, like string.sub but using utf-8 instead of ASCII
-- @param s utf-8 string
-- @param st start of substring
-- @param en end of substring
sub = function(s, st, en)
                   local len = len(s)
                   st = st<0 and len+st or st
                   en = en and (en<0 and len+en or en) or len
                   st = st<1 and 1 or st
                   en = en>len and len or en
                   if st>len then return "" end
                   if en<1 then return "" end

                   local ret={}
                   local i=0
                   for uchar in s:gfind("([%z\1-\127\194-\244][\128-\191]*)") do
                      i=i+1
                      if i>=st and i<=en then
                         ret[#ret + 1] = uchar
                      end
                      if i>=en then
                         return table.concat(ret)
                      end
                   end
                   return ""
                end

--- Return !!! BYTE offset NOT utf8 of pattern in input string
-- @param s utf-8 string
-- @param p search pattern
-- @param i initial utf8 offset
find = function(s,p,i)
                    local len = len(s)
                    i = i<0 and len+i or i
                    i = i<1 and 1 or i
                    if i> len then return nil end
                    local sEnd = sub(s,i)
                    local sSta = sub(s,1,i-1)
                    local a,b,c = sEnd:find(p)
                    return (a and (a+#sSta) or nil),(b and (b+#sSta) or nil),c
                 end
