module(..., package.seeall)
--just helps
local helps = {
   _H = "webJobs",
   _H = "pleaseWait",
   _H = "runFirst",
   _H = "Welcome to NSG Linux",
}

local nsg = require'nsg'
local mod_scheme = require"scheme"
local deepCopy = nsg.deepcopy
local clonePath = mod_scheme.clonePath
local ser = nsg.serialize
local utf8 = require"utf8"
local locale = "en"
localeString =  mod_scheme.localeString

local safe
local ifThen
local fstr
local genDir
local genParam
local isExpanded
local expandedOrCollapsed
local normalOrDisabled
local genLiClass
local genParamName
local genParamValue
local genParamControls
local genDirControls
local genDirBlock
local genDirItem

local mainPage
local htmlEncode
local escapeForScript
local replaceNil
local toTypeString
local findSchemeNode
local findSchemeKey
local strongValue
local genInput
local splitString
local xmlConf

splitString =
   function (s)
      s = s or ""
      local out = {}
      local i = 70
      local i2 = 0
      local len = utf8.len(s) or 0
      while i2<len do
         out[#out+1] = utf8.sub(s,i2 + 1, i2 + i)
         i2 = i2 + i
      end
      return out
   end

escapeForScript =
   function (s)
      s = splitString(s)
      for k,v in ipairs(s) do
         s[k] = '"'..((((((tostring(v or "" )):gsub("\\","\\\\")):gsub("\r","\\r")):gsub("\n","\\n")):gsub('"','\\"')):gsub("'","\\'") or "")..'"'
      end
      s[#s+1] = '""'
      return table.concat(s,"+")
   end

htmlEncode =
   function(s)
      if type(s) == 'boolean' then
         return tostring(s)
      end
      return ((((tostring(s) or ""):gsub( "&", "&amp;")):gsub("\"", "&quot;")):gsub("<", "&lt;")):gsub(">", "&gt;")
   end

replaceNil =
   function (a, ...)
      if(#{...} == 0) then
         return a
      else
         return (a or ""), replaceNil(...)
      end
   end

fstr =
   function (...)
      return table.concat({replaceNil(...)})
   end


ifThen =
   function (test, ontrue, onfalse)
      if(test) then
         return ontrue
      else
         return onfalse
      end
   end

toTypeString =
   function (...)
      local out = {}
      for k,v in pairs({...}) do
         table.insert(out,
                      fstr(tostring(k),
                           ":",
                           type(v),
                           "(",
                           ifThen(type(v)=='table',ser(v or {}),tostring(v))
                           ,")"
                        )
                   )
      end
      return table.concat(out,", ")
   end

local debugBuf = ""
local lastRet = nil

xsafe =
   function (func, funcName, ...)
      return func(...)
   end

safe =
   function (func, funcName, ...)
      local res = { pcall(func, ...) }
      if res[1] then
         table.remove(res,1)
         lastRet = res
         return unpack(res)
      else
         return error(
            fstr(funcName,
                 "=",
                 type(func),
                 "(",
                 toTypeString(...),
                 ")-><br>lastRet=",
                 toTypeString(lastRet),
                 "<br>",
                 toTypeString(debugBuf),
                 "<br>",
                 tostring(res[2])
              )
         )
      end
   end


local specChar = {
   ["\a"] = "\\a",
   ["\b"] = "\\b",
   ["\t"] = "\\t",
   ["\r"] = "\\r",
   ["\n"] = "\\n",
   ["\v"] = "\\v",
   ["\f"] = "\\f"
}

strongValue =
   function (conf, scheme)
      local val = conf
      if conf == nil then
         if scheme._D == nil then
            if scheme._T == "boolean" then
               val =  "false"
            else
               val = ""
            end
         else
            if scheme._T == "boolean" then
               val =  tostring(scheme._D)
            else
               val = scheme._D
            end
         end
      end
      if type(conf or scheme._D) == "string" and (scheme._T ~= "choice") then
         val = ("%q"):format(val):gsub("\\\n","\\n"):gsub("[%c%z]", function (char) return specChar[char] or ("\\%.3d"):format((char:byte())) end)
      end
      return tostring(val)
   end


isExpanded =
   function (htmlEnv, path)
      local s="LC"
      if  #path>0 then
         for i=1,#path do
            s = fstr(s ,".",type(path[i])=="string" and  ("%q"):format(path[i]) or tostring(path[i]))
         end
      end
      if #path==0 or (htmlEnv and (htmlEnv.data[s]))then
         return true
      else 
         return false
      end
   end

xmlConf =
   function (xml, conf, confParent, scheme, schemeParent, path, htmlEnv, level, sortIndex)
      sortIndex = sortIndex or 0
      xml.path = ser(path)
      if type(scheme) ~= 'table' then
         xml.name ="Error:scheme is not 'table'"
         return xml
      end
      level =  isExpanded(htmlEnv, path) and 1 or level
      xml.luaName = ser(path[#path] or "root")
      xml.name = path[#path] or "root"
      xml.prio = sortIndex
      xml.type = scheme._T or 'directory'
      if xml.type == 'choice' then
         xml.arguments = {}
         for _ ,v in ipairs(scheme._A) do
            xml.arguments[#xml.arguments + 1 ] = type(v) == 'table' and v[1] or v
         end
      elseif xml.type == 'boolean' then
         xml.arguments = {'true', 'false'}
      end
      xml.executable = scheme._X and 'true' or nil
      xml.interactive = scheme._XI and 'true' or nil
      xml.default = scheme._D or nil
      xml.help = scheme._H or nil
      xml.apply = scheme._Z and 'true'
      xml.plus = scheme._LIST and 'insert' or scheme._NAME and 'new' or nil
      xml.plusHelp = scheme._LIST and 'webDlgAddList' or scheme._NAME and (scheme._NAME_H or 'webDlgAddName') or nil
      xml.html = scheme._HTML or nil
      xml.dependence = scheme._DEP or nil
      xml.value = strongValue(conf, scheme)
      xml.configured = scheme._T and conf or nil
      xml.class = level>0 and 'ex' or 'cl'
      xml.description = type(conf)=='table' and type(conf.description) =='string' and conf.description
      if (level < 1) then
         xml.loaded = 'no'
         return xml
      end

      conf = conf or {}

      local newPath = clonePath(path, nil, "")
      local confScheme = mod_scheme.confScheme(scheme or {}, conf or {}, path)
      -- Generate sub items of DIR
      local iter = confScheme._pairs or pairs
      for k,v in iter(confScheme or {}, path) do
         if not (tostring(k):find('^[_].*$')) then
            newPath[#newPath] = k
            local subsch = v(nil, newPath)
            if subsch then
               xml.parameter = xml.parameter or {}
               sortIndex = sortIndex + 1
               xml.parameter[#xml.parameter + 1] = safe(xmlConf, "xmlConf", {}, conf[k], conf, subsch, scheme, newPath, htmlEnv, level-1, sortIndex)
            end
         end
      end
      return xml
   end

mainPage =
   function (xml, conf, confParent, scheme, schemeParent, htmlEnv)
      local release = C.getVersion()
      xml.head = {

         title ='NSG Web Configuration Manager',
         initScript = fstr('window.NSGVERSION="',release:gsub("[^%w]*",""),'";',
            'window.activity=0;window.webMode="xml";\nwindow.SID="',
            htmlEnv.data.sesnum,
            '";\nwindow.LOGIN=',
            escapeForScript(htmlEnv.data.login),
            ';\nwindow.PASS=',
            escapeForScript(htmlEnv.data.pass),
            ';\nwindow.AUTH="mode=xml&login="+encodeURIComponent(LOGIN)+ "&pass=" + encodeURIComponent(PASS) +"&sesnum="+ SID +"&";\nwindow.PAGENAME="',
            htmlEnv.data.pagename,
            '";\nwindow.LOCALE="',
            locale,'";\n'),
         welcome = mod_scheme.localeString("Welcome to NSG Linux",htmlEnv.data.locale),
         release = release,
         mach = BI and BI.mach or nil,
         sessionMessage = (htmlEnv.data.sessionMessage>'') and htmlEnv.data.sessionMessage or nil,
         hostName  = io.popen("hostname", "r"):read("*l"),
         login = htmlEnv.data.login,
         sessionNumber = htmlEnv.data.sesnum,
         locale = locale,
         usage = htmlEnv.usage,
      }

      xml.parameter = {
         [1] = xmlConf({}, conf, confParent, scheme, schemeParent, {}, htmlEnv, 1)
      }

      return xml
   end

----------------------------------------------------------------------
--  global methods

function getConf(args)
   locale = args.locale
   if(args.par[1] == false) then
      -- text mode
      local conf = args.getConfCommon(args) or ""
      return nsg.xmlize("EDIT", {text = conf }, 1)
   else
      -- tree mode
      if args.virtPath then
         --sub tree
         collectgarbage("collect")
         local xml={}
         return nsg.xmlize("CONF",xmlConf(xml, args.conf, args.confParent, args.sch, args.schParent, args.path or args.virtPath, args.par[1].htmlEnv, 1), 1)
      else
         --root tree
         local xml={}
         collectgarbage("collect")
         args.par[1].htmlEnv.usage = args.getUsage()
         return nsg.xmlize("ROOT",mainPage(xml, args.conf, args.confParent, args.sch, args.schParent, args.par[1].htmlEnv),1)
      end
   end
end

function setValue(args)
   locale = args.locale
   local res, err, val
   if args.val == nil then
      res, err, val = true, "", nil 
   else
      res, err, val = args.setValueCommon(args)
   end
   local xml = {}
   local conf = C.getConfByPath(args.path, args.sch._LOCAL and args.sesNum)
   if res then
      return true, res, nsg.xmlize("ITEM",xmlConf(xml, val, args.confParent, args.sch, args.schParent, args.path or args.virtPath, nil, 1),1)
   else
      local conf = C.getConfByPath(args.path, args.sch._LOCAL and args.sesNum)
      return false, res==nil and "Error:Incorrect value: Expected: "..err or err, nsg.xmlize("ITEM",xmlConf(xml, conf, args.confParent, args.sch, args.schParent, args.path or args.virtPath, nil, 1), 1)  or err or ""
   end
end

function executeTask(args)
   locale = args.locale
   return args.executeTaskCommon(args)
end

function newKey(args)
   locale = args.locale
   local path = htmlEncode("root."..table.concat(args.path,"."))
   local ret, err = args.funcCommon(args)
   if ret then
      return ret, nil 
   else
      return ret, err
   end
end

function insertKey(args)
   locale = args.locale
   local path = htmlEncode("root."..table.concat(args.path,"."))
   local ret, err = args.funcCommon(args)
   if ret then
      return ret, nil 
   else
      return ret, err
   end
end

function removeKey(args)
   locale = args.locale
   local ret, err = args.funcCommon(args)
   args.path[#args.path] = nil
   local path = htmlEncode("root."..table.concat(args.path,"."))

   if ret then
      return ret, nil 
   else
      return ret, err
   end
end

function getManual(args)
   return args.man
end
